/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileSystemView;

import dialogs.MtgFileView;
import entreesSorties.MathGraphDataOutputStream;
import entreesSorties.MtgFileManager;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class ActionFichierEnregistrerSous extends MtgAction {
	public ActionFichierEnregistrerSous(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuFichierEnregistrerSous"), null, null, null);
	}

	public void actionPerformed(ActionEvent e) {
		enregistreSous();
	}
	
	/**
	 * Fonction enregistrant la figure sous un autre nom et renvoyant true seulement si l'enregisrement a t accept
	 * et s'est bien pass
	 * @return
	 */	
	public boolean enregistreSous() {
		class MyFilter extends javax.swing.filechooser.FileFilter {
			public boolean accept(File file) {
				if (file.isDirectory())
					return true;
				String filename = file.getName().toLowerCase();
				return filename.endsWith(Global.GLOBAL_COMMA_MGJ)
					|| filename.endsWith(Global.GLOBAL_COMMA_MGF);
			}

			public String getDescription() {
				return Global.GLOBAL_STAR_COMMA_MGJ;
			}
		}
		MtgInternalFrame cadre = frame.cadreActif();
		JFileChooser fc;
		if (MtgFrame.repertoireFichiers.isEmpty()) {
			FileSystemView vueSysteme = FileSystemView.getFileSystemView(); 
			File defaut = vueSysteme.getDefaultDirectory(); 
			fc = new JFileChooser(defaut);
		}
		else {
			fc = new JFileChooser(MtgFrame.repertoireFichiers);
		}
  	if (cadre.path != null) {
  		if (cadre.path.toLowerCase().endsWith(Global.GLOBAL_COMMA_MGF)) 
  			cadre.path = cadre.path.substring(0, cadre.path.length() - 3) + Global.GLOBAL_MGJ;
  		File file = new File(cadre.path);
  		fc.setSelectedFile(file);
  	}
  	else {
  		File file = new File(MtgFrame.repertoireFichiers + File.separator + cadre.nomDocument() + Global.GLOBAL_COMMA_MGJ);
  		fc.setSelectedFile(file);
  	}
		fc.setFileView(new MtgFileView());
		fc.setFileFilter(new MyFilter());
    // int rVal = fc.showSaveDialog(frame); // Modifi version 4.7 spcial applet
		int rVal = fc.showSaveDialog(null);
    if(rVal == JFileChooser.APPROVE_OPTION) {
  		File file = fc.getSelectedFile();
			String ch = file.getAbsolutePath();
			if (!ch.toLowerCase().endsWith(Global.GLOBAL_COMMA_MGJ)) {
				ch = ch + Global.GLOBAL_COMMA_MGJ;
				file = new File(ch);
			}
			// cadre.path = ch;
			// cadre.setTitle(cadre.nomDocument());
			int ind = ch.lastIndexOf(File.separatorChar);
			MtgFrame.repertoireFichiers = ch.substring(0, ind);
			// On regarde si un fichier du mme nom existe dj.
			// Si oui on demande confirmation pour l'crasement
			if (!MtgFileManager.accepteEnregistrement(frame, ch)) return false;
				
			try {
				FileOutputStream fos = new FileOutputStream(file);
				MathGraphDataOutputStream oups = new MathGraphDataOutputStream(fos);
				try {
					try { // Rajout version 4.6.2
						cadre.document.write(oups);
					}
					finally {
						oups.close();
					}
					cadre.document.setDirty(false);
				} catch (IOException e1) {
					frame.dialogueAvertissement(getString("ch45"));
					return false;
				}
			} catch (FileNotFoundException e2) {
				frame.dialogueAvertissement(getString("ch45"));
				return false;
			}
			cadre.path = ch;
			cadre.setTitle(cadre.nomDocument());			
			cadre.document.estDocumentVersionC = false;
			cadre.document.path = ch;
			/* Supprim version Applet
			for (int i =0; i < frame.menuFenetre.getMenuComponentCount(); i++) {
				JMenuItem item = (JMenuItem) frame.menuFenetre.getMenuComponent(i);
				if (item.getClass() == MtgMenuFileItem.class) {
					MtgMenuFileItem mtgi = (MtgMenuFileItem) item;
					if (mtgi.cadre == cadre) mtgi.setText(cadre.getTitle());
				}
			}
			*/
			return true;
    }
    else return false;
	}
}
