/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import mathgraph32.MtgFrame;
import objets.CAbscisseOrigineRep;
import objets.CListeObjets;

/**
 * Classe cre pour la version 4.5
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class AbscisseOrigineDlg extends MtgDlg {
	private boolean modification; // True si on modifie les coordonnes d'un point dj cr
	private CAbscisseOrigineRep abs;
	private ListeReperes listeReperes;
	private JLabel label1, label2;
  private JButton btnOK, btnCancel;
	private EditeurNomCalcul editeurNom;

  public AbscisseOrigineDlg(MtgFrame frameInit, CAbscisseOrigineRep absInit, boolean modificationInit) {
    super (getString("abscisseOrigineDlg1"), true, frameInit);
    abs = absInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    editeurNom.requestFocus();
  }
  
  public AbscisseOrigineDlg(MtgFrame frameInit, String title, CAbscisseOrigineRep ordInit, boolean modificationInit) {
    super (title, true, frameInit);
    abs = ordInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    editeurNom.requestFocus();
  }
  
	public void initComponents() {
    setResizable(false);
    // Spcial version applet
    // CListeObjets liste = frame.cadreActif().listePr;
    CListeObjets liste = frame.listePourConstruction();
    int indiceObj;
    if (modification) indiceObj = liste.indexOf(abs);
    else indiceObj = -1;
    // listeReperes = new ListeReperes(frame.cadreActif().listePr, indiceObj);
    listeReperes = new ListeReperes(liste, indiceObj);
    if (modification) listeReperes.setSel(abs.rep);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollList = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		label1 = new JLabel(getString("creerCalculDlg1"));
		label2 = new JLabel(getString("choixRepereDlg1"));
		editeurNom = new EditeurNomCalcul();
		editeurNom.setColumns(15);
		if (modification) editeurNom.setEditable(false);
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    getContentPane().setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(editeurNom, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 50, 80, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    getContentPane().add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 50, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    getContentPane().add(scrollList, constraints);
    setConstraints(constraints, 2, 0, 1, 2, 50, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    getContentPane().add(paneRight, constraints);
    if (modification) listeReperes.setSel(abs.rep);
    
    
    
    pack();
	}  

  public void btnOKActionPerformed() {
    // MtgInternalFrame cadre = frame.cadreActif();
    // CListeObjets listePr = cadre.listePr;
    CListeObjets listePr = frame.listePourConstruction(); // spcial applet
    String chNom = editeurNom.getText();
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    if (!modification) abs.donneNom(chNom);
    abs.rep = listeReperes.repereChoisi();
    setRetourOK(true);
    dispose();
  }
  
  public void btnCancelActionPerformed() {
  	dispose();
  }
  
	
}
