
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.NumberFormat;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;


import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalcul;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPrototype;
import objets.CResultatValeur;
import objets.CValeurAssocieeAVariable;
import objets.CValeurDynamique;

@SuppressWarnings("serial")
public class AjoutGraduationRepereDlg extends MtgDlg {
	private boolean graduationTrigo;
	private JLabel label1, label2, label3, label4, label5;
	private EditeurValeurPositiveStrict editeurgradx, editeurgrady;
	private EditeurNombreGraduation editeurnbgradx, editeurnbgrady;
  private CValeurAssocieeAVariable gradx;
  private CValeurAssocieeAVariable grady;
  private CValeurAssocieeAVariable nbgradx;
  private CValeurAssocieeAVariable nbgrady;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions1;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions2;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions3;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions4;
  private ListeReperes listeReperes;
  private JButton btnOK, btnCancel;

  public AjoutGraduationRepereDlg(MtgFrame frameInit, boolean graduationTrigoInit) {
    super (getString("graduationDlg1"), true, frameInit);
    graduationTrigo = graduationTrigoInit;
    initComponents();
    if (graduationTrigo) editeurnbgradx.requestFocus(); else editeurgradx.requestFocus();
    getRootPane().setDefaultButton(btnOK);
  }

	public void initComponents() {
    setResizable(false);
    CListeObjets liste = frame.cadreActif().listePr;
    listeReperes = new ListeReperes(frame.cadreActif().listePr, -1);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollList = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    gradx = new CValeurAssocieeAVariable(liste,1);
    grady = new CValeurAssocieeAVariable(liste,1);
    nbgradx = new CValeurAssocieeAVariable(liste,20);
    nbgrady = new CValeurAssocieeAVariable(liste,20);
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
	  label1 = new JLabel(getString("graduationDlg2"));
	  label2 = new JLabel(getString("graduationDlg3"));
	  label3 = new JLabel(getString("graduationDlg4"));
	  label4 = new JLabel(getString("graduationDlg5"));
	  label5 = new JLabel(getString("choixRepereDlg2"));
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurgradx = new EditeurValeurPositiveStrict(frame, gradx, null);
    editeurgradx.setText("1");
    editeurgrady = new EditeurValeurPositiveStrict(frame, grady, null);
    editeurgrady.setText("1");
    editeurnbgradx = new EditeurNombreGraduation(frame, nbgradx, null);
    editeurnbgradx.setText("20");
    editeurnbgrady = new EditeurNombreGraduation(frame, nbgrady, null);
    editeurnbgrady.setText("20");
    
    int indmax = liste.size() - 1;
    paneListeValeurEtFonctions1 = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurgradx, true, indmax, false);
    paneListeValeurEtFonctions2 = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurnbgradx, true, indmax, false);
    paneListeValeurEtFonctions3 = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurgrady, true, indmax, false);
    paneListeValeurEtFonctions4 = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurnbgrady, true, indmax, false);
    

    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    getContentPane().setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
     
    int lig = 0;
    if (!graduationTrigo) {
    	lig += 2;
	    setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	    getContentPane().add(label1, constraints);
	    setConstraints(constraints, 1, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	    getContentPane().add(editeurgradx, constraints); 
	    setConstraints(constraints, 1, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	    getContentPane().add(paneListeValeurEtFonctions1, constraints); 
    }
    setConstraints(constraints, 0, lig, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label2, constraints);
    setConstraints(constraints, 1, lig++, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(editeurnbgradx, constraints);
    setConstraints(constraints, 1, lig++, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(paneListeValeurEtFonctions2, constraints); 
    setConstraints(constraints, 0, lig, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label3, constraints);
    setConstraints(constraints, 1, lig++, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(editeurgrady, constraints);
    setConstraints(constraints, 1, lig++, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(paneListeValeurEtFonctions3, constraints); 
    setConstraints(constraints, 0, lig, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label4, constraints);
    setConstraints(constraints, 1, lig++, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(editeurnbgrady, constraints);
    setConstraints(constraints, 1, lig++, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(paneListeValeurEtFonctions4, constraints); 
    
    setConstraints(constraints, 2, 0, 1, 3, 30, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(paneRight, constraints);
    setConstraints(constraints, 2, 3, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label5, constraints);
    setConstraints(constraints, 2, 4, 1, 2, 30, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(scrollList, constraints);
    pack();
    
	}
	
  public void btnOKActionPerformed() {
  	CValeurDynamique valeurgradx; // La valeur qui sera associ au premier objet source
  	CValeurDynamique valeurnbgradx; // La valeur qui sera associ au deuxime objet source
  	CValeurDynamique valeurgrady; // La valeur qui sera associ au troisime objet source
  	CValeurDynamique valeurnbgrady; // La valeur qui sera associ au quatrime objet source
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;

		if (graduationTrigo) {		
	    if (!editeurnbgradx.verifieValidite()) {
	    	editeurgradx.requestFocus();
	    	return;
	    }
	    if (!editeurgrady.verifieValidite()) {
	    	editeurgrady.requestFocus();
	    	return;
	    }
	    if (!editeurnbgrady.verifieValidite()) {
	    	editeurnbgrady.requestFocus();
	    	return;
	    }
	  	if (nbgradx.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("nbgradx", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, nbgradx.calcul.chaineCalculSansPar(null), nbgradx.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurnbgradx = calcul;
	  	}
	  	else {
	  		valeurnbgradx = ((CResultatValeur)nbgradx.calcul).valeurAssociee;
	  	}
	  	if (grady.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("grady", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, grady.calcul.chaineCalculSansPar(null), grady.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurgrady = calcul;
	  	}
	  	else {
	  		valeurgrady = ((CResultatValeur)grady.calcul).valeurAssociee;
	  	}
	  	if (nbgrady.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("nbgrady", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, nbgrady.calcul.chaineCalculSansPar(null), nbgrady.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurnbgrady = calcul;
	  	}
	  	else {
	  		valeurnbgrady = ((CResultatValeur)nbgrady.calcul).valeurAssociee;
	  	}
    	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("GraduationReperePourTrigoAvecPas.mgc");
    	//On implmente maintenant ce prototype dans la figure
  		((CElementGenerique)proto.get(0)).elementAssocie = listeReperes.repereChoisi();
  		((CElementGenerique)proto.get(1)).elementAssocie = valeurnbgradx;
  		((CElementGenerique)proto.get(2)).elementAssocie = valeurgrady;
  		((CElementGenerique)proto.get(3)).elementAssocie = valeurnbgrady;
  		CImplementationProto impProto = new CImplementationProto(liste, proto);
  		impProto.implemente(liste, proto, cadre);
  		proto.retireTout(); // Ajout version 4.6.2
  		frame.cadreActif().redessineAvecCalcul();
		}
		else {
	    if (!editeurgradx.verifieValidite()) {
	    	editeurgradx.requestFocus();
	    	return;
	    }
	    if (!editeurnbgradx.verifieValidite()) {
	    	editeurnbgradx.requestFocus();
	    	return;
	    }
	    if (!editeurgrady.verifieValidite()) {
	    	editeurgrady.requestFocus();
	    	return;
	    }
	    if (!editeurnbgrady.verifieValidite()) {
	    	editeurnbgrady.requestFocus();
	    	return;
	    }
	  	if (gradx.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("gradx", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, gradx.calcul.chaineCalculSansPar(null), gradx.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurgradx = calcul;
	  	}
	  	else {
	  		valeurgradx = ((CResultatValeur)gradx.calcul).valeurAssociee;
	  	}
	  	if (nbgradx.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("nbgradx", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, nbgradx.calcul.chaineCalculSansPar(null), nbgradx.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurnbgradx = calcul;
	  	}
	  	else {
	  		valeurnbgradx = ((CResultatValeur)nbgradx.calcul).valeurAssociee;
	  	}
	  	if (grady.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("grady", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, grady.calcul.chaineCalculSansPar(null), grady.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurgrady = calcul;
	  	}
	  	else {
	  		valeurgrady = ((CResultatValeur)grady.calcul).valeurAssociee;
	  	}
	  	if (nbgrady.calcul.getClass() != CResultatValeur.class) {
	  		String nom = liste.genereNomPourCalcul("nbgrady", false);
	  		CCalcul calcul = new CCalcul(liste, null, false, nom, nbgrady.calcul.chaineCalculSansPar(null), nbgrady.calcul);
	  		frame.ajouteElement(calcul);
	  		valeurnbgrady = calcul;
	  	}
	  	else {
	  		valeurnbgrady = ((CResultatValeur)nbgrady.calcul).valeurAssociee;
	  	}
    	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("GraduationAxesRepereAvecPas.mgc");
    	//On implmente maintenant ce prototype dans la figure
  		((CElementGenerique)proto.get(0)).elementAssocie = listeReperes.repereChoisi();
  		((CElementGenerique)proto.get(1)).elementAssocie = valeurgradx;
  		((CElementGenerique)proto.get(2)).elementAssocie = valeurnbgradx;
  		((CElementGenerique)proto.get(3)).elementAssocie = valeurgrady;
  		((CElementGenerique)proto.get(4)).elementAssocie = valeurnbgrady;
  		CImplementationProto impProto = new CImplementationProto(liste, proto);
  		impProto.implemente(liste, proto, cadre);
  		proto.retireTout(); // Ajout version 4.6.2
      cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil154"));
      cadre.repaintFigureAvecCalcul();
      frame.activeOutilCapture();
		}
		setRetourOK(true);
  	dispose();
  
  }
	
  public void btnCancelActionPerformed() {
  	dispose();
  }
  

}
