/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;

import objets.CMacro;
import objets.CMacroAvecListe;
import types.FonteNom;

@SuppressWarnings("serial")
public class ChoixMacroAvecListeDlg extends MtgDlg {
	private JLabel label1, label2, label3;
	private JButton btnOK, btnCancel;
	private ListeMacrosAvecListe listeMacros;
	private JScrollPane scrollPaneMacro;
	private JTextArea infoCommentaire, info;
	private JScrollPane scrollPaneInfoCommentaire, scrollPaneInfo;
	public CMacroAvecListe macroChoisie;
	
  public ChoixMacroAvecListeDlg(MtgFrame frameInit) {
  	super(getString("choixMacroDlg1"), true, frameInit);
  	initComponents();
    getRootPane().setDefaultButton(btnOK);      	
  }
	
  private void initComponents() {
    label1 = new JLabel(getString("choixMacroDlg2"), JLabel.LEFT);
    label2 = new JLabel(getString("InformationSurMacro"), JLabel.LEFT);
    label3 = new JLabel(getString("CommentaireMacro"), JLabel.LEFT);
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    listeMacros = new ListeMacrosAvecListe(frame.cadreActif().listePr);
    listeMacros.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeMacrosSelectionChanged();
      }
    });
    listeMacros.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent evt) {
				listeMacrosMouseClicked(evt);
			}

			public void mouseEntered(MouseEvent evt) {
			}

			public void mouseExited(MouseEvent evt) {
			}

			public void mousePressed(MouseEvent evt) {
			}

			public void mouseReleased(MouseEvent evt) {
			}
    });
    
    scrollPaneMacro = new JScrollPane(listeMacros, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);    
		infoCommentaire = new JTextArea();
		infoCommentaire.setFont(FonteNom.getFonteNom(1, FonteNom.Normal, -1));    
		infoCommentaire.setColumns(20);
		infoCommentaire.setLineWrap(false);
		infoCommentaire.setRows(4);
    scrollPaneInfoCommentaire = new JScrollPane(infoCommentaire, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		info = new JTextArea();
		info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
		info.setColumns(40);
		info.setLineWrap(true);
		info.setRows(2);
    scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneMacro, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfo, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfoCommentaire, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
    listeMacrosSelectionChanged();
  }
  
  private void btnOKActionPerformed() {
  	macroChoisie = listeMacros.macroChoisie();
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void listeMacrosSelectionChanged() {
  	int i = listeMacros.getSelectedIndex();
  	CMacro mac = listeMacros.macroChoisie();
  	if (i != -1) {
  		infoCommentaire.setText(mac.commentaireMacro);
  		infoCommentaire.setCaretPosition(0);
  		info.setText(mac.infoPourHistorique());
  		info.setCaretPosition(0);
  		pack();
  	}
  }
  
  public void listeMacrosMouseClicked(MouseEvent evt) {
  	if ((evt.getClickCount() == 2) && (listeMacros.getSelectedIndex() != -1)) btnOKActionPerformed();
  	}

}
