/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import objets.CBarycentre;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;


import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class CoefficientBarycentreDlg extends MtgDlg {
	private CBarycentre barycentre;
	private boolean modification; // True si on modifie un coefficient de barycentre dj cr
	private CValeurAssocieeAVariable coefficientAssocie;
  private EditeurValeurDynamique editeurCoef;
  private JButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

  public CoefficientBarycentreDlg(MtgFrame frameInit, CBarycentre barycentreInit, CValeurAssocieeAVariable coefficientAssocieInit,
  		boolean modificationInit) {
    super(getString("coefficientBarycentreDlg1"), true, frameInit);
    barycentre = barycentreInit;
    coefficientAssocie = coefficientAssocieInit;
    modification = modificationInit;
    JLabel label1 = new JLabel(getString("coefficientBarycentreDlg2"), SwingConstants.LEFT);
    editeurCoef = new EditeurValeurDynamique(frame, coefficientAssocie, barycentre);
    editeurCoef.setText(coefficientAssocie.chaineInfo());
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    // Modification version 4.6
    CListeObjets liste = frame.listePourConstruction();
  	int indmax;
  	if (modification) indmax = liste.indexOf(barycentre);
  	else indmax = liste.size() - 1;
    btnOK.setPreferredSize(btnCancel.getPreferredSize());
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurCoef, true, indmax, false);
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurCoef, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctions, constraints);

    setConstraints(constraints, 2, 1, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnCancel, constraints);    
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(4);
    order.add(editeurCoef);
    order.add(btnOK);
    order.add(paneListeValeurEtFonctions);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    setContentPane(pane);    
    getRootPane().setDefaultButton(btnOK);  
    editeurCoef.setSelectionStart(0);
    editeurCoef.setSelectionEnd(editeurCoef.getText().length());
    pack();
  }
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
    if (editeurCoef.verifieValidite()) {
    	if (editeurCoef.verifieValidite()) {
        setRetourOK(true);
        dispose();
    	}
    }
  }

  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }    
}
