/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CPointParMultVect;

@SuppressWarnings("serial")
public class CoefficientDlg extends MtgDlg {
	private boolean modification;
  private EditeurValeurDynamique editeurCoef;
  private JButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;
  private CPointParMultVect pointParMultVect;
  
  /** Creates a new instance of RayonDlg */
  public CoefficientDlg(MtgFrame frameInit, CPointParMultVect pointParMultVectInit, boolean modificationInit) {
    super(getString("coefficientBarycentreDlg1"), true, frameInit);
    modification = modificationInit;
    setResizable(false);
    pointParMultVect = pointParMultVectInit;
    JLabel label = new JLabel(getString("coefficientBarycentreDlg2"), SwingConstants.LEFT);
    editeurCoef = new EditeurValeurDynamique(frame, pointParMultVect.coef, pointParMultVect);
    btnCancel = new JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });    
    btnOK = new JButton(getString("btnOK"));
    btnOK.setPreferredSize(btnCancel.getPreferredSize());
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    // Spcial version applet
    // CListeObjets liste = frame.cadreActif().listePr;
    CListeObjets liste = frame.listePourConstruction();
    int indmax;
    if (modification) indmax = liste.indexOf(pointParMultVect) - 1;
    else indmax = liste.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurCoef, true, indmax, false);
    if (modification) editeurCoef.setText(pointParMultVect.coef.chaineInfo());
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
    JPanel paneBas = new JPanel(); // Pour contenir les boutons OK et Cancel
    paneBas.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    pane.add(editeurCoef, constraints);
    constraints.insets.bottom = 5;
    
    setConstraints(constraints, 0, 1, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctions, constraints);
    

    setConstraints(constraints, 0, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneBas.add(btnOK, constraints);

    setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneBas.add(btnCancel, constraints);
    
    setConstraints(constraints, 0, 2, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    pane.add(paneBas, constraints);
    
    setContentPane(pane);
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void btnOKActionPerformed() {
  	// VerifieValidite se charge de mettre la valeur entre dans coef de pointParMultVec
  	if (editeurCoef.verifieValidite()) {
      setRetourOK(true);
      dispose();  		
  	}
  }

}
