/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.DimensionFenetre;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class CurseurDlg extends MtgDlg {

	public JRadioButton radioHorizontal;
	public JRadioButton radioVertical;
	private ButtonGroup btnGroup;
	// Ajout version 4.7.2
	public JRadioButton radioPteTaille;
	public JRadioButton radioGdeTaille;
	private ButtonGroup btnGroup2;
	public JCheckBox checkValeursEntieres;
	private JLabel label1, label2, label3, label4, label5;
	private EditeurValeurDynamique editeurValeurMini, editeurValeurMaxi;
	public JTextField editeurNom;
	public TextFieldAvecCarSpeciaux editeurPreChaine;
	public TextFieldAvecCarSpeciaux editeurPostChaine;
	private JButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsMin;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsMax;
	public PaneListeDecimales paneDecimales;
	public JCheckBox checkExtremitesPunaisees;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
	public CValeurAssocieeAVariable valMin;
	public CValeurAssocieeAVariable valMax;
	
  public CurseurDlg(MtgFrame frameInit) {
    super(getString("curseurDlg1"), true, frameInit);
    CListeObjets liste = frame.cadreActif().listePr;
    valMin = new CValeurAssocieeAVariable(liste);
    valMax = new CValeurAssocieeAVariable(liste);
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
  	radioHorizontal = new JRadioButton(getString("curseurDlg2"));
  	radioHorizontal.setSelected(true);
  	radioVertical =new JRadioButton(getString("curseurDlg3"));
  	btnGroup = new ButtonGroup();
  	btnGroup.add(radioHorizontal);
  	btnGroup.add(radioVertical);
  	radioPteTaille = new JRadioButton(getString("curseurDlg11"));
  	radioGdeTaille = new JRadioButton(getString("curseurDlg12"));
  	btnGroup2 = new ButtonGroup();
  	btnGroup2.add(radioPteTaille);
  	btnGroup2.add(radioGdeTaille);
  	radioPteTaille.setSelected(true);

  	// Pas de valeurs entires par dfaut
  	checkValeursEntieres = new JCheckBox(getString("curseurDlg4"), false);
  	checkValeursEntieres.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        checkValeursEntieresActionPerformed();
      }
  	});
  	
		btnOK = new javax.swing.JButton(getString("btnOK"));
		btnOK.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOKActionPerformed();
			}
		});
		btnCancel = new javax.swing.JButton(getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		btnOK.setMaximumSize(btnCancel.getPreferredSize());
		label1 = new JLabel(getString("curseurDlg5"));
		label2 = new JLabel(getString("curseurDlg6"));
		label3 = new JLabel(getString("curseurDlg7"));
		label4 = new JLabel(getString("curseurDlg8"));
		label5 = new JLabel(getString("curseurDlg9"));
		checkExtremitesPunaisees = new JCheckBox(getString("curseurDlg10"));
		checkExtremitesPunaisees.setSelected(true);
		editeurValeurMini = new EditeurValeurDynamique(frame, valMin, null);
		editeurValeurMaxi = new EditeurValeurDynamique(frame, valMax, null);
  	int indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctionsMin = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurMini, true, indmax, false);
    paneListeValeurEtFonctionsMax = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurMaxi, true, indmax, false);
    editeurNom = new JTextField(8);
  	editeurPreChaine = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
  	editeurPostChaine = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
  	paneDecimales = new PaneListeDecimales(2);
  	
  	JPanel panePrincipal = (JPanel) getContentPane();
		JPanel paneRight = new JPanel();
		JPanel paneCenter = new JPanel();
		panePrincipal.setLayout(new GridBagLayout());
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
		paneCenter.setLayout(new GridBagLayout());
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		setConstraints(constraints, 0, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(editeurValeurMini, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(paneListeValeurEtFonctionsMin, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(label2, constraints);
		setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(editeurValeurMaxi, constraints);
		setConstraints(constraints, 1, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneCenter.add(paneListeValeurEtFonctionsMax, constraints);
		setConstraints(constraints, 2, 0, 1, 4, 20, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(paneDecimales, constraints);
		
		
		setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioHorizontal, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioVertical, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioPteTaille, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(radioGdeTaille, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkValeursEntieres, constraints);
		
		setConstraints(constraints, 0, 3, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneCenter, constraints);
	
		setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label3, constraints);
		setConstraints(constraints, 1, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurNom, constraints);
		setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label4, constraints);
		setConstraints(constraints, 1, 5, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurPreChaine, constraints);
		setConstraints(constraints, 0, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label5, constraints);
		setConstraints(constraints, 1, 6, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurPostChaine, constraints);
		setConstraints(constraints, 0, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkExtremitesPunaisees, constraints);
		
		paneRight.add(btnOK);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5)));
		paneRight.add(btnCancel);

		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(paneRight, constraints);
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(7);
    order.add(editeurValeurMini);
    order.add(editeurValeurMaxi);
    order.add(editeurNom);
    order.add(editeurPreChaine);
    order.add(editeurPostChaine);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		
		pack();
  }
  
  private void btnOKActionPerformed() {
  	// On vrifie si le nom choisi pour la calcul final du curseur est valide
    if (!frame.cadreActif().listePr.validationNomCalcul(frame, editeurNom.getText())) {
      editeurNom.requestFocus();
      return;
    }
    if (!editeurValeurMini.verifieValidite()) {
    	editeurValeurMini.requestFocus();
    	return;
    }
    if (!editeurValeurMaxi.verifieValidite()) {
    	editeurValeurMaxi.requestFocus();
    	return;
    }
    // Amlioration version 3.5.1
    // Un curseur  valeurs entires n'accepte pas que la valeur maxi soit plus petite que la valeur mini
    // et un curseur  valeurs non entires n'accepte pas que la valeur maxi soit gale  la valeur maxi
    DimensionFenetre dimfen = frame.cadreActif().dimf;
	  valMin.positionne(false, dimfen);
	  double min = valMin.valeur;
	  valMax.positionne(false, dimfen);
	  double max = valMax.valeur;
	  if (min == max) {
    	MtgDlg.dialogueAvertissementCentreEcran(getString("ch71"));
    	editeurValeurMini.requestFocus();
    	return;
	  }
    if (checkValeursEntieres.isSelected()) {
	    if (max < min) {
	    	MtgDlg.dialogueAvertissementCentreEcran(getString("ch71"));
	    	editeurValeurMini.requestFocus();
	    	return;
	    }
	    if (min != Math.floor(min)) {
	    	MtgDlg.dialogueAvertissementCentreEcran(getString("ch71"));
	    	editeurValeurMini.requestFocus();
	    	return;
	    }
	    if (max != Math.floor(max)) {
	    	MtgDlg.dialogueAvertissementCentreEcran(getString("ch71"));
	    	editeurValeurMaxi.requestFocus();
	    	return;
	    }
    }
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
     
  private void checkValeursEntieresActionPerformed() {
  	if (checkValeursEntieres.isSelected()) paneDecimales.setVisible(false);
  	else paneDecimales.setVisible(true);
  	pack();
  }
	
}
