/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import mathgraph32.MtgFrame;
import objets.CListeObjets;

/**
 * Dialogue permettant de crer une construction affichant une quation de droite dans un repre
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class EquationDroiteDlg extends MtgDlg {
	public ListeReperes listeReperes;
  private JButton btnOK, btnCancel;
	public PaneListeDecimales paneListeDecimales;
  private PaneTaillePolice paneTaillePolice;
  public PaneModeAffichage paneModeAffichage;
	
  public EquationDroiteDlg(MtgFrame frameInit) {
    super(getString("equationDroiteDlg1"), true, frameInit);
    // Spcial version applet
    // CListeObjets liste = frame.cadreActif().listePr;
    CListeObjets liste = frame.listePourConstruction();
    setResizable(false);
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    JLabel label1 = new JLabel(getString("pointSurCourbeDlg2"), SwingConstants.LEFT);
    listeReperes = new ListeReperes(frame.cadreActif().listePr, liste.size() - 1);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollPaneListeReperes = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
 		paneListeDecimales = new PaneListeDecimales(1); // 1 dcimale par dfaut
    paneTaillePolice = new PaneTaillePolice(frame.pref_IndiceTaillePoliceAffichages);
    paneModeAffichage = new PaneModeAffichage(frame, null, true);
    JPanel paneLow = new JPanel();
    paneLow.setLayout(new FlowLayout());
    paneLow.add(paneTaillePolice);
    paneLow.add(paneModeAffichage); 		


		JPanel panePrincipal = (JPanel)getContentPane();
		panePrincipal.setLayout(new GridBagLayout());
		JPanel paneRight = new JPanel();
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeReperes, constraints);
		setConstraints(constraints, 2, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneListeDecimales, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneLow, constraints);
		
		paneRight.add(btnOK);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5)));
		paneRight.add(btnCancel);

		setConstraints(constraints, 3, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(paneRight, constraints);
		pack();
    getRootPane().setDefaultButton(btnOK);    
  }
  
  private void btnOKActionPerformed() {
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
    
  
}
