/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CFonc;
import objets.CIntegrale;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.FonteNom;

@SuppressWarnings("serial")
public class IntegraleDlg extends MtgDlg {
	private CIntegrale integrale; // L'objet c=qu'on modifie
	private boolean modification; // True si on modifie un objet dj cr
	private EditeurNomCalcul editeurNom;
	private ListeFonctionsReelles listeFonctions;
	private JButton btnOK, btnCancel;
	private EditeurValeurDynamique editeurValeura;
	private EditeurValeurDynamique editeurValeurb;
	private EditeurNombreTermesIntegrale editeurValeurn;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsa;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsb;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsn;
	private JTextArea info;
	// On utilise des copies des valeurs dynamiques en cas d'abandon aprs avoir valid
	private CValeurAssocieeAVariable valeura, valeurb, valeure;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
  public IntegraleDlg(MtgFrame frameInit, CIntegrale integraleInit, boolean modificationInit) {
    super(getString("integraleDlg1"), true, frameInit);
    integrale = integraleInit;
    modification = modificationInit;
    setResizable(false);
    JLabel label1 = new JLabel(getString("integraleDlg2"), SwingConstants.LEFT);
    JLabel label1Bis = new JLabel(getString("integraleDlg2Bis"), SwingConstants.LEFT);
    JLabel label2 = new JLabel(getString("integraleDlg3"), SwingConstants.LEFT);
    JLabel label3 = new JLabel(getString("integraleDlg4"), SwingConstants.LEFT);
    JLabel label4 = new JLabel(getString("integraleDlg5"), SwingConstants.LEFT);
    JLabel label5 = new JLabel(getString("integraleDlg6"), SwingConstants.LEFT);
    JLabel label6 = new JLabel(getString("integraleDlg7"), SwingConstants.LEFT);
    JLabel label7 = new JLabel(getString("integraleDlg8"), SwingConstants.LEFT);
    editeurNom = new EditeurNomCalcul();
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    info.setEditable(false);
    info.setColumns(40);
    // info.setLineWrap(false);
    info.setRows(1);
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // info.setColumns(40);
    // Spcial verion applet
    // CListeObjets liste = frame.cadreActif().listePr;
    CListeObjets liste = frame.listePourConstruction();
    valeura = new CValeurAssocieeAVariable(integrale.a);
    valeurb = new CValeurAssocieeAVariable(integrale.b);
    valeure = new CValeurAssocieeAVariable(integrale.n);
    editeurValeura = new EditeurValeurDynamique(frame, valeura, integrale);
    editeurValeurb = new EditeurValeurDynamique(frame, valeurb, integrale);
    editeurValeurn = new EditeurNombreTermesIntegrale(frame, valeure, integrale);
    int indmax;
    if (modification) indmax = liste.indexOf(integrale) - 1;
    else indmax = liste.size() - 1;
    listeFonctions = new ListeFonctionsReelles(liste, indmax, false);
    listeFonctions.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
        	listeFonctionsSelectionChanged();
        }
      });
    if (modification) {
    	listeFonctions.setSel(integrale.fonctionAssociee);
    	editeurNom.setText(integrale.nom());
    	editeurValeura.setText(integrale.a.chaineInfo());
    	editeurValeurb.setText(integrale.b.chaineInfo());
    	editeurValeurn.setText(integrale.n.chaineInfo());
    }
    else editeurValeurn.setText("20");
    JScrollPane scrollPane = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setPreferredSize(btnCancel.getMaximumSize());    
    paneListeValeurEtFonctionsa = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeura, true, indmax, false);
    paneListeValeurEtFonctionsb = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurb, true, indmax, false);
    paneListeValeurEtFonctionsn = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurn, true, indmax, false);
   
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 4, 1, 100, 5, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 0, 1, 4, 1, 100, 5, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1Bis, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurNom, constraints);
    setConstraints(constraints, 3, 2, 1, 1, 25, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label3, constraints);
    setConstraints(constraints, 1, 3, 1, 7, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(scrollPane, constraints);
    setConstraints(constraints, 3, 3, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnCancel, constraints);
    setConstraints(constraints, 2, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label4, constraints);
    setConstraints(constraints, 3, 4, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeura, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 3, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsa, constraints);
    constraints.insets.top = 5;
    setConstraints(constraints, 2, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label5, constraints);
    setConstraints(constraints, 3, 6, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeurb, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 3, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsb, constraints);
    constraints.insets.top = 5;

    setConstraints(constraints, 2, 8, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label6, constraints);
    setConstraints(constraints, 3, 8, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeurn, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 3, 9, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsn, constraints);
    constraints.insets.top = 5;

    setConstraints(constraints, 0, 10, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label7, constraints);
    setConstraints(constraints, 0, 11, 4, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(scrollPaneInfo, constraints);
    
    listeFonctionsSelectionChanged();
    setContentPane(pane);

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(7);
    order.add(editeurNom);
    order.add(listeFonctions);
    order.add(editeurValeura);
    order.add(editeurValeurb);
    order.add(editeurValeurn);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }

  public void btnCancelActionPerformed() {
  	dispose();
  }
  
  public void btnOKActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
    if (!modification) {
      if (!modification) {
  	    if (!listePr.validationNomCalcul(frame, chNom)) {
  	      editeurNom.requestFocus();
  	      return;
  	    }
      }
    }
    if (editeurValeura.verifieValidite()) {
    	if (editeurValeurb.verifieValidite()) {
      	if (editeurValeurn.verifieValidite()) {
	        setRetourOK(true);
	        if (!modification)integrale.donneNom(chNom);
	        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
	        integrale.a = valeura;
	        integrale.b = valeurb;
	        integrale.n = valeure;
	        integrale.fonctionAssociee = listeFonctions.fonctionSelectionnee();
	        dispose();
      	}
    	}
    }
  }
  
  public void listeFonctionsSelectionChanged() {
  	CFonc fonc = listeFonctions.fonctionSelectionnee();
  	info.setText(fonc.infoFormule());
  	info.setCaretPosition(0);
  	pack();
  }
}
