/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CPointBaseEnt;
import objets.CValeurAssocieeAVariable;

@SuppressWarnings("serial")
public class PointLibreEntDlg extends MtgDlg {
	private boolean modification; // True si on modifie les coordonnes d'un point dj cr
	private CPointBaseEnt pointBaseEnt;
	
	private JLabel label1;
	private JLabel labelAbsMin, labelAbsMax, labelOrdMin, labelOrdMax;
	private ListeReperes listeReperes;
	private JCheckBox checkAscissesLibres;
	private JCheckBox checkOrdonneesLibres;
	private EditeurValeurDynamique editeurXmin;
	private EditeurValeurDynamique editeurXmax;
	private EditeurValeurDynamique editeurYmin;
	private EditeurValeurDynamique editeurYmax;
	private JButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsXMin;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsYMin;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsXMax;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsYMax;
  private CValeurAssocieeAVariable valXmin;
  private CValeurAssocieeAVariable valXmax;
  private CValeurAssocieeAVariable valYmin;
  private CValeurAssocieeAVariable valYmax;
	
	
  public PointLibreEntDlg(MtgFrame frameInit, CPointBaseEnt pointBaseEntInit, boolean modificationInit) {
    super(getString("pointLibreEntDlg1"), true, frameInit);
    modification = modificationInit;
    pointBaseEnt = pointBaseEntInit;
    initComponents();
  	if (checkAscissesLibres.isSelected()) {
  		paneListeValeurEtFonctionsXMin.setEnabled(false);
  		editeurXmin.setEditable(false);
  		paneListeValeurEtFonctionsXMax.setEnabled(false);
  		editeurXmax.setEditable(false);	
  	}
  	else {
  		paneListeValeurEtFonctionsXMin.setEnabled(true);
  		editeurXmin.setEditable(true);
  		paneListeValeurEtFonctionsXMax.setEnabled(true);
  		editeurXmax.setEditable(true);	
  		
  	}
  	if (checkOrdonneesLibres.isSelected()) {
  		paneListeValeurEtFonctionsYMin.setEnabled(false);
  		editeurYmin.setEditable(false);
  		paneListeValeurEtFonctionsYMax.setEnabled(false);
  		editeurYmax.setEditable(false);	
  	}
  	else {
  		paneListeValeurEtFonctionsYMin.setEnabled(true);
  		editeurYmin.setEditable(true);
  		paneListeValeurEtFonctionsYMax.setEnabled(true);
  		editeurYmax.setEditable(true);	
  	}
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
  	// Spcial version applet
    // CListeObjets liste = frame.cadreActif().listePr;
    CListeObjets liste = frame.listePourConstruction();
    valXmin = new CValeurAssocieeAVariable(pointBaseEnt.absMin);
    valXmax = new CValeurAssocieeAVariable(pointBaseEnt.absMax);
    valYmin = new CValeurAssocieeAVariable(pointBaseEnt.ordMin);
    valYmax = new CValeurAssocieeAVariable(pointBaseEnt.ordMax);
    editeurXmin = new EditeurValeurDynamique(frame, valXmin, pointBaseEnt);
    editeurXmin.setColumns(12); // 30 trop grand pour ce dialogue
    editeurXmin.setText(pointBaseEnt.absMin.chaineInfo());
    editeurXmax = new EditeurValeurDynamique(frame, valXmax, pointBaseEnt);
    editeurXmax.setColumns(12); // 30 trop grand pour ce dialogue
    editeurXmax.setText(pointBaseEnt.absMax.chaineInfo());
    editeurYmin = new EditeurValeurDynamique(frame, valYmin, pointBaseEnt);
    editeurYmin.setColumns(12); // 30 trop grand pour ce dialogue
    editeurYmin.setText(pointBaseEnt.ordMin.chaineInfo());
    editeurYmax = new EditeurValeurDynamique(frame, valYmax, pointBaseEnt);
    editeurYmax.setColumns(12); // 30 trop grand pour ce dialogue
    editeurYmax.setText(pointBaseEnt.ordMax.chaineInfo());
    
    checkAscissesLibres = new JCheckBox(getString("pointLibreEntDlg2"));
    checkAscissesLibres.setSelected(pointBaseEnt.absLibres);
    checkAscissesLibres.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	checkAscissesLibresActionPerformed();
      }
 		});    
    checkOrdonneesLibres = new JCheckBox(getString("pointLibreEntDlg3"));
    checkOrdonneesLibres.setSelected(pointBaseEnt.ordLibres);
    checkOrdonneesLibres.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	checkOrdonneesLibresActionPerformed();
      }
 		});
  	int indmax;
  	if (modification) indmax = liste.indexOf(pointBaseEnt) - 1;
  	else indmax = liste.size() - 1;
    paneListeValeurEtFonctionsXMin = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurXmin, true, indmax, false);
    paneListeValeurEtFonctionsXMax = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurXmax, true, indmax, false);
    paneListeValeurEtFonctionsYMin = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurYmin, true, indmax, false);
    paneListeValeurEtFonctionsYMax = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurYmax, true, indmax, false);
   
    if (modification) {
    	editeurXmin.setText(pointBaseEnt.absMin.chaineInfo());    
    	editeurXmax.setText(pointBaseEnt.absMax.chaineInfo());    
    	editeurYmin.setText(pointBaseEnt.ordMin.chaineInfo());    
    	editeurYmax.setText(pointBaseEnt.ordMax.chaineInfo());
    	checkAscissesLibres.setSelected(pointBaseEnt.absLibres);
     	checkOrdonneesLibres.setSelected(pointBaseEnt.ordLibres);
    }
    
  	btnOK = new javax.swing.JButton(getString("btnOK"));
  	btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });
  	btnCancel = new javax.swing.JButton(getString("btnCancel"));
  	btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    label1 = new JLabel(getString("choixRepere"), SwingConstants.LEFT);
    labelAbsMin = new JLabel(getString("abscisseMini"), SwingConstants.LEFT);
    labelAbsMax = new JLabel(getString("abscisseMaxi"), SwingConstants.LEFT);
    labelOrdMin = new JLabel(getString("OrdonneeMini"), SwingConstants.LEFT);
    labelOrdMax = new JLabel(getString("OrdonneeMaxi"), SwingConstants.LEFT);
    
    int indiceObj;
    if (modification) indiceObj = liste.indexOf(pointBaseEnt);
    else indiceObj = -1;
    listeReperes = new ListeReperes(frame.cadreActif().listePr, indiceObj);
    if (modification) listeReperes.setSel(pointBaseEnt.rep);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollList = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);    
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 2;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    JPanel paneBas = new JPanel();
    paneBas.setLayout(new GridBagLayout());
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(checkAscissesLibres, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 25, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(labelAbsMin, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 25, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneBas.add(editeurXmin, constraints);
    setConstraints(constraints, 2, 1, 1, 1, 25, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(labelAbsMax, constraints);
    setConstraints(constraints, 3, 1, 1, 1, 25, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneBas.add(editeurXmax, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(paneListeValeurEtFonctionsXMin, constraints);
    setConstraints(constraints, 3, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(paneListeValeurEtFonctionsXMax, constraints);

    setConstraints(constraints, 0, 3, 2, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(checkOrdonneesLibres, constraints);
    setConstraints(constraints, 0, 4, 1, 1, 25, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(labelOrdMin, constraints);
    setConstraints(constraints, 1, 4, 1, 1, 25, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneBas.add(editeurYmin, constraints);
    setConstraints(constraints, 2, 4, 1, 1, 25, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(labelOrdMax, constraints);
    setConstraints(constraints, 3, 4, 1, 1, 25, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneBas.add(editeurYmax, constraints);
    setConstraints(constraints, 1, 5, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(paneListeValeurEtFonctionsYMin, constraints);
    setConstraints(constraints, 3, 5, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(paneListeValeurEtFonctionsYMax, constraints);
    

    
 		JPanel paneRight = new JPanel();
 		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
 		paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
 		paneRight.add(btnCancel);
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(scrollList, constraints);
    
    setConstraints(constraints, 0, 1, 2, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneBas, constraints);
 
    setConstraints(constraints, 1, 0, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneRight, constraints);
    
    setContentPane(pane);
    pack();
    if (pointBaseEnt.absLibres) {
			paneListeValeurEtFonctionsXMin.setEnabled(false);
			editeurXmin.setEditable(false);
			paneListeValeurEtFonctionsXMax.setEnabled(false);
			editeurXmax.setEditable(false);
    }
    if (pointBaseEnt.ordLibres) {
			paneListeValeurEtFonctionsYMin.setEnabled(false);
			editeurYmin.setEditable(false);
			paneListeValeurEtFonctionsYMax.setEnabled(false);
			editeurYmax.setEditable(false);
    }    
  }
	
  private void btnOKActionPerformed() {
  	boolean reponseValide = true;
  	if (!checkAscissesLibres.isSelected()) {
  		reponseValide = editeurXmin.verifieValidite() && editeurXmax.verifieValidite();
  		if (!reponseValide) return;
  	}
  	if (!checkOrdonneesLibres.isSelected()) {
  		reponseValide = editeurYmin.verifieValidite() && editeurYmax.verifieValidite();
  		if (!reponseValide) return;
  	}
		double xmin = valXmin.rendValeur(false);
		double xmax = valXmax.rendValeur(false);
		double ymin = valYmin.rendValeur(false);
		double ymax = valYmax.rendValeur(false);
		if (!checkAscissesLibres.isSelected() && (xmax < xmin)) {
			dialogueAvertissement(getString("ch123"));
			editeurXmax.requestFocus();
			return;
		}
		if (!checkOrdonneesLibres.isSelected() && (ymax < ymin)) {
			dialogueAvertissement(getString("ch123"));
			editeurYmax.requestFocus();
			return;
		}
    setRetourOK(true);
    // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
    pointBaseEnt.absLibres = checkAscissesLibres.isSelected();
    pointBaseEnt.ordLibres = checkOrdonneesLibres.isSelected();
    if (!pointBaseEnt.absLibres) {
	    pointBaseEnt.absMin = valXmin;
	    pointBaseEnt.absMax = valXmax;
    }
    if (!pointBaseEnt.ordLibres) {
	    pointBaseEnt.ordMin = valYmin;
	    pointBaseEnt.ordMax = valYmax;
    }
    pointBaseEnt.rep = listeReperes.repereChoisi();
    dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void checkAscissesLibresActionPerformed() {
  	if (checkAscissesLibres.isSelected()) {
  		paneListeValeurEtFonctionsXMin.setEnabled(false);
  		editeurXmin.setEditable(false);
  		paneListeValeurEtFonctionsXMax.setEnabled(false);
  		editeurXmax.setEditable(false);	
  	}
  	else {
  		paneListeValeurEtFonctionsXMin.setEnabled(true);
  		editeurXmin.setEditable(true);
  		paneListeValeurEtFonctionsXMax.setEnabled(true);
  		editeurXmax.setEditable(true);			
  	}
  }
  
  private void checkOrdonneesLibresActionPerformed() {
  	if (checkOrdonneesLibres.isSelected()) {
  		paneListeValeurEtFonctionsYMin.setEnabled(false);
  		editeurYmin.setEditable(false);
  		paneListeValeurEtFonctionsYMax.setEnabled(false);
  		editeurYmax.setEditable(false);	
  	}
  	else {
  		paneListeValeurEtFonctionsYMin.setEnabled(true);
  		editeurYmin.setEditable(true);
  		paneListeValeurEtFonctionsYMax.setEnabled(true);
  		editeurYmax.setEditable(true);	
  	}
  	
  }

}
