/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CProduit;
import objets.CValeurAssocieeAVariable;
import types.FonteNom;

@SuppressWarnings("serial")
public class ProduitIndiceDlg extends MtgDlg {
	private boolean modification;
	private CProduit produit;
	private JLabel label1, label2, label3, label4, label5, label6, label7;
	private JTextField editeurNom;
	private EditeurValeurDynamique editeurIndiceMini, editeurIndiceMaxi;
	private ListeValeursReelles listeValeurs;
	private ListeVariables listeVariables;
	private JTextArea info;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions1;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions2;
  private javax.swing.JButton btnCancel;
  private javax.swing.JButton btnOK;
  private CValeurAssocieeAVariable indmin, indmax;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
  public ProduitIndiceDlg(MtgFrame frameInit, CProduit produitInit, boolean modificationInit) {
    super (getString("produitIndiceDlg1"), true, frameInit);
    produit = produitInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
  
  private void initComponents() {
  	indmin = new CValeurAssocieeAVariable(produit.indiceMin);
	  indmax = new CValeurAssocieeAVariable(produit.indiceMax);
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setPreferredSize(btnCancel.getPreferredSize());
    label1 = new JLabel(getString("produitIndiceDlg2"), JLabel.RIGHT);
    label2 = new JLabel(getString("produitIndiceDlg3"), JLabel.RIGHT);
    label3 = new JLabel(getString("produitIndiceDlg4"), JLabel.RIGHT);
    label4 = new JLabel(getString("sommeIndiceeDlg5"), JLabel.RIGHT);
    label5 = new JLabel(getString("produitIndiceDlg6"), JLabel.RIGHT);
    label6 = new JLabel(getString("sommeIndiceeDlg7"), JLabel.RIGHT);
    label7 = new JLabel(getString("sommeIndiceeDlg8"), JLabel.RIGHT);
    editeurNom = new javax.swing.JTextField(null,8);
    editeurIndiceMini = new EditeurValeurDynamique(frame, indmin, produit);
    editeurIndiceMini.setColumns(15); // Plus petit que la valeur par dfaut
    editeurIndiceMaxi = new EditeurValeurDynamique(frame, indmax, produit);
    editeurIndiceMaxi.setColumns(15);
    // Spcial version applet
		// CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
		int indmax;
		if (modification) {
			indmax = liste.indexOf(produit) - 1;
		}
		else {
			indmax = liste.size() - 1;
		}
		listeValeurs = new ListeValeursReelles(liste, indmax, false);
    listeValeurs.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeValeursSelectionChanged();
      }
    });		
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    info.setEditable(false);
    info.setColumns(20);
    info.setLineWrap(false);
    info.setRows(4);
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneInfo.setPreferredSize(new Dimension(300, 90));
    listeVariables = new ListeVariables(liste, indmax, false);
    paneListeValeurEtFonctions1 = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurIndiceMini, true, indmax, false);
    paneListeValeurEtFonctions2 = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurIndiceMaxi, true, indmax, false);
		if (modification) {
			editeurNom.setEditable(false);
			editeurNom.setText(produit.nom());
			listeValeurs.setSel(produit.valeurAMultiplier);
			listeVariables.setSel(produit.indice);
			editeurIndiceMini.setText(produit.indiceMin.chaineInfo());
			editeurIndiceMaxi.setText(produit.indiceMax.chaineInfo());
		}
			
    JScrollPane scrollPaneListeValeurs = new JScrollPane(listeValeurs, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    JScrollPane scrollPaneListeVariables = new JScrollPane(listeVariables, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
		JPanel panePrincipal = new JPanel();
		panePrincipal.setLayout(new GridBagLayout());
		JPanel paneIndices = new JPanel();
		paneIndices.setLayout(new GridBagLayout());

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 25, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(editeurNom, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
		panePrincipal.add(label3, constraints);
		setConstraints(constraints, 0, 3, 2, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeValeurs, constraints);
		setConstraints(constraints, 2, 2, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
		panePrincipal.add(label4, constraints);
		setConstraints(constraints, 2, 3, 2, 1, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeVariables, constraints);

		setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label6, constraints);
		setConstraints(constraints, 1, 4, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIndiceMini, constraints);
		setConstraints(constraints, 1, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneListeValeurEtFonctions1, constraints);
		setConstraints(constraints, 2, 4, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label7, constraints);
		setConstraints(constraints, 3, 4, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIndiceMaxi, constraints);
		setConstraints(constraints, 3, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneListeValeurEtFonctions2, constraints);
		
		setConstraints(constraints, 0, 6, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label5, constraints);
		setConstraints(constraints, 0, 7, 4, 1, 0, 20, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneInfo, constraints);
		setConstraints(constraints, 3, 0, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
		panePrincipal.add(btnOK, constraints);
		setConstraints(constraints, 3, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
		panePrincipal.add(btnCancel, constraints);	

		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(7);
		order.add(editeurNom);
		order.add(listeValeurs);
		order.add(listeVariables);
		order.add(editeurIndiceMini);
		order.add(editeurIndiceMaxi);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();
		listeValeursSelectionChanged();
  }
  
  private void btnOKActionPerformed() { 
		MtgInternalFrame cadre = frame.cadreActif();
    // Spcial version applet		
		// CListeObjets listePr = cadre.listePr;
		CListeObjets listePr = frame.listePourConstruction();
		String chNom = editeurNom.getText();
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
		// Le nom est correct. On regarde les champs d'entre
		if (editeurIndiceMini.verifieValidite()) {
			if (editeurIndiceMaxi.verifieValidite()) {
				// On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour
				// les remplacer par les locaaux
				produit.nomCalcul = chNom;
				produit.indiceMin = indmin;
				produit.indiceMax = indmax;
				produit.valeurAMultiplier = listeValeurs.valeurSelectionnee();
				produit.indice = listeVariables.variableSelectionnee();
				// Modification version 3.4.2
				// produit.metAJour();
				produit.etablitListeElementsAncetres();
	      if (modification) cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
				setRetourOK(true);
				dispose();
			}
		}  	
  }
    
  private void btnCancelActionPerformed() {                                          
    dispose();
  }           

	public void listeValeursSelectionChanged() {
		int i = listeValeurs.getSelectedIndex();
		if (i != -1) {
			info.setText(listeValeurs.valeurSelectionnee().infoPourHistorique());
			info.setCaretPosition(0);
			pack();
		}
	}
  
}