/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CListeObjets;
import objets.CValeurDynamique;
import objets.CVariableBornee;
import types.FonteNom;
import types.NatureCalcul;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class RenommerCalculDlg extends MtgDlg {
	int indiceMini; // L'indice mini  partir duquel on propose de renommer les calculs
	private JLabel label1, label2, label3, label4;
	private JButton btnFermer, btnRenommer;
	private ListeObjetsCalculsNommesNonConstants listeObjetsCalcul;
	private JScrollPane scrollPaneListe;
	private JTextField editeurNomCalcul;
	private JTextArea info;
	private JScrollPane scrollPaneInfo;
	
  public RenommerCalculDlg(MtgFrame frameInit, int indiceMiniInit) {
    super(getString("renommerCalculDlg1"), true, frameInit);
    indiceMini = indiceMiniInit;
    initComponents();
    getRootPane().setDefaultButton(btnRenommer);
    listeObjetsCalculSelectionChanged();
  }
	
  private void initComponents() {
  	// Spcial version applet
  	// CListeObjets liste = frame.cadreActif().listePr;
  	CListeObjets liste = frame.listePourConstruction();
  	label1 = new JLabel(getString("renommerCalculDlg2"));
  	label2 = new JLabel(getString("renommerCalculDlg3"));
  	label3 = new JLabel(getString("renommerCalculDlg4"));
  	label4 = new JLabel(getString("renommerCalculDlg5"));
  	
  	btnFermer = new javax.swing.JButton(getString("btnFermer"));
  	btnFermer.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnFermerActionPerformed();
      }
    });
  	btnRenommer = new javax.swing.JButton(getString("btnRenommer"));
  	btnRenommer.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnRenommerActionPerformed();
      }
    });
  	btnFermer.setMaximumSize(btnRenommer.getPreferredSize());
  	
  	listeObjetsCalcul = new ListeObjetsCalculsNommesNonConstants(liste, indiceMini, false);
  	scrollPaneListe = new JScrollPane(listeObjetsCalcul, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    listeObjetsCalcul.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeObjetsCalculSelectionChanged();
      }
    });
    editeurNomCalcul = new javax.swing.JTextField(null,8);
    
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info.setEditable(false);
    info.setColumns(20);
    info.setEditable(false);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPaneInfo.setPreferredSize(new Dimension(450, 90));    
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
 		JPanel paneRight = new JPanel();
 		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
 		paneRight.add(label2);
 		paneRight.add(label3);
    paneRight.add(Box.createRigidArea(new Dimension(0,5))); 		
 		paneRight.add(editeurNomCalcul);
    paneRight.add(Box.createRigidArea(new Dimension(0,5))); 		
 		paneRight.add(btnRenommer);
    paneRight.add(Box.createRigidArea(new Dimension(0,5))); 		
 		paneRight.add(btnFermer);
 		
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    pane.add(scrollPaneListe, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label4, constraints);
    setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    pane.add(scrollPaneInfo, constraints);
    setConstraints(constraints, 1, 0, 1, 3, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneRight, constraints);
   
    setContentPane(pane);
    pack();
  }
  
  private void btnFermerActionPerformed() {
  	dispose();
  }
  
  private void btnRenommerActionPerformed() {
    // MtgInternalFrame cadre = frame.cadreActif();
    // Spcial version applet
    // CListeObjets listePr = cadre.listePr;
    CListeObjets listePr = frame.listePourConstruction();
  	// On regarde si le nom choisi est valide
    String chNom = editeurNomCalcul.getText();
    // Modifi version 4.7 car on doit accepter que le nom commence par une fonction prdfinie
    /*
    if ((chNom.length() == 0) || !EditeurNomCalcul.validationNomVariableOuCalcul(chNom)) {
      dialogueAvertissement(getString("ch1"));
      editeurNomCalcul.requestFocus();
      return;
    }
    if (chNom.equals("i")) {
      dialogueAvertissement(getString("ch2"));
      editeurNomCalcul.requestFocus();
      return;
    }
    CValeurDynamique val = (CValeurDynamique) listeObjetsCalcul.valeurSelectionnee();
    if ((!val.nom().equals(chNom)) && listePr.egalNomValeurOuFonctionOuMesure(chNom, listePr.size()-1)) {
      dialogueAvertissement(getString("ch3"));
      editeurNomCalcul.requestFocus();
      return;
    }
    if (CCalculBase.commenceParNomFonctionPredefinie(chNom)) {
      dialogueAvertissement(getString("ch91"));
      editeurNomCalcul.requestFocus();
      return;	    	
    }
    */
    if (!listePr.validationNomCalcul(frame, chNom)) {
    	editeurNomCalcul.requestFocus();
    	return;
    }
    CValeurDynamique val = (CValeurDynamique) listeObjetsCalcul.valeurSelectionnee();
    // Le nom est correct.
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil142"));
    // On modifie les commentaires faisant un affichage dynamique de la valeur dont le nom a t modifi
    listePr.remplaceNomValeurDynamiqueDansCommentairesOuLatexDependantDe(val.nom(), chNom, val);
    // On reconstruit les formules de tous les calculs ou fonctions dpendant de ce calcul
    val.donneNom(chNom);
    listePr.reconstruitChainesCalculDependantsDe(listePr.indexOf(val) + 1, val);
    listePr.determineDependancesCommentaires(); // Ajout version 4.6
    listeObjetsCalcul.metAJourListe();
		if (val.getNatureCalcul() == NatureCalcul.NVariable)
			// frame.cadreActif().metAJourDialogueAffichageVariables((CVariableBornee) val);
 			frame.cadreActif().metAJourPaneAffichageVariables((CVariableBornee) val);
  }

  public void listeObjetsCalculSelectionChanged() {
  	int i = listeObjetsCalcul.getSelectedIndex();
  	if (i != -1) {
  		CValeurDynamique val = (CValeurDynamique) listeObjetsCalcul.pointeurs.get(i);
  		info.setText(val.infoPourHistorique());
  		info.setCaretPosition(0);
  		editeurNomCalcul.setText(val.nom());
  		editeurNomCalcul.requestFocus();
  		pack();
  	}
  }
 
}
