/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CTestExistence;
import objets.CValeurDynamique;
import types.FonteNom;

@SuppressWarnings("serial")
public class TestExistenceDlg extends MtgDlg {
	private boolean modification;
	private CTestExistence testExistence;
	private JLabel label1, label2, label3, label4;
	private JTextField editeurNom;
	private JTextArea info;
  private JButton btnOK, btnCancel;  
  private ListeValeursReellesOuComplexesNonConstantes listeValeurs;

  public TestExistenceDlg(MtgFrame frameInit, CTestExistence testExistenceInit, boolean modificationInit) {
  	super(getString("testExistenceDlg1"), true, frameInit);
  	modification = modificationInit;
  	testExistence = testExistenceInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);      	
  }

  private void initComponents() {
    label1 = new JLabel(getString("testExistenceDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new JLabel(getString("testExistenceDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new JLabel(getString("testExistenceDlg4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    label4 = new JLabel(getString("testExistenceDlg5"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    editeurNom = new javax.swing.JTextField(null,8);
		// Spcial version applet
    // CListeObjets liste = frame.cadreActif().listePr;
    CListeObjets liste = frame.listePourConstruction();
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(testExistence) - 1;
    	listeValeurs = new ListeValeursReellesOuComplexesNonConstantes(liste, indmax, false);
    	editeurNom.setText(testExistence.nom());
    	editeurNom.setEditable(false);
    }
    else {
    	indmax = liste.size() - 1;
    	listeValeurs = new ListeValeursReellesOuComplexesNonConstantes(liste, indmax, false);
    }
    
    JScrollPane scrollPaneListe = new JScrollPane(listeValeurs, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    if (modification) {
    	CValeurDynamique val = testExistence.valeurAssociee;
    	listeValeurs.setSel(val);
    	listeValeurs.ensureIndexIsVisible(listeValeurs.pointeurs.indexOf(val));    	
    }
    listeValeurs.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeValeursSelectionChanged();
      }
    });
    
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    info.setEditable(false);
    info.setColumns(20);
    info.setLineWrap(false);
    info.setRows(4);
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneInfo.setPreferredSize(new Dimension(300, 90));
    
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());

    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);    
    
    setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);

    setConstraints(constraints, 0, 2, 2, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);
    
    setConstraints(constraints, 0, 3, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);

    setConstraints(constraints, 0, 4, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);

    setConstraints(constraints, 0, 5, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfo, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
    listeValeursSelectionChanged();
  }
  
  public void listeValeursSelectionChanged() {
  	info.setText(listeValeurs.valeurSelectionnee().infoPourHistorique());
  	pack();
  }
  
  private void btnOKActionPerformed() {                                      
    // MtgInternalFrame cadre = frame.cadreActif();
		// Spcial version applet
    // CListeObjets listePr = cadre.listePr;
    CListeObjets listePr = frame.listePourConstruction();
    String chNom = editeurNom.getText();
    if (!modification) {
	    // Modifi version 4.6
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
	    testExistence.nomCalcul = chNom;
    }
    testExistence.valeurAssociee = listeValeurs.valeurSelectionnee();
  	setRetourOK(true);
  	dispose();
  }  
	
  public void btnCancelActionPerformed() {
  	dispose();
  }

}
