/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mathgraph32;

import glob.Kernel;

import java.util.StringTokenizer;
import objets.CCalcul;
import objets.CCalculComplexe;
import objets.CFonc;
import objets.CFoncNVar;
import objets.CLatex;
import objets.CValeurDynamique;
import org.scilab.forge.jlatexmath.dynamic.ExternalConverter;

import types.Complexe;
import types.NatureCalcul;

/**
 * Classe servant au parser latex  remplacer les affichages dynamiques par leur valeur
 * Avant utilisation, setAssociation devra tre appel pour spcifier quelle est la liste active
 * et si mes tirages alatoires sont permis ou non
 * @author Yves
 *
 */
public class MtgConverter implements ExternalConverter {
	private CLatex latex;
	private static final String errorFrac = "Frac\\;error";
	private static final String errorIf = "If\\;error";
	private static final String errorFor = "For\\;error";
	private static final String errorSimp = "ForSimp\\;error";
	private static final String errorCalc = "Calc\\;error";
	private static final String errorCalcSimp = "CalcSimp\\;error";
	
	public MtgConverter(CLatex latexAssocieInit) {
		latex = latexAssocieInit;
	}
	
	public String getLaTeXString(String ch) {
		String chnombre = "\\,";
    StringBuffer bufferAffichage = new StringBuffer(ch.length() + 64);
    bufferAffichage.append(ch);
		if (ch.startsWith("\0Val\0")) {
			bufferAffichage.delete(0, 5);
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }
	    // On spare cette chaine avec les virgules
	    StringTokenizer st = new StringTokenizer(bufferAffichage.toString(), ",", false);
	    if (st.hasMoreTokens()) {
	      String nomvaleur = st.nextToken();
	      // On recherche si c'est bien le nom d'une valeur valide
	      // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
	      // Si c'est le nom d'une valeur utilise, elle est comprise dans la
	      // liste listeValeursDynamiquesUtilisees
	      int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
	      // Modifi version 4.7.2
	      // CValeurDynamique pValeur = latex.listeValeursDynamiquesUtilisees.pointeurValeurOuFonction(nomvaleur,indfin);
	      CValeurDynamique pValeur = latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
	      		(NatureCalcul.NTouteValeurPourCommentaireDynamiqueSaufFonction, nomvaleur,indfin);
	      if (pValeur != null) {
	        // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
	        if (pValeur.existe) {
	          chnombre = new String("");
	          if (st.hasMoreTokens()) {
	            String chnbdec = st.nextToken();
	            if (chnbdec.equals("+")) {
	              chnombre = pValeur.rendChaineValeurPourCommentaire(2, false);
              	if (chnombre.startsWith("-")) chnombre = "\\,-\\," + chnombre.substring(1);
              	else chnombre = "\\,+\\," + chnombre;
	            }
	            else {
	              Integer nbdec = new Integer(chnbdec);
	              int nbdecimales = nbdec.intValue();
	              chnombre = pValeur.rendChaineValeurPourCommentaire(nbdecimales, false) + "\\,";
	              // Si le troisime paramtre :
	              // commence par + : on impose un signe + devant l'affichage si le nombre est positif ou nul
	              // finit par un zro : on impose un signe + devant l'affichage si le nombre est positif ou nul
	              // et on n'affiche rien si le nombre est nul
	              // finit par 1 : on n'affiche pas le nombre si le nombre est gal  1 et on affiche un signe - sil est gal  -1
	              if (st.hasMoreTokens()) {
	                String chplus = st.nextToken();
	                boolean signePlusImpose = chplus.startsWith("+");
	                if (signePlusImpose) {
	                	if (chnombre.startsWith("-")) chnombre = "\\,-\\," + chnombre.substring(1);
	                	else chnombre = "\\,+\\," + chnombre;
	                }
                	if (chplus.endsWith("0")) {
                		double nombre = pValeur.rendValeur(false);
                		if (nombre == 0) return "\\,";
                	}
                	else {
                		double nombre = pValeur.rendValeur(false);
	                	if (chplus.endsWith("1")) {
	                		if (nombre == 1) return signePlusImpose ? "\\,+\\," : "\\,";
	                		else if (nombre == -1) return "\\,-\\,";
	                	}
                	}
	              }
	            }
	          }
	          else
	          {
	            // Si le nombre de dcimales n'est pas prcis, on en met deux par dfaut
	            return pValeur.rendChaineValeurPourCommentaire(2, false);
	          }
	        }
	      }
	    }
	    return chnombre;
		}
  	// Nouveaut version 3.4.3 \FracRed{numerateur, denominateur, parametre} provoquera l'affichage
		// d'une fraction irrductible quivalente  numerateur/denominateur
		// Si le troisime paramtre est +, un signe + sera rajout devant si le signe est positif ou nul
		// Si le troisime paramtre est 1, quand la fraction est gale  1, elle ne sera pas affiche
		// et quand elle est gale  -1 elle sera remplace par un signe -
		// Si le troisime paramtre est 0, quand la fraction est gale  0, rien ne sera affich
		if (ch.startsWith("\0FracRed\0")) {
			bufferAffichage.delete(0, 9);
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }

	    // On spare cette chaine avec les virgules
	    StringTokenizer st = new StringTokenizer(bufferAffichage.toString(), ",", false);
	    if (st.hasMoreTokens()) {
	      String nomvaleur1 = st.nextToken();
	      // On recherche si c'est bien le nom d'une valeur valide
	      // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
	      // Si c'est le nom d'une valeur utilise, elle est comprise dans la
	      // liste listeValeursDynamiquesUtilisees
	      int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
	      CValeurDynamique pValeurNum = latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
	      		(NatureCalcul.NTouteValeurReellePourCommentaireDynamique, nomvaleur1,indfin);
	      if (pValeurNum == null) return errorFrac;
	      if (pValeurNum.existe) {
          double num = pValeurNum.rendValeur(false);
          if (st.hasMoreTokens()) {
  		      String nomvaleur2 = st.nextToken();
  		      CValeurDynamique pValeurDen = latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
  		      		(NatureCalcul.NTouteValeurReellePourCommentaireDynamique, nomvaleur2,indfin);
  		      if (pValeurDen == null) return errorFrac;
  		      if (pValeurDen.existe) {
  		      	String res;
  	          double den = pValeurDen.rendValeur(false);
  	          if (den == 0) return errorFrac;
  	          double numabs = Math.abs(num);
  	          double denabs = Math.abs(den);
  	          if ((numabs != Math.floor(numabs)) || (numabs > Integer.MAX_VALUE)
  	          		|| (denabs != Math.floor(denabs)) || (denabs > Integer.MAX_VALUE) || (denabs == 0)) return errorFrac;
  	          else {
	  	          boolean signePlusImpose= false;
	  	          String chsig = "";
	  	          if (st.hasMoreTokens()) {	          	
			            chsig = st.nextToken();
			            if (chsig.startsWith("+")) signePlusImpose = true;
	  	          }
	  	          if ((num == 0) && chsig.endsWith("0")) return "\\,";
  	          	double g = Kernel.pgcd(numabs, denabs);
  	          	double num1 = numabs/g;
  	          	double den1 = denabs/g;
  	          	boolean positif = (num*den >= 0);
  	          	if (den1 == 1) {
  	          		if ((num1 == 1) && chsig.endsWith("1")) {
  	          			if (positif) return  signePlusImpose ? "\\,+\\," : "\\,";
  	          			else return "\\,-\\,";
  	          		}
  	          		res = Kernel.chaineNombre(num1, 0);
  	          	}
  	          	else res = "\\,\\frac{" + Kernel.chaineNombre(num1, 0) + "}{" + Kernel.chaineNombre(den1, 0) + "}\\,";
  	          	if (positif && signePlusImpose) res = "\\,+" + res;
  	          	
  	          	else if (!positif) res = "\\,-" + res;
  	          	return res;
  	          }
  		      }
          }
	      }
	    }
		}
		if (ch.startsWith("\0If\0")) {
			bufferAffichage.delete(0, 4);
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }
	    ch = bufferAffichage.toString();
	    // On spare cette chaine avec sparateurs \\0
	    // StringTokenizer st = new StringTokenizer(bufferAffichage.toString(), "\0", true);
	    String[] tab = ch.split("\0");
	    String nomvaleur = tab[0];
	    String arg1;
	    String arg2;
	    if (tab.length == 1) {
	    	arg1 = "";
	    	arg2 = "";
	    }
	    else {
	    	arg1 = tab[1];
	    	if (tab.length <= 2) arg2 = ""; else arg2 = tab[2];
	    }
	    int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
	    // Modifi version 4.7.2
      // CValeurDynamique pValeurNum = latex.listeValeursDynamiquesUtilisees.pointeurValeurOuFonction(nomvaleur,indfin);
	    CValeurDynamique pValeurNum = latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
	    		(NatureCalcul.NTouteValeurPourCommentaireDynamiqueSaufFonction, nomvaleur, indfin);
      if (pValeurNum == null) return errorIf;
      if (pValeurNum.existe) {
      	/*
      	 * Ajout version 4.7.2
      	 */
      	if (pValeurNum.getNatureCalcul() == NatureCalcul.NCalculComplexe) {
      		Complexe z =new Complexe();
      		((CCalculComplexe) pValeurNum).rendValeurComplexe(false, z);
      		if ((z.x == 1) && (z.y == 0)) return arg1; else return arg2;
      	}
      	//
      	else {
      		if (pValeurNum.estDeNatureCalcul(NatureCalcul.NTouteValeurReellePourCommentaireDynamique)) {
		        double num = pValeurNum.rendValeur(false);
		        if (num == 1) return arg1; else return arg2;
      		}
      		else return arg2;
      	}
      }
	    else return errorIf;
		}
		// Nouveaut version 4.5.2
		if (ch.startsWith("\0For\0")) {
			bufferAffichage.delete(0, 5);
	    // On retire les ventuels espaces de dbut cette chaine
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }
      int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
      CCalcul calcul = (CCalcul) latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
      		(NatureCalcul.NTouteValeurPourCommentaireDynamique, bufferAffichage.toString(),indfin);
      if (calcul != null) {
      	String[] varFor = null;
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
    			varFor = new String[1];
    			varFor[0] = ((CFonc) calcul).chaineVariable;
    		}
    		else {
      		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
      				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
      			CFoncNVar fnv = (CFoncNVar) calcul;
      			int nbv = fnv.nbVar;
      			varFor = new String[nbv];
      			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
      		}            			
    		}
    		return calcul.calcul.chaineLatexSansPar(varFor);          		
    	}
    	else return errorFor;
		}
		// Ajout version 4.6 : fournit le calcul en ligne
		if (ch.startsWith("\0Calc\0")) {
			bufferAffichage.delete(0, 6);
	    // On retire les ventuels espaces de dbut cette chaine
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }
      int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
      CCalcul calcul = (CCalcul) latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
      		(NatureCalcul.NTouteValeurPourCommentaireDynamique, bufferAffichage.toString(),indfin);
      if (calcul != null) {
      	String[] varFor = null;
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
    			varFor = new String[1];
    			varFor[0] = ((CFonc) calcul).chaineVariable;
    		}
    		else {
      		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
      				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
      			CFoncNVar fnv = (CFoncNVar) calcul;
      			int nbv = fnv.nbVar;
      			varFor = new String[nbv];
      			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
      		}            			
    		}
    		String chret = "\\text{" + calcul.calcul.chaineCalculSansPar(varFor) + "}";
    		// Il faut remplacer les caractres ^ par \^{}
    		StringBuffer stb = new StringBuffer(chret.length()+256);
    		stb.append(chret);
    		for (int i = 0; i < stb.length(); i++) {
    			if (stb.charAt(i) == '^') {
    				stb.replace(i, i+1, "\\;\\^{}\\;");
    				i = i + 8;
    			}
    		}
    		return stb.toString();
    	}
    	else return errorCalc;
		}
		// Ajout version 4.6 : Fournit le calcul en ligne simplifi
		if (ch.startsWith("\0CalcSimp\0")) {
			bufferAffichage.delete(0, 10);
	    // On retire les ventuels espaces de dbut cette chaine
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }
      int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
      CCalcul calcul = (CCalcul) latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
      		(NatureCalcul.NTouteValeurPourCommentaireDynamique, bufferAffichage.toString(),indfin);
      if (calcul != null) {
      	String[] varFor = null;
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
    			varFor = new String[1];
    			varFor[0] = ((CFonc) calcul).chaineVariable;
    		}
    		else {
      		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
      				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
      			CFoncNVar fnv = (CFoncNVar) calcul;
      			int nbv = fnv.nbVar;
      			varFor = new String[nbv];
      			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
      		}            			
    		}
    		String chret = "\\text{" + calcul.calcul.calculAvecValeursRemplacees().
    				calculNormalise(true, false).chaineCalculSansPar(varFor) + "}";          		
    		// Il faut remplacer les caractres ^ par \^{}
    		StringBuffer stb = new StringBuffer(chret.length()+256);
    		stb.append(chret);
    		for (int i = 0; i < stb.length(); i++) {
    			if (stb.charAt(i) == '^') {
    				stb.replace(i, i+1, "\\;\\^{}\\;");
    				i = i + 8;
    			}
    		}
    		return stb.toString();
    	}
    	else return errorCalcSimp;
		}
		
		if (ch.startsWith("\0ForSimp\0")) {
			bufferAffichage.delete(0, 9);
	    // On retire les ventuels espaces de dbut cette chaine
	    // On retire les ventuels espaces de dbut cette chaine
	    while(bufferAffichage.charAt(0) == ' ') {
	    	bufferAffichage.deleteCharAt(0);
	    }
      int indfin = latex.listeValeursDynamiquesUtilisees.size() - 1;
      CCalcul calcul = (CCalcul) latex.listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
      		(NatureCalcul.NTouteValeurPourCommentaireDynamique, bufferAffichage.toString(),indfin);
      if (calcul != null) {
      	String[] varFor = null;
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
    			varFor = new String[1];
    			varFor[0] = ((CFonc) calcul).chaineVariable;
    		}
    		else {
      		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
      				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
      			CFoncNVar fnv = (CFoncNVar) calcul;
      			int nbv = fnv.nbVar;
      			varFor = new String[nbv];
      			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
      		}            			
    		}
    		return calcul.calcul.calculAvecValeursRemplacees().calculNormalise(true, false).chaineLatexSansPar(varFor);          		
    	}
    	else return errorSimp;
		}
		return ""; // Pour satisfaire le compilateur
	}
}
