/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import javax.swing.JPanel;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import objets.CAffichageLieAPoint;
import objets.CCalcul;
import objets.CCommentaire;
import objets.CConstante;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroSuiteMacros;
import objets.CMathGrafDoc;
import objets.CPointBase;
import objets.CPrototype;
import objets.CValeurAssocieeAVariable;
import types.Couleur;
import types.DimensionFenetre;
import types.NatureObjet;
import types.StyleEncadrement;

/**
 * JPanel contenant une figure MathGraph32
 * @author Yves
 */
@SuppressWarnings("serial")
public class MtgFigurePanel extends JPanel implements MouseListener,
		MouseMotionListener, Printable {
	private MtgInternalFrame cadre;
	// Un commentaire destin  tre affich au voisinage du pointeur souris
	public CCommentaire commentaireDesignation;
	public CPointBase pointSouris; // Reflte la position du pointeur souris dans

	/**
	 * Constructeur
	 * @param cadreInit : le cadre qui contient la figure
	 */
	public MtgFigurePanel(MtgInternalFrame cadreInit) {
		cadre = cadreInit;
		setDoubleBuffered(false);
		// Le point qui sert  suivre les mouvements de la souris pour la cration
		// visuelle
		pointSouris = new CPointBase(cadre.listePr, null, false);
		commentaireDesignation = new CCommentaire(cadre.listePr, null, false,
				Couleur.TraduitCouleur(4), 0d, 0d, 10d, 12d, false, pointSouris, 2,
				StyleEncadrement.Sans, true, cadre.couleurFond, "",
				CAffichageLieAPoint.alignementHorizontalCentre,CAffichageLieAPoint.alignementVerticalHaut);
		
		addMouseMotionListener(this);
		addMouseListener(this);
	} 

	/** Il faut redfinir la mthode update car par dfaut elle efface le contenu
	 * de la fentre avec la couleur de fond avant d'appeler paint(), ce qui provoque
	 * un effet de flash
	 * */
	@Override
	public void update(Graphics graphics) {
		paint(graphics);
	}

	@Override
	public void paint(Graphics g) {
		// Ajout version 4.6 : Si l'ordre paint arrive avant que la figure soit charge, on en renvoie un nouveau
		// puis retour en attendant que tout la figure soit charge.
		if (cadre.frame.chargementEnCours) {
			// repaint(); Supprim version 4.6.2
			Thread.yield();
			return;
		}
		MtgFrame frame = cadre.frame;
		CMathGrafDoc doc = cadre.document;
		CListeObjets liste = doc.listePrincipale;
		Graphics2D g2D = (Graphics2D) g;
		setBackground(cadre.couleurFond);
    if (frame.premierAffichage) {
	    // C'est  ce niveau qu'on peut savoir combien mettre d'icnes par colonne  gauche
			frame.metAJourIconesEtMenus(frame.cadreActif().document);
			// liste.ajouteConstantePi(); Modification version 4.7.2. Ne doit pas tre fait  ce niveau
			if (frame.mtgOnInitParam.isEmpty() && (frame.path == null) && (frame.dataBase64 == null)) {
				liste.uniteAngle = frame.pref_angleUnit;
				if (frame.initialFigure.equalsIgnoreCase("orthonormalFrame")) {
					liste.ajouteRepereOrthonormal(cadre.dimf, false, false, true, "O", "I", "J", "", "",
							new CValeurAssocieeAVariable(liste, 0d), new CValeurAssocieeAVariable(liste, 0d),
							new CValeurAssocieeAVariable(liste, 1), new CValeurAssocieeAVariable(liste, 1), true);
					AjouteGraduationRepere();
				}
				else {
					if (frame.initialFigure.equalsIgnoreCase("orthogonalFrame")) {
						liste.ajouteRepereOrthogonal(cadre.dimf, false, false, true, "O", "I", "J", "", "",
								new CValeurAssocieeAVariable(liste, 0d), new CValeurAssocieeAVariable(liste, 0d),
								new CValeurAssocieeAVariable(liste, 1), new CValeurAssocieeAVariable(liste, 1), true);
						AjouteGraduationRepere();
					}
					else { // Par dfaut figure avec segment unit
						if (!frame.initialFigure.equalsIgnoreCase("empty")) {
							doc.listePrincipale.ajouteSegmentLongueurUnite();			
						}
					}
				}
			}
    	
      frame.listeObjetsVisuels.uniteAngle = cadre.listePr.uniteAngle;
      // Modification version 4.7.2 : On ne recalcule les tirages alatoires que lorsque le paramre allowRandom de l'applet est true
      // liste.positionne(true, cadre.dimf);
      liste.positionne(frame.allowRandom, cadre.dimf);
      frame.activeOutilCapture();
      // Modification version 4.7.2.6
      /*
  		if (!(frame.allowExercise && cadre.isConstruction) && (doc.listePrincipale.nombreObjetsParNatureVisibles(NatureObjet.NMacro) != 0)) frame.activeOutilExecutionMacro();
  		else frame.activeOutilCapture();
  		*/
  		if (!(frame.allowExercise && cadre.isConstruction) && (frame.allowLeftToolbar || frame.allowTopToolbar || frame.allowMenuBar)
  				&& (doc.listePrincipale.nombreObjetsParNatureVisibles(NatureObjet.NMacro) != 0)) frame.activeOutilExecutionMacro();
  		else frame.activeOutilCapture();
      
      frame.premierAffichage = false;
  		// Lors du premier chargement via un flux URL c'est ici qu'il faut initialiser le gestionnaire d'dition
  		try {
  			cadre.gestionnaire.initialise();
  		}
  		catch (IOException e) {
  			// TODO: handle exception
  		}
  		frame.isReady = true; // Ajout version 4.6.1. Permettra lors de l'appel de isReady de l'applet de savoir si l'applet a bien t affiche.
    }
		if (frame.outilPointageActif != null) {
			if (frame.outilPointageActif.aRedessinerDansPaint) frame.outilPointageActif.actionDansPaint(cadre);
		}

		// Si une macro est en cours d'excution
		if (cadre.macroEnCours != null) {
			CMacro mac = cadre.macroEnCours.macroEnCours(); // Dans le cas o la
			// macro en cours est une  CMacroSuiteMacros, mac pointe sur la macro
			// de la liste en cours d'excution
			if (mac.executionEnCours) {
				mac.actionDansPaint(cadre);
				if ((mac.executionEnCours == false)
						&& (mac.typeAnimation() == CMacro.sansAnimation)) {
					// termineAtion est  true sauf dans le cas d'une
					// CMacroSuiteMacro quand la macro en cours
					// n'est pas la dernire
					mac.termineAction(cadre);
					if (!(cadre.macroEnCours instanceof CMacroSuiteMacros))
						cadre.macroEnCours = null;

				} else {
					if (mac.dureePourAnimation() != 0) {
						if (System.currentTimeMillis() >= mac.timeFin) {
							mac.termineAction(cadre);
							// mac.executionEnCours = false;
							if (!(cadre.macroEnCours instanceof CMacroSuiteMacros))
								cadre.macroEnCours = null;
						}
					}
				}
			}
			if (cadre.macroEnCours != null) {
				if (cadre.macroEnCours instanceof CMacroSuiteMacros) {
					if (!cadre.macroEnCours.macroEnCours().executionEnCours) {
						((CMacroSuiteMacros) cadre.macroEnCours).passageMacroSuiv(cadre);
						// passageMacroSuiv redonne ventuellement un ordre paint()
						// suivant la nature de la macro suivante
						// passageMacroSuiv met executionEnCours  false si la dernire
						// macro a t excute
						if (!cadre.macroEnCours.executionEnCours) {
							cadre.macroEnCours.termineAction(cadre);
							cadre.macroEnCours = null;
						}
					}
				}
			}
			dessinegbuffer(g2D);
		} else {
	    if (frame.aRecalculerEtRedessiner) {
	    	frame.positionne();
	  		cadre.effacegbuffer();
	  		cadre.afficheToutDansgbuffer();
	  		frame.aRecalculerEtRedessiner = false;
	  		cadre.infRand = false;
	    }
	    else {
		  	if (frame.aRedessiner) {
		  		cadre.effacegbuffer();
		  		cadre.afficheToutDansgbuffer();
		  		frame.aRedessiner = false;
		  	}
	    }
			if ((frame.cadreActif() == cadre) && frame.pref_AideVisuelle && (frame.listeObjetsVisuels.size() != 0)) {
				dessinegbuffer(g2D);
				frame.listeObjetsVisuels.positionne(cadre.infRand, cadre.dimf);
				// Ligne suivante modifie version 2.0
				frame.listeObjetsVisuels.afficheTout(g2D, cadre.dimf, true, doc.couleurFond, doc.transparencePourSurfaces,
						frame.pref_antialiasing);
				if (commentaireDesignation.chaineCommentaire.length() != 0) {
					commentaireDesignation.positionne(cadre.infRand, cadre.dimf);
					commentaireDesignation.affiche(g2D, true, doc.couleurFond, doc.transparencePourSurfaces);
				}
			} else {
				if (commentaireDesignation.chaineCommentaire.length() != 0) {
					commentaireDesignation.positionne(cadre.infRand, cadre.dimf);
					dessinegbuffer(g2D);
					commentaireDesignation.affiche(g2D, true, doc.couleurFond, doc.transparencePourSurfaces);
				} else
					dessinegbuffer(g2D);
			}
		}
	}

	public void dessinegbuffer(Graphics2D gc) {
		gc.drawImage(cadre.imageBuffer, 0, 0, cadre.document.couleurFond, this);
	}

	/**
	 * Affichage dans le buffer lors de l'historique y compris les objets de la
	 * liste clignotante
	 */
	public void afficheToutDansgbufferPourHist(boolean antialiasing) {
		CMathGrafDoc doc = cadre.document;
		//  revoir
		Image imageFond = doc.imageFond;
		if ((imageFond != null) && doc.imageFondVisible) {
			cadre.gbuffer.drawImage(imageFond, 0, 0, this);
		}
		if (cadre.isConstruction) {
	    cadre.listePr.afficheJusque(cadre.gbuffer, cadre.dimf,cadre.listeSauve.size()-1, false,
	    		cadre.couleurFond, doc.transparencePourSurfaces, antialiasing);
	    cadre.listeConstruction.afficheJusque(cadre.gbuffer, cadre.dimf,
	            cadre.frame.historiqueDlg.indiceObjetEnCours, false, doc.couleurFond,
	            doc.transparencePourSurfaces, antialiasing);			
		}
		else cadre.listePr.afficheJusque(cadre.gbuffer, cadre.dimf,
				cadre.frame.historiqueDlg.indiceObjetEnCours, cadre.traceObjetsInterm
				, doc.couleurFond, doc.transparencePourSurfaces, antialiasing);
	}

	/**
	 * Affichage dans le buffer lors de l'historique y compris les objets de la
	 * liste clignotante
	 */
	public void afficheToutDansgbufferPourHistSaufListeClig(boolean antialiasing) {
		CMathGrafDoc doc = cadre.document;
		Image imageFond = doc.imageFond;
		MtgFrame frame = cadre.frame;
		if ((imageFond != null) && doc.imageFondVisible) {
			cadre.gbuffer.drawImage(imageFond, 0, 0, this);
		}
		if (cadre.isConstruction) {
	    cadre.listePr.afficheNomMasquesJusqueSaufElementsDe(cadre.gbuffer, cadre.dimf, cadre.listeSauve.size()-1,
	    		frame.listeClignotante, doc.couleurFond, doc.transparencePourSurfaces);    
	    cadre.listeConstruction.afficheJusqueSaufElementsDe(cadre.gbuffer, cadre.dimf, frame.historiqueDlg.indiceObjetEnCours,
	    		frame.listeClignotante, false, doc.couleurFond, doc.transparencePourSurfaces, antialiasing);		
		}
		else cadre.listePr.afficheJusqueSaufElementsDe(cadre.gbuffer, cadre.dimf,
				frame.historiqueDlg.indiceObjetEnCours, frame.listeClignotante, cadre.traceObjetsInterm,
				doc.couleurFond, doc.transparencePourSurfaces, antialiasing);
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
		Cursor c = cadre.frame.outilActif.curseurUtilise();
		if (getCursor() != c)
			setCursor(c);
	}

	public void mouseExited(MouseEvent e) {
		if (cadre.frame.cadreActif() == cadre) {
			Cursor c = MtgFrame.defaultCursor;
			if (getCursor() != c)
				setCursor(c);
			if (commentaireDesignation.chaineCommentaire.length() != 0) {
				commentaireDesignation.donneChaine("");
				repaint();
			}
		}
	}

	public void mousePressed(MouseEvent e) {
		if (cadre.frame.cadreActif() == cadre) {
			pointSouris.placeEn(e.getX(), e.getY());
			cadre.frame.outilPointageActif.mousePressed(e, cadre);
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (cadre.frame.cadreActif() == cadre) {
			pointSouris.placeEn(e.getX(), e.getY());
			cadre.frame.outilPointageActif.mouseReleased(e, cadre);
			Cursor c = cadre.frame.outilActif.curseurUtilise();
			if (getCursor() != c)
				setCursor(c);

		}
	}

	public void mouseDragged(MouseEvent e) {
		if (cadre.frame.cadreActif() == cadre) {
			pointSouris.placeEn(e.getX(), e.getY());
			cadre.frame.outilPointageActif.mouseDragged(e, cadre);
			Cursor c = cadre.frame.outilActif.curseurUtilise();
			if (getCursor() != c)
				setCursor(c);
		}
	}

	public void mouseMoved(MouseEvent e) {
		if (cadre.frame.cadreActif() == cadre) {
			pointSouris.placeEn(e.getX(), e.getY());
			cadre.frame.outilPointageActif.mouseMoved(e, cadre);
			Cursor c = cadre.frame.outilActif.curseurUtilise();
			if (getCursor() != c) setCursor(c);
		}
	}

	// Modifi version 4.7.0.3 pour bug avec CEditeur et image de fond
	/**
	 * Appel lors de l'impression de la figure. Pour une raison que j'ignore
	 * cette mthode est appele deux fois
	 */
	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex)
			throws PrinterException {
		if (pageIndex != 0)
			return NO_SUCH_PAGE;
		Graphics2D g2D = (Graphics2D) graphics;
		AffineTransform oldat = g2D.getTransform();
		// oldat.getScaleX() contient le nombre de pixels par 1/72 de pouce
		// horizontalement pour l'imprimante
		// oldat.getScaleY() contient le nombre de pixels par 1/72 de pouce
		// horizontalement pour l'imprimante
		double resx = oldat.getScaleX();
		double resy = oldat.getScaleY();
		double widthEffective = pageFormat.getImageableWidth(); // Contient la
		// largeur en 1/72
		// pouce de la zne
		// d'impression
		double heightEffective = pageFormat.getImageableHeight(); // Contient la
		// largeur en 1/72
		// pouce de la
		// zne
		// d'impression
		double rapport1 = widthEffective * resx
				/ (double) cadre.dimf.longueurAbscisses /4d;
		double rapport2 = heightEffective * resy
				/ (double) cadre.dimf.longueurOrdonnees /4d;
		double rapport = rapport1;
		if (rapport2 < rapport1)
			rapport = rapport2;
		// Je ne comprends pas pourquoi mais il manque une partie de l'image alors
		// je rduis un peu
		rapport *= 0.9;
		AffineTransform at = new AffineTransform(rapport, 0, 0, rapport, 0, 0);
		g2D.setTransform(at);
		BufferedImage image = new BufferedImage(cadre.dimf.longueurAbscisses*4, cadre.dimf.longueurOrdonnees*4, BufferedImage.TYPE_4BYTE_ABGR);
		Graphics2D pDC = image.createGraphics();
		CMathGrafDoc doc = cadre.document;
		Image imageFond = doc.imageFond;
		if (cadre.document.modeTraceActive) pDC.drawImage(cadre.imageTraces, 0, 0, cadre.dimf.longueurAbscisses*4, cadre.dimf.longueurOrdonnees*4,
				cadre.couleurFond, null);
		else {
			if ((imageFond != null) && doc.imageFondVisible) {
				pDC.drawImage(imageFond, 0, 0, imageFond.getWidth(null)*4, imageFond.getHeight(null)*4, cadre.couleurFond,  null);
			}
		}
		// On change la rsolution. Pour cela on fait avant une sauvegarde de la
		// figure dans un
		// flux afin de la recharer ensuite dans le mme tat
		try {
			ByteArrayOutputStream sauvegarde = new ByteArrayOutputStream();
			MathGraphDataOutputStream oups = new MathGraphDataOutputStream(sauvegarde);
			cadre.listePr.write(oups);
			oups.close();
			CListeObjets listeCopie = new CListeObjets(cadre.listePr);
	    listeCopie.bCreationComposants = false; // Pour qu'il n'y ait pas recration des composants pour les CEditeurFormule
	    ByteArrayInputStream ba = new ByteArrayInputStream(sauvegarde.toByteArray());
	    MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
	    listeCopie.read(inps);
	    inps.close();
			
			listeCopie.basseDefinition = false;
			listeCopie.adaptePourHauteResolution(4);
			listeCopie.bAttenuationEpaisseurTrait = true;
			DimensionFenetre dim = new DimensionFenetre(cadre.dimf.longueurAbscisses * 4, cadre.dimf.longueurOrdonnees * 4);
			listeCopie.positionne(cadre.infRand, dim);
			listeCopie.afficheTout(pDC, dim, cadre.masquageObjetsCaches, doc.couleurFond,
					doc.transparencePourSurfaces, cadre.frame.pref_antialiasing);
			listeCopie.retireTout();
			g2D.drawImage(image, 0, 0, (int) (cadre.dimf.longueurAbscisses*4), (int) (cadre.dimf.longueurOrdonnees*4), null);
			g2D.setTransform(oldat);
			
			return PAGE_EXISTS;
		} catch (IOException err) {
			g2D.setTransform(oldat);

			return NO_SUCH_PAGE;
		}
	}
	
	private void AjouteGraduationRepere() {
		MtgFrame frame = cadre.frame;
		CListeObjets liste = cadre.listePr;
		String nom1 = "nbgradx";
		CCalcul calcul1 = new CCalcul(liste, null, false, nom1, "20", new CConstante(liste, 20));
		frame.ajouteElement(calcul1);
		String nom2 = "nbgrady";
		CCalcul calcul2 = new CCalcul(liste, null, false, nom2, "20", new CConstante(liste, 20));
		frame.ajouteElement(calcul2);
  	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("GraduationAxesRepere.mgc");
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = liste.premierRepere();
		((CElementGenerique)proto.get(1)).elementAssocie = calcul1;
		((CElementGenerique)proto.get(2)).elementAssocie = calcul2;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		proto.retireTout(); // Ajout version 4.6.2
		frame.cadreActif().redessineAvecCalcul();
		
	}

}
