/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.swing.JLayeredPane;
import javax.swing.SpringLayout;
import objets.CAffichageLieAPoint;
import objets.CAppelFonction;
import objets.CCalcul;
import objets.CCalculBase;
import objets.CCommentaire;
import objets.CDemiDroiteOA;
import objets.CDroiteAB;
import objets.CDroiteAncetre;
import objets.CElementBase;
import objets.CFonc;
import objets.CLieuDePoints;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroAnimationPointLie;
import objets.CMacroApparition;
import objets.CMacroPause;
import objets.CMathGrafDoc;
import objets.CMesureX;
import objets.CPointDansRepere;
import objets.CPointImage;
import objets.CPointLieDroite;
import objets.CRepere;
import objets.CResultatValeur;
import objets.CSegment;
import objets.CSousListeObjets;
import objets.CTranslation;
import objets.CValeurAssocieeAVariable;
import objets.CVariableBornee;
import types.Couleur;
import types.DimensionFenetre;
import types.InfoLieu;
import types.MotifPoint;
import types.StylePoint;
import types.StyleTrait;
import dialogs.PaneAffichageVariables;
import entreesSorties.MathGraphTimerTask;

/**
 * MtgInternalFrame est la classe reprsentant une sous-fentre de la fentre
 * principale associe  un document
 * 
 * @author Yves
 */
@SuppressWarnings("serial")
// Modification version Applet : MtgInternalFrame drive de JPanel et pas de MouseWheelListener car la molette sert pour la navigation dans les pages
public class MtgInternalFrame extends JLayeredPane implements Runnable, KeyListener
	{
	private String title; // Ajout version Applet
	public MtgFrame frame; // Le frame propritaire qui est dans cette version l'applet
	// Variable servant  savoir si la touche Ctl a t appuye. Utilise pour l'outil de nommage d'un point ou une droite
	public boolean ctlKeyPressed = false;
	// Variable rajoute  la version 1.3.5 servant  mmoriser la couleur de fond utilise pour construire
	// le dernier commentaire ou affichage de valeur ou intitul de macro
	public Color couleurFondAffichage;

	// Thread utilis pour certaines macros
	public Thread runner;
	// Timer pour les macros
	public MathGraphTimerTask timerTask;
	public java.util.Timer timer;
	public CMacro macroEnCours; // Macro ventuellement en cours d'excution dans
															// le cadre
	public boolean masquageObjetsCaches;
	public GestionnaireEdition gestionnaire = new GestionnaireEdition(this);
	// Chemin d'accs  la figure. Null s'il s'agit d'une nouvelle figure pas
	// encore enregistre
	public String path;
	// Dimensions du JInternalFrame
	public Dimension size = new Dimension(0, 0);
	// Dimensions du panel qui contiendra la figure
	public DimensionFenetre dimf = new DimensionFenetre(0, 0);
	public boolean infRand; // Mis  true quand des expriences alatoires sont en
													// cours dans la fentre
	public CMathGrafDoc document;
  public boolean isConstruction = false; // Spcial applet : true si l'applet sert  faire un exercice de construction
  public CMacroApparition macroPourCons;	// Spcial applet : dans le cas d'un exercice de construction, pointe sur la cmacro
  public boolean isExercise = false;  // Spcial Applet : true si l'applet contient via la macro ci-dessus une liste de calculs autoris
  public CMacroPause macroPourCalculs;	// Spcial applet : Pour un exercice de validation interne, ventuelle macro dont le commentaire contiendra les noms
  																				// des calculs ou variables autoriss  l'utilisateur
  																				// Pour un tel exercice, aucune cration d'objet n'est permise mais on peut y mettre des champs d'dition
  																				// Un calcul nomm reponse doit valoir 1 si l'exercice a t rsolu et 0 sinon.
  public CListeObjets listeProposee;
  public CSousListeObjets listeAConstruire; // Pointe sur une liste forme des lments
  // que l'utilisateur doit construire avant de valider sa constriction.
  // null s'il ne s'agit pas d'un exercice de construction
  // La liste d'objets que l'utilisateur dsignera comme solutions
  																				// d'apparition d'objets faisant apparatre les objets  construire
	public CListeObjets listePr; // Pointeur sur document.listePrincipale
  public CListeObjets listeSauve; // Spcial applet : Utilise comme sauvegarde de la figure dans son tat initial au chargement de la figure
  public CListeObjets listeConstruction;	// Spcial applet version 4.6 : Liste utilise pour stocker tous les lments construits par l'utilisateur
  																				// dans le cas d'un exercice de construction
  public CListeObjets listeCalculsAutorises;	// Spcial applet. Si l'exercice est un exercice de validation interne, le commentaire
  																						// de la macro contient entre accolades les calculs autoriss
	public CListeObjets listeDependantsDansPrincipale; // Utilise pour les captures de points mobiles
	public Graphics2D gbuffer = null;
	public MtgFigurePanel paneFigure; // Le panneau dans lequel se passent tous les affichages
  public PaneAffichageVariables paneAffichageVariables;
  // Supprim version Applet c'est this qui correspond  paneCenter
  // public JLayeredPane paneCenter; // Le panneau recevant les deux prcdents
  public SpringLayout springl;  
	// un DeviceContext pour les macros laissant une trace
	public BufferedImage imageTraces;
	// L'ventuelle image de fond
	public BufferedImage imageBuffer = null;
	// Nouveau pour la version 1.8 : un DeviceContext pour les macros laissant
	// une trace
	public Graphics2D tbuffer;
	public Color couleurFond = Color.white; // Couleur de fond du document de la
	// Variable mise  true quand le dialogue d'historique trace dans les objets intermdiaires des constructions;
	public boolean traceObjetsInterm = false;
	// Macro d'animation servant dans le menu Mouvoir Animation
	public CMacroAnimationPointLie macroAnimation;
	// Listes contenant les objets sources et finaux des futures constructions (prototypes)
	// public CListeObjets listeObjetsSourcesG; supprim version Applet
	// public CListeObjets listeObjetsSourcesNG; supprim version Applet
	// public CListeObjets listeObjetsFinauxG; supprim version Applet
	// public CListeObjets listeObjetsFinauxNG; supprim version Applet

	// fentre. A revoir

	/**
	 * Constructeur crant une nouvelle fentre avec un nouveau document
	 */
	// public MtgInternalFrame(MtgFrame frameInit, String titre, CMathGrafDoc documentInit) { Modifi version Applet
	public MtgInternalFrame(MtgFrame frameInit, CMathGrafDoc documentInit) {
		// super(titre, true, true, true, true); // Modifi version Applet
		// document = documentInit; // Supprim version Applet
		// document.cadreProprietaire = this; // Supprim version Applet
		// URL adr = MtgFrame.class.getResource("images/IconeDoc.png"); Supprim version Applet
		// ImageIcon imic = new ImageIcon(adr); Supprim version Applet
		// setFrameIcon(imic); Modifi version Applet
		frame = frameInit;
		document = documentInit;
		setDocument(document);
		masquageObjetsCaches = true;
		path = null;
		paneFigure = new MtgFigurePanel(this);
    springl = new SpringLayout();
    setLayout(springl);
    add(paneFigure, JLayeredPane.DEFAULT_LAYER);
    springl.putConstraint(SpringLayout.NORTH, paneFigure, 0, SpringLayout.NORTH, this);
    springl.putConstraint(SpringLayout.WEST, paneFigure, 0, SpringLayout.WEST, this);
		setDoubleBuffered(false);
		setFocusable(true);
		addKeyListener(this);
		macroAnimation = new CMacroAnimationPointLie(document.listePrincipale, null, false, Color.black,
				0d, 0d, 0d, 0d, false, null, 2, false, Color.white, "","",true,  20, 100, 0, null,
				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
		setVisible(true);
		requestFocus();
	}
	
	public void setDocument(CMathGrafDoc doc) {
		document = doc;
		document.cadreProprietaire = this;
		listePr = document.listePrincipale;
		listeSauve = new CListeObjets(); // Spcial applet
		listeConstruction = new CListeObjets(doc); // Spcial applet
		listeCalculsAutorises = new CListeObjets(); // Spcial applet
    listeProposee = new CListeObjets(listePr.uniteAngle, listePr.pointeurLongueurUnite);
		Couleur.setCouleurFond(document.couleurFond);
		couleurFond = document.couleurFond;
		couleurFondAffichage = couleurFond;
		setBackground(couleurFond);
		activeModeTrace(document.modeTraceActive);
		initialiseListes();
	}
	
	public final void activeModeTrace(boolean bactive) {
		document.modeTraceActive = bactive;
		frame.buttonModeTrace.setSelected(bactive);
		frame.cbModeTrace.setSelected(bactive);
	}	
	
	/**
	 * Ajout version Applet
	 * @param titleInit
	 */
	public final void setTitle(String titleInit) {
		title = titleInit;
	}
	
	/**
	 * Ajout version Applet
	 * @return
	 */
	public final String getTitle() {
		return title;
	}

	/**
	 * A appeler aprs avoir rajout des objets  une figure vide, comme un repre
	 */
	public void initialiseListes() {
		listeDependantsDansPrincipale = new CListeObjets(listePr);
		/* supprim version Applet
		listeObjetsSourcesG = new CListeObjets(listePr);
		listeObjetsSourcesNG = new CListeObjets(listePr);
		listeObjetsFinauxG = new CListeObjets(listePr);
		listeObjetsFinauxNG = new CListeObjets(listePr);
		*/
	}
	
	/* Retire version Applet
	public void internalFrameActivated() {
		
		requestFocus(); // Pour pouvoir arrter une animation par la touche
										// Echappement
		ctlKeyPressed = false;
		if (imageBuffer == null) {
			prepareBuffers();
			repaintFigureAvecCalcul();
		}
		frame.listeClignotante.retireTout();
		frame.listeExclusionDesignation.retireTout();
		frame.listeObjetsVisuels.retireTout();
		// creeDialogueAffichageVariables();
		// montreDialogueAffichageVariables();
		frame.metAJourIconesEtMenus(document);
		// Ajout version 2.0
		frame.paneTransparence.donneValeurTransparence(document.transparencePourSurfaces, this);
		// Ajout version3.2.6
		frame.buttonModeTrace.setSelected(modeTrace);
		// Si une macro est en cours d'excution on active l'outil d'excution de
		// macro
		if ((macroEnCours != null) && (frame.outilActif != frame.outilExecutionMacro))
			frame.activeOutilExecutionMacro();
		// On adapte les objets visuels  l'unit d'angle de la figure
		frame.listeObjetsVisuels.uniteAngle = document.listePrincipale.uniteAngle;
		// Pour une raison que je n'ai pas trouve, la barre d'ouutils de gauche ne s'affiche pas correctement
		// tant qu'on a pas envoy un ordre d'affichage au frame. D'o la ligne suivante...
		frame.indication(frame.indicationArea.getText());
	}
	*/
		
	/* supprim version Applet
	public void internalFrameClosing(InternalFrameEvent e) {
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		if (document.isDirty()) {
			int rep = frame.dialogueConfirmation_YES_NO_CANCEL(textes.Chaines.ch119 + " " + nomDocument());
			if (rep == JOptionPane.CANCEL_OPTION) {
				setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
				return;
			}
			if (rep == JOptionPane.YES_OPTION)
				((ActionFichierEnregistrer) frame.actionFichierEnregistrer).enregistreFigure();
		}
	}
	*/

	/* supprim version Applet
	public void internalFrameDeactivated(InternalFrameEvent e) {
		// supprimeDialogueAffichageVariables();
		// cacheDialogueAffichageVariables();
	}

	public void internalFrameDeiconified(InternalFrameEvent e) {
	}

	public void internalFrameIconified(InternalFrameEvent e) {
	}

	public void internalFrameOpened(InternalFrameEvent e) {
		// Ajout version 2.0
		frame.paneTransparence.donneValeurTransparence(document.transparencePourSurfaces, this);
		// Ajout version3.2.6
		frame.buttonModeTrace.setSelected(modeTrace);
	}
	*/

	/**
	 * Prparation des buffers
	 */
	public void prepareBuffers() {
		dimf.setSize(paneFigure.getWidth(), paneFigure.getHeight());
		// Modifi version 4.6.2 pour utiliser des BufferedImage
		imageBuffer = new BufferedImage(dimf.longueurAbscisses, dimf.longueurOrdonnees, BufferedImage.TYPE_4BYTE_ABGR);
		gbuffer = (Graphics2D) imageBuffer.getGraphics();
		imageTraces = new BufferedImage(dimf.longueurAbscisses, dimf.longueurOrdonnees, BufferedImage.TYPE_4BYTE_ABGR);
		tbuffer = (Graphics2D) imageTraces.getGraphics();
		effacetBuffer();  // Ajout version 4.4.4
	}
	
	/**
	 * Fonction ajoute version 4.6.2 pour librer les ressources des images utilises pour les Buffers quand on quitte l'applet
	 */
	public void flushImageBuffers() {
		imageBuffer.flush();
		imageTraces.flush();
	}

	/**
	 * Affichage de la figure dans le buffer
	 */
	public synchronized void afficheToutDansgbuffer() {
		listePr.afficheTout(gbuffer, dimf, masquageObjetsCaches, couleurFond, document.transparencePourSurfaces, frame.pref_antialiasing);
		/* Correction d'un dfaut version 4.7.2 le point laissait une mauvaise trace quand l'outil rideau tait activ
		if (document.modeTraceActive) listePr.affichePointsMarquesPourTrace(tbuffer, dimf, masquageObjetsCaches, couleurFond,
				document.transparencePourSurfaces, frame.pref_antialiasing);
		*/
		if (document.modeTraceActive && (frame.outilActif != frame.outilRideau)) listePr.affichePointsMarquesPourTrace(tbuffer, dimf, masquageObjetsCaches, couleurFond,
				document.transparencePourSurfaces, frame.pref_antialiasing);
	}
	
	/**
	 * Affichage de la figure dans le buffer sauf les lments de listeExclue
	 * sans effacement initial du buffer
	 * @param listeExclue
	 */
	public synchronized void afficheToutDansgbufferSaufElementsDe(CListeObjets listeExclue) {
		Image imageFond = document.imageFond;
		if ((imageFond != null) && document.imageFondVisible) {
			gbuffer.drawImage(imageFond, 0, 0, this);
		}
		listePr.afficheToutSaufElementsDe(gbuffer, dimf, masquageObjetsCaches, couleurFond, document.transparencePourSurfaces,
				listeExclue, frame.pref_antialiasing);
	}
	

	public final synchronized void effaceFenetre(Graphics2D gc) {
		gc.setColor(couleurFond); // Ncessaire car peut tre appel en dehors
		// de paint
		gc.fillRect(0, 0, getWidth(), getHeight());
	}
	
	// Synchronized rajout version 2.0 pour viter que dans certains cas la couelur de fond soit dans une
	// couleur alatoire
	public synchronized void effacegbuffer() {
		BufferedImage imageFond = document.imageFond;
		if (document.modeTraceActive) {
			transferetbufferversgbuffer();
		}
		else {
			effaceFenetre(gbuffer);
			if ((imageFond != null) && document.imageFondVisible) {
				gbuffer.drawImage(imageFond, 0, 0, this);
			}
		}
	}

	/**
	 * Recalcul de la figure et affichage aprs effacement du gbuffer
	 */
	public final void redessineAvecCalcul() {
		listePr.positionne(infRand, dimf);
		// effaceFenetre(gbuffer);
		// if (modeTrace) transferetbufferversgbuffer();
		effacegbuffer();
		afficheToutDansgbuffer();
	}

	/**
	 * Version 1.8 : mthode publique servant  recalculer et rafficher
	 * compltement la figure en effaant tout et raffichant tout sans recalculer
	 * tout et sans sauvegarde pour rtroaction.
	 */
	public void redessineSansCalcul() {
		// effaceFenetre(gbuffer); Supprim version 4.4. Inutile
		// if (modeTrace) transferetbufferversgbuffer();
		effacegbuffer();
		afficheToutDansgbuffer();
	}

	/**
	 * Par dfaut, lors d'un repaint, la figure n'est pas recalcule Appeler cette
	 * fonction pour recalculer la figure puis la rafficher en particulier lors
	 * de termineAction() des macros
	 * La figure est efface avant d'tre redessine
	 **/
	public void repaintFigureAvecCalcul() {
		effacegbuffer();
		// if (modeTrace) transferetbufferversgbuffer();
		listePr.positionne(infRand, dimf);
		afficheToutDansgbuffer();
		paneFigure.repaint();
	}

	/**
	 * Par dfaut, lors d'un repaint, la figure n'est pas recalcule Appeler cette
	 * fonction pour recalculer la figure puis la rafficher en particulier lors
	 * de termineAction() des macros
	 * La figure est efface avant d'tre redessine
	 **/
	public void repaintFigureSansCalcul() {
		effacegbuffer();
		// if (modeTrace) transferetbufferversgbuffer();
		afficheToutDansgbuffer();
		paneFigure.repaint();
	}
	
	// Ajout version 4.6
	/**
	 * Envoie un message repaint immdiat  la figure
	 */
	public void repaintFigureImmediatly() {
		paneFigure.paintImmediately(0, 0, dimf.longueurAbscisses, dimf.longueurOrdonnees);
	}
	
	/**
	 * Pour les macros devant laisser une trace, initialise le device context de
	 * traces en l'effaant et en affichant l'image de fond s'il y en a une
	 * Modifi version 4.7.7. Mme si le mode trace est actif on l'excute.
	 */
	public void initialisetbuffer() {
		//if (!document.modeTraceActive) {
		effaceFenetre(tbuffer);
		Image imageFond = document.imageFond;
		if ((imageFond != null) && document.imageFondVisible)
			tbuffer.drawImage(imageFond, 0, 0, this);
		//}
	}

	/**
	 * Fonction appele pour initialiser le buffer de traces lors de sa cration
	 * Ajout version 4.4.4 pour corriger un bug qui faisait que, quand on ouvrait une image
	 * avec le mode trace activ, la couleur de fond n'tait pas la bonne et l'image de fond pas affiche s'il y en avait une
	 */
	public void effacetBuffer() {
		effaceFenetre(tbuffer);
		Image imageFond = document.imageFond;
		if ((imageFond != null) && document.imageFondVisible)
			tbuffer.drawImage(imageFond, 0, 0, this);
	}

	/**
	 * Transfre l'image contenue dans le tbuffer (traces) dans le gbuffer
	 */
	public void transferetbufferversgbuffer() {
		gbuffer.drawImage(imageTraces, 0, 0, this);
	}

	/**
	 * Fonction mettant  chane vide la chaine utilise pour la dsignation
	 * d'objets
	 */
	public void videDesignation() {
		paneFigure.commentaireDesignation.donneChaine("");
	}

	/**
	 * 
	 * @return un pointeur sur le commentaire de dsignation du cadre actif qui
	 *         appartient au paneFigure
	 */
	public CCommentaire commentaireDesignation() {
		return paneFigure.commentaireDesignation;
	}

	/**
	 * Appel lors d'un chanement de taille de la fentre
	 */
	/* supprim version Applet
	@Override
	public void reshape(int x, int y, int width, int height) {
		super.reshape(x, y, width, height);
		if (isVisible()) {
			if ((width != size.width) || (height != size.height)) {
				size.width = width;
				size.height = height;
				// Les deux ligne ssuivantes rajoutes pour la version 3.5  cause du paneCenter qui contient paneFigure
				paneFigure.setPreferredSize(paneCenter.getSize());
				paneFigure.setSize(paneCenter.getSize());
				document.sizeWindowx = paneFigure.getWidth();
				document.sizeWindowy = paneFigure.getHeight();
				prepareBuffers();
				repaintFigureAvecCalcul();
			}
		}
	}
	*/


	// A tudier pour future version
	/*
	 @Override public void setMaximum(boolean b) throws PropertyVetoException {
		 javax.swing.plaf.InternalFrameUI ui = getUI();
		 ((javax.swing.plaf.basic.BasicInternalFrameUI)ui).setNorthPane(null);
		 setBorder(null);
		 super.setMaximum(b);
		 prepareBuffers();
		 repaintFigureAvecCalcul();
	 }
	 */

	public void run() {
		if (macroEnCours != null) { // Deuxime test indispensable
			macroEnCours.macroEnCours().executePourThread(this);
		}
	}

	// Modifi version Applet
	public void keyPressed(KeyEvent evt) {
		/*
	}
		if ((evt.getKeyCode() == KeyEvent.VK_F7)) {
			//Affichage d'un dialogue avec les infos d'une construction si on est en train de l'implmenter
			if ((frame.outilActif == frame.outilConstructionImplementationFichier)
					|| (frame.outilActif == frame.outilConstructionImplementationFigure)) {
				CPrototype proto = null;
				if (frame.outilActif == frame.outilConstructionImplementationFichier) proto = frame.outilConstructionImplementationFichier.proto;
				else proto = frame.outilConstructionImplementationFigure.proto;
				InformationConstructionDlg dlg = new InformationConstructionDlg(frame, proto);
				dlg.setLocationRelativeTo(frame);
				dlg.setVisible(true);
			}
		}
		else {
		*/
			if ((evt.getKeyCode() == KeyEvent.VK_CONTROL)) {
				ctlKeyPressed = true;
			}
			else ctlKeyPressed = false;

		//}
	}

	public void keyReleased(KeyEvent evt) {
		if ((evt.getKeyCode() == KeyEvent.VK_CONTROL)) {
			ctlKeyPressed = false;
		}
	}

	public void keyTyped(KeyEvent evt) {
		if ((evt.getKeyChar() == KeyEvent.VK_ESCAPE)) {
			if (macroEnCours != null) {
				macroEnCours.termineAction(this);
				macroEnCours = null;
				try {
					Thread.sleep(100); // Pour laisser le temps aux boucles en cours
															// d'excution de se terminer
				} catch (InterruptedException e) {
				}
				redessineSansCalcul();
			}
			else frame.activeOutilCapture();
		}
	}
	
	public void creePaneAffichageVariables() {
		if (listePr.nombreVariablesAvecDialogueAssocie() == 0) {
			if (paneAffichageVariables != null) {
				remove(paneAffichageVariables);
			}
		}
		else {
			if (paneAffichageVariables != null) {
				remove(paneAffichageVariables);
				paneAffichageVariables = null;
			}
	    paneAffichageVariables = new PaneAffichageVariables(this);
	    remove(paneAffichageVariables);
	    add(paneAffichageVariables, JLayeredPane.DRAG_LAYER);
	    springl.putConstraint(SpringLayout.SOUTH, paneAffichageVariables, 0, SpringLayout.SOUTH, this);
	    springl.putConstraint(SpringLayout.EAST, paneAffichageVariables, 0, SpringLayout.EAST, this);
		}
	}
	
	public void metAJourPaneAffichageVariables(CVariableBornee var) {
		if (paneAffichageVariables != null) paneAffichageVariables.metAJourAffichageDe(var);
	}


	/**
	 * Renvoie le nom du document associ
	 * @return
	 */
	public String nomDocument() {
		if (path == null)
			return getTitle();
		int ind = path.lastIndexOf(File.separatorChar);
		return path.substring(ind + 1);
	}
	
	/**
	 * Procdure crant une courbe de fonction dans le repre RepereCourbe
	 * avec un point li  DroiteIntervalle (qui peut pointer sur une droite,
	 * un segment ou une demi-droite.
	 * Si aucun point ne semble visible, un dialogue demande confirmation
	 * de la cration de la courbe. En cas de refus, les objets crs
 	 * sont dtruits et on renvoie false. Sinon on renvoie true
	 * @param repere Le repre dans lequel tracer la courbe
	 * @param droiteIntervalle L'objet sur lequek varie un point dont les positions gnreront la courbe
	 * peut tre une droite, un segmen ou une demi-droite
	 * @param fonction : La fonction dont on doit tracer la courbe
	 * @param nomPointLieAxe : Le nom du point li  l'axe des abscisses
	 * @param nomCalculAbscisse : le nom du calcul contenant l'abscisse du point mobile
	 * @param nomCalculOrdonnee : le nom du calcul contenant l'image de l'abscisse
	 * @param nombrePoints : le nombre de points de la courbe
	 * @param pointLieCache : true si le point li doit tre cach
	 * @param pointCourbeCache : true si le point gnrant la courbe doit tre cach
	 * @param gestionAutoDiscontinuite : true pour gestion auto des discontinuits
	 */
	public final boolean ajouteCourbe(CRepere repere, CDroiteAncetre droiteIntervalle,
			CFonc fonction, String nomPointLieAxe, String nomCalculAbscisse, String nomCalculOrdonnee,
			int nombrePoints, boolean pointLieCache, boolean pointCourbeCache,boolean gestionAutoDiscontinuite) {
		// On gnre une abscisse alatoire pour le point li  l'axe des
		// abscisses
		double abs1 = droiteIntervalle.abscisseMinimale();
		double abs2 = droiteIntervalle.abscisseMaximale();
		double d = abs2 - abs1;
		abs1 = abs1 + d/10;
		abs2 = abs2 - d/10;
		double abs = abs1 + (abs2 - abs1)*Math.random();
		CPointLieDroite pointLieDroite = new CPointLieDroite(listePr, null, false,
				Color.black, false, 0d, 0d, pointLieCache, nomPointLieAxe, 2, MotifPoint.petitRond, true, abs,
				droiteIntervalle, false);
		pointLieDroite.donneNom(nomPointLieAxe);
		frame.ajouteElement(pointLieDroite);
		// On cre la mesure de l'abscisse du point qu'on vient de crer
		CMesureX mesureAbscisse = new CMesureX(listePr, null, false, repere, pointLieDroite);
		frame.ajouteElement(mesureAbscisse);
		// Il faut crer un calcul algbrique contenant comme formule
		// l'abscisse qu'on vient de mesurer
		CCalculBase abscisse = new CResultatValeur(listePr, mesureAbscisse);
		CCalcul calculAbscisse = new CCalcul(listePr, null, false, nomCalculAbscisse,
				mesureAbscisse.nom(), abscisse);
		frame.ajouteElement(calculAbscisse);
		// Il faut crer un calcul algbrique contenant comme valeur
		// l'image par la fonction du calcul contenant l'abscisse
		// mesure
		CCalculBase ordonnee = new CAppelFonction(listePr, fonction, new CResultatValeur(listePr, calculAbscisse));
		CCalcul calculOrdonnee = new CCalcul(listePr, null, false, nomCalculOrdonnee,
				ordonnee.chaineCalculSansPar(null), 
				ordonnee);
		frame.ajouteElement(calculOrdonnee);
		// Il faut maintenant crer le point repr de la courbe dont
		// les positions successives gnreront cette courbe
		CPointDansRepere pointCourbe = new CPointDansRepere(listePr, null, false, Color.black,
				false, 0d, 0d, pointCourbeCache, "", 2, MotifPoint.petitRond, repere, 
				new CValeurAssocieeAVariable(listePr, new CResultatValeur(listePr, calculAbscisse)),
				new CValeurAssocieeAVariable(listePr, new CResultatValeur(listePr, calculOrdonnee)), false);
		frame.ajouteElement(pointCourbe);
		// Il faut maintenant crer la courbe comme un lieu de points
		InfoLieu infoLieu = new InfoLieu(nombrePoints, false, gestionAutoDiscontinuite);
		CLieuDePoints lieu = new CLieuDePoints(listePr, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), pointCourbe, infoLieu, pointLieDroite);
		frame.ajouteElement(lieu);
		lieu.etablitListeElementsAncetres();
		// On regarde si le lieu cr est visible
		listePr.positionne(false, dimf);
		if (!lieu.dansFenetre(dimf)) {
			if (!frame.dialogueConfirmation_YES_NO(MtgFrame.getString("ch134")))
				frame.detruitNDerniersElements(6);
			return false;
		}
		return true;
		
	}
	
	/**
	 * Ajout d'une courbe sur R
	 * @param repere
	 * @param fonction
	 * @param nomPointLieAxe
	 * @param nomCalculAbscisse
	 * @param nomCalculOrdonnee
	 * @param nombrePoints
	 * @param pointLieCache
	 * @param pointCourbeCache
	 * @param gestionAutoDiscontinuite
	 */
	public final void ajouteCourbeSurR(CRepere repere, CFonc fonction, String nomPointLieAxe, String nomCalculAbscisse, String nomCalculOrdonnee,
			int nombrePoints, boolean pointLieCache, boolean pointCourbeCache,boolean gestionAutoDiscontinuite) {
		// On regarde si le repre possde dj un axe des abscisses
		CDroiteAB axe = listePr.existeDroiteABDefiniePar(repere.O, repere.I);
		if (axe == null) {
  		axe = new CDroiteAB(frame.cadreActif().listePr, null, false, Color.black, 
  				StyleTrait.traitFin, repere.O, repere.I, frame.pref_indiceTaillePoliceNom);
  		frame.ajouteElement(axe);
		}
		axe.positionne(false, dimf); 
		ajouteCourbe(repere, axe, fonction, nomPointLieAxe, nomCalculAbscisse, nomCalculOrdonnee,
				nombrePoints, pointLieCache, pointCourbeCache, gestionAutoDiscontinuite);
	}
	
	/**
	 * Ajout d'une courbe sur un intervalle
	 * @param repere
	 * @param fonction
	 * @param nomPointLieAxe
	 * @param nomCalculAbscisse
	 * @param nomCalculOrdonnee
	 * @param nombrePoints
	 * @param pointLieCache
	 * @param pointCourbeCache
	 * @param gestionAutoDiscontinuite
	 * @param a
	 * @param b
	 */
	public final void ajouteCourbeSurab(CRepere repere, CFonc fonction, String nomPointLieAxe, String nomCalculAbscisse, String nomCalculOrdonnee,
			int nombrePoints, boolean pointLieCache, boolean pointCourbeCache,boolean gestionAutoDiscontinuite,
			CValeurAssocieeAVariable a, CValeurAssocieeAVariable b) {
		a.positionne(false, dimf);
		b.positionne(false, dimf);
		// On regarde si le repre possde dj un point d'abscisse a
		CPointDansRepere point1 = listePr.existeabscrissea(a, repere);
		if (point1 == null) {
			point1 = new CPointDansRepere(listePr, null, false, Color.black,false, 0d, 0d, true,
				"", 2, StylePoint.petitRond, repere, a, new CValeurAssocieeAVariable(listePr, 0), false);
			frame.ajouteElement(point1);
			point1.positionne(false, dimf);
		}
		CPointDansRepere point2 = listePr.existeabscrissea(b, repere);
		if (point2 == null) {
			point2 = new CPointDansRepere(listePr, null, false, Color.black,false, 0d, 0d, true,
				"", 2, StylePoint.petitRond, repere, b, new CValeurAssocieeAVariable(listePr, 0), false);
			frame.ajouteElement(point2);
			point2.positionne(false, dimf);
		}
		CSegment seg = listePr.existeSegmentDefiniPar(point1, point2);
		if (seg == null) {
			seg = new CSegment(listePr, null, false, Color.black, true, StyleTrait.traitFin, point1, point2);
			frame.ajouteElement(seg);
			seg.positionne(false, dimf);
		}
		ajouteCourbe(repere, seg, fonction, nomPointLieAxe, nomCalculAbscisse, nomCalculOrdonnee,
				nombrePoints, pointLieCache, pointCourbeCache, gestionAutoDiscontinuite);
	}
	
	/**
	 * Ajout d'une courbe sur un intervalle [a; + inf [
	 * @param repere
	 * @param fonction
	 * @param nomPointLieAxe
	 * @param nomCalculAbscisse
	 * @param nomCalculOrdonnee
	 * @param nombrePoints
	 * @param pointLieCache
	 * @param pointCourbeCache
	 * @param gestionAutoDiscontinuite
	 * @param a
	 */
	public final void ajouteCourbeAPartirdea(CRepere repere, CFonc fonction, String nomPointLieAxe, 
			String nomCalculAbscisse, String nomCalculOrdonnee, int nombrePoints, boolean pointLieCache,
			boolean pointCourbeCache,boolean gestionAutoDiscontinuite, CValeurAssocieeAVariable a) {
		a.positionne(false, dimf);
		// On regarde si le repre possde dj un point d'abscisse a
		CPointDansRepere point1 = listePr.existeabscrissea(a, repere);
		if (point1 == null) {
			point1 = new CPointDansRepere(listePr, null, false, Color.black,false, 0d, 0d, true,
				"", 2, StylePoint.petitRond, repere, a, new CValeurAssocieeAVariable(listePr, 0), false);
			frame.ajouteElement(point1);
			point1.positionne(false, dimf);
		}
		CTranslation trans = listePr.existeTranslationDefiniePar(repere.O, repere.I);
		if (trans == null) {
			trans =	new CTranslation(listePr, null, false, repere.O, repere.I);
			frame.ajouteElement(trans);
			trans.positionne(false, dimf);
		}
		CPointImage im = listePr.pointImageDePar(point1, trans);
		if (im == null) {
			im =new CPointImage(listePr, null, false, Color.black, true, 0d, 0d, true, "", 2,
				StylePoint.petitRond, point1, trans, false);
			frame.ajouteElement(im);
			im.positionne(false, dimf);
		}
		CDemiDroiteOA demidte = listePr.existeDemidroiteDefiniePar(point1, im);
		if (demidte == null) {
			demidte = new CDemiDroiteOA(listePr, null, false, Color.black, StyleTrait.traitFin, point1, im);
			frame.ajouteElement(demidte);
			demidte.positionne(false, dimf);
		}
		ajouteCourbe(repere, demidte, fonction, nomPointLieAxe, nomCalculAbscisse, nomCalculOrdonnee,
				nombrePoints, pointLieCache, pointCourbeCache, gestionAutoDiscontinuite);
	}

	/**
	 * Ajout d'une courbe sur un intervalle ]- inf; a]
	 * @param repere
	 * @param fonction
	 * @param nomPointLieAxe
	 * @param nomCalculAbscisse
	 * @param nomCalculOrdonnee
	 * @param nombrePoints
	 * @param pointLieCache
	 * @param pointCourbeCache
	 * @param gestionAutoDiscontinuite
	 * @param a
	 */
	public final void ajouteCourbeJusquea(CRepere repere, CFonc fonction, String nomPointLieAxe, 
			String nomCalculAbscisse, String nomCalculOrdonnee, int nombrePoints, boolean pointLieCache,
			boolean pointCourbeCache,boolean gestionAutoDiscontinuite, CValeurAssocieeAVariable a) {
		a.positionne(false, dimf);
		// On regarde si le repre possde dj un point d'abscisse a
		CPointDansRepere point1 = listePr.existeabscrissea(a, repere);
		if (point1 == null) {
			point1 = new CPointDansRepere(listePr, null, false, Color.black,false, 0d, 0d, true,
				"", 2, StylePoint.petitRond, repere, a, new CValeurAssocieeAVariable(listePr, 0), false);
			frame.ajouteElement(point1);
			point1.positionne(false, dimf);
		}
		CTranslation trans = listePr.existeTranslationDefiniePar(repere.I, repere.O);
		if (trans == null) {
			trans =	new CTranslation(listePr, null, false, repere.I, repere.O);
			frame.ajouteElement(trans);
			trans.positionne(false, dimf);
		}
		CPointImage im = listePr.pointImageDePar(point1, trans);
		if (im == null) {
			im =new CPointImage(listePr, null, false, Color.black, true, 0d, 0d, true, "", 2,
				StylePoint.petitRond, point1, trans, false);
			frame.ajouteElement(im);
			im.positionne(false, dimf);
		}
		CDemiDroiteOA demidte = listePr.existeDemidroiteDefiniePar(point1, im);
		if (demidte == null) {
			demidte = new CDemiDroiteOA(listePr, null, false, Color.black, StyleTrait.traitFin, point1, im);
			frame.ajouteElement(demidte);
			demidte.positionne(false, dimf);
		}
		ajouteCourbe(repere, demidte, fonction, nomPointLieAxe, nomCalculAbscisse, nomCalculOrdonnee,
				nombrePoints, pointLieCache, pointCourbeCache, gestionAutoDiscontinuite);
	}	
	

	
	/**
	 * Remise  zro des listes utilises pour les constructions
	 * Pas utilis pour l'applet mais gard pour compatibilit de code
	 */
	public void videListesPourConstructions() {
		// listeObjetsSourcesG.retireTout(); Supprim version Applet	
		// listeObjetsSourcesNG.retireTout(); Supprim version Applet	
		// listeObjetsFinauxG.retireTout(); Supprim version Applet	
		// listeObjetsFinauxNG.retireTout(); Supprim version Applet	
	}

	/* Pas de MouseWhhelListener pour la version Applet
	public void mouseWheelMoved(MouseWheelEvent e) {
		int notches = e.getWheelRotation();
		if (notches > 0) for (int j = 0; j < notches; j++) frame.actionZoomMoins.actionPerformed(null);
		else
			if (notches < 0) for (int j = 0; j < -notches; j++) frame.actionZoomPlus.actionPerformed(null);
	}
	*/
	
	@Override
	protected void paintComponent(Graphics g) {
		// TODO Auto-generated method stub
		super.paintComponent(g);
		// Ligne suivante ajoute version 4.6 pour que les barres d'outils soient dj affiches  avant
		// le premier affichage de la figure pour qu'on puisse ensuite centrer un repre ajout  la figure
		//  l'initialisation
		if (frame.premierAffichage) frame.metAJourIconesEtMenus(frame.cadreActif().document);
		if (!paneFigure.getSize().equals(getSize())) {
    	Dimension d = getSize();
    	paneFigure.setSize(d);
      paneFigure.setPreferredSize(d); // Ligne ncessaire car this n'a pas de taille. C'est paneCenter qui en une.
  		dimf.setSize((int)(d.getWidth()), (int)(d.getHeight()));
			prepareBuffers();
			// redessineAvecCalcul(); // Supprim version 4.6.2. Inutile. Sera fait dans paint()
			// paneFigure.repaint();
		} 
	}
	
	/**
	 * Important : paint doit tre redfini pour que prepareBuffers soit appel aprs que les barres
	 * d'icnes aient t mises  jour.
	 */
	@Override
	public void paint(Graphics g) {
		// TODO Auto-generated method stub
		super.paint(g);
		if (!paneFigure.getSize().equals(getSize())) {
    	Dimension d = getSize();
    	paneFigure.setSize(d);
      paneFigure.setPreferredSize(d); // Ligne ncessaire car this n'a pas de taille. C'est paneCenter qui en une.
  		dimf.setSize((int)(d.getWidth()), (int)(d.getHeight()));
			prepareBuffers();
			redessineAvecCalcul();
		}
	}
	
  // Ajout version 4.5.2
  public void retireComposants() {
  	if (paneAffichageVariables != null) {
			remove(paneAffichageVariables);
			paneAffichageVariables = null;
  	}
    java.util.ListIterator <CElementBase> li = listePr.listIterator();
    while (li.hasNext()) {
    	CElementBase elb = li.next();
    	elb.retireComposant(this);
    }
    if (frame.allowExercise && isConstruction) frame.retireBoutonPropSol();
		validate();
  }
  
  /**
   * Fonction appele lors de l'applet de destroy() de l'applet pour nettoyer les objets utiliss
   */  
  public final void cleanUp() {
  	if (listeAConstruire != null) listeAConstruire.retireTout();
  	listeAConstruire = null;
  	listeCalculsAutorises.retireTout();
  	listeCalculsAutorises = null;
  	listeConstruction.retireTout();
  	listeConstruction = null;
  	listeDependantsDansPrincipale.retireTout();
  	listeDependantsDansPrincipale = null;
  	listePr.retireTout();
  	listePr = null;
  	listeProposee.retireTout();
  	listeProposee = null;
  	listeSauve.retireTout();
  	listeSauve = null;
  	gbuffer = null;
  	tbuffer = null;
  }

}
