/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;

// Modification pour la version 1.9.5
// A la suite du rajout des lieux d'objets cette classe n'est plus abstract
public  class CAffichageLieAPoint extends CElementGraphique {
	// Ajouts version 4.5
	public static final int alignementHorizontalGauche = 0;
	public static final int alignementHorizontalCentre = 1;
	public static final int alignementHorizontalDroit = 2;
	public static final int alignementVerticalHaut = 0;
	public static final int alignementVerticalCentre = 1;
	public static final int alignementVerticalBas = 2;
	
  public CPointAncetre pointLie;  // null si li  aucun point
  // Dans la version Pascal, il y avait un membre TailleCaractere
  // qui est maintenant remplac par IndiceTaillePolice
  public int indiceTaillePolice;
  // A partir de la version 2.5, encadrement n'est plus booln
  // Si encadrement vaut 0, pas d'encadrement. S'il vaut 1, encadrement simple
  // et s'il vaut 2 encadrement 3D
  public int encadrement; // mis  true pour que l'affichage soit encadr
  public boolean effacementFond; // mis  true pour que le fond soit effac
  //avant l'affichage des caractres
  public Color couleurFondAffichage; // Ajout pour la version 2.5. N'est srialis que si EffacementFond est  true;
  // Ajout version 4.5
  public int alignementHorizontal = 0; // Ancien mode de fonctionnemment par dfaut  la cration
  public int alignementVertical = 0; // Ancien mode de fonctionnemment par dfaut  la cration
  
  public Rectangle rectangleAffichage; // Coordonnes du rectangle contenant la
  //valeur affichee en coordonnes logiques
  // InfoRandom n'tant disponible que dans positionne(), il faut mmoriser la chaine  afficher
  private String chaineAAfficher;
  // Pour l'affichage en relief des intituls de macros
  int xEncadrement;
  int yEncadrement;
  int widthEncadrement;
  int heightEncadrement;
  
  /** Creates new CAffichageLieAPoint */
  public CAffichageLieAPoint() {
    rectangleAffichage = new Rectangle();
  }
  
  // Constructeur pour clonage
  public CAffichageLieAPoint(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
    boolean effacementFondInit, Color couleurFondInit, int 	alignementHorizontalInit, int alignementVerticalInit) {
      super(liste, impProtoInit, estElementFinalInit, couleurInit, dec_xInit, dec_yInit, masqueInit);
      x_nom = x_nomInit;
      y_nom = y_nomInit;
      pointLie = pointLieInit;
      indiceTaillePolice = indiceTaillePoliceInit;
      encadrement = encadrementInit;
      effacementFond = effacementFondInit;
      couleurFondAffichage = couleurFondInit;
      rectangleAffichage = new Rectangle();
      alignementHorizontal = alignementHorizontalInit;
      alignementVertical = alignementVerticalInit;
  }
  

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    chaineAAfficher = ((CAffichageLieAPoint)ptel).chaineAAfficher;
    indiceTaillePolice = ((CAffichageLieAPoint)ptel).indiceTaillePolice;
    encadrement = ((CAffichageLieAPoint)ptel).encadrement;
    effacementFond = ((CAffichageLieAPoint)ptel).effacementFond;
    couleurFondAffichage = ((CAffichageLieAPoint)ptel).couleurFondAffichage;
    rectangleAffichage.setRect(((CAffichageLieAPoint)ptel).rectangleAffichage);
  }
  
  // Version 1.9.5 : cette mthode n'est plus abstract et doit tre redfinie
  // pour les descendants
  protected String rendChaineAffichage(boolean infoRandom){
    return "";
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    if (pointLie != null) liste.add(pointLie);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
    if (pointLie == null)
      return super.dependDe(p);
    else
      return super.dependDe(p) || pointLie.dependDe(p) ;
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    if (pointLie == null)
      return super.dependDePourReclassement(p);
    else
      return super.dependDePourReclassement(p) || pointLie.dependDePourReclassement(p) ;
  }
  
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    if (pointLie == null)
      return (p == this);
    else
      return (p == this) || pointLie.dependDePourBoucle(p) ;
  }
  
// Lors des captures de points
// inutile d'effacer les affichages sauf si le point auquel il
// est ventuellement li en dpend.
  @Override
  public boolean dependDePourCapture(CElementBase p) {
    return dependDe(p);
  }
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    if (pointLie != null) existe = pointLie.existe;
    else existe = true;
    if (!existe) return;
    if (pointLie != null) {
      if (pointLie.horsEcran) {
        existe = false;
        return;
      }
      // Java :  revoir.
      x_nom = pointLie.x; //
      y_nom = pointLie.y; //
    }
    chaineAAfficher = rendChaineAffichage(infoRandom);
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!existe || (masquage && masque)) return -1;
    else {
      if (rectangleAffichage.contains(xp, yp)) return 0;
      else return -1;
    }
  }
  
  // Java : changement par rapport au C++. Lors du positionnement, le device context n'est pas
  // accessible et on ne peut pas avoir d'objet FontMetrics. Le membre rectangleAffichage ne peut
  // donc pas tre tabli  ce niveau.
  // Il le sera dans l'affichage puisque l'effacement arrive toujours aprs.
  /* Midifi version 4.5
  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
  	// Il faut afficher un tel objet mme avec l'outil rideau quand il est masqu
  	if (!masque || (masque && !masquage)) {
      rectangleAffichage.x = (int) (x_nom + dec_x);
      rectangleAffichage.y = (int) (y_nom + dec_y);
      Kernel.dimensionsChaine(pDC, indiceTaillePolice, chaineAAfficher, rectangleAffichage, listeProprietaire.basseDefinition );
      trace(pDC, couleurFond, transparence);
    }
    }
  */
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
  	// Il faut afficher un tel objet mme avec l'outil rideau quand il est masqu
  	if (!masque || (masque && !masquage)) {
      rectangleAffichage.x = (int) (x_nom + dec_x);
      rectangleAffichage.y = (int) (y_nom + dec_y);
      Kernel.dimensionsChaine(pDC, indiceTaillePolice, chaineAAfficher, rectangleAffichage, listeProprietaire.coefMult);
      switch (alignementHorizontal) {
	      case alignementHorizontalCentre :
	      	rectangleAffichage.x -= rectangleAffichage.width/2;
	      	break;
	      case alignementHorizontalDroit :
	      	rectangleAffichage.x -= rectangleAffichage.width;
      }
      switch (alignementVertical) {
      	case alignementVerticalCentre :
      		rectangleAffichage.y -= rectangleAffichage.height/2;
      		break;
      	case alignementVerticalBas :
      		rectangleAffichage.y -= rectangleAffichage.height;
      }
      trace(pDC, couleurFond, transparence);
    }
  }
  
  
  // Java :  revoir. On peut optimiser l'effacement dans le cas d'un effacement
  // du fond
  // Modifi pour la version 1.3.1
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    boolean basseDefinition = listeProprietaire.basseDefinition;
    if ((chaineAAfficher.length() == 0)) return;
    pDC.setStroke(Pinceau.CreePinceau(StyleTrait.traitFin, listeProprietaire)); // Pour le rectangle
    // d'encadrement ventuel et les flches de vecteur ventuelles
    if (basseDefinition) {
	    xEncadrement = rectangleAffichage.x - 3;
	    yEncadrement = rectangleAffichage.y - 2;
	    widthEncadrement = rectangleAffichage.width + 5;
	    heightEncadrement = rectangleAffichage.height + 2;
    } else {
      // Les dimensions suivantes sont diffrentes dans l'applet car l'espace de coordonnes est 100 fois plus grand
      // et il y a des problmes d'arrondi
      xEncadrement = rectangleAffichage.x - 30;
      yEncadrement = rectangleAffichage.y - 20;
      widthEncadrement = rectangleAffichage.width + 50;
      heightEncadrement = rectangleAffichage.height + 20;    	
    }
    
 
    if (effacementFond) {
      pDC.setColor(couleurFondAffichage);
      // Les bords bas et droits ne sont pas compris semble-t-il par fillRect
      // Diffrence par rapport  la version C++ V1.8
      if (encadrement != 0)
        pDC.fillRect(xEncadrement, yEncadrement, widthEncadrement, heightEncadrement);
      else
        pDC.fillRect(rectangleAffichage.x, rectangleAffichage.y,
          rectangleAffichage.width, rectangleAffichage.height);
      
    }
    pDC.setColor(couleur);
    Kernel.afficheChaine(pDC, indiceTaillePolice, chaineAAfficher, rectangleAffichage.x,
    		rectangleAffichage.y, listeProprietaire.coefMult);
    if (encadrement != 0) {
      pDC.setColor(couleur);
      pDC.drawRect(xEncadrement, yEncadrement, widthEncadrement, heightEncadrement);
    }
    
    if (encadrement == StyleEncadrement.Effet3D) {
    	if (basseDefinition) {
	      int x1 = xEncadrement + widthEncadrement + 1;
	      int y1 = yEncadrement + heightEncadrement + 1;
	      pDC.drawLine(xEncadrement + 1, y1, x1, y1);
	      pDC.drawLine(x1, yEncadrement + 1, x1, y1);
    	}
    	else {
    		int x = xEncadrement + widthEncadrement;
    		int y = yEncadrement + heightEncadrement;
        int x1 = x + 10;
        int y1 = y + 10;
        for (int j = 1; j <= 10; j++) {
	        pDC.drawLine(xEncadrement + j, y + j, x1, y + j);
	        pDC.drawLine(x + j, yEncadrement + j, x + j, y1);
        }
    	}
    }
  }
  
  public void translateDe(double decalagex, double decalagey) {
  	x_nom += decalagex;
  	y_nom += decalagey;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	if (pointLie == null) return true;
  	else return (listeOb.contains(pointLie) || pointLie.estDefiniParObjetDans(listeOb));
	} 
  
  @Override
  public void adaptePourHauteResolution(double coef) {
  	super.adaptePourHauteResolution(coef);
		x_nom *= coef;
		y_nom *= coef;  	
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    if (ind1 == -1) pointLie = null;
    else pointLie = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    indiceTaillePolice = inps.readByte();
    x_nom = inps.readCDouble()/10;
    y_nom = inps.readCDouble()/10;
    encadrement = inps.readUnsignedByte();
    effacementFond = inps.readBoolean();
    // Modification pour le passage en version RVB  partir de la version 2.5 (n5)
    if ((effacementFond) && (listeProprietaire.numeroVersion >=5)) {
      int r = inps.readUnsignedByte();
      int g = inps.readUnsignedByte();
      int b = inps.readUnsignedByte();
      @SuppressWarnings("unused")
			int a = inps.readUnsignedByte();
      couleurFondAffichage = new Color(r,g,b,255);
    }
    // Ligne suivante modifie version 2.0
    // else couleurFond = Couleur.TraduitCouleur(0);
    // else couleurFondAffichage = listeProprietaire.documentProprietaire.couleurFond; // Provoque null pointer exception
    else couleurFondAffichage = Couleur.TraduitCouleur(0); // Ne sert pas
  }
  /* Chargement depuis un flux java */
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    if (ind1 == -1) pointLie = null;
    else pointLie = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    indiceTaillePolice = inps.readByte();
    // Amlioration version 4.7.7 : Si pointLie n'est pas null, il est inutile d'enregistrer x_Nom et y_Nom dans le flux
    if ((listeProprietaire.numeroVersion < 11)) {
    	x_nom = inps.readDouble();
    	y_nom = inps.readDouble();
    }
    else {
    	if (pointLie == null) {
      	x_nom = inps.readDouble();
      	y_nom = inps.readDouble();    		
    	}
    }
    encadrement = inps.readUnsignedByte();
    effacementFond = inps.readBoolean();
    // Modification pour le passage en version RVB  partir de la version 2.5 (n5)
    if (effacementFond) {
      int r = inps.readUnsignedByte();
      int g = inps.readUnsignedByte();
      int b = inps.readUnsignedByte();
      couleurFondAffichage = new Color(r,g,b,255);
    }
    else couleurFondAffichage = Couleur.TraduitCouleur(0); // Ne sert pas
    // Modification version 4.5 : On charge les indicateurs d'alignement horizontal et vertical
    if (listeProprietaire.numeroVersion >= 9) {
    	alignementHorizontal = inps.readInt();
    	alignementVertical = inps.readInt();
    }
    else {
    	alignementHorizontal = alignementHorizontalGauche;
    	alignementVertical = alignementVerticalHaut;
    }
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1;
    if (pointLie == null) ind1 = -1;
    else ind1 = list.indexOf(pointLie);
    oups.writeInt(ind1);
    oups.writeByte(indiceTaillePolice);
    // Amlioration version 4.7.7 : Si pointLie n'est pas null, il est inutile d'enregistrer x_Nom et y_Nom dans le flux
    if (pointLie == null) {
	    oups.writeDouble(x_nom);
	    oups.writeDouble(y_nom);
    }
    oups.writeByte(encadrement);
    oups.writeBoolean(effacementFond);
    if (effacementFond) {
      oups.writeByte(couleurFondAffichage.getRed());
      oups.writeByte(couleurFondAffichage.getGreen());
      oups.writeByte(couleurFondAffichage.getBlue());
    }
    // Modification version 4.5 : On enregistre les indicateurs d'alignement horizontal et vertical
  	oups.writeInt(alignementHorizontal);
  	oups.writeInt(alignementVertical);
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointLie == ancienPoint) pointLie = nouveauPoint;
  }
  
  // Inutile. Supprim version 4.5.2
  /*
  @Override
  public void positionneNom() {
  	// TODO Auto-generated method stub
  	super.positionneNom();
  }
  */
}