/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
// Java : Important : en C++ CDemiDroite hritait de CDroite mais appelait la mthode de srialization
// de CElement ligne car il est inutile de mmoriser l'origine et le vecteur
// c'est impossible en java. Une demi-droite ne pouvant pas tre nomme, inutile s'avoir
// un champ abscisseNom
// Mais cela pose un problme car dans la mthode de serialization de CDemiDroiteOA qui hrite de
// CdemiDroite, c'est la mthode de srialization de CDroite qui tait appele, ce qui est illogique.
// Dans chargeDepuisFlux de CDemiDroiteOA, il faut donc dpiler les donnes inutiles
// Mais pourtant une demi-droite doit descendre de l'anctre des droites. J'ai rsolu le problme
// en rajoutant un type CDroiteAncetre qui soit l'anctre de toutes les droites
import glob.*;
import java.awt.*;
import java.awt.geom.*;
import types.*;


// Modification pour la version 1.9.5 : la classe n'est plus abstract
// car elle peut tre instancie dans les leiux d'objets

public class CDemiDroite extends CDroiteAncetre {
  private Point2D.Double pointr = new Point2D.Double();

  /** Creates new CDemiDroite */
  public CDemiDroite() {
    super(); // Pour que le vecteur soit cr
  }
  // Constructeur pour clonage
  public CDemiDroite(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
  }
  
  @Override
  public String complementInfo() {
    return getString("chinfo42");
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	// Modification version 3.2.0 pour optimisation
  	/*
  	double a,xa,ya;
		existe = (existe && vecteur.nonNul());
    if (!(existe)) return;
    xext1 = point_x;
    yext1 = point_y;
    if (vecteur.x == 0)
		{
      if (vecteur.y < 0)
			{
        xext2 = point_x;
        yext2 = 0;
			}
      else
			{
        xext2 = point_x;
        yext2 = dimfen.longueurOrdonnees;
			}
      return;
    }; // if vecteur.x
    if (vecteur.y == 0)
		{
      yext2 = point_y;
      if (vecteur.x > 0)
			{
        xext2 = dimfen.longueurAbscisses;
			}
      else
			{
        xext2 = 0;
			}
      return;
    }// if vecteur.y
    a = vecteur.y/vecteur.x;
		if (vecteur.y < 0)
    {
      // on cherche l'intersection avec la ligne horizontale haute
      xa = (-point_y)/a+point_x;
      ya = 0;
      if (xa < 0)
			{ // on coupe la verticale gauche
        xa = 0;
        ya = a*(xa-point_x)+point_y;
			}
      else
			{
				if (xa > dimfen.longueurAbscisses)
        { // on coupe la verticale droite
					xa = dimfen.longueurAbscisses;
					ya = a*(xa-point_x)+point_y;
        };
			}
      // sinon on coupe la verticale haute
    }
    else
		{
      // on cherche l'intersection avec la ligne horizontale basse
      xa = (dimfen.longueurOrdonnees-point_y)/a + point_x;
      ya = dimfen.longueurOrdonnees;
      if (xa < 0)
			{ // on coupe la verticale gauche}
        xa = 0;
        ya = a*(xa-point_x)+point_y;
			}
      else
			{
				if (xa > dimfen.longueurAbscisses)
				{ // on coupe la verticale droite}
					xa = dimfen.longueurAbscisses;
					ya = a*(xa-point_x)+point_y;
				}
			}
      // sinon on coupe la verticale basse
    } //if vecteur.y<0
		xext2 = xa;
		yext2 = ya;
		*/
  	super.positionne(infoRandom, dimfen);
  	// Si la droite est hors-fentre, la demi-droite l'est aussi
  	if (horsFenetre) return;
  	// Si l'origine de la demi-droite  est dans la fentre
  	if (dimfen.dansFenetre(point_x, point_y)) {
  		VecteurDirecteur u = new VecteurDirecteur(point_x, point_y, xext1, yext1);
  		if (u.presqueNul()) {
  			VecteurDirecteur v = new VecteurDirecteur(point_x, point_y, xext2, yext2);
	   		if (Kernel.colineairesMemeSens(v, vecteur)) {
	  			xext1 = point_x;
	  			yext1 = point_y;
	  		}
	  		else {
	  			xext2 = xext1;
	  			yext2 = yext1;
	  			xext1 = point_x;
	  			yext1 = point_y;
	  		}
  		}
  		else {
	   		if (Kernel.colineairesMemeSens(u, vecteur)) {
	  			xext2 = xext1;
	  			yext2 = yext1;
	  			xext1 = point_x;
	  			yext1 = point_y;
	  		}
	  		else {
	  			xext1 = point_x;
	  			yext1 = point_y;
	  		}
  		}
  	}
  	else {
  		// Si l'origine est hors-fentre
  		VecteurDirecteur u = new VecteurDirecteur(point_x, point_y, xext1, yext1);
  		if (!Kernel.colineairesMemeSens(u, vecteur)) horsFenetre = true;
  		// Si un des points d'intersection de la droite avec la fentre est sur la demi-droite cela revient  tracer la droite en entier
  		
  	}
  }

  /*
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {

    if (Kernel.testToile(xext1, yext1) && Kernel.testToile(xext2, yext2)) {
	    int styleTrait;
	    if (masque) styleTrait = StyleTrait.traitPointille;
	    else styleTrait = style;
	    pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire.basseDefinition));
	    pDC.setColor(couleur);
	    pDC.drawLine((int)Math.round(xext1),(int)Math.round(yext1),
	      (int)Math.round(xext2),(int)Math.round(yext2));
    }
  }
  */
  
  // Ajout version 3.2.0
  // Pour ne rien faire lorsque CDroite.positionne appelle positionneNom()
  @Override
  public void positionneNom() {
  }
  
  public boolean appartientA(double x, double y) {
    if (!(existe)) return false;
    else
    {
      if (Kernel.zero(vecteur.x))
      {
        return ((y-point_y)*vecteur.y >= 0);
      }
      else // if vecteur.x<>0
      {
        return ((x-point_x)*vecteur.x >= 0);
      }
    }
	}
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque))
    return -1;
    else
    {
      Kernel.projetteOrtho(xp, yp, point_x, point_y, vecteur, pointr);
      if (appartientA(pointr.x,pointr.y)) {
        return Kernel.distancePointPoint(xp,yp,pointr.x, pointr.y);
      }
      else return -1;
    }      
  }
  
  // Supprim version 3.2.4
  // CDroiteAncetre renvoie !horsFenetre qui est calcul dans positionne
  /*
  @Override
  public boolean dansFenetre(DimensionFenetre dimfen) {
    Point2D.Double[] res = new Point2D.Double[4];
    for (int i = 0; i < 4; i++) res[i] = new Point2D.Double();

    Kernel.projetteOrtho(0,0,point_x,point_y,vecteur,res[0]);
    Kernel.projetteOrtho(dimfen.longueurAbscisses,0,point_x,point_y,vecteur,res[1]);
    Kernel.projetteOrtho(0,dimfen.longueurOrdonnees,point_x,point_y,vecteur,res[2]);
    Kernel.projetteOrtho(dimfen.longueurAbscisses,dimfen.longueurOrdonnees,point_x,point_y,vecteur,res[3]);
     
    // Si l'origine appartient  la fenetre, la demi-droite est visible
    if (dimfen.dansFenetre(point_x,point_y)) return true;
    else {
			// On regarde si un ou moins des projets 
			// orthogonaux des sommets de la
			// fentre sur la droite appartient  la demi_droite
			boolean resultat = false;
			for (int i=0; i <= 3; i++)
			{
				resultat = resultat || ((dimfen.dansFenetre(res[i].x,res[i].y) &&
					appartientA(res[i].x,res[i].y)));
			}
			return resultat;
  	}
  }
  */
  
  
  @Override
  public boolean coincideAvec(CElementBase p) {
    if (p.getNature() != NatureObjet.NDemiDroite) return false;
    CDemiDroite dmd = (CDemiDroite) p;
    return Kernel.zero(point_x-dmd.point_x) && Kernel.zero(point_x-dmd.point_x)
      && Kernel.colineairesMemeSens(vecteur, dmd.vecteur);
  }  

  @Override
  public long getNature() {
    return NatureObjet.NDemiDroite;
  }

  @Override
  public String chaineDesignation() {
    return getString("designationDemiDroite");
  }
  
}
