
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;
import mathgraph32.MtgFrame;
import dialogs.DroiteParEquationDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import types.DimensionFenetre;
import types.ErreurCalculException;
import types.Int;

public class CDroiteParEq extends CDroite {
	public CRepere rep;
	public CCalculBase equation; // Le calcul reprsentant l'quation avec un test d'galit
	private CCalculBase fonction;		// Un calcul reprsentant une fonction de deux variables x et y de la forme ax+by+c = 0
																	// correspondant  l'quation de la droite
	private CCalculBase calcula; // Un calcul reprsentant le coefficient a, drive partielle de fonction par rapport  x
	private CCalculBase calculb; // Un calcul reprsentant le coefficient b, drive partielle de fonction par rapport  y
	private double a, b, c; // Reprsentent les valeurs des coefficients de l'quation ax+by+c = 0 une fois la droite positionne
	private boolean bEquationValide; // True si l'quation est bien une quation de droite. Est vrifi  la cration de l'objet

  // Pour la version 4.1, modification de la faon dont l'objet s'enregistre dans le flux.
	// calcula et calculb ne doivent plus tre enregistrs dans le flux mais recalculs au chargement
	// car es calculs de drives utlisant des fonctions de plusieurs variables ne peuvent pas tre enregistrs dans le flux
	
  @Override
  public int numeroVersion() {
  	return 2;
  }  

	/** Creates new CDroiteOm */
  public CDroiteParEq() {
  }
 
  
  /**
   * Constructeur pour cration par bote de dialoge
   * @param liste
   * @param impProtoInit
   * @param estElementFinalInit
   * @param couleurInit
   * @param nomMasqueInit
   * @param dec_xInit
   * @param dec_yInit
   * @param masqueInit
   * @param nomInit
   * @param indiceTailleNomInit
   * @param styleInit
   * @param abscisseNomInit
   * @param repInit
   */
  public CDroiteParEq(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit, int styleInit,
      double abscisseNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
    rep = null;
    equation = null;
    fonction = null;
  }
  
  // Constructeur pour clonage
  public CDroiteParEq(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit, int styleInit,
      double abscisseNomInit, CRepere repInit, CCalculBase equationInit, CCalculBase fonctionInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
    rep = repInit;
    equation = equationInit;
    fonction = fonctionInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);
    CCalculBase equationClone = equation.getClone(listeSource, listeCible);
    CCalculBase fonctionClone = fonction.getClone(listeSource, listeCible);
    CDroiteParEq ptelb = new CDroiteParEq(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y,
      masque, nom, indiceTailleNom, style, abscisseNom, (CRepere)(listeCible.get(ind1, CRepere.class)),
      equationClone, fonctionClone);
    if (listeCible.getClass() != CPrototype.class) {
	    ptelb.calcula = CCalculBase.creeDerivee(fonctionClone, 0);
	    ptelb.calculb = CCalculBase.creeDerivee(fonctionClone, 1);
	    ptelb.bEquationValide = equationValide();
    }
    return ptelb;
  }
  
  
  // Il faut redfinir metAJour() pour une droite car quand on modifie une formule dont l'objet dpend les coefficients
  // a et b qui sont des drives partielles doivent tre recalculs.
  @Override
  public void metAJour() {
  	// TODO Auto-generated method stub
		calcula = CCalculBase.creeDerivee(fonction, 0); // Drive partielle par rapport  x
		calculb = CCalculBase.creeDerivee(fonction, 1); // Drive partielle par rapport  y
		bEquationValide = equationValide();
  }
  
  @Override
  // Amlior version 3.8
  public String infoPourHistorique() {
  	String[] parametre = new String[2];
  	parametre[0] = "x";
  	parametre[1] = "y";
    return getString("chinfo175") + " " + equation.chaineCalculSansPar(parametre) + "\n" + getString("chinfo32") + " " + rep.nom();
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || rep.dependDe(p) || equation.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || rep.dependDePourReclassement(p) 
    	|| equation.dependDePourReclassement(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || rep.dependDePourBoucle(p)
      || equation.dependDePourBoucle(p));
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	if (!bEquationValide) {
  		existe = false;
  		return;
  	}
		existe = rep.existe && equation.existe(infoRandom);
		if (existe)
		{
	    try {
	      listeProprietaire.initialiseNombreIterations();
	      a = calcula.resultat(infoRandom);
	      b = calculb.resultat(infoRandom);
	      if ((a==0) && (b == 0)) {
	      	existe = false;
	      	return;
	      }
	      // Le coefficient c est calcul comme l'image par la fonction de deux variables du couple (0;0)
	      double[] param = new double[2];
	      param[0] = 0;
	      param[1] = 0;
	      c = fonction.resultatFonction(infoRandom, param);  
	    } catch (ErreurCalculException e) {
	      existe = false;
	      return;
	    }
	    if (Double.isNaN(a) || Double.isInfinite(a)
	    		|| Double.isNaN(b) || Double.isInfinite(b)
	    		|| Double.isNaN(c) || Double.isInfinite(c)
	    		|| ((a == 0) && (b == 0))) {
	      existe = false;
	      return;
	    }
	    double nor = Math.sqrt(a*a+b*b);
	    // Modification version 4.0 car les repres ont une unit sur chacun des axes
	    double unitex = rep.unitex;
	    double unitey = rep.unitey;
	    // double x1 = -b/nor;
	    // double y1 = a/nor;
	    double x1 = -b/nor/unitex;
	    double y1 = a/nor/unitey;
	    vecteur.x = rep.u.x*x1 + rep.v.x*y1;
	    vecteur.y = rep.u.y*x1 + rep.v.y*y1;
	    if (b!=0) {
	    	double d = -c/b/unitey;
	    	if ((Math.abs(d) > 1e5) && (a!=0)) { // Droite quasi parallle  l'axe des ordonnes
	    		// On cherche alors le point d'intersection avec l'axe des abscisses
		    	d = -c/a/unitex;
		    	point_x = rep.origine.x + d*rep.u.x;
		    	point_y = rep.origine.y + d*rep.u.y;	    		
	    	}
	    	// Sinon point d'intersection avec l'axe des ordonnes
	    	else {    		
		    	point_x = rep.origine.x + d*rep.v.x ;
		    	point_y = rep.origine.y + d*rep.v.y;
	    	}
	    }
	    else {
	    	double d = -c/a/unitex;
	    	point_x = rep.origine.x + d*rep.u.x;
	    	point_y = rep.origine.y + d*rep.u.y;
	    }
			super.positionne(infoRandom, dimfen);
    }
  }
  
  @Override
  public boolean modifiableParMenu() {
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new DroiteParEquationDlg(frame, this, true); //  revoir
  }
   
   @Override
   public String informationModification() {
   	return getString("chaineOutil146");
   }
   
  @Override
 	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
 		boolean res = listeOb.contains(rep) || rep.estDefiniParObjetDans(listeOb);
		java.util.ListIterator<CElementBase> li = listeOb.listIterator();
		while (li.hasNext()) {
			CElementBase ptel = li.next();
			res = res || dependDe(ptel);
		}
		return res && equation.estDefiniParObjetDans(listeOb);
 	}      
  
   @Override
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
     super.read(inps, list);
     int ind1 = inps.readInt();
     rep = (CRepere) (list.get(ind1,CRepere.class));
     equation = (CCalculBase) (inps.readObject(list));
     fonction = (CCalculBase) (inps.readObject(list));
     // Modification version 4.1
     if (nVersion < 2) {
	     calcula = (CCalculBase) (inps.readObject(list));
	     calculb = (CCalculBase) (inps.readObject(list));
	     bEquationValide = true;
     }
     else {
       if (list.getClass() != CPrototype.class) {
      	 calcula = CCalculBase.creeDerivee(fonction, 0); // Drive partielle par rapport  x
      	 calculb = CCalculBase.creeDerivee(fonction, 1); // Drive partielle par rapport  y
      	 bEquationValide = equationValide();
       }
     }
   } 
   
   @Override
   public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
     super.write(oups, list);
     int ind1 = list.indexOf(rep);
     oups.writeInt(ind1);
     oups.writeObject(equation);
     oups.writeObject(fonction);
     // Modification version 4.1 : on ne doit pas enregistrer des drives dans le flux
     // oups.writeObject(calcula);
     // oups.writeObject(calculb);
   }
   
   /**
    * Fonction renvoyant true si et seulement si la chaine cheq contient une quation de droite correcte
    * Si l'quation est correcte, les membres equation, fonction, calcula et calculb sont tablis en consquence
    * @param cheq
    * @return
    */
   public boolean verifieFormules (String cheq, Int indiceErreur) {
  	 // On regarde d'abord si la syntaxe est correcte comme fonction de deux variables x et y
  	 String[] parametre = new String[2];
  	 parametre[0] = "x";
  	 parametre[1] = "y";
  	 int indiceMax = listeProprietaire.indexOf(this);
  	 if (indiceMax == -1) indiceMax = listeProprietaire.size() - 1;
		 boolean synt = CCalculBase.verifieSyntaxe(listeProprietaire, cheq, indiceErreur, indiceMax, parametre);
		 if (!synt) return false;
		 // Si la syntaxe est correcte il faut vrifier qu'il n'y a qu'un seul signe d'galit
		 int nbegal = 0;
		 int par = 0;
		 for (int i = 0; i < cheq.length(); i++) {
			 char car = cheq.charAt(i);
			 if (car == '(') par++;
			 else if (car == ')') par --;
			 else if ((car == '=') && (par == 0)) nbegal++;
			 if (nbegal > 1) {
				 indiceErreur.setValue(i);
				 break;
			 }
		 }
  	 if (nbegal != 1) return false;
  	 // La syntaxe est correcte. On cre la fonction qui sera de la forme ax+by+c en replaant
  	 // l'galit par une soustraction
  	 String chfon = new String(cheq);
		 par = 0;
		 for (int i = 0; i < chfon.length(); i++) {
			 char car = chfon.charAt(i);
			 if (car == '(') par++;
			 else if (car == ')') par --;
			 else if ((car == '=') && (par == 0)) {
				 chfon = chfon.substring(0, i) + "-(" + chfon.substring(i+1) + ")";
				 break;
			 }
		 }
		 CCalculBase fonction1 = CCalculBase.creeCalculBase(chfon, listeProprietaire, 0, chfon.length() - 1, parametre);
		 // Il faut vrifier si fonction1 dpend bien de au moins une des deux variables x et y
		//  if (!(fonction1.dependDeVariable(0) || fonction1.dependDeVariable(1))) return false;
		 // Il faut vrifier que c'est bien une quation de droite qui a t entre.
		 // Pour cela on cre les drives partielles de fonction1 par rapport  x et y et on vrifie que ces drives
		 // partielles ne dpendent ni de x ni de y
		 CCalculBase calcula1 = CCalculBase.creeDerivee(fonction1, 0); // Drive partielle par rapport  x
		 // if (calcula1.dependDeVariable(0) || calcula1.dependDeVariable(1)) return false;
		 CCalculBase calculb1 = CCalculBase.creeDerivee(fonction1, 1); // Drive partielle par rapport  y
		 // if (calculb1.dependDeVariable(0) || calculb1.dependDeVariable(1)) return false;
		 equation = CCalculBase.creeCalculBase(cheq, listeProprietaire, 0, cheq.length() - 1, parametre);
		 fonction = fonction1;
		 calcula = calcula1;
		 calculb = calculb1;
		 bEquationValide = equationValide();
		 return bEquationValide;
   }
   
   /**
    * Fonction renvoyant true si l'quation n'est pas une quation de droite
    * Cela peut arriver dans de rares cas,  savoir quand l'objet est un objet final
    * de construction avec des objets sources qui sont des fonctions de une ou plusieurs variables
    * @return
    */
   public boolean equationValide() {
		 return (fonction.dependDeVariable(0) || fonction.dependDeVariable(1))
				 && !calcula.dependDeVariable(0) && !calcula.dependDeVariable(1)
				 && !calculb.dependDeVariable(0) && !calculb.dependDeVariable(1);
   }
}
