/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import types.*;
import dialogs.FonctionComplexeDlg;
import dialogs.MtgDlg;
import mathgraph32.MtgFrame;


/**
 *
 * @author yves.bitton
 */
public class CFoncComplexe extends CFonc {
  
  /** Creates a new instance of CFoncComplexe */
  public CFoncComplexe() {
  }
  
  /* Constructeur pour outil de cration */
  public CFoncComplexe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  /* Constructeur pour clonage */
  public CFoncComplexe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomInit,String chaineCalculInit, CCalculBase calculInit, String chaineVariableInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit, chaineCalculInit, calculInit, chaineVariableInit);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1= listeSource.indexOf(impProto);        
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    CElementBase ptelb = new CFoncComplexe(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone, chaineVariable);
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NFonctionComplexe;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = getString("ch102") + " " + nom() + " " + getString("ch87") + "\n" + nom() 
    	+ "(" + chaineVariable + ")" + " = " + chaineCalcul;
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
  
  void rendValeurFonctionComplexe(boolean infoRandom, Complexe parametre, Complexe zRes)
      throws ErreurCalculException {
    calcul.resultatFonctionComplexe(infoRandom, parametre, zRes);
  }

  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new FonctionComplexeDlg(frame, this, true);
  }
  
   // Inutile de redfinir informationModification() pour les objets de type calcul    
  
   /**
    * Pour savoir si le calcul est une fonction relle, une suite relle ou non
    * Redfini seulement pour CFonc, CFoncComplexe, CSuiteRec et CSuiteRecComplexe;
    */ 
   // A revoir. Inutile car dj dfini sans CFonc. Supprim version 4.7.5.2
   /*
   public boolean estFonctionOuSuite() {
     return true;
   }
   */
   
}
