/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CLieuDiscretParPointLie extends CLieuDiscretDeBase {
  CPointLie pointLieGenerateur;// le point li gnrateur du lieu
  
  /** Creates new CLieuDiscretParPointLie */
  public CLieuDiscretParPointLie() {
  }
  
  // Constructeur pour clonage
  public CLieuDiscretParPointLie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, CPointAncetre pointATracerInit, int nombreDePointsInit, int motifInit, CPointLie pointLieGenerateurInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit,
      masqueInit, pointATracerInit, nombreDePointsInit, motifInit);
    pointLieGenerateur = pointLieGenerateurInit;
    // Rajout version 3.4.2
    // etablitListeElementsAncetres(); Inutile et provoquait un bug sous version 4.4 en appelant getClone
  }
  
  /**
   * A partir de la version 3.4.2, changement de version (version 1 -> 2) car on enregistre dans le flux la CSousListeObjetstres
   * contenant les pointeurs sur les lments anctres
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    
    int ind1 = listeSource.indexOf(pointATracer);
    int ind2 = listeSource.indexOf(pointLieGenerateur);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CLieuDiscretParPointLie(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      nombreDePoints, motif, (CPointLie)(listeCible.get(ind2, CPointLie.class)));
    if (listeCible.getClass() != CPrototype.class) {
	    // Version 3.4.2 Inutile ((CLieuDiscretParPointLie) ptelb).listeElementsAncetres = new CListeObjets(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    	// Bug corrig version 4.7.6
	    //listeElementsAncetres.setImage(((CLieuDiscretParPointLie)ptelb).listeElementsAncetres, ptelb.listeProprietaire,
	    //  listeProprietaire);
    	((CLieuDiscretParPointLie)ptelb).listeElementsAncetres.setImage(listeElementsAncetres, listeProprietaire,ptelb.listeProprietaire);
    }
    return ptelb;
  }
  
  // Un membre ayant t rajout et servant lors du positionnement, il faut redfinir setClone()
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    indiceDernierPointTrace = ((CLieuDiscretParPointLie) ptel).indiceDernierPointTrace;
  }
  
  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  @Override
  public void metAJour() {
  	super.metAJour();
  	// Bug corrig version 4.0 : il faut mettre  jour la liste d'lments anctres car metAJour est appel quand
  	// on modifie un objet dont dpend cet objet
  	// if (nVersion == 1) etablitListeElementsAncetres();
  	etablitListeElementsAncetres();
  }
  
  // Inutile de redfinir dependDe() var le point  tracer dpend du point li gnrateur  
  public boolean dependDePourCapture(CElementBase p) {
    // Un lieu de points n'a pas  tre redessin si on bouge le point
    // li qui l'a engendr}
    return ((p == this) || (pointATracer.dependDe(p) && (p != pointLieGenerateur)));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double abscisseMini, abscisseMaxi, abscisse, abscisseInitiale, pas, abscisseFinale;
    
    if (!pointLieGenerateur.existe) {
      existe = false;
      return;
    };
    abscisseMini = pointLieGenerateur.abscisseMinimale();
    abscisseMaxi = pointLieGenerateur.abscisseMaximale();
    if (abscisseMini == abscisseMaxi) {
      existe = false;
      return;
    } else {
      // On range dans l'ordre croissant
      if (abscisseMini > abscisseMaxi) {
        double d = abscisseMini;
        abscisseMini = abscisseMaxi;
        abscisseMaxi = d;
      }
    }
    abscisseInitiale = pointLieGenerateur.abscisse;
    // Modification pour la version 1.9.5
    int nbValeurs;
    if (pointLieGenerateur.lieALigneFermee()) {
    	nbValeurs = nombreDePoints;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi - pas;
    }
    else {
    	nbValeurs = nombreDePoints - 1;
    	pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    	abscisseFinale = abscisseMaxi;
    }
    pas = (abscisseMaxi-abscisseMini)/nbValeurs;
    abscisse = abscisseMini - pas;
    indiceDernierPointTrace = -1;
    int nb = nombreDePoints - 1;
    for (int i = 0; i < nombreDePoints; i++) {
    	abscisse = (i == 0) ? abscisseMini : ((i == nb) ? abscisseFinale : abscisse + pas);
      pointLieGenerateur.donneAbscisse(abscisse);
      listeElementsAncetres.positionne(infoRandom, dimfen);
      if (pointATracer.existe) {
        if (Kernel.testToile(pointATracer.x, pointATracer.y)) {
          indiceDernierPointTrace++;
          abscissesCoordonneesPoints[indiceDernierPointTrace] = (int)Math.round(pointATracer.x);
          ordonneesCoordonneesPoints[indiceDernierPointTrace] = (int)Math.round(pointATracer.y);
        }
      }
    } // for
    
    // On  remet le point anctre  sa position initiale
    pointLieGenerateur.donneAbscisse(abscisseInitiale);
    listeElementsAncetres.positionne(infoRandom, dimfen);
    if (indiceDernierPointTrace >= 0)
      existe = true;
    else existe = false;
  }
  
  @Override
  public String infoPourHistorique() {
  	return getString("chinfo146");
  }
  
  
  public boolean confonduAvec(CElementBase p) {
  	CLieuDiscretParPointLie pt;
    if (p.getClass() == this.getClass()) {
      pt = (CLieuDiscretParPointLie) p;
      return (pointATracer == pt.pointATracer) && (pointLieGenerateur == pt.pointLieGenerateur);
    } else return false;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		super.estDefiniParObjetDans(listeOb)
		&&
		(listeOb.contains(pointLieGenerateur) || pointLieGenerateur.estDefiniParObjetDans(listeOb));
	}
  
  public void etablitListeElementsAncetres() {
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (listeProprietaire.getClass() != CPrototype.class) {
	    int indicePointATracer = listeProprietaire.indexOf(pointATracer);
	    int indicePointLieGenerateur = listeProprietaire.indexOf(pointLieGenerateur);
	    listeElementsAncetres.retireTout();
    	listeElementsAncetres.add(pointLieGenerateur);
    	listeElementsAncetres.add(pointLieGenerateur);
      for (int i = indicePointLieGenerateur + 1; i < indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(listeProprietaire.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
      listeElementsAncetres.add(pointATracer);
    }	
  }
  
	/**
	 * Fonction renvoyant true si l'objet dpend des positions gnres pas les dplacements d'un point li
	 * A redfinir pour les lieux de points et lieux d'objets et macros utilisant les dplacements d'un point li
	 * @param pointlie
	 * @return
	 */
  @Override
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return pointLieGenerateur == pointlie;
	}
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    pointLieGenerateur = (CPointLie) (list.get(ind1,CPointLie.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      int indicePointATracer = list.indexOf(pointATracer);
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    }
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointLieGenerateur = (CPointLie) (list.get(ind1,CPointLie.class));
    // Il faut prparer la liste des lments dont dpend le point  tracer
    // Version 1.9.5 : modifi pour optimisation
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    // Modifi version 3.4.2
    /*
    if (list.getClass() != CPrototype.class) {
      int indicePointATracer = list.indexOf(pointATracer);
      for (int i = 0; i <= indicePointATracer; i++) {
        CElementBase ptelb = (CElementBase)(list.get(i));
        if ((pointATracer.dependDe(ptelb)) && (ptelb.dependDe(pointLieGenerateur)))
          listeElementsAncetres.add(ptelb);
      }
    }
    */
    if (nVersion == 1) etablitListeElementsAncetres();
    else listeElementsAncetres.read(inps, list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointLieGenerateur);
    oups.writeInt(ind1);
    listeElementsAncetres.write(oups, listeProprietaire);
  }   
}
