/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import glob.*;

import java.awt.*;


import types.*;

public class CMarqueAngleOrienteDirecte extends CMarqueAngleOriente {
	double angle; // Mesure principale de l'angle (OA,OB)

  /** Creates new CMarqueAngleOrienteDirecte */
  public  CMarqueAngleOrienteDirecte() {
  }
  
  // Constructeur pour clonage  
  public CMarqueAngleOrienteDirecte(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit, int styleMarqueInit, double rayonInit, int motifFlecheInit,
      	CPointAncetre AInit, CPointAncetre OInit, CPointAncetre BInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, styleMarqueInit,
    		rayonInit, motifFlecheInit, AInit, OInit, BInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(O);
    int ind3 = listeSource.indexOf(B);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMarqueAngleOrienteDirecte(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, styleMarque, rayon, motifFleche, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
  }
  
  /**
   * Fonction renvoyant la chane  afficher dans la bote de dialogue d'historique
   * pour dcrire l'objet.
   * @return
   */
  @Override
  public String infoPourHistorique() {
    return getString("chinfo68");
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	double petitAngle; // Reprsente l'angle de correction pour la direction de la flche
  	// due  la longueur de la flche
    super.positionneAncetre(infoRandom, dimfen);
    if (!existe) return;
    // Optimisation pour la version 1.9.5
    double longueurFleche =StyleFleche.longeurFlechePourArc(motifFleche, listeProprietaire);
    v.vecteurColineaire(rayon,v0);
    angle = u.mesureAngleVecteurs(v);
    if (Kernel.zeroAngle(angle - Math.PI/2)) petitAngle = 0;
    else petitAngle = longueurFleche/rayon;
    
    if (Kernel.zero(angle))
    {
      existe = false;
      return;
    }
    else
    {
      v0.tourne(-Math.PI/2-petitAngle,vecteurDirection);
    }
    xdeb = O.x + v0.x;
    ydeb = O.y + v0.y;
  }

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	  	
    if (!(existe) || (masquage && masque)) return -1;
    else {
    	boolean appsecteur;    	
    	VecteurDirecteur w = new VecteurDirecteur(centre_x, centre_y, xp, yp);
  		double ang = w.angleRad();
  		if (Kernel.zeroAngle(Math.abs(ang2rad - ang1rad) - Math.PI))
  		{
  			if (ang1rad < ang2rad) appsecteur = (ang>=ang1rad) && (ang<=ang2rad);
  			else appsecteur = (ang >= ang1rad) || (ang <= ang2rad);
  		}
  		else
  		{
  			if (ang1rad <= ang2rad) appsecteur = (ang>=ang1rad) && (ang<ang2rad);
  			else appsecteur = (ang>=ang1rad) || (ang<=ang2rad);
  		}
  		// Ici on ne peut pas appeler super.distancePoint car on appellerait la mthode
  		// de CMarqueAngleOriente
  		if (appsecteur) return Math.abs(Kernel.distancePointPoint(centre_x,centre_y,xp,yp) - rayon);
  		else return -1;
    	
    }
  }  
  
  @Override
  protected void remplitSecteur(Graphics2D pDC, double ang, float transparence) {
    Composite originalComposite = pDC.getComposite();
    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
    Kernel.secteurDirectPlein(pDC, A.x, A.y, O.x, O.y, rayon, ang);	  	
    pDC.setComposite(originalComposite);    		
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	if (angle >= 0) super.trace(pDC, couleurFond, transparence);
  	else {
	    if (A.horsEcran || O.horsEcran || B.horsEcran) return;
	    int styleTrait;
	    if (masque) styleTrait = StyleTrait.traitPointille;
	    else styleTrait = style;
	    pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
	    pDC.setColor(couleur);
	  	double difang = ang2rad - ang1rad;
	    Kernel.secteurDirect(pDC,A.x,A.y,O.x,O.y,rayon,difang);
	    if ((styleMarque & StyleMarqueAngle.marqueRemplie) != 0) remplitSecteur(pDC, difang, transparence);
      if ((styleMarque & StyleMarqueAngle.marqueAvecTrait) != 0) 
      	traceMarqueAngle(pDC, A.x, A.y, O.x, O.y, Math.PI*2 + angle);
	    StyleFleche.traceFlechePourArc(pDC,xdeb,ydeb,vecteurDirection,motifFleche, listeProprietaire);
	  }
  }
}
