/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.util.ListIterator;
import java.util.Vector;
import javax.imageio.ImageIO;
import types.Int;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMathGrafDoc {
	MtgFrame frame; // Le MtgFrame de l'application
	public boolean estDocumentVersionC = false; // True si on a ouvert un document cr avec la version C++
	// A partir de la version 1.5, on rajoute un membre numro de version
	// On commence  2
	int numeroVersion;
	// Pour la version java on mmorise la langue qui a servi  construire la figure
	public String langage;
	// Nouveau pour la version java 2.0 : Effet de transparence ou non pour les surfaces
	public float transparencePourSurfaces;	
	public MtgInternalFrame cadreProprietaire;
	// Nouveau pour le java : Variable qui servira  savoir si la figure a t
	// charge depuis un flux C++
	public boolean ancienneVersion = false;
	// Chemin d'accs  la figure. Null s'il s'agit d'une nouvelle figure pas
	// encore enregistre
	public String path;
	public int sizeWindowx;
	public int sizeWindowy;
	int sizeDocx;
	int sizeDocy;
	// ///////////////////////////////////////
	// Membres rajouts pour la version 1.8//
	public Color couleurFond;
	// public String pathImageFond; // Plus utilis version 4.4
	// public Image imageFond = null; // Version 4.4 : imageFonf devient un BufferedImage
	public BufferedImage imageFond = null;
	// Deux rajouts pour le version 4.7.7  cause de le vaersion JavaScript. Le numro de version passe  11.
	// En JavaScript on a besoin de savoir quelles sont les dimensions initiales de l'image de fond.
	public int widthImageFond;
	public int heightImageFond;
	public byte natImageFond; // Dsignera la nature de imageGod parmi gif, jpg et png
	// Ajout version 4.4 : un boolean disant si l'image de fond doit tre affiche ou non
	public boolean imageFondVisible = true;
	// Nouveau version Java : chaque document doit savoir si le mode tarce est activ ou non
	public boolean modeTraceActive = false;
	// Membres rajouts pour la version 4.6 (numro de version 10) spcifiant si les champs
	// d'dition doivent avoir ou non des boutons Valeurs et Fonctions prdfinies
	public boolean buttonValueAllowed = false;
	public boolean buttonFunctionAllowed = false;
	// ///////////////////////////////////////
	// Variable rajoute pour la versino ajva permettant de savoir si le document
	// a t modifi
	private boolean isDirty;

	public boolean isDirty() {
		return isDirty;
	}

	public void setDirty(boolean val) {
		isDirty = val;
	}

	public Vector<CPrototype> tablePrototypes = new Vector<CPrototype>();
	// Membres enregistrs dans le flux qui n'taient pas dans la version Pascal
	public boolean affichagesFiges; // Sera true si on ne veut pas
	// que l'utilisateur puisse dplacer les commentaires et
	// affichages de valeurs
	public boolean marquesAnglesFigees; // Idem pour les marques d'angles
	// n'a rien spcifi ou si la figure ne comporte pas de longueur
	// unit. Sinon contiendra la longueur en cm qu'aura la figure quand elle
	// sera copie dans un document  l'aide du menu Edition Copier
	// A partir de la version2, on rajoute un objet CListeIdMenu qui contient
	// un mot de passe et une liste contenant les identificateurs des items
	// de menus interdits pour la figure
	//  revoir en java. Inutile de charger les items de menus permis ou interdits
	// CListeIdMenus m_ListeIdMenus;
	// Toujours  partir de la version 2, on rajoute un mot de passe qui permet
	// de protger la figure ainis que 4 membres boolens spcifiant le type
	// de protection.
	// Il se dfinit et se change dans le menu Options>>Figure en cours.
	String motDePasseFigure;
	boolean protectionOptions;
	boolean protectionCopie;
	boolean protectionFigure;
	boolean protectionMenus;
	boolean protectionSecret;

	public String TexteFigure; // Le texte associ  la figure
	public CListeObjets listePrincipale;
	// Membre  true quand les items contenus dans la liste d'items de menus sont
	// interdits
	// (dans le cas o la liste n'est pas vide)
	public boolean itemsCochesInterdits;

	// Une liste d'objets qui contiendra la liste des menus autoriss ou interdits
	public Vector<Integer> listeIdMenus = new Vector<Integer>();

	/** Creates new CMathGrafDoc */
	public CMathGrafDoc(MtgFrame frameInit) {
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		frame = frameInit;
		langage = frame.pref_language;
		path = null;
		listePrincipale = new CListeObjets(this);
		// Par dfaut un nouveau document autorise tous les items de menu
		itemsCochesInterdits = true;
		// A revoir
		couleurFond = Color.white;
		path = "";
		// pathImageFond = "";
		motDePasseFigure = "";
		itemsCochesInterdits = true;
		motDePasseFigure = "";
		protectionOptions = false;
		protectionCopie = false;
		protectionFigure = false;
		protectionMenus = false;
		protectionSecret = false;
		TexteFigure = "";
		// sizeWindowx = MtgInternalFrame.largeurNouvelleFigure; Supprim version Applet
		// sizeWindowy = MtgInternalFrame.hauteurNouvelleFigure; Supprim version Applet
		affichagesFiges = false;
		marquesAnglesFigees = false;
		// Ajout version 2.0
		transparencePourSurfaces = MtgFrame.transparenceParDefaut;
		
		setDirty(false);
	}

	public final boolean itemPermis(int item) {
		if (item != 0) {
			boolean trouve = false;
			for (int i = 0; i < listeIdMenus.size(); i++) {
				Integer nb = (Integer) listeIdMenus.get(i);
				if (nb.intValue() == item) {
					trouve = true;
					break; // Rajout version 4.3
				}
			}
			if (itemsCochesInterdits)
				return !trouve;
			else
				return trouve;
		}
		else return true;
	}
	
	public final void retireItem(int item) {
		for (int i = 0; i < listeIdMenus.size(); i++) {
			Integer nb = (Integer) listeIdMenus.get(i);
			if (nb.intValue() == item) {
				listeIdMenus.remove(i);
			}
		}
		
	}
	
	public final boolean itemContenu(int item) {
		if (item != 0) {
			boolean trouve = false;
			for (int i = 0; i < listeIdMenus.size(); i++) {
				Integer nb = (Integer) listeIdMenus.get(i);
				if (nb.intValue() == item) {
					trouve = true;
					break; // Ajout version 4.3
				}
			}	
			return trouve;
		}
		return false;
	}
	
	

	public final boolean menuModifie() {
		return listeIdMenus.size() != 0;
	}

	/**
	 * Chargement du document depuis un flux C++ d'objets polymorphe
	 * 
	 * @param inps
	 * @throws java.io.IOException
	 */
	public final void chargeDepuisFlux(MathGraphDataInputStream inps, boolean bfromURL) throws IOException {
		String chaineIdentification = inps.readCString(16);
		if ((!chaineIdentification.equals(textes.Chaines.chaineIdentification))
				&& (!chaineIdentification.equals(textes.Chaines.chaineIdentificationDemo))
				&& (!chaineIdentification.equals(textes.Chaines.ancienneChaineIdentification))
				&& (!chaineIdentification.equals(textes.Chaines.ancienneChaineIdentificationDemo))
				&& (!chaineIdentification.equals(textes.Chaines.chaineIdentificationJava)))
			throw new IOException();
		if ((chaineIdentification.equals(textes.Chaines.chaineIdentification))
				|| (chaineIdentification.equals(textes.Chaines.chaineIdentificationDemo))
				|| (chaineIdentification.equals(textes.Chaines.ancienneChaineIdentification))
				|| (chaineIdentification.equals(textes.Chaines.ancienneChaineIdentification))) {
			estDocumentVersionC = true;
			chargeDepuisFluxC(inps, chaineIdentification, bfromURL);
		} else
			chargeDepuisFluxJava(inps, bfromURL);
		// Nouveaut pour la version java : si la langue dans laquelle a t cr le document
		// on reconstruit toutes les formules mathmatiques
		if (!langage.equals(frame.pref_language)) listePrincipale.reconstruitChainesCalcul();
	}
	
	/**
	 * Spcial applet : chargement d'une figure en ligne.
	 * Les CMacroJouantSon ne sont pas charges de la mme faon.
	 * @param inps
	 * @throws IOException
	 */
	public final void chargeDepuisFluxFromURL(MathGraphDataInputStream inps) throws IOException {
		String chaineIdentification = inps.readCString(16);
		if ((!chaineIdentification.equals(textes.Chaines.chaineIdentification))
				&& (!chaineIdentification.equals(textes.Chaines.chaineIdentificationDemo))
				&& (!chaineIdentification.equals(textes.Chaines.ancienneChaineIdentification))
				&& (!chaineIdentification.equals(textes.Chaines.ancienneChaineIdentificationDemo))
				&& (!chaineIdentification.equals(textes.Chaines.chaineIdentificationJava)))
			throw new IOException();
		if ((chaineIdentification.equals(textes.Chaines.chaineIdentification))
				|| (chaineIdentification.equals(textes.Chaines.chaineIdentificationDemo))
				|| (chaineIdentification.equals(textes.Chaines.ancienneChaineIdentification))
				|| (chaineIdentification.equals(textes.Chaines.ancienneChaineIdentification))) {
			estDocumentVersionC = true;
			chargeDepuisFluxC(inps, chaineIdentification, true);
		} else
			chargeDepuisFluxJava(inps, true);
		// Nouveaut pour la version java : si la langue dans laquelle a t cr le document
		// on reconstruit toutes les formules mathmatiques
		if (!langage.equals(frame.pref_language)) listePrincipale.reconstruitChainesCalcul();		
	}

	/** Chargement depuis un flux C++ une fois que les 16 caractres donnant la
	 * chaine d'identification ont t lus
	 * 
	 * @param inps
	 * @param chaineId
	 * @throws IOException
	 */
	public void chargeDepuisFluxC(MathGraphDataInputStream inps, String chaineId, boolean bfromURL) throws IOException {
		langage = "fra"; // Franais pour les anciennes figures
		ancienneVersion = true;
		if ((chaineId.equals(textes.Chaines.ancienneChaineIdentification))
				|| (chaineId.equals(textes.Chaines.ancienneChaineIdentificationDemo))) {
			// Rajoute pour version 2.5
			// NumeroVersionDocumentActif = 1;
			numeroVersion = 1;
			// Couleur de fond blanc pour les anciennes figures
			couleurFond = new Color(255, 255, 255);
		} else {
			numeroVersion = inps.readCInt();
			// Changements pour la version 1.8 (version 3)
			if (numeroVersion > MathGraphDataInputStream.numeroVersionActif) {
				// frame.dialogueAvertissement(MtgFrame.getString("erreurVersion")); // Modifi version 4.6.2
				// Pas de message pour si une age contient plusieurs applets les messages ne s'empilent pas.
				throw new IOException();
			}
			// Changement pour version 2.0 :apparition du degr de transparence des surfaces pleines dans un float
			transparencePourSurfaces = 1f; // Pas de transparence pour anciennes figures ( modifier dans Options Figure en cours
			if (numeroVersion >= 3) {
				// Version 1.8 : on enregistre la couleur de fond de la figure
				int couleurRouge, couleurVert, couleurBleu;
				couleurRouge = inps.readUnsignedByte();
				couleurVert = inps.readUnsignedByte();
				couleurBleu = inps.readUnsignedByte();
				couleurFond = new Color(couleurRouge, couleurVert, couleurBleu);
				// Modifications version 4.4
				String pathImageFond = inps.readCString();
				if (pathImageFond.length() != 0) {
					// ImageIcon imic = new ImageIcon(cheminRacine() + pathImageFond);
					// imageFond = imic.getImage();
					File file = new File(cheminRacine() + pathImageFond);
		    	imageFond = ImageIO.read(file);
		    	natImageFond = MathGraphDataOutputStream.getNatImage(pathImageFond);
				}
				else {
					imageFond = null;
					natImageFond = MathGraphDataOutputStream.nat_noImage; // -1 quand il n'y a pas d'image de fonc
				}

				// Pas utilis par l'applet : absoluteImageFondPath
				@SuppressWarnings("unused")
				String absoluteImageFondPath = inps.readCString();
				// Pas utilis en java : la taille de l'image de fond en
				// coordonnes logiques
				@SuppressWarnings("unused")
				int i = inps.readCInt();
				i = inps.readCInt();
				// On enregistre aussi les dimensions de la vue qui contenait la fentre
				// (nouveau pour la version 1.8)
				// Pas utilis dans l'applet java
				i = inps.readCInt();
				i = inps.readCInt();
			} else {
				couleurFond = Color.WHITE;
				// pathImageFond = "";
			}
			// On charge l'ventuelle liste d'items de menus
			int nombreItems = inps.readCCount();
			if (nombreItems != 0) {
				// Il faut tromper le flux dans lequel ont t enregistrs des objets
				// C++
				// du type CIdMenu et du type Point
				inps.listeNomsClasse.add(new String("CIdMenu"));
				inps.listeNumerosVersion.add(new Int());
				// Deux fois ! (dfaut du C++
				inps.listeNomsClasse.add(new String("CIdMenu"));
				inps.listeNumerosVersion.add(new Int());
				for (int i = 0; i < nombreItems; i++) {
					// On saute 13 octets pour le nom de la classe CIdMenu pour le premier
					// item CIdMenu
					if (i == 0) {
						for (int j = 0; j < 13; j++) {
							@SuppressWarnings("unused")
							byte a = inps.readByte();
						}
						int a = inps.readCInt();
						listeIdMenus.add(new Integer(a));
					} else {
						int a = inps.readCShort();
						a = inps.readCInt();
						listeIdMenus.add(new Integer(a));
						inps.listeNomsClasse.add(new String("CIdMenu"));
						inps.listeNumerosVersion.add(new Int());
					}
				}
			}
			// Un octet pour itemsCochesInterdits
			itemsCochesInterdits = inps.readBoolean();
			motDePasseFigure = inps.readCString();
			protectionOptions = inps.readBoolean();
			protectionCopie = inps.readBoolean();
			protectionFigure = inps.readBoolean();
			protectionMenus = inps.readBoolean();
			protectionSecret = inps.readBoolean();
		}
		TexteFigure = inps.readCString();
		sizeWindowx = inps.readCInt();
		sizeWindowy = inps.readCInt();
		affichagesFiges = inps.readBoolean();
		marquesAnglesFigees = inps.readBoolean();
		@SuppressWarnings("unused")
		boolean affichageSurfacesPendantCaptures = inps.readBoolean(); // Plus
																																		// utilis
																																		// en java
		@SuppressWarnings("unused")
		double longueurReellePourUnite = inps.readCDouble();  // Plus utilis en java
		if (numeroVersion > 3) chargePrototypes(inps);
		// Nouveaut pour la version 2.5 : La liste doit tre aucourant du n de
		// version
		// de document
		listePrincipale.numeroVersion = numeroVersion;
		listePrincipale.chargeDepuisFlux(inps, bfromURL);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		listePrincipale.numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		// on reconstruit toutes les formules mathmatiques pour une figure c++ car on ne sait pas la langue de l'utilisateur
		listePrincipale.reconstruitChainesCalcul();
		// A revoir plus tard pour la version java
		// if (ListePrincipale.PointeurLongueurUnite != NULL)
		// ListePrincipale.MetAJourListeAncetresPourLieuxDependantDe
		// (ListePrincipale.PointeurLongueurUnite);
		// COleServerDoc::Serialize(ar);

	}

	/**
	 * Chargement depuis un flux java d'objets polymorphe
	 * 
	 * @throws IOException
	 */
	public final void chargeDepuisFluxJava(MathGraphDataInputStream inps, boolean bfromURL) throws IOException {
		numeroVersion = inps.readInt();
		// Ligne suivante pour corriger un oubli dans la premire version o le numro de document
		// tait nul
		if (numeroVersion == 0) numeroVersion = 5;
		if (numeroVersion > MathGraphDataInputStream.numeroVersionActif) { // Amlior version 4.4
			// frame.dialogueAvertissement(MtgFrame.getString("erreurVersion"));
			// Pas de message pour si une age contient plusieurs applets les messages ne s'empilent pas.
			throw new IOException();
		}
		if (numeroVersion >= 6) transparencePourSurfaces = inps.readFloat();
		else transparencePourSurfaces = 1f; //pas de transparence pour les anciennes figures. A modifier dans Options Figure en cours
		langage = inps.readUTF(); // la langue dans laquelle a t cr le document
		// Changements pour la version 1.8 (version 3)
		// Changement pour version 2.0 : on enregistre le degr de transparence des surfaces pleines dans un float
		int couleurRouge, couleurVert, couleurBleu;
		couleurRouge = inps.readUnsignedByte();
		couleurVert = inps.readUnsignedByte();
		couleurBleu = inps.readUnsignedByte();
		couleurFond = new Color(couleurRouge, couleurVert, couleurBleu);
		// Modifications version 4.4
		if (numeroVersion < 8) { // Jusqu' la version 4.4, la rfrence  l'image de fond tait externe. Depuis l'image de fond est enregistre dans le flux
			imageFondVisible = true;
			modeTraceActive = false;
			String pathImageFond = inps.readUTF();
			if (pathImageFond.length() != 0) {
				// ImageIcon imic = new ImageIcon(cheminRacine() + pathImageFond);
				// imageFond = imic.getImage();
	    	File file = new File(cheminRacine() + pathImageFond);
	    	imageFond = ImageIO.read(file);
	    	natImageFond = MathGraphDataOutputStream.getNatImage(pathImageFond);
			}
		}
		else {
			imageFondVisible = inps.readBoolean(); // Nouveau version 4.4
			modeTraceActive = inps.readBoolean(); // Nouveau version 4.4
			natImageFond = inps.readByte();
			// Modification version 4.7.7 : Pour la version JavaScript on a besoin de connatre les dimensions initiales de l'imagee de fond.
			// Donc on change de numro de version de document qui passe  11
			if (natImageFond != MathGraphDataOutputStream.nat_noImage) {
				imageFond = inps.readBufferedImage();
				if (numeroVersion > 10) {
					widthImageFond = inps.readInt();
					heightImageFond = inps.readInt();
				}
			}
		}	
		// On lit un boolen indiquant si les codes d'items de menu correspondent 
		// des items interdits ou non
		itemsCochesInterdits = inps.readBoolean();
		// On charge l'ventuelle liste d'items de menus
		int nombreItems = inps.readInt();
		if (nombreItems != 0) {
			for (int i = 0; i < nombreItems; i++) {
				int a = inps.readInt();
				listeIdMenus.add(new Integer(a));
			}
		}
		motDePasseFigure = inps.readUTF();
		protectionOptions = inps.readBoolean();
		protectionCopie = inps.readBoolean();
		protectionFigure = inps.readBoolean();
		protectionMenus = inps.readBoolean();
		protectionSecret = inps.readBoolean();
		TexteFigure = inps.readUTF();
		sizeWindowx = inps.readInt();
		sizeWindowy = inps.readInt();
		affichagesFiges = inps.readBoolean();
		marquesAnglesFigees = inps.readBoolean();
		// Nouveau pour la version 4.6 : Deux boolean disant si les champs d'dition peuvent avoir des boutons
		// Valeur et Fontions prdfinies
		if (numeroVersion >= 10) {
			buttonValueAllowed = inps.readBoolean();
			buttonFunctionAllowed = inps.readBoolean();
		}
		readPrototypes(inps);
		// Nouveaut pour la version 2.5 : La liste doit tre au courant du n de
		// version
		// de document
		listePrincipale.numeroVersion = numeroVersion;
		if (bfromURL) listePrincipale.readfromURL(inps);
		else listePrincipale.read(inps);
		listePrincipale.numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
	}

	/**
	 * Enregistrement dans un flux java d'objets polymorphe
	 */
	public final void write(MathGraphDataOutputStream oups) throws IOException {
		oups.writeCString(textes.Chaines.chaineIdentificationJava);
		oups.writeInt(numeroVersion);
		// Changement pour version 2.0 : on enregistre le degr de transparence des surfaces pleines dans un float
		oups.writeFloat(transparencePourSurfaces);
		oups.writeUTF(langage);
		oups.write(couleurFond.getRed());
		oups.write(couleurFond.getGreen());
		oups.write(couleurFond.getBlue());
		// oups.writeUTF(pathImageFond); Supprim version 4.4 L'image de fond est directement enregistre dans le flux.
		// Nouveau version 4.4 : un boolean disant si l'image de fond est visible ou non
		oups.writeBoolean(imageFondVisible);
		// Nouveau version 4.4 : un boolean disant si le mode trace est activ
		oups.writeBoolean(modeTraceActive);
		if (imageFond != null) {
			oups.writeByte(natImageFond);
			oups.writeBufferedImage(imageFond, natImageFond); // Si pas d'image Fond un int de vaeur nulle est enregistr dans le flux.
			// Modification version 4.7.7 : On enregistre les dimensiosn de l'image de fond.
			oups.writeInt(imageFond.getWidth(null));
			oups.writeInt(imageFond.getHeight(null));
		}
		else oups.writeByte(MathGraphDataOutputStream.nat_noImage);
		// On lit un boolen indiquant si les codes d'items de menu correspondent 
		// des items interdits ou non
		oups.writeBoolean(itemsCochesInterdits);
		// On enregistre l'ventuelle liste d'items de menus
		int nombreItems = listeIdMenus.size();
		oups.writeInt(nombreItems);
		if (nombreItems != 0) {
			for (int i = 0; i < nombreItems; i++) {
				oups.writeInt(listeIdMenus.get(i));
			}
		}
		oups.writeUTF(motDePasseFigure);
		oups.writeBoolean(protectionOptions);
		oups.writeBoolean(protectionCopie);
		oups.writeBoolean(protectionFigure);
		oups.writeBoolean(protectionMenus);
		oups.writeBoolean(protectionSecret);
		oups.writeUTF(TexteFigure);
		// Spcial version Applet
		// oups.writeInt(sizeWindowx);
		// oups.writeInt(sizeWindowy);
		MtgInternalFrame cadre = frame.cadreActif();
		Dimension d = cadre.getSize();
		oups.writeInt(d.width);
		oups.writeInt(d.height);		
		oups.writeBoolean(affichagesFiges);
		oups.writeBoolean(marquesAnglesFigees);
		oups.writeBoolean(buttonValueAllowed);
		oups.writeBoolean(buttonFunctionAllowed);
		writePrototypes(oups);
		// Nouveaut pour la version 2.5 : La liste doit tre aucourant du n de
		// version
		// de document
		listePrincipale.write(oups);
	}
	public String cheminRacine() {
		String sep = System.getProperty("file.separator");
		int ind = path.lastIndexOf(sep);
		if (ind == -1)
			return "";
		else
			return path.substring(0, ind) + sep;
	}
	
	/**
	 * Fonction renvoyant true si le document contient un prototype ayant pour nom chNom
	 *  l'exception ventuelle de exclu
	 * @param chNom
	 * @return
	 */
	public final boolean existePrototype(String chNom, CPrototype exclu){
		for (int i =0; i < tablePrototypes.size(); i++ ) {
			CPrototype pr = tablePrototypes.get(i);
			if ((pr != exclu) && (pr.nom.equals(chNom))) return true;
		}
		return false;
	}

	/**
	 * Chargement des ventuels prototypes de la figure depuis un flux C++.
	 */
	final void chargePrototypes(MathGraphDataInputStream inps) throws IOException {
		// On lit d'abord le nombre de prototypes
		int nbProt = inps.readCInt();
		for (int i = 0; i < nbProt; i++) {
			CPrototype prot = new CPrototype();
			prot.chargeDepuisFlux(inps);
			tablePrototypes.add(prot);
		}
	}

	/**
	 * Chargement des ventuels prototypes de la figure depuis un flux java.
	 */
	private final void readPrototypes(MathGraphDataInputStream inps) throws IOException {
		// On lit d'abord le nombre de prototypes
		int nbProt = inps.readInt();
		for (int i = 0; i < nbProt; i++) {
			CPrototype prot = new CPrototype();
			prot.read(inps);
			tablePrototypes.add(prot);
		}
	}

	/**
	 * Enregistrement des ventuels prototypes de la figure dans un flux java.
	 */
	private final void writePrototypes(MathGraphDataOutputStream oups) throws IOException {
		ListIterator<CPrototype> li = tablePrototypes.listIterator();
		try {
			oups.writeInt(tablePrototypes.size());
		} catch (IOException e1) {
			throw e1;
		}
		while (li.hasNext()) {
			CPrototype prot = li.next();
			try {
				prot.write(oups);
			} catch (IOException e2) {
				throw e2;
			}
		}
	}
	
	public final void initialiseItemsMenu() {
		itemsCochesInterdits = true;
		listeIdMenus = new Vector<Integer>();
	}
	
	public final void addItem(int item) {
		listeIdMenus.add(new Integer(item));
	}

}
