/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CPointBase extends CPointAncetrePointsMobiles {

  /** Creates new CPointBase */
  public CPointBase() {
  }
  
  /**
   * Constructeur pour objets internes qui ont besoin de connatre la rsolution de la liste
   * @param liste
   */
  protected CPointBase(CListeObjets liste) {
  	listeProprietaire = liste;
  }
  
  public  CPointBase(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
    int motifInit, boolean pointLibreInit, double xInit, double yInit, boolean marquePourTraceInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, marquePourTraceInit);
      x = xInit;
      y = yInit;
  }
  // Constructeur plus court pour objets visuels
  public CPointBase(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit, Couleur.TraduitCouleur(1), false,
      0d, 0d, false, "", 2, MotifPoint.croix, true, false);
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointBase(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, pointLibre, x, y, marquePourTrace);
    return ptelb;
  }
  // Inutile de dfinir setClone car les membres x et y ont t dj clons par CPointAncetre

  @Override
  public String infoPourHistorique() {
    return getString("chinfo1");
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPointBase;
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    super.positionne(infoRandom, dimfen);
    existe = true;
  }

  // La procdure suivante test si lengthdplacement du point correspondant  la position
  // (xtest, ytest)  de la souris est possible et si oui renvoyant dans pointr les
  // nouvelles coordonnes et dans pointAbs.x l'abscisse pour un point li

  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr)
  {
    pointr.x = xtest;
    pointr.y = ytest;
    return testFenetre(dimfen, xtest, ytest);
  }
   

  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CPointBase pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointBase) p;
      return (Kernel.zero(pt.x-x)&&Kernel.zero(pt.y-y));
    }
    else return false;
  }
  
  public void translateDe(double decalagex, double decalagey) {
  	x += decalagex;
  	y += decalagey;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(this);
	}
  
  /**
   * Fonction utilise quand on zoome ou dzoome sur la figure
   * @param xcentre
   * @param ycentre
   * @param rapport
   */
  public final void zoom(double xcentre, double ycentre, double rapport) {
    x = xcentre + rapport*(x - xcentre);
    y = ycentre + rapport*(y - ycentre);  	
  }
  
  @Override
  public void adaptePourHauteResolution(double coef) {
  	super.adaptePourHauteResolution(coef);
		x *= coef;
		y *= coef;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
      super.chargeDepuisFlux(inps, list);
      x = inps.readCDouble()/10;
      y = inps.readCDouble()/10;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    x = inps.readDouble();
    y = inps.readDouble();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeDouble(x);
    oups.writeDouble(y);
  }
}
