/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.io.IOException;
import types.Bool;
import types.DimensionFenetre;
import types.Doub;
import types.Int;
import types.NatureObjet;
import mathgraph32.MtgFrame;

public class CPointInterieurPolygone extends CPointAncetrePointsMobiles {
	CPolygone polygoneAssocie;
	private boolean estSurCote; // true si le point est sur un des sommets du polygone
	private int indicePremierSommet; // L'indice du premier segment sur lequel est le point ou alors l'indice
	// du premier des trois sommets du type BCD tels que le point soit intrieur  BCD et CDE
	private double alpha; // Le coefficient barycentrique normalis du premier point
	private double beta; // Le coefficient barycentrique normalis du second point
	private double gamma; // Le coefficient barycentrique normalis du troisime point
	
	
	
  /** Creates new CPointInterieurPolygone */
  public CPointInterieurPolygone() {
  }
  
  // Constructeur pour clonage
  public CPointInterieurPolygone(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit, boolean pointLibreInit, CPolygone polygoneInit,
      boolean estSurCoteInit, int indicePremierSommetInit, double alphaInit, double betaInit, double gammaInit, boolean marquePourTraceInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
       dec_xInit, dec_yInit, masqueInit, nomInit,
      indiceTailleNomInit, motifInit, pointLibreInit, marquePourTraceInit);
     polygoneAssocie = polygoneInit;
     estSurCote = estSurCoteInit;
     indicePremierSommet = indicePremierSommetInit;
     alpha = alphaInit;
     beta = betaInit;
     gamma = gammaInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(polygoneAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointInterieurPolygone(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, pointLibre, (CPolygone)(listeCible.get(ind1, CPolygone.class)),
      estSurCote, indicePremierSommet, alpha, beta, gamma, marquePourTrace);
    return ptelb;
  }
  

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(polygoneAssocie);
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo166");
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || polygoneAssocie.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || polygoneAssocie.dependDePourReclassement(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || polygoneAssocie.dependDePourBoucle(p);
  }  
  
  /**
   * Pour cet objet, on commence par calculer les coordonnes du point tel qu'il doir
   * tre calcul puis on regarde s'il est bien toujours  l'intrieur du polygone
   * S'il ne l'est pas on met existe  false
   */
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	double xn, yn;
  	existe = polygoneAssocie.existe;
  	if (!existe) return;
  	// int nbSommets = polygoneAssocie.collectionPoints.size() - 1;
		CNoeudPointeurSurPoint n = polygoneAssocie.collectionPoints.get(0);
		CPointAncetre a = n.pointeurSurPoint;
		n = polygoneAssocie.collectionPoints.get(indicePremierSommet);
		CPointAncetre b = n.pointeurSurPoint;
		n = polygoneAssocie.collectionPoints.get(indicePremierSommet + 1);
		CPointAncetre c = n.pointeurSurPoint;
  	if (estSurCote) {
  		xn = b.x*alpha + c.x*beta;
  		yn = b.y*alpha + c.y*beta;
  		super.placeEn(xn, yn);
  		// return; Version 4.6.4
  	}
  	else {
			xn = a.x*alpha + b.x*beta + c.x*gamma;
			yn = a.y*alpha + b.y*beta + c.y*gamma;
  		if (!polygoneAssocie.pointInterieur(xn, yn)) {
  			double d = beta + gamma;
				xn = (b.x*beta + c.x*gamma)/d;
				yn = (b.y*beta + c.y*gamma)/d;
  		}
  		super.placeEn(xn, yn);
  	}
  	super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  }
  
  @Override
  public void placeEn(double xn, double yn) {
  	Bool estSurCote1 = new Bool(false);
  	Int indicePremierSommet1 = new Int();
  	Doub alpha1 = new Doub();
  	Doub beta1 = new Doub();
  	Doub gamma1 = new Doub();
  	polygoneAssocie.determinePositionInterieure(xn, yn, indicePremierSommet1, estSurCote1, alpha1, beta1, gamma1);
  	estSurCote = estSurCote1.getValue();
  	indicePremierSommet = indicePremierSommet1.getValue();
  	alpha = alpha1.getValue();
  	beta = beta1.getValue();
  	gamma = gamma1.getValue();
  	super.placeEn(xn, yn);
  }
  
  @Override
  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr) {
  	// On regarde si le point test est bien intrieur au polygone
  	if (polygoneAssocie.pointInterieur(xtest, ytest)) {
  		pointr.x = xtest;
  		pointr.y = ytest;
  		return true;
  	}
  	else return false;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPointInterieurPolygone;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    if (p.getClass() == this.getClass()) {
	  	CPointInterieurPolygone pt = (CPointInterieurPolygone) p;
	  	if (Kernel.zero(pt.x-x)&& Kernel.zero(pt.y-y))return (polygoneAssocie == pt.polygoneAssocie);
	  	else return false;
    }
    else return false;
  }  

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(polygoneAssocie) || polygoneAssocie.estDefiniParObjetDans(listeOb);
	}  

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    polygoneAssocie = (CPolygone) (list.get(ind1,CDroiteAncetre.class));
    estSurCote = inps.readBoolean();
    indicePremierSommet = inps.readInt();
    alpha = inps.readDouble();
    beta = inps.readDouble();
    gamma = inps.readDouble();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(polygoneAssocie);
    oups.writeInt(ind1);
    oups.writeBoolean(estSurCote);
    oups.writeInt(indicePremierSommet);
    oups.writeDouble(alpha);
    oups.writeDouble(beta);
    oups.writeDouble(gamma);
  }  
}
