/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSymetrieAxiale extends CTransformation {
	private CDroiteAncetre axe;

  /** Creates new CSymetrieAxiale */
  public CSymetrieAxiale() {
  }
  
  @Override
	public int natureTransformation() {
		return symetrieAxiale;
	}
  
  // Constructeur pour clonage
  public CSymetrieAxiale(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CDroiteAncetre axeInit) {
    super(liste, impProtoInit, estElementFinalInit);
    axe = axeInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(axe);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSymetrieAxiale(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
    estElementFinal, (CDroiteAncetre)(listeCible.get(ind1, CDroiteAncetre.class)));
    return ptelb;    
  }
 

  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    liste.add(axe);
  }
  
  @Override
  String designationTransf() {
    return getString("chinfo20");
  }  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || axe.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || axe.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || axe.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	existe = axe.existe;
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CSymetrieAxiale pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSymetrieAxiale) p;
      return axe.confonduAvec(pt.axe);
    } else return false;
  }
  
  @Override
  public double transformeLongueur(double L) {
    return L;
  }

  @Override
  public void transformePoint(double x, double y, Point2D.Double pointr) {

    VecteurDirecteur u = new VecteurDirecteur(axe.point_x, axe.point_y, x, y);
    VecteurDirecteur uimage = new VecteurDirecteur();
    transformeVecteur(u, uimage);
    pointr.x = axe.point_x + uimage.x;
    pointr.y = axe.point_y + uimage.y;
  }

  // Mthode rajoute pour la version 1.8
  void transformeAngle(double ang, Point2D.Double pointr) {
    pointr.x = - ang;
  }
  
  // Java : transformeVecteur a t modifi par rapport au C++. Il est inutile de tester
  // que lengthvecteur directeur de l'axe est non  nul
  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
  double norm2;
  double ax,ay,x,y,k1,k2;

    x = u.x;
    y = u.y;
    ax = axe.vecteur.x;
    ay = axe.vecteur.y;
    norm2 = ax*ax + ay*ay;
    k1 = ax*ax - ay*ay;
    k2 = 2*ax*ay;
    ur.x = (k1*x + k2*y)/norm2;
    ur.y = (k2*x - k1*y)/norm2;
  }
    
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(axe) || axe.estDefiniParObjetDans(listeOb);
	}  	    

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    axe = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    axe = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(axe);
    oups.writeInt(ind1);
  }    
}
