/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;

import types.*;

import dialogs.ChoixModificationImageDlg;
import dialogs.MtgDlg;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;


// Modification de la versio 2.5 : la classe n'est plus abstract pour cause de chargement
// de prototype. Peut tre instancie dans CListeObjets.get(int indice, Class cla)
/** Classe anctre de toutes les transformations gomtriques*/
public abstract class CTransformation extends CElementBase {
	protected static final int translation = 1;
	protected static final int rotation = 2;
	protected static final int symetrieCentrale = 3;
	protected static final int symetrieAxiale = 4;
	protected static final int homothetie = 5;
	protected static final int similitude = 6;
	protected static final int inversion = 7;
	protected static final int translationParCoord = 8;

	/** Creates new CTransformation */
	public CTransformation() {
	}

	public CTransformation(CListeObjets liste, CImplementationProto impProtoInit,
			boolean estElementFinalInit) {
		super(liste, impProtoInit, estElementFinalInit);
	}

	@Override
	public long getNature() {
		return NatureObjet.NTransformation;
	}
	
	/**
	 * Renvoie la nature de la transformation
	 * @return
	 */
	public abstract int natureTransformation();
	

	/**
	 * Sert lors de l'historique Est redfini pour la translation et ermet
	 * d'ajouter  la liste visuelle un vecteur
	 * 
	 * @param liste
	 */
	void AjouteElementsVisuelsPourHist(CListeObjets liste) {
	}

	/**
	 * Chaine pour l'historique indiquant la nature de la transformation
	 * 
	 * @return
	 */
	abstract String designationTransf();

	/**
	 * Certaine transformation renvoient dans cette chaine des complments 
	 * rajouter l'historique
	 * 
	 * @return
	 */
	String complementPourHist() {
		return "";
	}

	public void transformePoint(double x, double y, Point2D.Double pointr) {
	}

	void transformeAngle(double ang, Point2D.Double pointr) {
		pointr.x = ang;
	}

	void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
	}

	public double transformeLongueur(double L) {
		return 0;
	}

	/**
	 * Procdure renvoyant true si le point a une image Sera true sauf pour
	 * l'inversion si on veut l'image du centre
	 * 
	 */
	public boolean pointAUneImage(CPointAncetre point) {
		return true;
	}

	/**
	 * Fonction renvoyant true lorsque les images par ce type de transformation
	 * peuvent tre modifies via un menu. C'est le cas pour les rotations,
	 * homothties, similitudes, inversions pour lequels il faudra redfinir cette
	 * fonction
	 * 
	 * @return
	 */
	public boolean imageModifiableParMenu() {
		return false;
	}

	/**
	 * Fonction renvoyant un dialogue seulement dans le cas o
	 * imageModifiableParMenu() renvoie true Renverra le dailogue associ  la
	 * modification de la transformation. Par exemple pour une rotation renverra
	 * un angleRotDlg.
	 * 
	 * @param image
	 * @return
	 */
	public MtgDlg dialogueModificationImage(MtgFrame frame) {
		return null;
	}

	public void modificationImage(MtgFrame frame, CElementGraphique image) {
		MtgInternalFrame cadre = frame.cadreActif();
		boolean modification = false; // Sera true si un objet a t modifi
		CListeObjets liste = frame.cadreActif().listePr;
		int nbImages = liste.nombreImagesParTransformation(this);
		if (nbImages == 1) {
			// Une seule image pour cette transformation
			// On appelle le dialogue que fournit cette transformation pour se modifier
			MtgDlg dlg = dialogueModificationImage(frame);
			dlg.setVisible(true);
			if (dlg.retourOK()) modification = true;
		}
		else {
			ChoixModificationImageDlg modDlg = new ChoixModificationImageDlg(frame);
			modDlg.setVisible(true);
			if (modDlg.retourOK()) {
				if (modDlg.getModificationMultiple()) {
					// La modification s'applique  la transformation et aussi  toutes les images par celle-ci
					MtgDlg dlg = dialogueModificationImage(frame);
					dlg.setVisible(true);
					if (dlg.retourOK()) modification = true;				
				}
				else {
					// Il y a d'autres objets images par cette transformation et l'utilisateur a choisi de ne pas modifier
					// les autres images. Il faut donc cloner cette transformation
					// et appeler le dialogue de modification pour ce clone
					CTransformation transclone = (CTransformation) this.getClone(listeProprietaire, listeProprietaire);
					frame.insereElement(transclone, liste.indexOf(image));
					MtgDlg dlg = transclone.dialogueModificationImage(frame);
					dlg.setVisible(true);
					if (dlg.retourOK()) {
						modification = true;
						image.associeATransformation(transclone);
					}
					else {
						// Si l'utilisateur a abandonn la mdification il faut retirer la nouvelle transformation
						frame.retireDependants(transclone);
					}
				}
			}
		}
		if (modification) {
		  cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil122"));
		  cadre.listePr.metAJourObjetsDependantDe(this);
		  cadre.repaintFigureAvecCalcul();
		}
	}

}
