/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;

import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CValeurAffichee;
import types.StyleEncadrement;

import dialogs.AffichageValeurDlg;

import mathgraph32.MtgFrame;

public class OutilAffichageValeur extends Outil {
  Point pointDesigne;
 
  /** Creates a new instance of OutilPtLib */
  public OutilAffichageValeur(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
	public Cursor curseurUtilise() {
		return MtgFrame.cornerCursor;
	}  
    
  @Override
	public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("affichageValeur1"));
  }
  
  /** Appel lorsqu'un objet a t cliqu pour tre trait par l'outil
   * par exemple le premier point pour crer une droite passant par deux points
   **/
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    pointDesigne = pointClique;
    creeObjet();
  }
  @Override
  public void creeObjet() {
    int x = pointDesigne.x;
    int y = pointDesigne.y;
    CListeObjets liste = frame.cadreActif().listePr;
    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
    // pour matrialiser l'emplacement de l'affichage
    CCommentaire com = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
    com.positionne(false, frame.cadreActif().dimf);
    ajouteClignotementDe(com);
    resetClignotement();
    CValeurAffichee affval = new CValeurAffichee(liste, null, false, frame.couleurActive(),
    		x, y, 0d, 0d, false, null, frame.pref_IndiceTaillePoliceAffichages, StyleEncadrement.Sans, false, frame.cadreActif().couleurFondAffichage, null, "", "", 2,
				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
    frame.indication("");
    AffichageValeurDlg dlg = new AffichageValeurDlg(frame, affval, false);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
	    if (frame.ajouteElementAvecVerification(affval)) {
	    	annuleClignotement();
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil46"));
	      frame.cadreActif().repaintFigureAvecCalcul();
	    }
    }
    annuleClignotement();
    select();
  }
}
