/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Color;
import java.awt.Point;
import java.util.ArrayList;

import objets.CAffichageLieAPoint;
import objets.CBarycentre;
import objets.CCommentaire;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CNoeudPondere;
import objets.CPointAncetre;
import objets.CValeurAssocieeAVariable;
import types.NatureObjet;
import types.StyleEncadrement;

import dialogs.CoefficientBarycentreDlg;
import mathgraph32.MtgFrame;

public class OutilBarycentre extends Outil {
	private CPointAncetre point; // Pointera sur les points cliqus
	private CBarycentre barycentre; // Le barycentre cr par l'outil. null  la slection
  /** Creates a new instance of OutilRot */
  public OutilBarycentre(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("barycentre1"));
    barycentre = null;
  }

  @Override
  public void deselect() {
    barycentre = null;
    System.gc();
    super.deselect();
  } 

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    point = (CPointAncetre) elg;
    CListeObjets liste = elg.listeProprietaire;
    ajouteClignotementDe(elg);
    if (barycentre == null) {
    	ArrayList<CNoeudPondere> arrayList = new ArrayList<CNoeudPondere>();
     	barycentre =new CBarycentre(liste, null, false, frame.couleurActive(), false, 0, 0, false, "", frame.pref_indiceTaillePoliceNom,
    			frame.stylePointActif, arrayList, false);
    }
    // On cre un nouvel objet CValeurAssocieeAVariable qui sera le coefficient avec 1 commecoefcient par dfaut propos
    CValeurAssocieeAVariable coef = new CValeurAssocieeAVariable(liste, 1);
    // On cre un nouveau CNoeudPondere associ au point cliqu
    CNoeudPondere noeud = new CNoeudPondere(liste, point, coef);
    barycentre.col.add(noeud);
    // On lance un dialogue charg de choisir le coefficient
    frame.cadreActif().videDesignation();
    CoefficientBarycentreDlg dlg = new CoefficientBarycentreDlg(frame, barycentre, coef, false);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
    	// On cre un commentaire li au point qui vient d'tre cliqu reprsentant le coefficient choisi
    	// et on l'ajoute  la liste des objets clignotants
    	CCommentaire com = new CCommentaire(liste, null, false, Color.blue, 0d, 0d, 0d, 10d, false, point, 2,
    			StyleEncadrement.Simple, true,  new Color(230,230,246), coef.chaineInfo(),
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
    	com.positionne(false, frame.cadreActif().dimf);
    	ajouteClignotementDe(com);
    }
    else {
    	barycentre.col.remove(noeud);
    	frame.listeClignotante.remove(frame.listeClignotante.size() - 1);
    	frame.cadreActif().repaintFigureSansCalcul();
    	resetClignotement();
    }
  }
  
  @Override
  public void creeObjetApresClicDroit() {
  	if (barycentre.col.size()<2) {
			frame.dialogueAvertissement(getString("ch104"));
			select();
  	}
  	else {
  		// On vrifie que la somme des coefficients n'est pas nulle
  		if (barycentre.sommeCoefficients(false) == 0) {
  			frame.dialogueAvertissement(getString("ch105"));
  			select(); 			
  		}
  		else {
  			// On rajoute le barycentre  la liste des obejts crs si le mme objet n'a pas t cr
  			if (frame.ajouteElementAvecVerification(barycentre)) {
  				barycentre.donneNom(frame.genereNomPourPoint());
  			  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil65"));
  			  frame.cadreActif().repaintFigureAvecCalcul();
  			}
  	    annuleClignotement();			
  			select();
  		}
  	}
  }
 
}
