/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.JFileChooser;

import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CPrototype;
import dialogs.ChoixPrototypeFigureDlg;
import entreesSorties.MathGraphDataOutputStream;
import entreesSorties.MtgFileManager;

public class OutilConstructionEnregistrer extends Outil {

	CPrototype proto;
	
	public OutilConstructionEnregistrer(MtgFrame frameInit) {
    super(frameInit, false);
  }
	
  @Override
	public void select() {
    super.select();
    boolean choixProto = choixProto();
    if (!choixProto) {
    	frame.activeOutilCapture();
    	return;
    }
    enregistrePrototype();
    frame.activeOutilCapture();
	}
	
	public void enregistrePrototype() {
		class MyFilter extends javax.swing.filechooser.FileFilter {
			public boolean accept(File file) {
				if (file.isDirectory())
					return true;
				String filename = file.getName();
				return filename.endsWith(Global.GLOBAL_COMMA_MGC);
			}

			public String getDescription() {
				return Global.GLOBAL_STAR_COMMA_MGC;
			}
		}
		MtgInternalFrame cadre = frame.cadreActif();
		// JFileChooser fc = new JFileChooser(frame.pref_constDirectory); Modifi version Applet
		JFileChooser fc = new JFileChooser();
		fc.setFileFilter(new MyFilter());
		String chNom = proto.nom + Global.GLOBAL_COMMA_MGC;
		File file1 = new File(chNom);
		fc.setSelectedFile(file1);
    int rVal = fc.showSaveDialog(frame);
    if(rVal == JFileChooser.APPROVE_OPTION) {
  		File file = fc.getSelectedFile();
			String ch = file.getAbsolutePath();
			if (!ch.toLowerCase().endsWith(Global.GLOBAL_COMMA_MGC)) {
				ch = ch + Global.GLOBAL_COMMA_MGC;
				file = new File(ch);
			}
			// frame.repertoireFichiers = ch.substring(0, ind);
			// On regarde si un fichier du mme nom existe dj.
			// Si oui on demande confirmation pour l'crasement
			if (!MtgFileManager.accepteEnregistrement(frame, ch)) return;
				
			try {
				FileOutputStream fos = new FileOutputStream(file);
				MathGraphDataOutputStream oups = new MathGraphDataOutputStream(fos);
				try {
					proto.write(oups);
					oups.close();
					cadre.document.setDirty(false);
				} catch (IOException e1) {
					frame.dialogueAvertissement(getString("ch45"));
					return;
				}
			} catch (FileNotFoundException e2) {
				frame.dialogueAvertissement(getString("ch45"));
				return;
			}
    }
	}
	
	
	protected boolean choixProto() {
		ChoixPrototypeFigureDlg dlg = new ChoixPrototypeFigureDlg(frame, getString("btnEnregistrer"), getString("choixPrototypeFigureDlg3"));
		dlg.setLocationRelativeTo(null);
		dlg.setVisible(true);
		if (dlg.retourOK()) proto = dlg.prototypeChoisi;
		return dlg.retourOK();
	}
 
  @Override
  public boolean activationValide() {
    return frame.cadreActif().document.tablePrototypes.size()!= 0;
  }
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeConstruction");
	}	
	
	
}
