/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import objets.*;
import types.*;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 */
public class OutilDtMed extends OutilPar2pts {
  
  /** Creates a new instance of OutilMed */
  public OutilDtMed(MtgFrame frameInit) {
    super(frameInit);
  }
  
  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  @Override
  public void creeObjet() {
    CMediatrice pt = new CMediatrice(frame.cadreActif().listePr, null, false,
      frame.couleurActive(), frame.styleTraitActif(), point1, point2, frame.pref_indiceTaillePoliceNom);
    if (frame.ajouteElementAvecVerification(pt))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil11"));
    annuleClignotement();
    select();
  }

  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CMilieu ptm = new CMilieu(liste, null, false, Couleur.TraduitCouleur(1), MotifPoint.petitRond,
      point1, pane.pointSouris);
    ptm.cache();
    frame.ajouteObjetVisuel(ptm);
  	// On cre deux segments en pointills
  	CSegment pts1 = new CSegment(liste, null, false, point1, ptm);
    frame.ajouteObjetVisuel(pts1);
  	CSegment pts2 = new CSegment(liste, null, false, ptm, pane.pointSouris);
    frame.ajouteObjetVisuel(pts2);
    // On cre une marque de segment pour les deux
    CMarqueSegment ptMarque1 = new CMarqueSegment(liste, null, false,
      Couleur.TraduitCouleur(1), false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, pts1);
    frame.ajouteObjetVisuel(ptMarque1); 
    CMarqueSegment ptMarque2 = new CMarqueSegment(liste, null, false,
       Couleur.TraduitCouleur(1), false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, pts2);
    frame.ajouteObjetVisuel(ptMarque2);
    // On cre une perpendiculaire plutt qu'une mdiatrice pour pouvoir lui adjoindre
    // une marque d'angle droit
    CDroitePerpendiculaire ptPer = new CDroitePerpendiculaire(liste, null, false,
      frame.couleurActive(), frame.styleTraitActif(), ptm, pts1, 2);
    frame.ajouteObjetVisuel(ptPer);
    // Cration de la marque d'angle droit
    CMarqueAngleDroit marque = new CMarqueAngleDroit(liste, null, false, ptPer);
    frame.ajouteObjetVisuel(marque);
  }
  
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("mediatrice1");
      case 2 : return getString("mediatrice2");
      default: return "";
    }
  }
}
