/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CElementGraphique;
import objets.CLieuObjetParVariable;
import objets.CValeurAssocieeAVariable;
import objets.CVariableBornee;
import types.NatureCalcul;
import types.NatureObjet;

import dialogs.ChoixVariableDlg;
import dialogs.LieuObjetDlg;

import mathgraph32.MtgFrame;

public class OutilLieuObjetParVariable extends Outil {
	private CElementGraphique objetPourTrace;
	private CVariableBornee variableGeneratrice;
	
  public OutilLieuObjetParVariable(MtgFrame frameInit) {
    super(frameInit, true);
  }
	
  @Override
  public void deselect() {
    objetPourTrace = null;
    variableGeneratrice = null;
    super.deselect();
  }

  @Override
  public void select() {
    super.select();
    objetPourTrace = null;
    variableGeneratrice = null;
    frame.indication(getString("lieuParVariable1"));
    ChoixVariableDlg dlg = new ChoixVariableDlg(frame, frame.cadreActif().listePr.size()-1);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (!dlg.retourOK()) { 
    	frame.activeOutilCapture();
    	return;
    }
    variableGeneratrice = dlg.variableChoisie;
    frame.outilPointageObjetClignotant.designationDe(NatureObjet.NToutObjet);
    frame.listeClignotante.ajouteObjetsPouvantGenererLieuObjetDepDe(frame.cadreActif().listePr, variableGeneratrice);
    frame.outilPointageActif = frame.outilPointageObjetClignotant;
    frame.indication(getString("lieuParPointLie1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (variableGeneratrice == null) return;
    if (objetPourTrace == null) {
      objetPourTrace = elg;
      creeObjet();
    }
  }
  
  @Override
  public void creeObjet() {
  	CValeurAssocieeAVariable nbPositions = new CValeurAssocieeAVariable(frame.cadreActif().listePr, 40);
    CLieuObjetParVariable pt = new CLieuObjetParVariable(frame.cadreActif().listePr, null,
       false, frame.couleurActive(), false, objetPourTrace, nbPositions, variableGeneratrice);
    frame.cadreActif().videDesignation();
    LieuObjetDlg dlg = new LieuObjetDlg(frame, pt, false);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
	    if (frame.ajouteElementAvecVerification(pt)){
	    	pt.etablitListeElementsAncetres();
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil62"));
	      frame.cadreActif().repaintFigureAvecCalcul();
	    }
    }
    annuleClignotement();
    frame.activeOutilCapture();
  }  
  
  @Override
	public boolean activationValide() {
		return (frame.cadreActif().listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NVariable) != 0);
	}
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeVariable");
	}
	
	
  
}
