/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CElementGraphique;
import objets.CLieuParVariable;
import objets.CPointAncetre;
import objets.CVariableBornee;
import types.InfoLieu;
import types.NatureCalcul;
import types.NatureObjet;

import dialogs.ChoixVariableDlg;
import dialogs.LieuDlg;

import mathgraph32.MtgFrame;

public class OutilLieuParVariable extends Outil {
	private CPointAncetre pointPourTrace;
	private CVariableBornee variableGeneratrice;
	/**
	 * Constructeur
	 * @param frameInit
	 */
  public OutilLieuParVariable(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
  public void deselect() {
    pointPourTrace = null;
    variableGeneratrice = null;
    super.deselect();
  }  
  
  @Override
  public void select() {
    super.select();
    pointPourTrace = null;
    variableGeneratrice = null;
    frame.indication(getString("lieuParVariable1"));
    ChoixVariableDlg dlg = new ChoixVariableDlg(frame, frame.cadreActif().listePr.size()-1);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (!dlg.retourOK()) { 
    	frame.activeOutilCapture();
    	return;
    }
    variableGeneratrice = dlg.variableChoisie;
    frame.outilPointageObjetClignotant.designationDe(NatureObjet.NToutPoint);
    frame.listeClignotante.ajoutePointsDependantDe(frame.cadreActif().listePr, variableGeneratrice);
    frame.outilPointageActif = frame.outilPointageObjetClignotant;
    frame.indication(getString("lieuParPointLie1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (variableGeneratrice == null) return;
    if (pointPourTrace == null) {
      pointPourTrace = (CPointAncetre) elg;
      creeObjet();
    }
  }
  
  @Override
  public void creeObjet() {
    InfoLieu info = new InfoLieu(200, false, false);
    CLieuParVariable pt = new CLieuParVariable(frame.cadreActif().listePr, null,
       false, frame.couleurActive(), false, frame.styleTraitActif(), pointPourTrace, info, variableGeneratrice);
    frame.cadreActif().videDesignation();
    LieuDlg dlg = new LieuDlg(frame, pt);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
	    if (frame.ajouteElementAvecVerification(pt)){
	    	pt.etablitListeElementsAncetres();
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil58"));
	      frame.cadreActif().repaintFigureAvecCalcul();
	    }
    }
    annuleClignotement();
    frame.activeOutilCapture();
  }
  
  @Override
	public boolean activationValide() {
		return (frame.cadreActif().listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NVariable) != 0);
	}
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeVariable");
	}	  	
  
}
