/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.geom.Point2D;
import dialogs.PointLibreEntDlg;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointBaseEnt;
import objets.CRepere;
import objets.CValeurAssocieeAVariable;
import types.Doub;
import types.NatureCalcul;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilPointLibreEnt extends Outil {
  Point pointDesigne;
  CPointBaseEnt pointCree;
  
  /** Creates a new instance of OutilPtLibreEnt */
  public OutilPointLibreEnt(MtgFrame frameInit) {
    super(frameInit, false);
  }
  
	@Override
  public Cursor curseurUtilise() {
		return MtgFrame.crayonCursor;
	}  
  
  @Override
  public void select() {
    super.select();
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
		CRepere rep = null;
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere) == 1) rep = liste.premierRepere();
    pointCree = new CPointBaseEnt(frame.cadreActif().listePr, null, false,
        frame.couleurActive(), false, 0, 0, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom,
        frame.stylePointActif(), true, rep, true, true, new CValeurAssocieeAVariable(liste, -5),
        new CValeurAssocieeAVariable(liste, 5), new CValeurAssocieeAVariable(liste, -5),
        new CValeurAssocieeAVariable(liste, 5), 0, 0, false);
    if (rep == null) {
			PointLibreEntDlg dlg = new PointLibreEntDlg(frame, pointCree, false);
			dlg.setLocationRelativeTo(null);
			dlg.setVisible(true);
			if (!dlg.retourOK()) {
				frame.activeOutilCapture();
				return;
			}
    }
		frame.outilPointageActif = frame.outilPointageClic;
		frame.indication(getString("pointLibreEnt1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    pointDesigne = pointClique;
    creeObjet();
  }
  
  @Override
  public void creeObjet() {
  	// CListeObjets liste = frame.cadreActif().listePr;
    MtgInternalFrame cadre = frame.cadreActif();
    Point2D.Double pointres = new Point2D.Double();
    Doub pointabs = new Doub();
    pointCree.positionne(cadre.infRand, cadre.dimf);
    pointCree.testDeplacement(cadre.dimf, pointDesigne.x, pointDesigne.y, pointres, pointabs);
    pointCree.placeEn(pointres.x, pointres.y);
    if (frame.ajouteElementAvecVerification(pointCree))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil99"));
    frame.cadreActif().repaintFigureAvecCalcul();
    // Attention : version 3.5 : S'il y a plusieurs repres dans la figure on ne reslectionne pas l'outil
    // sinon la bote de dialogue va se rouvrir
    if (cadre.listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere) == 1) select();
    else frame.activeOutilCapture();
  }  

  @Override
	public boolean activationValide() {
		return frame.cadreActif().listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)>0;
	}
	
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeRepere");
	}	
  
}
