/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import dialogs.PointSurCourbeParAbscisseDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalcul;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPrototype;
import objets.CResultatValeur;
import objets.CValeurAssocieeAVariable;
import objets.CValeurDynamique;
import types.NatureCalcul;

public class OutilPointSurCourbeParAbscisse extends Outil {
	PointSurCourbeParAbscisseDlg dlg;
	
	public OutilPointSurCourbeParAbscisse(MtgFrame frameInit) {
		super(frameInit, false);
	}
	
  @Override
	public void select() {
		super.select();
  	dlg = new PointSurCourbeParAbscisseDlg(frame);
		dlg.setLocationRelativeTo(null);
  	dlg.setVisible(true);
  	if (dlg.retourOK()) creeObjet();
  	else frame.activeOutilCapture();
	}

  @Override
  public void creeObjet() {
  	CValeurDynamique valeurAbs;
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste= cadre.listePr;
  	cadre.videDesignation();
  	// On regarde d'abord si pour le champ d'dition de l'abscisse l'utilisateur a fait rfrence  une valeur
  	// existante ou entr un calcul
  	CValeurAssocieeAVariable valAbs =dlg.valAbs;
  	if (valAbs.calcul.getClass() != CResultatValeur.class) {
  		String nomMin = liste.genereNomPourCalcul("abs", true);
  		CCalcul calculMin = new CCalcul(liste, null, false, nomMin, valAbs.calcul.chaineCalculSansPar(null), valAbs.calcul);
  		frame.ajouteElement(calculMin);
  		valeurAbs = calculMin;
  	}
  	else {
  		valeurAbs = ((CResultatValeur)valAbs.calcul).valeurAssociee;
  	}
  	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("PointSurCourbeParAbscisse.mgc");;
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = dlg.listeFonctions.fonctionSelectionnee();
		((CElementGenerique)proto.get(1)).elementAssocie = valeurAbs;
		((CElementGenerique)proto.get(2)).elementAssocie = dlg.listeReperes.repereChoisi();
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		proto.retireTout(); // Ajout version 4.6.2
		// On donne aux calculs finaux les noms choisis dans la bote de dialogue
		((CValeurDynamique)liste.get(liste.size() - 2)).donneNom(dlg.editeurNomOrdonnee.getText());
		// On modifie le nom de l'implmentation (pour internationalisation)
		impProto.nomProto = getString("pointSurCourbeParAbscisse");
		// On modifie le dernier objet implment qui est un affichage de valeur li  un point
		CPointAncetre point = (CPointAncetre) liste.get(liste.size() - 1);
		point.donneMotif(frame.stylePointActif);
		point.donneCouleur(frame.couleurActive());
		point.indiceTailleNom = frame.pref_indiceTaillePoliceNom;
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil124"));
    frame.cadreActif().repaintFigureAvecCalcul();
    frame.activeOutilCapture();
  }	
  	
  @Override
	public boolean activationValide() {
  	// Modifi version Applet
  	/*
		CListeObjets liste = frame.cadreActif().listePr;
		return (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)!=0)
						&& (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NFonction)!=0);
		*/
		return (frame.cadreActif().listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)!=0)
						&& (frame.listePourConstruction().nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NFonction)!=0);
	}
	
	@Override
	public String messageActivationNonValide() {
		CListeObjets liste = frame.cadreActif().listePr;
		String ch = "";
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)==0) ch = ch + getString("messagePasDeRepere") + "\n";
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NFonction)==0) ch = ch + getString("messagePasDeFonction");
		return ch;
	}	  
	

}
