/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPrototype;
import objets.CValeurDynamique;
import types.NatureCalcul;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import dialogs.PointSurCourbeParPointLieDlg;

public class OutilPointSurCourbeParPointLie extends OutilParPoint {
	PointSurCourbeParPointLieDlg dlg;
	
	public OutilPointSurCourbeParPointLie(MtgFrame frameInit) {
		super(frameInit);
	}
	
  @Override
	public void select() {
		super.select();
  	dlg = new PointSurCourbeParPointLieDlg(frame);
		dlg.setLocationRelativeTo(null);
  	dlg.setVisible(true);
  	if (!dlg.retourOK()) {
  		frame.activeOutilCapture();
  	}
  	frame.indication(getString("pointSurCourbe1"));
  	// Il ne faut pourvoir dsigner qu'un point li
    frame.outilPointageCre.designationDe(NatureObjet.NPointLie);
	}
	
  @Override
  public void creeObjet() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste= cadre.listePr;
  	frame.cadreActif().videDesignation();
  	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("PointSurCourbe.mgc"); // Modifi version 4.6.2
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = dlg.listeFonctions.fonctionSelectionnee();
		((CElementGenerique)proto.get(1)).elementAssocie = dlg.listeReperes.repereChoisi();
		((CElementGenerique)proto.get(2)).elementAssocie = point;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		proto.retireTout(); // Ajout version 4.6.2
		// On donne aux calculs finaux les noms choisis dans la bote de dialogue
		((CValeurDynamique)liste.get(liste.size() - 3)).donneNom(dlg.editeurNomAbscisse.getText());
		((CValeurDynamique)liste.get(liste.size() - 2)).donneNom(dlg.editeurNomOrdonnee.getText());
		// On modifie le nom de l'implmentation (pour internationalisation)
		impProto.nomProto = getString("pointSurCourbeParPointLie");
		// On modifie le dernier objet implment qui est un affichage de valeur li  un point
		CPointAncetre point = (CPointAncetre) liste.get(liste.size() - 1);
		point.donneMotif(frame.stylePointActif);
		point.donneCouleur(frame.couleurActive());
		point.indiceTailleNom = frame.pref_indiceTaillePoliceNom;
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil124"));
    frame.cadreActif().repaintFigureAvecCalcul();
    frame.activeOutilCapture();
  }
	  
  @Override
	public boolean activationValide() {
  	// Modifi version Applet
  	/*
		CListeObjets liste = frame.cadreActif().listePr;
		return (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)!=0)
						&& (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NFonction)!=0)
						&& (liste.nombreObjetsParNatureVisibles(NatureObjet.NPointLie) != 0);
  	*/
  	CListeObjets liste = frame.listePourConstruction();
  	CListeObjets listePr = frame.cadreActif().listePr;
		return (listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)!=0)
						&& (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NFonction)!=0)
						&& (liste.nombreObjetsParNatureVisibles(NatureObjet.NPointLie) != 0);
	}
	
	@Override
	public String messageActivationNonValide() {
		CListeObjets liste = frame.cadreActif().listePr;
		String ch = "";
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)==0) ch = ch + getString("messagePasDeRepere") + "\n";
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NFonction)==0) ch = ch + getString("messagePasDeFonction");
		if (liste.nombreObjetsParNatureVisibles(NatureObjet.NPointLie) == 0) ch = ch + getString("messagePasDePointLie");
		return ch;
	}	  
	
}
