/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import types.InfoProx;
import types.NatureObjet;

public class OutilPointagePourRideau extends OutilPointageCre {
  
  /** Creates a new instance of OutilPointageCre */
  public OutilPointagePourRideau(MtgFrame frameInit) {
    super(frameInit);
  }
  
  /**
   * Renvoie la liste  utiliser.
   * Devra tre redfini pour l'outil de suppresion d'objet
   * qui ne permettra de supprimer que les objets de la liste
   * listeConstruction
   */
  @Override
  protected CListeObjets listeUtilisee() {
  	// Spcial version applet
    // return frame.cadreActif().listePr;
    return frame.listePourConstruction();
  }

  public void actionDansPaint(MtgInternalFrame cadre) {
  	if (!cadre.isConstruction) {
  		super.actionDansPaint(cadre);
  		return;
  	}
		CMathGrafDoc doc= cadre.document;
		cadre.effacegbuffer();
		Outil outilActif = frame.outilActif;
		if (outilActif.timer != null) {
    	if (outilActif.nbClignPair) {
    		// Ncessaire de ne pas afficher deux fois les mmes pour les affichages de surface avec transparence
    		// donc ne pas afficher les lemnts de listeClignotante (qui peuvent tre des objest provisoires
    		cadre.listeSauve.afficheTout(cadre.gbuffer, cadre.dimf, true, doc.couleurFond,
      			doc.transparencePourSurfaces, frame.pref_antialiasing);
    		frame.listePourConstruction().afficheToutSaufElementsDe(cadre.gbuffer, frame.cadreActif().dimf, true, doc.couleurFond,
    			doc.transparencePourSurfaces, frame.listeClignotante, frame.pref_antialiasing);
    		frame.listeClignotante.afficheTout(cadre.gbuffer, frame.cadreActif().dimf, false, doc.couleurFond,
    			doc.transparencePourSurfaces, frame.pref_antialiasing);
    	}
    	else {
    		cadre.listeSauve.afficheTout(cadre.gbuffer, cadre.dimf, true, doc.couleurFond,
      			doc.transparencePourSurfaces, frame.pref_antialiasing);    		
    		frame.listePourConstruction().afficheToutSaufElementsDe(cadre.gbuffer, cadre.dimf, false, doc.couleurFond,
    			doc.transparencePourSurfaces, frame.listeClignotante, frame.pref_antialiasing);
    	}
		}
		// else cadre.afficheToutDansgbuffer();
		else {
  		cadre.listeSauve.afficheTout(cadre.gbuffer, cadre.dimf, true, doc.couleurFond,
    			doc.transparencePourSurfaces, frame.pref_antialiasing);
  		frame.listePourConstruction().afficheTout(cadre.gbuffer, frame.cadreActif().dimf, false, doc.couleurFond,
    			doc.transparencePourSurfaces, frame.pref_antialiasing);
  		
		}
		aRedessinerDansPaint = false;
  }
  
  
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    int x = mouseEvent.getX();
    int y = mouseEvent.getY();
    CListeObjets liste = listeUtilisee();
    InfoProx info = frame.infoProx;
    int nbObjetsProches = liste.procheDePourOutilRideau(aDesigner, mouseEvent.getPoint(), info);
    int nbTypesProches = info.nombreTypesProches;
    boolean designationDejaAffichee = (cadre.commentaireDesignation().chaineCommentaire.length() != 0);
      // true si une dsignation est dj affiche prs du pointeur
    if (nbObjetsProches > 0) {
        String ch = "";
      // Si lors d'un dialogue de choix d'objet proche ona dj choisi un objet
      // et si celui-ci est proche du pointeur souris, c'est celui-l qui sera slectionnn
      if (info.premierPointVoisin != null) {
        ch = getString("designationPoint");
      }
      else {
        if (nbTypesProches == 1) {
          for (int i =0; i < NatureObjet.nombreTypesDesignables; i++) {
            if (info.infoParType[i].nombreVoisins > 0) {
              ch = info.infoParType[i].premierVoisin.chaineDesignation();
              break;
            }
          }
        }
        else {
          ch = " prciser"; // A revoir
        }
      }
      cadre.commentaireDesignation().donneChaine(ch);
      cadre.commentaireDesignation().placeEn(x, y);
      cadre.paneFigure.repaint();
    }
    else {
      setActiveCursor(MtgFrame.crossHairCursor);
      frame.cadreActif().videDesignation();
      if ((frame.listeObjetsVisuels.size() > 0) || designationDejaAffichee) {
        cadre.paneFigure.repaint();
      }
    }
  }
}
