/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import objets.*;
import dialogs.AbscisseDlg;
import mathgraph32.MtgFrame;

/**
 * Outil de cration d'un point dfini par son abscisse relativement  deux autres points
 * @author Yves
 */
public class OutilPtParAbs extends OutilPar2pts {
  CPointParAbscisse pt;
  /** Creates a new instance of OutilPar2pts */
  public OutilPtParAbs(MtgFrame frameInit) {
    super(frameInit);
  }
  
  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  boolean nomsPointsIndisp() {
    return true;
  }

  /**
   * Action aprs dsignation du deuxime point
   */
  @Override
  public void actionApresPoint2() {
    ajouteClignotementDe(point2);
    resetClignotement();    
    frame.indication(getString("pointParAbs3"));   
  }

  @Override
  public String chaineIndication(int indEtape) {
    switch(indEtape) {
      case 1 : return getString("pointParAbs1");
      case 2 : return getString("pointParAbs2");
      case 3 : return getString("pointParAbs3");
      default: return "";
    }
  }
  
 
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    CValeurAssocieeAVariable abs = new CValeurAssocieeAVariable(liste, 0);
    pt = new CPointParAbscisse(liste, null, false, frame.couleurActive(), false, 
            0d, 0d, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom,
            frame.stylePointActif(), point1, point2, abs, false);
    // On cre un dialogue modal permettant d'entrer l'abscisse
    AbscisseDlg abscisseDlg = new AbscisseDlg(frame, pt, false);
    // On redessine la figure pour liminer la dsignation ce point affiche pour le centre
    frame.cadreActif().videDesignation();
    frame.cadreActif().paneFigure.repaint();
		abscisseDlg.setLocationRelativeTo(null);
    abscisseDlg.setVisible(true);
    // On regarde si la bote de dialogue a bien t valide par OK
    if (abscisseDlg.retourOK()) {
      if (frame.ajouteElementAvecVerification(pt)) {
        objetCree = true;
        frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil21"));
      }
    }
    annuleClignotement();
    select();
  }
}
