/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import objets.CDroiteAB;
import objets.CListeObjets;
import objets.CRepere;
import objets.CValeurAssocieeAVariable;
import dialogs.RepereDlg;
import mathgraph32.MtgFrame;

public class OutilRepere extends OutilPar3pts {
  public OutilRepere(MtgFrame frameInit) {
    super(frameInit);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  @Override
  boolean nomsPointsIndisp() {
    return true;
  }
  
  @Override
  public void creeObjet() {
  	CListeObjets liste = frame.cadreActif().listePr;
    CRepere rep = new CRepere(frame.cadreActif().listePr,null, false, frame.couleurActive(), false,frame.styleTraitActif(),
    		point1, point2, point3, new CValeurAssocieeAVariable(liste, 0d), new CValeurAssocieeAVariable(liste, 0d),
    		false, false, false,new CValeurAssocieeAVariable(liste, 1d), new CValeurAssocieeAVariable(liste, 1d));
    // dialogue de choix des caractristiques du repre
    RepereDlg dlg = new RepereDlg(frame, rep, false);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
    	// On cre si ncessaire les axes
    	int nbObjetsSup = 0; // Le nombre d'objets supplmentaires crs (les axes)
    	if (dlg.checkCreerAxes.isSelected()) {
	    	CDroiteAB axeAbs = liste.existeDroiteABDefiniePar(point1, point2);
	    	if (axeAbs == null) {
	    		axeAbs = new CDroiteAB(frame.cadreActif().listePr, null, false, frame.couleurActive(), 
	    				frame.styleTraitActif(), point1, point2, frame.pref_indiceTaillePoliceNom);
	    		frame.ajouteElement(axeAbs);
	    		nbObjetsSup++;
	    	}
	    	CDroiteAB axeOrd = liste.existeDroiteABDefiniePar(point1, point3);
	    	if (axeOrd == null) {
	    		axeOrd = new CDroiteAB(frame.cadreActif().listePr, null, false, frame.couleurActive(), 
	    				frame.styleTraitActif(), point1, point3, frame.pref_indiceTaillePoliceNom);
	    		frame.ajouteElement(axeOrd);
	    		nbObjetsSup++;
	    	}
    	}
      // Attention : Pour cause de compatibilit avec la version C++ ce qui est appel
      // dans la bote de dialogue Quadrillage vertical fait rfrence au membre QuadrillageHorizontale du repre
    	rep.quadrillageHorizontal = dlg.checkQuadVer.isSelected();
    	rep.quadrillageVertical = dlg.checkQuadHor.isSelected();
    	rep.pointilles = dlg.checkPointilles.isSelected();
    	
	    if (frame.ajouteElementAvecVerification(rep)) {
	      objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil40"));
	    }
	    else frame.detruitNDerniersElements(nbObjetsSup);
    }
    annuleClignotement();
    frame.activeOutilCapture();
  }

/**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */          
  @Override
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("repere1");
      case 2 : return getString("repere2");
      case 3 : return getString("repere3");
      default: return "";
    }
  }
}
