/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Ajout version 4.6.3
package outils;

import java.awt.Color;
import java.awt.Point;
import dialogs.segmentParLongDlg;
import objets.CAffichageLieAPoint;
import objets.CCalcul;
import objets.CCommentaire;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointLieCercle;
import objets.CPrototype;
import objets.CResultatValeur;
import objets.CSegment;
import objets.CValeurAffichee;
import objets.CValeurAssocieeAVariable;
import objets.CValeurDynamique;
import types.NatureObjet;
import types.StyleEncadrement;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilSegmentParLong extends Outil {
	segmentParLongDlg dlg;
	private CPointAncetre point1;
	public static String nomCurseur = "Segment";

	public OutilSegmentParLong(MtgFrame frameInit) {
		super(frameInit, true);
		// TODO Auto-generated constructor stub
	}

  @Override
	public void select() {
		super.select();
    point1 = null;
    frame.outilPointageActif = frame.outilPointageCreationSegmentParLongueur;
    frame.outilPointageActif.designationDe(NatureObjet.NToutPoint);
    frame.indication(getString("segmentParLong1"));
	}

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null) {
      point1 = (CPointAncetre) elg;
      ajouteClignotementDe(elg);
      CListeObjets liste = frame.cadreActif().listePr;
      // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
      // pour matrialiser l'emplacement de l'affichage
      int x = pointClique.x;
      int y = pointClique.y;
      CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 7d, -6d, false, null, 2,
      		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
  				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
      comClignotant.positionne(false, frame.cadreActif().dimf);
      ajouteClignotementDe(comClignotant);
      resetClignotement();
      frame.cadreActif().videDesignation();
    	dlg = new segmentParLongDlg(frame);
  		dlg.setLocationRelativeTo(null);
    	dlg.setVisible(true);
    	if (dlg.retourOK()) creeObjet();
    	else {
    		frame.detruitDernierElement();
    		frame.activeOutilCapture();
    	}	
    }
  }
  
  @Override
  public void creeObjet() {
  	CValeurDynamique valeurLong; // La valeur qui sera associ au premier objet source
  	
  	MtgInternalFrame cadre = frame.cadreActif();
  	cadre.videDesignation();
  	CListeObjets liste = cadre.listePr;
  	// On regarde d'abord si pour les valeurs mini et maxi l'utilisateur a fait rfrence  une valeur
  	// existante ou entr un calcul
  	CValeurAssocieeAVariable valLong =dlg.valLong;
  	if (valLong.calcul.getClass() != CResultatValeur.class) {
  		String nomMin = liste.genereNomPourCalcul(getString("long"), false);
  		CCalcul calculMin = new CCalcul(liste, null, false, nomMin, valLong.calcul.chaineCalculSansPar(null), valLong.calcul);
  		frame.ajouteElement(calculMin);
  		valeurLong = calculMin;
  	}
  	else {
  		valeurLong = ((CResultatValeur)valLong.calcul).valeurAssociee;
  	}
  	// On regarde quel est le prototype  implmenter (ils ont t chargs dans le constructeur du MtgFrame)
  	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("SegmentParLongueur.mgc");
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = valeurLong;
		((CElementGenerique)proto.get(1)).elementAssocie = point1;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
 		impProto.nomProto = getString("segmentParLongDlg1");
		// On modifie le dernier objet implment qui est un affichage de valeur li  un point
		// suivant les choix de la bote de dialogue
		// On modifie le style de trait et de couleur des objets finaux
		Color col = frame.couleurActive();
		CValeurAffichee affval = (CValeurAffichee) liste.get(liste.size() - 1);
		affval.couleurFondAffichage = cadre.couleurFond;
		affval.effacementFond = false;
		affval.donneCouleur(col);
		int styleTrait = frame.styleTraitActif();
		int indImpProto = liste.indexOf(impProto);
		CPointLieCercle pt = (CPointLieCercle) (liste.get(indImpProto + 2));
		pt.donneCouleur(col);
		pt.donneMotif(frame.stylePointActif());
		pt.donneNom(frame.genereNomPourPoint()); // Ajout version 4.7.0.1
		CSegment seg = (CSegment) (liste.get(indImpProto + 3));
		seg.donneCouleur(col);
		seg.donneStyle(styleTrait);
		
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil157"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
		
  }

  // Ajout version 4.7.2 pour corriger oubli
  @Override
  public boolean activationValide() {
    return (frame.cadreActif().listePr.pointeurLongueurUnite != null);
  }
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeLongueurUnite");
	}	

}
