/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CArcDeCercleAncetre;
import objets.CCercle;
import objets.CElementGraphique;
import objets.CLieuDeBase;
import objets.CPolygone;
import objets.CSurfaceDisque;
import objets.CSurfaceLieu;
import objets.CSurfacePolygone;
import objets.CSurfaceSecteurCirculaire;
import types.NatureObjet;

import mathgraph32.MtgFrame;

/**
 * Outil de cration d'uen surface dlimite par un cercle, un arc de cercle, un polygone ou un lieu de points
 * @author Yves
 *
 */
public class OutilSurface extends Outil {
	CElementGraphique bord;

	public OutilSurface(MtgFrame frameInit) {
		super(frameInit, false);
	}
	
  @Override
	public void select() {
		super.select();
		bord = null;
		frame.outilPointageActif = frame.outilPointageCre;
		frame.outilPointageActif.designationDe(NatureObjet.NPolygone | NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NLieu);
		frame.indication(getString("surface1"));
    frame.outilPointageObjetClignotant.reset();
	}

	/*
	 * Appel lorsqu'un objet a t cliqu pour tre trait par l'outil par
	 * exemple le premier point pour crer une droite passant par deux points
	 */
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (bord == null) {
      bord = elg;
      creeObjet();
    }
	}

  @Override
	public void creeObjet() {
		if (bord.getNature() == NatureObjet.NPolygone) {
	    CSurfacePolygone pt = new CSurfacePolygone(frame.cadreActif().listePr, null,
	        false, frame.couleurActive(), false, frame.styleRemplissageActif(), (CPolygone) bord);
	     if (frame.ajouteElementAvecVerification(pt))
	       frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil38"));
	     frame.repaintFigure();
	     select();
    }
		else {
			if (bord.getNature()== NatureObjet.NCercle) {
		    CSurfaceDisque pt = new CSurfaceDisque(frame.cadreActif().listePr, null,
		        false, frame.couleurActive(), false, frame.styleRemplissageActif(), (CCercle) bord);
		     if (frame.ajouteElementAvecVerification(pt))
		       frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil38"));
		     frame.repaintFigure();
		     select();
			}
			else {
				if (bord.getNature() == NatureObjet.NArc) {
			    CSurfaceSecteurCirculaire pt = new CSurfaceSecteurCirculaire(frame.cadreActif().listePr, null,
			        false, frame.couleurActive(), false, frame.styleRemplissageActif(), (CArcDeCercleAncetre) bord);
			     if (frame.ajouteElementAvecVerification(pt))
			       frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil38"));
			     frame.repaintFigure();
			     select();
				}
				else {
					if (bord.getNature() == NatureObjet.NLieu) {
						// Pour crer une surface dlimite par un lieu le lieu doit tre form d'au moins une ligne
						CLieuDeBase lieu = (CLieuDeBase) bord;
						if (!lieu.contientAuMoinsUneLigne()) {
							frame.dialogueAvertissement(getString("ch4"));
							return;
						}
				    CSurfaceLieu pt = new CSurfaceLieu(frame.cadreActif().listePr, null,
				        false, frame.couleurActive(), false, frame.styleRemplissageActif(), lieu);
				     if (frame.ajouteElementAvecVerification(pt))
				       frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil38"));
				     frame.repaintFigure();
				     select();
					}
				}
			}
		}
	}
}
