/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package mathgraph32;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ListIterator;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.SpringLayout;
import javax.swing.ToolTipManager;
import org.scilab.forge.jlatexmath.TeXFormula;
import netscape.javascript.JSObject;
import outils.*;
import objets.*;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.InfoProx;
import types.Int;
import types.NatureCalcul;
import types.NatureObjet;
import types.StyleFleche;
import types.StyleMarqueSegment;
import types.StylePoint;
import types.StyleRemplissage;
import types.UniteAngle;
import dialogs.HistoriqueDlg;
import dialogs.MtgDlg;
import dialogs.PaneInsertionCaractereFleche;
import dialogs.PaneInsertionCaractereGrec;
import dialogs.PaneInsertionCaractereMath;
import dialogs.PaneLatexInsertionCaractereFleche;
import dialogs.PaneLatexInsertionCaractereGrec;
import dialogs.PaneLatexInsertionCaractereMath;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import entreesSorties.MathGraphTimerTask;
import glob.Kernel;
import actions.*;

/**allow
 * MtgPane est la fentre principale de MathGraph32 Version 4.4.6 : Un paramtre
 * permet de choisir le niveau d'utilisation.
 */
@SuppressWarnings("serial")
public class MtgFrame extends JApplet /*
																			 * implements WindowListener,
																			 * ActionListener
																			 */{
	// @SuppressWarnings("unused")
	// private static TeXFormula formula1, formula2, formula3, formula4, formula5, formula6, formula7;
	public String mtgOnInitParam; // Nouveau version 4.6 permet au javaScript de
																// connatre via mtgOnInit le nom de l'applet
																// qui a appel mtgOnInit()
	public static MtgFrame frameActif; // Variable globale pointant sur le frame
	// en cours d'activit
	private MtgInternalFrame cadreActif; // Ajout version Applet
	// Ajout version 4.6.1. isReady sert  savoir si l'applet a bien t affiche
	// Sera mis  true dans paint de MtgFigurePanel quand premierAffichage = true et aussi aprs appel  setScript()
	public boolean isReady;
	Locale locale; // Pour la gestion de la langue
	public static ResourceBundle messages; // Pour la gestion des messages suivant
																					// la langue
	// Modifi version 4.6. L'initialisation est reporte dans init()
	public static ImageIcon blankIcon;
	public boolean allowLeftToolbar; // Sera dtermin par le paramtre
																		// allowLeftToolbar. True par dfaut
	public boolean allowTopToolbar; // Sera dtermin par le paramtre
																		// allowTopToolbar. True par dfaut
	private boolean allowRightToolbar; // Sera dtermin par le paramtre
																			// allowRightToolbar. True par dfaut
	public boolean allowMenuBar; // Sera dtermin par le paramtre allowMenu.
																// True par dfaut
	private boolean allowToolsChoice; // Sera dtermin par la paramtre
																		// allowToolChoice. True par dfaut
	private boolean allowFileMenu; // Sera dtermin par le paramtre
																	// allowFileMenu. True par dfaut
	private boolean allowOptionsMenu; // Sera dtermin par le paramtre
																		// allowOptionsMenu. True par dfaut
	private boolean allowIndicationArea; // Sera dtermin par le paramtre
																				// allowIndicationArea. True par dfaut
	public boolean allowLevelChoice; // Sera dtermin par le paramtre
																		// allowLevelChoice. True par dfaut
	public boolean allowExercise; // Si true, dans le cas o la figure contient
																// une macro d'apparition d'objets nomme
																// solutionConstruction
																// ou une macro de pause nomme exercice, passe
																// ne mode exercice (on ne peut pas modifier les
																// objets
																// de la figure initiale)
	public boolean forceEditorsFormulaDisplay;	// Ajout version 4.7.2.2 Sera false par dfaut. Si true, les diteurs de formule
	// pour lesquels on a demand un affichage en LaTeX et demand que la formule ne soit pas affiche au chargement verront
	// ces formules affiches nnamoins.
	private boolean allowFileOpen; // Si true le bouton d'ouverture de fichier et le menu correspondants seront actifs
	public String action; // Pour le gestionnaire d'applets pour savoir si
												// l'applet doit
	// sauver via le gestionnaire d'applet les valeurs de res1, ..., res10 auquel
	// cas action
	// doit contenir "Sauvegarde" ou si ces valeurs doivent ?tre restitu?es aux
	// variables
	// res1, ..., res10 auquel cas action doit contenir "Restitution"
	public String initialFigure; // Ajout version 4.6 : Permet de savoir quelle
																// est le type de figure pour l'applet
																// au chargement.
																// empty : Figure vide sans longueur unit
																// unity : Figure vide avec un segment unit
																// orthonormalFrame : Figure avec un repre
																// orthonormal, graduation et pointills
																// orthogonalFrame : Figure avec un repre
																// orthogonal, graduation et pointills
	public boolean allowSolutionButton; // Ajout version 4.7.2. Si ce paramtre est false,
																// pour les exercices de construction, le bouton Solution ne sera pas autoris
																// sinon il le sera. True par dfaut
	public boolean allowRandom; 	// Ajout version 4.7 : Si true lorsque la figure est affiche pour la premire fois, les tirages alatoires seront recalculs
																// et sinon ils ne le seront pas. True par dfaut
	public int numberOfInitialObjects;	// Ajout version 4.7.2. Si ce nombre est foruni en paramtre, l'historique ne prsentera que les objets au-del
																			// du nombre spcifi par ce paramtre.
	public boolean allowTools;		// Ajout version 4.7.2. true par dfaut. Si le paramtre est false, les seuls outils activables seront l'outil de capture,
																// l'historique, l'outil d'affichage des objets numriques crs et l'outil d'excution d'une macro.

	boolean premierAffichage;
	// Si aRecalculer est true, la figure sera recalcule et redessine
	// la prochaine fois que MtgPaneFigure excutera paint();
	boolean aRecalculerEtRedessiner = false;
	// Si aRedessiner est true, la figure sera retrace dans le gbuffer
	// la prochaine fois que MtgPaneFigure excutera paint();
	boolean aRedessiner = false;
	public String chaineIndicationPourCons; // Spcial version applet
	public boolean chargementEnCours = false; // Variable de classe servant 
																						// connatre le nombre d'instances
																						// de l'applet (spcial version
																						// Applet)
	private static int nbInstances = 0;
	private static AppletManager appletManager;
	public String path; // Contiendra la valeur du paramtre path, null par dfaut
											// et sinon le chemin d'accs en ligne  une figure
	public String dataBase64; // null si le paramtre figureData n'est pas donn,
														// sinon contient la description de la figure
	// Constantes dfinissant les options de fonctionnementMathGraph32EspV3245
	public static final int niveauAvanceAvecComplexes = 1;
	public static final int niveauAvanceSansComplexes = 2;
	public static final int niveauMoyen = 3;
	public static final int niveauElementaire = 4;
	// Ajout version 3.2.4 : un tableau de 4 chanes contenant les descripteurs
	// des 4 niveaux
	// Si le texte associ aux 4 figures default1  default4 contenues dans le
	// mme rpertoire que
	// le Jar excutable n'est pas vide, les 50 premiers caraactres seront le
	// descripteur du niveau
	// sinon ce sont les chanes contenues dans
	// textes.Chaines.niveauAvanceAvecComplexes  niveau lmentaire qui seront
	// utilises
	public String[] descripteurNiveau = new String[5];
	// Proprits de l'application
	public Properties properties;
	// Variables correspondant aux prfrences contenues dans properties;
	public String pref_language = "fra";
	// Ajout version 4.6.4 : Sert  savoir s'il faut attnuer l'paisseur des traits quand on copie-colle
	// avec un coefficient multiplicateur suprieur  1
	public boolean pref_attenuation;
	public int pref_angleUnit; // Nombre de points utiliss quand on utilise le
	// menu Mouvoir Animer un point li
	public int pref_moveNbPoints; // Frquence utilise quand on utilise le menu
	// Mouvoir Animer un point li
	public int pref_moveFreq; // Position de la prochaine fentre MtgInternalFrame
	public boolean pref_moveCyclic; // Animation cyclique ou non
	/*
	 * Pas utilis version Applet public String pref_fileDirectory; // /
	 * rpertoire par dfaut d'ouverture des // figures public String
	 * pref_constDirectory; // / rpertoire par dfaut d'ouverture des //
	 * constructions
	 */
	public int pref_niveau;
	public boolean pref_AideVisuelle; // Aide  la cration visuelle d'objets
	public boolean pref_decalageAffichagesLibres; // Dcalage des affichages
	// libres lors des translations
	// de la figure
	// Taille de police pour commentaires et affichages de valeur
	public int pref_indiceTaillePoliceNom;
	public int pref_IndiceTaillePoliceAffichages;
	public boolean pref_NomsAuto;
	// Nombre de colonnes dont est form la palette d'outils de gauche
	public int pref_NbCol;
	// Petites ou grandes icnes
	public boolean pref_LittleIconSize;
	// Ajout version 3.2.4 possibilit de choisir un fonctionnement personnalis
	public boolean pref_NiveauPersonnalise;
	// Ajout version 3.2.4 possibilit de choisir l'affichage automatique des
	// mesures en mode personnalis
	public boolean[] pref_affichageMesuresNiveau = new boolean[5];
	// Ajout version 4.6 : Facteur d'agrandissement des images quand on colle dans
	// le presse-papier et quand on exporte en jpeg
	public double pref_ClipboardImagesRatio;
	private MtgToggleButton btnsDispo[];

	// Version 3.7 : possibilit de regrouper les icne de mme type par couleur
	// d'encadrement
	public boolean pref_ColoredIcons;
	// Version 4.5.1 : Possibilit de choisir l'antialiasing active ou non via le
	// paramtre antialiasing
	public boolean pref_antialiasing;

	// Ajout pour version 2.0. Transparence par dfaut pour les nouvelles figures
	public static float transparenceParDefaut = 0.2f;
	public static final String nombrePointsPourLieuPointsParDefaut = "500"; // Pass
																																					// de
																																					// 300
																																					// 
																																					// 500
																																					// pour
																																					// version
																																					// 4.5.1
	// Figure dont les options de menu disent quels outils doivent tre permis ou
	// interdits
	// public CMathGrafDoc docPreferencesOutils = null; Plus utilis pour la
	// version Applet
	public CMathGrafDoc docPreferencesNiveauPredefini[] = new CMathGrafDoc[5];
	// Les couleurs pour les icnes par type d'icne
	private static final Color couleurOutilsPoints = Color.red;
	private static final Color couleurOutilsLigne = Color.blue;
	private static final Color couleurOutilsMesure = Color.green;
	private static final Color couleurOutilsTransformation = Color.orange;
	private static final Color couleurOutilsPolygones = Color.pink;
	private static final Color couleurOutilsAffichages = Color.cyan;
	private static final Color couleurOutilsDivers = Color.gray;

	
	// Version 4.6.1 : les icnes des outils
	// Passs static version 4.7.4
	public static ImageIcon iconCreerPointLibre = null;
	public static ImageIcon iconCreerPointLibreEnt = null;
	public static ImageIcon iconCreerPointLie = null;
	public static ImageIcon iconCreerMilieu = null;
	public static ImageIcon iconCreerProjeteOrthogonal = null;
	public static ImageIcon iconCreerPointParAbscisse = null;
	public static ImageIcon iconCreerPointParCoordonnees = null;
	public static ImageIcon iconCreerPointParAffixe = null;
	public static ImageIcon iconCreerCentreGravite = null;
	public static ImageIcon iconCreerPointParSommeVect = null;
	public static ImageIcon iconCreerPointParMultVect = null;
	public static ImageIcon iconCreerBarycentre = null;
	public static ImageIcon iconCreerPointInterieur = null;
	public static ImageIcon iconCurseur = null;
	public static ImageIcon iconCreerIntersection = null;
	public static ImageIcon iconCreerDroiteAB = null;
	public static ImageIcon iconCreerDroiteParallele = null;
	public static ImageIcon iconCreerDroitePerpendiculaire = null;
	public static ImageIcon iconCreerDroiteBissectrice = null;
	public static ImageIcon iconCreerDroiteMediatrice = null;
	public static ImageIcon iconCreerDroiteHorizontale = null;
	public static ImageIcon iconCreerDroiteVerticale = null;
	public static ImageIcon iconCreerDroiteParCoef = null;
	public static ImageIcon iconCreerDroiteParEquation = null;
	public static ImageIcon iconCreerSegment = null;
	public static ImageIcon iconCreerDemiDroite = null;
	public static ImageIcon iconCreerVecteur = null;
	public static ImageIcon iconCreerCercleOA = null;
	public static ImageIcon iconCreerCercleOR = null;
	public static ImageIcon iconCreerPolygone = null;
	public static ImageIcon iconPolygoneRegulier = null;
	public static ImageIcon iconParallelogramme = null;
	public static ImageIcon iconCarreDirect = null;
	public static ImageIcon iconTriangleEquilateralDirect = null;
	public static ImageIcon iconCreerLigneBrisee = null;
	public static ImageIcon iconCreerArcPetitParPointFinal = null;
	public static ImageIcon iconCreerArcPetitParRapporteur = null;
	public static ImageIcon iconCreerArcGrandParPointFinal = null;
	public static ImageIcon iconCreerArcGrandParRapporteur = null;
	public static ImageIcon iconCreerArcDirectParPointFinal = null;
	public static ImageIcon iconCreerArcIndirectParPointFinal = null;
	public static ImageIcon iconCreerMarqueAngleNonOriente = null;
	public static ImageIcon iconCreerMarqueAngleOriente = null;
	public static ImageIcon iconCreerMarqueSegment = null;
	public static ImageIcon iconMesureLongueur = null;
	public static ImageIcon iconMesureAbscisse = null;
	public static ImageIcon iconMesureAbscisseDansRepere = null;
	public static ImageIcon iconMesureOrdonneeDansRepere = null;
	public static ImageIcon iconMesurerCoefficientDirecteur = null;
	public static ImageIcon iconMesureLongueurLigne = null;
	public static ImageIcon iconMesureAirePolygone = null;
	public static ImageIcon iconMesureAffixe = null;
	public static ImageIcon iconMesureAngleNonOriente = null;
	public static ImageIcon iconMesureAngleOriente = null;
	public static ImageIcon iconMesureProduitScalaire = null;
	public static ImageIcon iconCreerImageParTranslation = null;
	public static ImageIcon iconCreerImageParTranslationParCoord = null;
	public static ImageIcon iconCreerImageParSymetrieAxiale = null;
	public static ImageIcon iconCreerImageParSymetrieCentrale = null;
	public static ImageIcon iconCreerImageParRotation = null;
	public static ImageIcon iconCreerImageParHomothetie = null;
	public static ImageIcon iconCreerImageParSimilitude = null;
	public static ImageIcon iconRapporteur = null;
	public static ImageIcon iconCreerAffichageValeur = null;
	public static ImageIcon iconCreerAffichageValeurLiePoint = null;
	public static ImageIcon iconCreerCommentaire = null;
	public static ImageIcon iconCreerCommentaireLiePoint = null;
	public static ImageIcon iconCreerLatex = null;
	public static ImageIcon iconCreerLatexLiePoint = null;
	public static ImageIcon iconCreerImage = null;
	public static ImageIcon iconCreerImageLie = null;
	public static ImageIcon iconCreerEquationDroite = null;
	public static ImageIcon iconCreerEquationDroiteLie = null;
	public static ImageIcon iconCreerEditeurFormule = null;
	public static ImageIcon iconCreerCalcul = null;
	public static ImageIcon iconCreerRepere = null;
	public static ImageIcon iconCreerLieuParPointLie = null;
	public static ImageIcon iconCreerFonctionReelle = null;
	public static ImageIcon iconCreerCourbeFonction = null;
	public static ImageIcon iconCreerPointSurCourbeParAbscisse = null;
	public static ImageIcon iconTangente = null;
	public static ImageIcon iconCreerSurface = null;
	public static ImageIcon iconCreerSurfaceLieuDroite = null;
	public static ImageIcon iconCreerSurfaceLieuDeuxPoints = null;
	public static ImageIcon iconCreerSurfaceDeuxLieux = null;
	public static ImageIcon iconPunaiser = null;
	public static ImageIcon iconDepunaiser = null;
	public static ImageIcon iconSegmentParLongueur = null;
	public static ImageIcon iconAnnuler = null;
	public static ImageIcon iconRefaire = null;
	public static ImageIcon iconVoirObjetsNumeriques = null;
	public static ImageIcon iconCapture = null;
	public static ImageIcon iconCopier = null;
	public static ImageIcon iconExecuterMacro = null;
	public static ImageIcon iconExporterAvecUnite = null;
	public static ImageIcon iconNouvelleFigure = null;
	public static ImageIcon iconNouvelleFigureAvecRepere = null;
	public static ImageIcon iconOuvrir = null;
	public static ImageIcon iconGomme = null;
	public static ImageIcon iconTexteFigure = null;
	public static ImageIcon iconHistorique = null;
	public static ImageIcon iconModifierFormule = null;
	public static ImageIcon iconNommer = null;
	public static ImageIcon iconModifierObjetGraphique = null;
	public static ImageIcon iconPalette = null;
	public static ImageIcon iconReafficher = null;
	public static ImageIcon iconRideau = null;
	public static ImageIcon iconSupprimerObjetGraphique = null;
	public static ImageIcon iconSupprimerObjetNumerique = null;
	public static ImageIcon iconTranslaterFigure = null;
	public static ImageIcon iconZoomPlus = null;
	public static ImageIcon iconZoomMoins = null;
	public static ImageIcon iconDemarquerPoint = null;
	public static ImageIcon iconMarquerPoint = null;
	public static ImageIcon iconActiverModeTrace = null;
	public static ImageIcon iconRelationEntreObjets = null;
	
	// Deux pointeurs servant  ractiver l'outil en cours prcdent
	public MtgAction actionEnCours;

	public int xfenetre = 5;
	public int yfenetre = 5;
	// Numro d'instance pour la cration d'un nouveau document
	public int instanceNouvelleFenetre = 0;
	// Le rpertoire utilis par les documents. Sera actualis  l'ouverture ou
	// enregistrement d'un document
	// Spcial version Applet : static pour que si une instance de l'applet ouvre un fichier
	// puis une autre instance elle se retrouve par dfaut dans le dossier prcdent
	public static String repertoireFichiers = "";
	// Les panneaux contenant les composants et la figure
	JPanel paneLeft = new JPanel();
	JPanel paneSouth = new JPanel();
	public JPanel paneNorth = new JPanel();
	JPanel paneRight = new JPanel();
	MtgPaneStylePoint paneStylePoint;
	MtgPaneStyleFleche paneStyleFleche;
	public JPanel paneRightHaut = new JPanel();
	MtgPaneStyleMarqueSegment paneStyleMarqueSegment;
	MtgPaneStyleRemplissage paneStyleRemplissage;
	MtgPaneChoixCouleur paneCouleurs;
	public MtgPaneTransparence paneTransparence;
	JPanel paneInfo = new JPanel(); // Pour mettre les labels indiquant les optios
	// noms auto et aide visuelle
	// Une zne de texte de deux lignes en bas de la fentre pour les indications
	JTextArea indicationArea = new JTextArea(1, 60);

	public JLabel labelNomsAuto, labelAideVisuelle, labelUniteAngle;

	// Le tableau de barres d'outil pour la palette d'outils de gauche
	JToolBar leftbar[];

	// La barre d'outil horizontale
	public JToolBar topbar[] = new JToolBar[2];

	// Ajout version 4.3 : Les deux MtgMenuItem qui servent  annuler refaire
	public MtgMenuItem menuItemEditionAnnuler = null;
	public MtgMenuItem menuItemEditionRefaire = null;
	// Les menus principaux qui seront modifis quand on utilise le menu Options
	// Figure en cours Menu
	public MtgMenu menuFichier;
	public MtgMenu menuEdition;
	public MtgMenu menuCreer;
	public MtgMenu menuCalculs;
	public MtgMenu menuModifier;
	public MtgMenu menuMouvoir;
	public MtgMenu menuMacros;
	// public MtgMenu menuFenetre; Supprim version Applet
	public MtgMenu menuHelp;
	public MtgMenu menuOptions;
	public MtgMenu menuConstructions;
	// Les menus qui eux ne seront pas modifis et seront utilsis pour
	// initialiser le PaneOptionsFigureMenu dans le OptionsFigureDlg
	public MtgMenu menuFichierInitial;
	public MtgMenu menuEditionInitial;
	public MtgMenu menuCreerInitial;
	public MtgMenu menuCalculsInitial;
	public MtgMenu menuModifierInitial;
	public MtgMenu menuMouvoirInitial;
	public MtgMenu menuMacrosInitial;
	public MtgMenu menuFenetreInitial;
	public MtgMenu menuHelpInitial;
	public MtgMenu menuOptionsInitial;
	public MtgMenu menuConstructionsInitial;

	// La barre le contenant
	JMenuBar menuBar = new JMenuBar();
	// curseurs pointeurs de souris
	public static Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
	public static Cursor crossHairCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
	public static Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
	public static Cursor moveCursor = new Cursor(Cursor.MOVE_CURSOR);
	public static Cursor waitCursor = new Cursor(Cursor.WAIT_CURSOR);
	public static Cursor cornerCursor; // Curseur pour les affichages de valeurs ,
	// commentaires, macros, user defined
	public static Cursor macroCursor; // Curseur pour excution de macro
	public static Cursor supCursor; // Curseur poubelle
	public static Cursor modifCursor; // Curseur de modification d'objet graphique
	public static Cursor gommeCursor; // Curseur gomme
	public static Cursor paletteCursor; // Curseur de palette
	public static Cursor crayonCursor; // Curseur de cration de point libre
	// Les actions associess aux items de menus ou icones
	public JCheckBoxMenuItem cbAideVisuelle, cbNomsAuto, cbModeTrace,
			cbAffichageAutoMesures, cbImageFondVisible;
	MtgAction actionAnnuler;
	MtgAction actionRefaire;
	MtgAction actionCapture;
	MtgAction actionTraceObjets;
	MtgAction actionFichierOuvrir;
	MtgAction actionFichierEnregistrer;
	public MtgAction actionFichierEnregistrerSous; // public car appel par
	// ActionFichierEnregistrer
	ActionFichierExporterPNG actionFichierExporterPNG;
	ActionFichierExporterJPEG actionFichierExporterJPEG;
	ActionFichierExporterPNGAvecUnite actionFichierExporterPNGAvecUnite;
	// MtgAction actionFichierQuitter;Suppri version Applet
	MtgAction actionTranslaterFigure;
	MtgAction actionCreerPointLibre;
	MtgAction actionCreerMilieu;
	MtgAction actionCreerPointLie;
	MtgAction actionCreerDroiteAB;
	MtgAction actionCreerDroiteParallele;
	MtgAction actionCreerDroitePerpendiculaire;
	MtgAction actionCreerDroiteBissectrice;
	MtgAction actionCreerDroiteMediatrice;
	MtgAction actionCreerSegment;
	MtgAction actionCreerDemiDroite;
	MtgAction actionCreerIntersection;
	MtgAction actionCreerCercleOA;
	MtgAction actionCreerCercleOR;
	MtgAction actionCreerCalcul;
	MtgAction actionModifierNom;
	MtgAction actionHistorique;
	MtgAction actionMesureLongueur;
	MtgAction actionMesureLongueurLigne;
	MtgAction actionMesureAirePolygone;
	MtgAction actionMesureAbscisse;
	MtgAction actionMesureAngleNonOriente;
	MtgAction actionMesureAngleOriente;
	MtgAction actionRapporteur;
	MtgAction actionSuppressionObjetGraphique;
	MtgAction actionCreerMarqueAngleNonOriente;
	MtgAction actionCreerMarqueAngleOriente;
	MtgAction actionCreerMarqueAngleOrienteSensDirect;
	MtgAction actionCreerMarqueAngleOrienteSensIndirect;
	MtgAction actionCreerImageParTranslation;
	MtgAction actionCreerImageParTranslationParCoord;
	MtgAction actionCreerImageParSymetrieAxiale;
	MtgAction actionCreerImageParSymetrieCentrale;
	MtgAction actionCreerImageParHomothetie;
	MtgAction actionCreerImageParRotation;
	MtgAction actionCreerImageParInversion;
	MtgAction actionCreerArcPetitParPointFinal;
	MtgAction actionCreerArcGrandParPointFinal;
	MtgAction actionCreerArcDirectParPointFinal;
	MtgAction actionCreerArcIndirectParPointFinal;
	MtgAction actionCreerArcPetitParAngle;
	MtgAction actionCreerArcGrandParAngle;
	MtgAction actionCreerArcDirectParAngle;
	MtgAction actionCreerArcIndirectParAngle;
	MtgAction actionCreerArcPetitParRapporteur;
	MtgAction actionCreerArcGrandParRapporteur;
	MtgAction actionPalette;
	MtgAction actionCreer;
	MtgAction actionCreerDroiteHorizontale;
	MtgAction actionCreerDroiteVerticale;
	MtgAction actionCreerMarqueSegment;
	MtgAction actionCreerPolygone;
	MtgAction actionCreerLigneBrisee;
	MtgAction actionCreerSurface;
	MtgAction actionCreerSurfaceLieuDroite;
	MtgAction actionCreerSurfaceLieuDeuxPoints;
	MtgAction actionCreerSurfaceDeuxLieux;
	MtgAction actionCreerProjeteOrthogonal;
	MtgAction actionCreerRepere;
	MtgAction actionCreerPointParCoordonnees;
	MtgAction actionModifierObjetGraphique;
	MtgAction actionCreerDroiteParCoef;
	MtgAction actionCreerCalculComplexe; // Pas d'icne pour cet outil
	MtgAction actionCreerAffichageValeur;
	MtgAction actionModifierFormule; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionReelle;
	MtgAction actionCreerFonctionComplexe; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionReelle2Var; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionReelle3Var; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionComplexe2Var; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionComplexe3Var; // Pas d'icne pour cet outil

	MtgAction actionCreerCentreGravite;
	MtgAction actionCreerPointParAbscisse;
	MtgAction actionCreerAffichageValeurLiePoint; // Pas d'icne pour cet outil
	MtgAction actionCreerImageLibre; // Pas d'icne pour cet outil
	MtgAction actionCreerImageLieePoint; // Pas d'icne pour cet outil
	MtgAction actionCreerCommentaire;
	MtgAction actionCreerCommentaireLiePoint; // Pas d'icne pour cet outil
	MtgAction actionCreerLatex;
	MtgAction actionCreerLatexLiePoint;
	MtgAction actionCreerVariable;
	MtgAction actionCreerLieuParPointLie;
	MtgAction actionCreerLieuParVariable;
	MtgAction actionCreerLieuDiscretParPointLie;
	MtgAction actionCreerLieuDiscretParVariable;
	MtgAction actionCreerLieuObjetParPointLie;
	MtgAction actionCreerLieuObjetParVariable;
	MtgAction actionCreerImageParSimilitude;
	MtgAction actionCreerBarycentre;
	MtgAction actionExecuterMacro;
	MtgAction actionGomme;
	MtgAction actionCreerVecteur;
	MtgAction actionMesureAbscisseDansRepere;
	MtgAction actionMesureOrdonneeDansRepere;
	MtgAction actionMesureAffixe;
	MtgAction actionMesureProduitScalaire;
	MtgAction actionCreerDemiPlan;
	MtgAction actionCreerDerivee;
	MtgAction actionCreerSuiteRecurrenteReelle;
	MtgAction actionCreerSuiteRecurrenteComplexe;
	MtgAction actionCreerPointParAffixe;
	MtgAction actionCreerGrapheSuiteRecurrenteReelle;
	MtgAction actionCreerGrapheSuiteRecurrenteComplexe;
	MtgAction actionFichierNouvelleFigure;
	MtgAction actionFichierNouvelleFigureSansLongueurUnite;
	MtgAction actionFichierNouvelleFigureAvecRepere;
	MtgAction actionModifierRepere;
	MtgAction actionCreerSolutionEquation;
	MtgAction actionCreerMaximum;
	MtgAction actionCreerMinimum;
	MtgAction actionCreerIntegrale;
	MtgAction actionRideau;
	MtgAction actionCreerTestExistence;
	MtgAction actionCreerTestEqNatOp; // Ajout version 4.7.2
	MtgAction actionCreerPartieReelle;
	MtgAction actionCreerPartieImaginaire;
	MtgAction actionCreerModule;
	MtgAction actionCreerArgument;
	MtgAction actionCreerSommeIndicee;
	MtgAction actionCreerProduitIndice;
	MtgAction actionCreerMacroApparition;
	MtgAction actionCreerMacroDisparition;
	MtgAction actionCreerMacroAnimation;
	MtgAction actionCreerMacroAnimationAvecTrace;
	MtgAction actionCreerMacroAffectationValeurVariable;
	MtgAction actionCreerMacroModificationVariable;
	// MtgAction actionCreerMacroJouantSon; Supprim version Applet
	MtgAction actionCreerMacroIncrementationVariable;
	MtgAction actionCreerMacroDecrementationVariable;
	MtgAction actionCreerMacroTraceAutoParPointLie;
	MtgAction actionCreerMacroTraceAutoParVariable;
	MtgAction actionCreerMacroAffectationPointLie;
	MtgAction actionCreerMacroClignotement;
	MtgAction actionCreerMacroAffichantTout;
	MtgAction actionCreerMacroPause;
	MtgAction actionCreerMacroBoucleAvecAnimation;
	MtgAction actionCreerMacroBoucleAvecTrace;
	MtgAction actionCreerMacroSuiteMacros;
	MtgAction actionEditionCopier;
	MtgButton buttonExporterUnite; // Ajout version 4.6.3
	MtgButton buttonFichierNouvelleFigureAvecRepereSansVecteurs; // Ajout version 4.6.3
	ActionFichierImprimer actionFichierImprimer;
	ActionZoomPlus actionZoomPlus;
	ActionZoomMoins actionZoomMoins;
	ActionModeTraceActivation actionModeTrace;
	MtgAction actionHelpAbout;
	// MtgAction actionHelpNet; Supprim version Applet
	// MtgAction actionHelpCodeHTML; Supprim version Applet
	MtgAction actionHelpHelp;
	MtgAction actionSupprimerLiaison;
	MtgAction actionCreerLiaison;
	MtgAction actionSupprimerLiaisonAffichage;
	MtgAction actionCreerLiaisonAffichage;
	MtgAction actionReclassementFinObjetGraphique;
	MtgAction actionReclassementFinValeur;
	MtgAction actionReclassementDebutObjetGraphique;
	MtgAction actionReclassementDebutValeur;
	MtgAction actionCreerPointLibreEnt;
	MtgAction actionPunaiser;
	MtgAction actionDepunaiser;
	MtgAction actionCreerObjetDuplique;
	MtgAction actionOptionsFigureEnCours;
	MtgAction actionOptionsPreferences;
	MtgAction actionSuppressionImageFond;
	MtgAction actionImageFondVisible;
	MtgAction actionChoixImageFond;
	MtgAction actionCreerCourbeFonction;
	MtgAction actionReafficherTout;
	MtgAction actionCreerCentreCercle;
	MtgAction actionConstructionImplementerDepuisFigure;
	MtgAction actionSuppressionValeur;
	MtgAction actionAjoutObjetsMacro;
	MtgAction actionRetraitObjetsMacro;
	MtgAction actionModificationMacro;
	MtgAction actionMouvoirAnimationPointLie;
	MtgAction actionRenommerCalcul;
	MtgAction actionConstructionEnregistrer;
	MtgAction actionIncorporerConstruction;
	MtgAction actionModifierConstruction;
	MtgAction actionCreerLongueurUnite;
	// MtgAction actionChoixSourcesNonGraphiques; Supprim version Applet
	// MtgAction actionChoixSourcesGraphiques; Supprim version Applet
	// MtgAction actionChoixFinauxGraphiques; Supprim version Applet
	// MtgAction actionChoixFinauxNonGraphiques; Supprim version Applet
	// MtgAction actionConstructionCreer; Supprim version Applet
	// MtgAction actionConstructionReinitialiser; Supprim version Applet
	MtgAction actionConstructionImplementerDepuisFichier;
	MtgAction actionConstructionSupprimer;
	MtgAction actionConstructionFusionner;
	MtgAction actionCreationVisuelle;
	MtgAction actionAffichageAutoMesures;
	MtgAction actionNomsAuto;
	MtgAction actionAfficherObjetsNumeriques;
	MtgAction actionEditerTexteFigure;
	MtgAction actionCurseur;
	MtgAction actionPointSurCourbeParPointLie;
	MtgAction actionPointSurCourbeParAbscisse;
	MtgAction actionTangente;
	MtgAction actionMesureCoefficientDirecteur;
	MtgAction actionActiverOutilPrecedent;
	MtgAction actionCreerMacroApparitionAPartirAutreMacro;
	MtgAction actionCreerMacroDisparitionAPartirAutreMacro;
	MtgAction actionFichierNouvelleFigurePredefinie;
	MtgAction actionCreerPointParSommeVect;
	MtgAction actionCreerPointParMultVect;
	MtgAction actionCarreDirect;
	MtgAction actionTriangleEquilateralDirect;
	MtgAction actionParallelogramme;
	MtgAction actionPolygoneRegulier;
	MtgAction actionCreerPointInterieur;
	MtgAction actionCreationLiaisonPointInterieurPolygone;
	MtgAction actionCreationLiaisonPointInterieurCercle;
	ActionRelationEntreObjets actionRelationEntreObjets;
	MtgAction actionCreerMacroModeTraceActivation;
	MtgAction actionCreerMacroModeTraceDesactivation;
	MtgAction actionHelpTexteFigure;
	MtgAction actionCreerEquationDroite;
	MtgAction actionCreerEquationDroiteLiePoint;
	MtgAction actionCreerDroiteParEquation;
	MtgAction actionChoixOutils;
	MtgAction actionMarquerPointPourTrace;
	MtgAction actionDemarquerPointPourTrace;
	MtgAction actionCreerAbscisseOrigineRep;
	MtgAction actionCreerOrdonneeOrigineRep;
	MtgAction actionCreerUnitexRep;
	MtgAction actionCreerUniteyRep;
	MtgAction actionAjoutGraduationSimple;
	MtgAction actionAjoutGraduationTrigo;
	MtgAction actioncreerEditeurFormule;
	MtgAction actionCreerTestEquivalence; // Ajout version 4.6
	MtgAction actionCreerDeriveePartielle; // Ajout version 4.6
	MtgAction actionCreerTestDependanceVariable; // Ajout version 4.6
	MtgAction actionCreerSegmentParLong; // Ajout version 4.6.3
	MtgAction actionCreerSuiteRecurrenteComplexe2; // Ajout version 4.7.5
	MtgAction actionCreerSuiteRecurrenteReelle2; // Ajout version 4.7.5
	MtgAction actionCreerSuiteRecurrenteComplexe3; // Ajout version 4.7.5
	MtgAction actionCreerSuiteRecurrenteReelle3; // Ajout version 4.7.5

	public MtgAction actionPropSol; // Spcial version applet

	MtgButton buttonPropSol = null; // Spcial applet
	// Un groupe de boutons pour englober tous les boutons de la barre d'outil de
	// cration d'objet
	ButtonGroup btngroup1 = new ButtonGroup();
	// les boutons de la barre d'outils de cration d'objets
	MtgToggleButton buttonCapture;
	MtgToggleButton buttonCreerPointLibre;
	MtgToggleButton buttonCreerMilieu;
	MtgToggleButton buttonCreerPointLie;
	MtgToggleButton buttonCreerDroiteAB;
	MtgToggleButton buttonCreerDroiteParallele;
	MtgToggleButton buttonCreerDroitePerpendiculaire;
	MtgToggleButton buttonCreerDroiteBissectrice;
	MtgToggleButton buttonCreerDroiteMediatrice;
	MtgToggleButton buttonCreerSegment;
	MtgToggleButton buttonCreerDemiDroite;
	MtgToggleButton buttonCreerIntersection;
	MtgToggleButton buttonCreerCercleOA;
	MtgToggleButton buttonCreerCercleOR;
	MtgToggleButton buttonCreerCalcul;
	MtgToggleButton buttonModifierNom;
	MtgToggleButton buttonHistorique;
	MtgToggleButton buttonAfficherObjetsNumeriques;
	MtgToggleButton buttonMesureLongueur;
	MtgToggleButton buttonMesureAbscisse;
	MtgToggleButton buttonMesureAngleNonOriente;
	MtgToggleButton buttonMesureAngleOriente;
	MtgToggleButton buttonMesureLongueurLigne;
	MtgToggleButton buttonMesureAirePolygone;
	MtgToggleButton buttonRapporteur;
	MtgToggleButton buttonSuppressionObjetGraphique;
	MtgToggleButton buttonSuppressionValeur;
	MtgToggleButton buttonCreerMarqueAngleNonOriente;
	MtgToggleButton buttonCreerMarqueAngleOriente;
	MtgToggleButton buttonCreerImageParTranslation;
	MtgToggleButton buttonCreerImageParTranslationParCoord;
	MtgToggleButton buttonCreerImageParSymetrieAxiale;
	MtgToggleButton buttonCreerImageParSymetrieCentrale;
	MtgToggleButton buttonCreerImageParHomothetie;
	MtgToggleButton buttonCreerImageParRotation;
	MtgToggleButton buttonCreerArcPetitParPointFinal;
	MtgToggleButton buttonCreerArcGrandParPointFinal;
	MtgToggleButton buttonCreerArcDirectParPointFinal;
	MtgToggleButton buttonCreerArcIndirectParPointFinal;
	MtgToggleButton buttonCreerArcPetitParRapporteur;
	MtgToggleButton buttonCreerArcGrandParRapporteur;
	MtgToggleButton buttonCreerDroiteHorizontale;
	MtgToggleButton buttonCreerDroiteVerticale;
	MtgToggleButton buttonCreerMarqueSegment;
	MtgToggleButton buttonCreerPolygone;
	MtgToggleButton buttonCreerLigneBrisee;
	MtgToggleButton buttonCreerSurface;
	MtgToggleButton buttonCreerSurfaceLieuDroite;
	MtgToggleButton buttonCreerSurfaceLieuDeuxPoints;
	MtgToggleButton buttonCreerSurfaceDeuxLieux;
	MtgToggleButton buttonCreerProjeteOrthogonal;
	MtgToggleButton buttonCreerRepere;
	MtgToggleButton buttonCreerPointParCoordonnees;
	MtgToggleButton buttonCreerDroiteParCoef;
	MtgToggleButton buttonCreerImage;
	MtgToggleButton buttonCreerImageLie;
	MtgToggleButton buttonCreerAffichageValeur;
	MtgToggleButton buttonCreerAffichageValeurLiePoint;
	MtgToggleButton buttonCreerCentreGravite;
	MtgToggleButton buttonCreerPointParAffixe;
	MtgToggleButton buttonCreerPointParAbscisse;
	MtgToggleButton buttonCreerCommentaire;
	MtgToggleButton buttonCreerCommentaireLiePoint;
	MtgToggleButton buttonCreerLatex;
	MtgToggleButton buttonCreerLatexLiePoint;
	MtgToggleButton buttonCreerLieuParPointLie;
	MtgToggleButton buttonCreerImageParSimilitude;
	MtgToggleButton buttonCreerBarycentre;
	MtgToggleButton buttonExecuterMacro;
	MtgToggleButton buttonGomme;
	MtgToggleButton buttonCreerVecteur;
	MtgToggleButton buttonMesureAbscisseDansRepere;
	MtgToggleButton buttonMesureOrdonneeDansRepere;
	MtgToggleButton buttonMesureAffixe;
	MtgToggleButton buttonMesureProduitScalaire;
	MtgToggleButton buttonRideau;
	MtgToggleButton buttonCreerPointLibreEnt;
	MtgToggleButton buttonPunaiser;
	MtgToggleButton buttonDepunaiser;
	MtgToggleButton buttonCreerFonctionReelle;
	MtgToggleButton buttonCreerCourbeFonction;
	MtgToggleButton buttonCreerPointSurCourbeParAbscisse;
	// MtgToggleButton buttonCreerPointSurCourbeParPointLie;
	MtgToggleButton buttonCurseur;
	MtgToggleButton buttonTangente;
	MtgToggleButton buttonMesurerCoefficientDirecteur;
	MtgToggleButton buttonCreerPointParSommeVect;
	MtgToggleButton buttonCreerPointParMultVect;
	MtgToggleButton buttonCarreDirect;
	MtgToggleButton buttonTriangleEquilateralDirect;
	MtgToggleButton buttonParallelogramme;
	MtgToggleButton buttonPolygoneRegulier;
	MtgToggleButton buttonCreerPointInterieur;
	MtgToggleButton buttonHelpTexteFigure;
	MtgToggleButton buttonCreerEquationDroite;
	MtgToggleButton buttonCreerEquationDroiteLie;
	MtgToggleButton buttonCreerDroiteParEquation;
	MtgToggleButton buttonSegmentParLong;

	// Un groupe de boutons pour englober tous les boutons de la barre d'outil de
	// cration d'objet
	ButtonGroup btngroup2 = new ButtonGroup();
	// Les boutons de la barre d'outils du haut
	MtgButton buttonFichierOuvrir;
	MtgButton buttonFichierNouvelleFigure;
	// MtgButton buttonFichierEnregistrer; Supprim version Applet
	// MtgButton buttonEditionCopier; Supprim version Applet
	MtgButton buttonAnnuler;
	MtgButton buttonRefaire;
	MtgButton buttonZoomPlus;
	MtgButton buttonZoomMoins;
	MtgButton buttonReafficherTout;
	// MtgButton buttonFichierImprimer; Supprim version Applet
	public MtgToggleButton buttonModeTrace;
	MtgToggleButton buttonTranslationFigure;
	MtgToggleButton buttonPalette;
	MtgToggleButton buttonModifierObjetGraphique;
	MtgToggleButton buttonModifierFormule;
	MtgToggleButton buttonRelationEntreObjets;
	MtgToggleButton buttonCreerEditeurFormule;

	// Le style de point actif qui est dtermin quand on presse un
	// MtgStylePointButton
	public int stylePointActif;
	// Le style de flche actif qui est dtermin quand on presse un
	// MtgStyleFlecheButton
	public int styleFlecheActif;
	// Le style de marque de segment actif qui est dtermin quand on presse un
	// MtgStyleMarqueSegmentButton
	public int styleMarqueSegmentActif;
	// Le style de marque de segment actif qui est dtermin quand on presse un
	// MtgStyleMarqueAngleButton
	public int styleMarqueAngleActif;
	// Le style de remplissage actif
	public int styleRemplissageActif;

	// Une liste contenant les styles de trait
	MtgListeStyleTrait listeStyleTrait;
	// Un eliste contenant les styles de marque d'angle
	MtgPaneStyleMarqueAngle paneStyleMarqueAngle;

	// Dialogue servant  l'historique. Sera affich par l'outil outilHistorique
	public HistoriqueDlg historiqueDlg;
	// Spcial applet : Pour les figures de construction, une variable boolean qui
	// contiendra true
	// si les objets dsigns avec l'outil de validation outilPropSol sont bons
	public boolean constructionExacte;

	// Bouton de choix de la couleur active
	public JColorChooser colorChooser;
	public Outil outilActif;
	// public MtgDesktopPane mtgDesktopPane; Supprim version Applet
	public CListeObjets listeClignotante; // Liste pour faire clignoter
	public CListeObjets listeExclusionDesignation; // Liste des objets interdits
	// d'tre points  la souris
	public CListeObjets listeObjetsVisuels; // Liste des objets d'aide  la
	// cration visuelle
	// Les outils de pointage
	public OutilPointageCapt outilPointageCapture = new OutilPointageCapt(this);
	// Ajout version 4.7.2.6
	// Si les paramttres allowLetfToolbar et allowTopToolbar et allowMenu sont  false, c'est cet outil qui sera
	// l'outil par dfaut permettant de capturer un point mobile ou d'excuter une macro
	public OutilPointageCapturePlusExecMacro outilPointageCapturePlusExecMacro = new OutilPointageCapturePlusExecMacro(this);
	//
	public OutilPointageClic outilPointageClic = new OutilPointageClic(this);
	public OutilPointageCre outilPointageCre = new OutilPointageCre(this);
	public OutilPointageIntersection outilPointageIntersection = new OutilPointageIntersection(this);

	public OutilPointageSup outilPointageSup = new OutilPointageSup(this);
	public OutilPointagePourRideau outilPointagePourRideau = new OutilPointagePourRideau(this);
	public OutilPointageRapporteur outilPointageRapporteur = new OutilPointageRapporteur(this);
	public OutilPointageObjetClignotant outilPointageObjetClignotant = new OutilPointageObjetClignotant(
			this);
	public OutilPointagePropSol outilPointagePropSol = new OutilPointagePropSol(this);
	public OutilPointagePourNommer outilPointagePourNommer = new OutilPointagePourNommer(this);
	public OutilPointageDecalageFigure outilPointageDecalageFigure = new OutilPointageDecalageFigure(this);
	public OutilPointageModification outilPointageModification = new OutilPointageModification(this);
	public OutilPointageExecutionMacro outilPointageExecutionMacro = new OutilPointageExecutionMacro(
			this);
	public OutilPointageTraceObjet outilPointageTraceObjet = new OutilPointageTraceObjet(this);
	public OutilPointageCreationCurseur outilPointageCreationCurseur = new OutilPointageCreationCurseur(this);
	public OutilPointageCreationSegmentParLongueur outilPointageCreationSegmentParLongueur = new OutilPointageCreationSegmentParLongueur(this);
	// Nouveau version 2.2
	public OutilPointageHistorique outilPointageHistorique = new OutilPointageHistorique(
			this);
	// Pointeur sur l'outil de pointage actif qui est l'outil de capture au d?part
	public OutilPointage outilPointageActif = outilPointageCapture;

	public InfoProx infoProx = new InfoProx(); // Utilis pour savoir les objets
	// proches du pointeur souris
	// Outils de cration et modification
	public OutilAnnuler outilAnnuler = new OutilAnnuler(this);
	public OutilRefaire outilRefaire = new OutilRefaire(this);
	public OutilCapture outilCapture = new OutilCapture(this);
	public OutilTraceObjets outilTraceObjets = new OutilTraceObjets(this);
	public OutilPtLib outilCreerPointLibre = new OutilPtLib(this);
	public OutilMilieu outilMilieu = new OutilMilieu(this);
	public OutilDtAB outilDtAB = new OutilDtAB(this);
	public OutilDtPar outilDtPar = new OutilDtPar(this);
	public OutilDtPer outilDtPer = new OutilDtPer(this);
	public OutilSegment outilSeg = new OutilSegment(this);
	public OutilDemiDt outilDemiDt = new OutilDemiDt(this);
	public OutilCerOA outilCerOA = new OutilCerOA(this);
	public OutilDtBis outilDtBis = new OutilDtBis(this);
	public OutilDtMed outilDtMed = new OutilDtMed(this);
	public OutilInt outilInt = new OutilInt(this);
	public OutilTranslation outilTrans = new OutilTranslation(this);
	public OutilSymAxiale outilSymAxiale = new OutilSymAxiale(this);
	public OutilSymCentrale outilSymCentrale = new OutilSymCentrale(this);
	public OutilRapporteur outilRapporteur = new OutilRapporteur(this);
	public OutilRot outilRot = new OutilRot(this);
	public OutilCerOR outilCerOR = new OutilCerOR(this);
	public OutilSup outilSup = new OutilSup(this);
	public OutilPtLie outilPtLie = new OutilPtLie(this);
	public OutilPtParAbs outilPtParAbs = new OutilPtParAbs(this);
	public OutilHom outilHom = new OutilHom(this);
	public OutilMesLong outilMesLong = new OutilMesLong(this);
	public OutilMesAngNor outilMesAngnor = new OutilMesAngNor(this);
	public OutilMesAngOr outilMesAngOr = new OutilMesAngOr(this);
	public OutilMesAbs outilMesAbs = new OutilMesAbs(this);
	public OutilMarqueAngleNonOriente outilMarqueAngleNonOriente = new OutilMarqueAngleNonOriente(
			this);
	public OutilMarqueAngleOriente outilMarqueAngleOriente = new OutilMarqueAngleOriente(
			this);
	public OutilMarqueAngleSensDirect outilMarqueAngleSensDirect = new OutilMarqueAngleSensDirect(
			this);
	public OutilMarqueAngleSensIndirect outilMarqueAngleSensIndirect = new OutilMarqueAngleSensIndirect(
			this);
	public OutilTranslation outilCreerImageParTranslation = new OutilTranslation(
			this);
	public OutilTranslationParCoord outilCreerImageParTranslationParCoord = new OutilTranslationParCoord(
			this);
	public OutilSymAxiale outilCreerImageParSymetrieAxiale = new OutilSymAxiale(
			this);
	public OutilSymCentrale outilCreerImageParSymetrieCentrale = new OutilSymCentrale(
			this);
	public OutilHom outilCreerImageParHomothetie = new OutilHom(this);
	public OutilArcPetitParPointFinal outilCreerArcPetitParPointFinal = new OutilArcPetitParPointFinal(
			this);
	public OutilArcGrandParPointFinal outilCreerArcGrandParPointFinal = new OutilArcGrandParPointFinal(
			this);
	public OutilArcDirectParPointFinal outilCreerArcDirectParPointFinal = new OutilArcDirectParPointFinal(
			this);
	public OutilArcIndirectParPointFinal outilCreerArcIndirectParPointFinal = new OutilArcIndirectParPointFinal(
			this);
	public OutilArcPetitParAngle outilCreerArcPetitParAngle = new OutilArcPetitParAngle(
			this);
	public OutilArcGrandParAngle outilCreerArcGrandParAngle = new OutilArcGrandParAngle(
			this);
	public OutilArcDirectParAngle outilCreerArcDirectParAngle = new OutilArcDirectParAngle(
			this);
	public OutilArcIndirectParAngle outilCreerArcIndirectParAngle = new OutilArcIndirectParAngle(
			this);
	public OutilArcPetitParRapporteur outilArcPetitParRapporteur = new OutilArcPetitParRapporteur(
			this);
	public OutilArcGrandParRapporteur outilArcGrandParRapporteur = new OutilArcGrandParRapporteur(
			this);
	public OutilDroiteHorizontale outilDroiteHorizontale = new OutilDroiteHorizontale(
			this);
	public OutilDroiteVerticale outilDroiteVerticale = new OutilDroiteVerticale(
			this);
	public OutilMarqueSegment outilMarqueSegment = new OutilMarqueSegment(this);
	public OutilPolygone outilPolygone = new OutilPolygone(this);
	public OutilLigneBrisee outilLigneBrisee = new OutilLigneBrisee(this);
	public OutilSurface outilSurface = new OutilSurface(this);
	public OutilSurfaceLieuDroite outilSurfaceLieuDroite = new OutilSurfaceLieuDroite(
			this);
	public OutilSurfaceLieuDeuxPoints outilSurfaceLieuDeuxPoints = new OutilSurfaceLieuDeuxPoints(
			this);
	public OutilSurfaceDeuxLieux outilSurfaceDeuxLieux = new OutilSurfaceDeuxLieux(
			this);
	public OutilProjeteOrthogonal outilProjeteOrthogonal = new OutilProjeteOrthogonal(
			this);
	public OutilRepere outilRepere = new OutilRepere(this);
	public OutilPointParCoordonnees outilPointParCoordonnees = new OutilPointParCoordonnees(
			this);
	public OutilModifierObjetGraphique outilModifierObjetGraphique = new OutilModifierObjetGraphique(
			this);
	public OutilDroiteParCoef outilDroiteParCoef = new OutilDroiteParCoef(this);
	public OutilAffichageValeur outilAffichageValeur = new OutilAffichageValeur(
			this);
	public OutilModifierFormule outilModifierFormule = new OutilModifierFormule(
			this);
	public OutilFonctionReelle outilFonctionReelle = new OutilFonctionReelle(this);
	public OutilFonctionComplexe outilFonctionComplexe = new OutilFonctionComplexe(
			this);
	public OutilFonctionReelle2Var outilFonctionReelle2Var = new OutilFonctionReelle2Var(
			this);
	public OutilFonctionReelle3Var outilFonctionReelle3Var = new OutilFonctionReelle3Var(
			this);
	public OutilFonctionComplexe2Var outilFonctionComplexe2Var = new OutilFonctionComplexe2Var(
			this);
	public OutilFonctionComplexe3Var outilFonctionComplexe3Var = new OutilFonctionComplexe3Var(
			this);
	public OutilCentreGravite outilCentreGravite = new OutilCentreGravite(this);
	public OutilPtParAbs outilPointParAbscisse = new OutilPtParAbs(this);
	public OutilAffichageValeurLiePoint outilAffichageValeurLiePoint = new OutilAffichageValeurLiePoint(
			this);
	public OutilCommentaire outilCommentaire = new OutilCommentaire(this);
	public OutilCommentaireLiePoint outilCommentaireLiePoint = new OutilCommentaireLiePoint(
			this);
	public OutilLatexLiePoint outilLatexLiePoint = new OutilLatexLiePoint(this);
	public OutilVariable outilVariable = new OutilVariable(this);
	public OutilLieuPointParPointLie outilLieuPointParPointLie = new OutilLieuPointParPointLie(
			this);
	public OutilLieuParVariable outilLieuParVariable = new OutilLieuParVariable(
			this);
	public OutilLieuDiscretParPointLie outilLieuDiscretParPointLie = new OutilLieuDiscretParPointLie(
			this);
	public OutilLieuDiscretParVariable outilLieuDiscretParVariable = new OutilLieuDiscretParVariable(
			this);
	public OutilLieuObjetParPointLie outilLieuObjetParPointLie = new OutilLieuObjetParPointLie(
			this);
	public OutilLieuObjetParVariable outilLieuObjetParVariable = new OutilLieuObjetParVariable(
			this);
	public OutilSimilitude outilSimilitude = new OutilSimilitude(this);
	public OutilPalette outilPalette = new OutilPalette(this);
	public OutilBarycentre outilBarycentre = new OutilBarycentre(this);
	public OutilModifierBarycentre outilModifierBarycentre = new OutilModifierBarycentre(
			this);
	public OutilExecutionMacro outilExecutionMacro = new OutilExecutionMacro(this);
	public OutilGomme outilGomme = new OutilGomme(this);
	public OutilVecteur outilVecteur = new OutilVecteur(this);
	public OutilMesureAbscisseDansRepere outilMesureAbscisseDansRepere = new OutilMesureAbscisseDansRepere(
			this);
	public OutilMesureOrdonneeDansRepere outilMesureOrdonneeDansRepere = new OutilMesureOrdonneeDansRepere(
			this);
	public OutilMesureAffixe outilMesureAffixe = new OutilMesureAffixe(this);
	public OutilInversion outilInversion = new OutilInversion(this);
	public OutilDemiPlan outilDemiPlan = new OutilDemiPlan(this);
	public OutilDerivee outilDerivee = new OutilDerivee(this);
	public OutilSuiteRecurrenteReelle outilSuiteRecurrenteReelle = new OutilSuiteRecurrenteReelle(
			this);
	public OutilSuiteRecurrenteComplexe outilSuiteRecurrenteComplexe = new OutilSuiteRecurrenteComplexe(
			this);
	public OutilPointParAffixe outilPointParAffixe = new OutilPointParAffixe(this);
	public OutilGrapheSuiteRecurrenteReelle outilGrapheSuiteRecurrenteReelle = new OutilGrapheSuiteRecurrenteReelle(
			this);
	public OutilGrapheSuiteRecurrenteComplexe outilGrapheSuiteRecurrenteComplexe = new OutilGrapheSuiteRecurrenteComplexe(
			this);
	public OutilModifierRepere outilModifierRepere = new OutilModifierRepere(this);
	public OutilSolutionEquation outilSolutionEquation = new OutilSolutionEquation(
			this);
	public OutilMaximum outilMaximum = new OutilMaximum(this);
	public OutilMinimum outilMinimum = new OutilMinimum(this);
	public OutilIntegrale outilIntegrale = new OutilIntegrale(this);
	public OutilRideau outilRideau = new OutilRideau(this);
	public OutilTestExistence outilTestExistence = new OutilTestExistence(this);
	public OutilPartieReelle outilPartieReelle = new OutilPartieReelle(this);
	public OutilPartieImaginaire outilPartieImaginaire = new OutilPartieImaginaire(
			this);
	public OutilModule outilModule = new OutilModule(this);
	public OutilArgument outilArgument = new OutilArgument(this);
	public OutilSomme outilSomme = new OutilSomme(this);
	public OutilProduit outilProduit = new OutilProduit(this);
	public OutilMacroApparition outilMacroApparition = new OutilMacroApparition(
			this);
	public OutilMacroDisparition outilMacroDisparition = new OutilMacroDisparition(
			this);
	public OutilMacroAnimation outilMacroAnimation = new OutilMacroAnimation(this);
	public OutilMacroAnimationAvecTrace outilMacroAnimationAvecTrace = new OutilMacroAnimationAvecTrace(
			this);
	public OutilMacroAffectationValeurVariable outilMacroAffectationValeurVariable = new OutilMacroAffectationValeurVariable(
			this);
	// public OutilMacroJouantSon outilMacroJouantSon = new
	// OutilMacroJouantSon(this); Pas disponible version Applet
	public OutilImageLibre outilImageLibre = new OutilImageLibre(this);
	public OutilImageLieePoint outilImageLieePoint = new OutilImageLieePoint(this);
	public OutilSuppressionLiaison outilSuppressionLiaison = new OutilSuppressionLiaison(
			this);
	public OutilCreationLiaison outilCreationLiaison = new OutilCreationLiaison(
			this);
	public OutilSuppressionLiaisonAffichage outilSuppressionLiaisonAffichage = new OutilSuppressionLiaisonAffichage(
			this);
	public OutilCreationLiaisonAffichage outilCreationLiaisonAffichage = new OutilCreationLiaisonAffichage(
			this);
	public OutilReclassementFinObjetGraphique outilReclassementFinObjetGraphique = new OutilReclassementFinObjetGraphique(
			this);
	public OutilReclassementFinValeur outilReclassementFinValeur = new OutilReclassementFinValeur(
			this);
	public OutilReclassementDebutObjetGraphique outilReclassementDebutObjetGraphique = new OutilReclassementDebutObjetGraphique(
			this);
	public OutilReclassementDebutValeur outilReclassementDebutValeur = new OutilReclassementDebutValeur(
			this);
	public OutilPointLibreEnt outilPointLibreEnt = new OutilPointLibreEnt(this);
	public OutilPunaiserPointMobile outilPunaiserPointMobile = new OutilPunaiserPointMobile(
			this);
	public OutilDepunaiserPointMobile outilDepunaiserPointMobile = new OutilDepunaiserPointMobile(
			this);
	public OutilMacroModificationVariable outilMacroModificationVariable = new OutilMacroModificationVariable(
			this);
	public OutilMacroIncrementationVariable outilMacroIncrementationVariable = new OutilMacroIncrementationVariable(
			this);
	public OutilMacroDecrementationVariable outilMacroDecrementationVariable = new OutilMacroDecrementationVariable(
			this);
	public OutilMacroTraceAutoParPointLie outilMacroTraceAutoParPointLie = new OutilMacroTraceAutoParPointLie(
			this);
	public OutilMacroTraceAutoParVariable outilMacroTraceAutoParVariable = new OutilMacroTraceAutoParVariable(
			this);
	public OutilMacroAffectationPointLie outilMacroAffectationPointLie = new OutilMacroAffectationPointLie(
			this);
	public OutilMacroClignotement outilMacroClignotement = new OutilMacroClignotement(
			this);
	public OutilMacroAffichantTout outilMacroAffichantTout = new OutilMacroAffichantTout(
			this);
	public OutilMacroPause outilMacroPause = new OutilMacroPause(this);
	public OutilMacroBoucleAvecAnimation outilMacroBoucleAvecAnimation = new OutilMacroBoucleAvecAnimation(
			this);
	public OutilMacroBoucleAvecTrace outilMacroBoucleAvecTrace = new OutilMacroBoucleAvecTrace(
			this);
	public OutilMacroSuiteMacros outilMacroSuiteMacros = new OutilMacroSuiteMacros(
			this);
	public OutilObjetDuplique outilObjetDuplique = new OutilObjetDuplique(this);
	public OutilOptionsFigure outilOptionsFigure = new OutilOptionsFigure(this);
	public OutilOptionsPreferences outilOptionsPreferences = new OutilOptionsPreferences(
			this);
	public OutilMesureLongueurLigne outilMesureLongueurLigne = new OutilMesureLongueurLigne(
			this);
	public OutilMesureAirePolygone outilMesureAirePolygone = new OutilMesureAirePolygone(
			this);
	public OutilCourbeFonction outilcourbeFonction = new OutilCourbeFonction(this);
	public OutilReafficherTout outilReafficherTout = new OutilReafficherTout(this);
	public OutilCentreCercle outilCentreCercle = new OutilCentreCercle(this);
	public OutilConstructionImplementationFigure outilConstructionImplementationFigure = new OutilConstructionImplementationFigure(
			this);
	public OutilSuppressionValeur outilSuppressionValeur = new OutilSuppressionValeur(this);
	public OutilAjoutObjetMacro outilAjoutObjetMacro = new OutilAjoutObjetMacro(this);
	public OutilRetraitObjetsMacro outilRetraitObjetsMacro = new OutilRetraitObjetsMacro(this);
	public OutilModificationMacro outilModificationMacro = new OutilModificationMacro(this);
	public OutilMouvoirAnimationPointLie outilMouvoirAnimationPointLie = new OutilMouvoirAnimationPointLie(this);
	public OutilRenommerCalculDlg outilRenommerCalculDlg = new OutilRenommerCalculDlg(this);
	public OutilConstructionEnregistrer outilConstructionEnregistrer = new OutilConstructionEnregistrer(this);
	public OutilConstructionModifier outilModifierConstruction = new OutilConstructionModifier(this);
	public OutilCreerLongueurUnite outilCreerLongueurUnite = new OutilCreerLongueurUnite(this);
	/*
	 * Supprim version Applet public OutilConstructionChoixSourcesGraphiques
	 * outilConstructionChoixSourcesGraphiques = new
	 * OutilConstructionChoixSourcesGraphiques(this); public
	 * OutilConstructionChoixFinauxGraphiques
	 * outilConstructionChoixFinauxGraphiques = new
	 * OutilConstructionChoixFinauxGraphiques(this);
	 */
	public OutilConstructionImplementationFichier outilConstructionImplementationFichier = new OutilConstructionImplementationFichier(
			this);
	public OutilConstructionSupprimer outilConstructionSupprimer = new OutilConstructionSupprimer(this);
	public OutilCurseur outilCurseur = new OutilCurseur(this);
	public OutilPointSurCourbeParPointLie outilPointSurCourbeParPointLie = new OutilPointSurCourbeParPointLie(this);
	public OutilPointSurCourbeParAbscisse outilPointSurCourbeParAbscisse = new OutilPointSurCourbeParAbscisse(this);
	public OutilTangente outilTangente = new OutilTangente(this);
	public OutilMesureCoefficientDirecteur outilMesureCoefficientDirecteur = new OutilMesureCoefficientDirecteur(this);
	public OutilMacroApparitionAPartirAutreMacro outilMacroApparitionAPartirAutreMacro = new OutilMacroApparitionAPartirAutreMacro(this);
	public OutilMacroDisparitionAPartirAutreMacro outilMacroDisparitionAPartirAutreMacro = new OutilMacroDisparitionAPartirAutreMacro(this);
	public OutilMesureProduitScalaire outilMesureProduitScalaire = new OutilMesureProduitScalaire(this);
	public OutilPointParSommeVect outilPointParSommeVect = new OutilPointParSommeVect(this);
	public OutilPointParMultVect outilPointParMultVect = new OutilPointParMultVect(this);
	public OutilCarreDirect outilCarreDirect = new OutilCarreDirect(this);
	public OutilTriangleEquilateralDirect outilTriangleEquilateralDirect = new OutilTriangleEquilateralDirect(this);
	public OutilParallelogramme outilParallelogramme = new OutilParallelogramme(this);
	public OutilPolygoneRegulier outilPolygoneRegulier = new OutilPolygoneRegulier(this);
	public OutilPointInterieur outilPointInterieur = new OutilPointInterieur(this);
	public OutilCreationLiaisonInterieurPolygone outilCreationLiaisonInterieurPolygone = new OutilCreationLiaisonInterieurPolygone(this);
	public OutilCreationLiaisonInterieurCercle outilCreationLiaisonInterieurCercle = new OutilCreationLiaisonInterieurCercle(this);
	public OutilLatex outilLatex = new OutilLatex(this);
	public OutilRelationEntreObjets outilRelationEntreObjets = new OutilRelationEntreObjets(this);
	public OutilMacroModeTraceActivation outilMacroModeTraceActivation = new OutilMacroModeTraceActivation(this);
	public OutilMacroModeTraceDesactivation outilMacroModeTraceDesactivation = new OutilMacroModeTraceDesactivation(this);
	public OutilEquationDroite outilEquationDroite = new OutilEquationDroite(this);
	public OutilEquationDroiteLiePoint outilEquationDroiteLiePoint = new OutilEquationDroiteLiePoint(this);
	public OutilDroiteParEquation outilDroiteParEquation = new OutilDroiteParEquation(this);
	public OutilMarquerPointPourTrace outilMarquerPointPourTrace = new OutilMarquerPointPourTrace(this);
	public OutilDemarquerPointPourTrace outilDemarquerPointPourTrace = new OutilDemarquerPointPourTrace(this);
	public OutilAbscisseOrigineRep outilAbscisseOrigineRep = new OutilAbscisseOrigineRep(this);
	public OutilOrdonneeOrigineRep outilOrdonneeOrigineRep = new OutilOrdonneeOrigineRep(this);
	public OutilUnitexRep outilUnitexRep = new OutilUnitexRep(this);
	public OutilUniteyRep outilUniteyRep = new OutilUniteyRep(this);
	public OutilAjoutGraduationSimple outilAjoutGraduationSimple = new OutilAjoutGraduationSimple(this);
	public OutilAjoutGraduationTrigo outilAjoutGraduationTrigo = new OutilAjoutGraduationTrigo(this);
	public OutilEditeurFormule outilEditeurFormule = new OutilEditeurFormule(this); // Ajout
																																									// version
																																									// 4.5.2
	public OutilTestEquivalence outilTestEquivalence = new OutilTestEquivalence(this); // Ajout version version 4.6
	public OutilTestEqNatOp outilTestEqNatOp = new OutilTestEqNatOp(this); // Ajout version version 4.7.2
	public OutilDeriveePartielle outilDeriveePartielle = new OutilDeriveePartielle(this); // Ajout version version 4.6
	public OutilTestDependanceVariable outilTestDependanceVariable = new OutilTestDependanceVariable(this);
	public OutilSegmentParLong outilSegmentParLong = new OutilSegmentParLong(this); // Ajout version 4.6.3
	public OutilSuiteRecurrenteComplexe2 outilSuiteRecurrenteComplexe2 = new OutilSuiteRecurrenteComplexe2(this); // Ajout version 4.7.5
	public OutilSuiteRecurrenteReelle2 outilSuiteRecurrenteReelle2 = new OutilSuiteRecurrenteReelle2(this); // Ajout version 4.7.5
	public OutilSuiteRecurrenteComplexe3 outilSuiteRecurrenteComplexe3 = new OutilSuiteRecurrenteComplexe3(this); // Ajout version 4.7.5
	public OutilSuiteRecurrenteReelle3 outilSuiteRecurrenteReelle3 = new OutilSuiteRecurrenteReelle3(this); // Ajout version 4.7.5

	// Outil servant  faire apparatre le dialogue d'historique
	public OutilHistorique outilHistorique = new OutilHistorique(this);
	// Outil servant  nommer un point ou une droite
	public OutilEntrerNom outilModifierNom = new OutilEntrerNom(this);
	// Spcial applet : Outil servant  cliquer sur les objets qu'on a demand de
	// construire
	public OutilPropSol outilPropSol = new OutilPropSol(this);
	// Outil servant  translater la figure
	public OutilTranslationFigure outilTranslationFigure = new OutilTranslationFigure(this);
	// Panneaux servant dans le dialogue de choix de caractre spciaux pour les
	// commentaires
	public PaneInsertionCaractereGrec paneInsertionCaractereGrec = new PaneInsertionCaractereGrec();
	public PaneInsertionCaractereMath paneInsertionCaractereMath = new PaneInsertionCaractereMath();
	public PaneInsertionCaractereFleche paneInsertionCaractereFleche = new PaneInsertionCaractereFleche();
	// Panneaux servant dans le dialogue de choix de caractre spciaux pour les
	// affichages latex
	public PaneLatexInsertionCaractereGrec paneLatexInsertionCaractereGrec = new PaneLatexInsertionCaractereGrec();
	public PaneLatexInsertionCaractereMath paneLatexInsertionCaractereMath = new PaneLatexInsertionCaractereMath();
	public PaneLatexInsertionCaractereFleche paneLatexInsertionCaractereFleche = new PaneLatexInsertionCaractereFleche();

	public int instanceNouvelleFenetre() {
		instanceNouvelleFenetre++;
		return instanceNouvelleFenetre;
	}

	/**
	 * Constructeur
	 */
	public MtgFrame() throws IOException {
		setVisible(false);
		// On met le dlai des info-bulles  1/4 s
		ToolTipManager.sharedInstance().setInitialDelay(250);
		frameActif = this;
		topbar[0] = new JToolBar(JToolBar.HORIZONTAL);
		topbar[1] = new JToolBar(JToolBar.HORIZONTAL);
		if (nbInstances == 0) {
			appletManager = new AppletManager();
			// Pour viter un bug de nullPointerException rcursif :
			TeXFormula.registerFonts(false);
			// On cre une formule latex pour initialiser tous les fichiers associs
			// et on en profite pour crer un alias de la commande \jlmDynamic qui
			// sert aux insertions dynamiques
			/* Abandonn version 4.7.4
			formula1 = new TeXFormula(
					"\\newcommand{\\Val}[1]{\\jlmDynamic{\0Val\0#1}}");
			formula2 = new TeXFormula(
					"\\newcommand{\\For}[1]{\\jlmDynamic{\0For\0#1}}");// Ajout version
																															// 4.5.2
			formula3 = new TeXFormula(
					"\\newcommand{\\ForSimp}[1]{\\jlmDynamic{\0ForSimp\0#1}}");// Ajout
																																			// version
																																			// 4.6
			formula4 = new TeXFormula(
					"\\newcommand{\\Calc}[1]{\\jlmDynamic{\0Calc\0#1}}");// Ajout version
																																// 4.6
			formula5 = new TeXFormula(
					"\\newcommand{\\CalcSimp}[1]{\\jlmDynamic{\0CalcSimp\0#1}}");// Ajout
																																				// version
																																				// 4.6
			formula6 = new TeXFormula(
					"\\newcommand{\\FracRed}[1]{\\jlmDynamic{\0FracRed\0#1}}");
			formula7 = new TeXFormula(
					"\\newcommand{\\If}[3]{\\jlmDynamic{\0If\0#1\0#2\0#3}}");
			*/
		}
		nbInstances++;

	}

	private void initialiseBoutons() {
		btnsDispo = new MtgToggleButton[88];		
		btnsDispo[0] = buttonCreerPointLibre;
		btnsDispo[1] = buttonCreerPointLibreEnt;
		btnsDispo[2] = buttonCreerPointLie;
		btnsDispo[3] = buttonCreerMilieu;
		btnsDispo[4] = buttonCreerProjeteOrthogonal;
		btnsDispo[5] = buttonCreerPointParAbscisse;
		btnsDispo[6] = buttonCreerPointParCoordonnees;
		btnsDispo[7] = buttonCreerPointParAffixe;
		btnsDispo[8] = buttonCreerCentreGravite;
		btnsDispo[9] = buttonCreerPointParSommeVect;
		btnsDispo[10] = buttonCreerPointParMultVect;
		btnsDispo[11] = buttonCreerBarycentre;
		btnsDispo[12] = buttonCreerPointInterieur;
		btnsDispo[13] = buttonCurseur;
		btnsDispo[14] = buttonCreerIntersection;
		btnsDispo[15] = buttonCreerDroiteAB;
		btnsDispo[16] = buttonCreerDroiteParallele;
		btnsDispo[17] = buttonCreerDroitePerpendiculaire;
		btnsDispo[18] = buttonCreerDroiteBissectrice;
		btnsDispo[19] = buttonCreerDroiteMediatrice;
		btnsDispo[20] = buttonCreerDroiteHorizontale;
		btnsDispo[21] = buttonCreerDroiteVerticale;
		btnsDispo[22] = buttonCreerDroiteParCoef;
		btnsDispo[23] = buttonCreerDroiteParEquation;		
		btnsDispo[24] = buttonCreerSegment;
		btnsDispo[25] = buttonSegmentParLong;
		btnsDispo[26] = buttonCreerDemiDroite;
		btnsDispo[27] = buttonCreerVecteur;
		btnsDispo[28] = buttonCreerCercleOA;
		btnsDispo[29] = buttonCreerCercleOR;
		btnsDispo[30] = buttonCreerPolygone;
		btnsDispo[31] = buttonPolygoneRegulier;
		btnsDispo[32] = buttonParallelogramme;
		btnsDispo[33] = buttonCarreDirect;
		btnsDispo[34] = buttonTriangleEquilateralDirect;
		btnsDispo[35] = buttonCreerLigneBrisee;
		btnsDispo[36] = buttonCreerArcPetitParPointFinal;
		btnsDispo[37] = buttonCreerArcPetitParRapporteur;
		btnsDispo[38] = buttonCreerArcGrandParPointFinal;
		btnsDispo[39] = buttonCreerArcGrandParRapporteur;
		btnsDispo[40] = buttonCreerArcDirectParPointFinal;
		btnsDispo[41] = buttonCreerArcIndirectParPointFinal;
		btnsDispo[42] = buttonCreerMarqueAngleNonOriente;
		btnsDispo[43] = buttonCreerMarqueAngleOriente;
		btnsDispo[44] = buttonCreerMarqueSegment;
		btnsDispo[45] = buttonMesureLongueur;
		btnsDispo[46] = buttonMesureAbscisse;
		btnsDispo[47] = buttonMesureAbscisseDansRepere;
		btnsDispo[48] = buttonMesureOrdonneeDansRepere;
		btnsDispo[49] = buttonMesurerCoefficientDirecteur;
		btnsDispo[50] = buttonMesureLongueurLigne;
		btnsDispo[51] = buttonMesureAirePolygone;
		btnsDispo[52] = buttonMesureAffixe;
		btnsDispo[53] = buttonMesureAngleNonOriente;
		btnsDispo[54] = buttonMesureAngleOriente;
		btnsDispo[55] = buttonMesureProduitScalaire;
		btnsDispo[56] = buttonCreerImageParTranslation;
		btnsDispo[57] = buttonCreerImageParTranslationParCoord;
		btnsDispo[58] = buttonCreerImageParSymetrieAxiale;
		btnsDispo[59] = buttonCreerImageParSymetrieCentrale;
		btnsDispo[60] = buttonCreerImageParRotation;
		btnsDispo[61] = buttonCreerImageParHomothetie;
		btnsDispo[62] = buttonCreerImageParSimilitude;
		btnsDispo[63] = buttonRapporteur;
		btnsDispo[64] = buttonCreerAffichageValeur;
		btnsDispo[65] = buttonCreerAffichageValeurLiePoint;
		btnsDispo[66] = buttonCreerCommentaire;
		btnsDispo[67] = buttonCreerCommentaireLiePoint;
		btnsDispo[68] = buttonCreerLatex;
		btnsDispo[69] = buttonCreerLatexLiePoint;
		btnsDispo[70] = buttonCreerImage;
		btnsDispo[71] = buttonCreerImageLie;		
		btnsDispo[72] = buttonCreerEquationDroite;
		btnsDispo[73] = buttonCreerEquationDroiteLie;
		btnsDispo[74] = buttonCreerEditeurFormule;
		btnsDispo[75] = buttonCreerCalcul;
		btnsDispo[76] = buttonCreerRepere;		
		btnsDispo[77] = buttonCreerLieuParPointLie;
		btnsDispo[78] = buttonCreerFonctionReelle;
		btnsDispo[79] = buttonCreerCourbeFonction;
		btnsDispo[80] = buttonCreerPointSurCourbeParAbscisse;
		btnsDispo[81] = buttonTangente;
		btnsDispo[82] = buttonCreerSurface;
		btnsDispo[83] = buttonCreerSurfaceLieuDroite;
		btnsDispo[84] = buttonCreerSurfaceLieuDeuxPoints;
		btnsDispo[85] = buttonCreerSurfaceDeuxLieux;
		btnsDispo[86] = buttonPunaiser;
		btnsDispo[87] = buttonDepunaiser;		
	}

	@Override
	public void init() {
		super.init();
		isReady = false; // Ajout version 4.6.1. Sera mis  true dans paint de MtgFigurePanel lors du premier affichage et aprs appel  setScript().
		chargementEnCours = true;
		constructionExacte = false;
		loadToolsIcons();
		blankIcon = new ImageIcon(MtgFrame.class.getResource("images/BlankIcon24.gif"));
		URL adr = MtgFrame.class.getResource("images/CurseurCoin.gif");
		ImageIcon icon = new ImageIcon(adr);
		Image img = icon.getImage();
		cornerCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurMacro.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		macroCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(6, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurSup.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		supCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurModif.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		modifCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurGomme.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		gommeCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurPalette.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		paletteCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurCrayon.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		crayonCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 31), "");
		// Pour savoir si cette applet doit sauver les valeurs de res1, res2, ...,
		// res10
		// ou au contraire les restituer  var1, var2, ...., var10
		action = getParameter("Action");
		if (action == null)
			action = "";
		// Pour savoir quel est le type de figure initial
		initialFigure = getParameter("initialFigure");
		if (initialFigure == null)
			initialFigure = "unity";
		// Pour savoir si on autorise le bouton Solution dans le cas d'un exercice de construction
		String challowSolutionButton = getParameter("allowSolutionButton");
		if (challowSolutionButton == null) allowSolutionButton = true;
		else {
			if (challowSolutionButton.equalsIgnoreCase("false")) allowSolutionButton = false;
			else allowSolutionButton = true;
		}
		// Pour savoir si on recalcule les tirages alatoires lors du premier affichage
		String challowRandom = getParameter("allowRandom");
		if (challowRandom == null) allowRandom = true;
		else {
			if (challowRandom.equalsIgnoreCase("false")) allowRandom = false;
			else allowRandom = true;
		}
		// Ajout version 4.7.2. allowTools est true par dfaut. Si false, seuls les outils de capture, d'excution d'une macro,
		// d'affichages des objets numriques crs et l'outil historique peuvent re excuts.
		String challowTools = getParameter("allowTools");
		if (challowTools == null) allowTools = true;
		else {
			if (challowTools.equalsIgnoreCase("false")) allowTools = false;
			else allowTools = true;
		}
		
		// Ajout version 4.7.2. Si le paramtre numberOfInitialObjects est spcifi, il reprsente le nombre de caractres initaux de la figure
		// de faon  savoir quels sont les objets qui ont t construits
		// N'est pris en compte que si allowExercise est true
		String chNumberOfInitialObjects = getParameter("numberOfInitialObjects");
		if (chNumberOfInitialObjects == null) numberOfInitialObjects = -1;
		else {
			try { // Rajout version 4.7.3.1
				numberOfInitialObjects = Integer.parseInt(chNumberOfInitialObjects);
			}
			catch (Exception e) {
				numberOfInitialObjects = -1;
			}
		}
		
		// Si l'applet contient un paramtre ToolsChoice et si celui-si est false,
		// on ne propose pas de choisir les outils
		// dans les menus
		dataBase64 = getParameter("figureData");
		path = getParameter("path");
		String toolChoice = getParameter("allowToolsChoice");
		if (toolChoice == null)
			allowToolsChoice = true;
		else
			allowToolsChoice = !toolChoice.equalsIgnoreCase("false");
		// Si l'applet contient un paramtre allowFileMenu et si celui-si est false,
		// on ne propose pas de choisir les outils
		// dans les menus
		String fileMenu = getParameter("allowFileMenu");
		if (fileMenu == null)
			allowFileMenu = true;
		else
			allowFileMenu = !fileMenu.equalsIgnoreCase("false");
		String leftToolbar = getParameter("allowLeftToolbar");
		if (leftToolbar == null)
			allowLeftToolbar = true;
		else
			allowLeftToolbar = !leftToolbar.equalsIgnoreCase("false");
		String topToolbar = getParameter("allowTopToolbar");
		if (topToolbar == null)
			allowTopToolbar = true;
		else
			allowTopToolbar = !topToolbar.equalsIgnoreCase("false");
		String rightToolbar = getParameter("allowRightToolbar");
		if (rightToolbar == null)
			allowRightToolbar = true;
		else
			allowRightToolbar = !rightToolbar.equalsIgnoreCase("false");
		String opmenu = getParameter("allowMenuBar");
		if (opmenu == null)
			allowMenuBar = true;
		else
			allowMenuBar = !opmenu.equalsIgnoreCase("false");

		String optionsMenu = getParameter("allowOptionsMenu");
		// Si l'applet contient un paramtre allowFileMenu et si celui-si est false,
		// on ne propose pas de choisir les outils
		// dans les menus
		if (optionsMenu == null)
			allowOptionsMenu = true;
		else
			allowOptionsMenu = !optionsMenu.equalsIgnoreCase("false");

		String paramIndicationArea = getParameter("allowIndicationArea");
		if (paramIndicationArea == null)
			allowIndicationArea = true;
		else
			allowIndicationArea = !paramIndicationArea.equalsIgnoreCase("false");

		// Ajout version 4.5.1 / Un paramtre permet de savoir si on utilise ou non
		// le mode antialiasing
		String antialiasing = getParameter("antialiasing");
		if (antialiasing == null)
			pref_antialiasing = true;
		else
			pref_antialiasing = !antialiasing.equalsIgnoreCase("false");

		// Nouveau versio 4.6 : Un papamtre qui permet au javascript dans
		// l'ventuelle fonction mtgOnInit de la page
		// de connatre le nom de l'applet qui a appel mtgOninit
		mtgOnInitParam = getParameter("onInitParam");
		if (mtgOnInitParam == null)
			mtgOnInitParam = "";

		// Ajout version 4.6 / Un paramtre permet de savoir quel est le niveau de
		// fonctionnement
		String challowLevelChoice = getParameter("allowLevelChoice");
		if (challowLevelChoice == null)
			allowLevelChoice = true;
		else
			allowLevelChoice = !challowLevelChoice.equalsIgnoreCase("false");

		// Ajout version 4.6 / Un paramtre permet de savoir si on utilise ou non le
		// mode exercice
		String challowExercise = getParameter("allowExercise");
		if (challowExercise == null)
			allowExercise = false;
		else
			allowExercise = challowExercise.equalsIgnoreCase("true");
		// Ajout version 4.7.2.2 Sera false par dfaut. Si true, les diteurs de formule
		// pour lesquels on a demand un affichage en LaTeX et demand que la formule ne soit pas affiche au chargement verront
		// ces formules affiches nnamoins.		
		String chforceEditorsFormulaDisplay = getParameter("forceEditorsFormulaDisplay");
		if (chforceEditorsFormulaDisplay == null)
			forceEditorsFormulaDisplay = false;
		else
			forceEditorsFormulaDisplay = chforceEditorsFormulaDisplay.equalsIgnoreCase("true");
		
		String challowFileOpen = getParameter("allowFileOpen");
		if (challowFileOpen == null)
			allowFileOpen = true;
		else
			allowFileOpen = !challowFileOpen.equalsIgnoreCase("false");

		set_pref_language_Systeme(); // Ligne spciale version Applet.
		// On charge les prfrences
		chargePreferences();
		if (pref_LittleIconSize)
			indicationArea.setFont(Font.decode("Dialog-14"));
		else
			indicationArea.setFont(Font.decode("Dialog-16"));
		indicationArea.setForeground(Color.blue);
		labelNomsAuto = new JLabel(getString("labelNomsAuto"));
		labelNomsAuto.setFont(Font.decode("Dialog-11"));
		labelNomsAuto.setBorder(BorderFactory.createRaisedBevelBorder());
		labelAideVisuelle = new JLabel(getString("labelAideVisuelle"));
		labelAideVisuelle.setFont(Font.decode("Dialog-11"));
		labelAideVisuelle.setBorder(BorderFactory.createRaisedBevelBorder());
		labelUniteAngle = (pref_angleUnit == UniteAngle.Degre) ? new JLabel(
				getString("radioDegre")) : new JLabel(getString("radioRadian"));
		labelUniteAngle.setFont(Font.decode("Dialog-11"));
		labelUniteAngle.setBorder(BorderFactory.createRaisedBevelBorder());
		paneTransparence = new MtgPaneTransparence(this, transparenceParDefaut);
		paneCouleurs = new MtgPaneChoixCouleur(this);
		paneStylePoint = new MtgPaneStylePoint(this);
		paneStyleMarqueSegment = new MtgPaneStyleMarqueSegment(this);
		paneStyleFleche = new MtgPaneStyleFleche(this);
		paneStyleRemplissage = new MtgPaneStyleRemplissage(this);

		listeStyleTrait = new MtgListeStyleTrait(this);
		paneStyleMarqueAngle = new MtgPaneStyleMarqueAngle(this);
		historiqueDlg = new HistoriqueDlg(this);

		/*
		 * Abandonn car problmes avec les jTree avec JCheckBox try { String lf =
		 * UIManager.getSystemLookAndFeelClassName().toLowerCase(); if
		 * (lf.indexOf("windows") != -1 || lf.indexOf("apple") != -1) {
		 * UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); } }
		 * catch (Exception e) {} // Si pb, garder le look par defaut }
		 */
		// addWindowListener(this); Supprim version applet
		// Dfaut corrig version 4.6.1. Doit tre if (bInstances == 1)
		// if (nbInstances == 0) blankIcon = new ImageIcon(MtgFrame.class.getResource("images/BlankIcon24.gif")); // modifi version 4.6.1
		
		// Prparation des curseurs utilisateurs

		listeClignotante = new CListeObjets();
		listeObjetsVisuels = new CListeObjets();
		listeExclusionDesignation = new CListeObjets();
		indicationArea.setEditable(false);
		// On cre le dialogue de choix de couleurs
		colorChooser = new JColorChooser();
		colorChooser.setPreviewPanel(new JPanel()); // Pour qu'il n'y ait pas de
		// preview
		colorChooser.setColor(Color.black);
		Insets ins = new Insets(0, 0, 0, 0);
		topbar[0].setMargin(ins);
		topbar[0].setFloatable(false);
		topbar[1].setMargin(ins);
		topbar[1].setFloatable(false);

		// Cration des outils

		creeBoutonsEtActions();
		initialiseBoutons();
		// ajouteBoutonsBarreOutilsNiveauLycee();

		prepareBarreOutilsGauche(0);
		paneNorth.setLayout(new GridLayout(2,1));
		paneNorth.add(topbar[0]);
		paneNorth.add(topbar[1]);

		paneSouth.setLayout(new BorderLayout());
		paneSouth.add("Center", indicationArea);
		paneInfo.add(labelUniteAngle);
		paneInfo.add(labelNomsAuto);
		paneInfo.add(labelAideVisuelle);
		if (!pref_NomsAuto)
			labelNomsAuto.setVisible(false);
		if (!pref_AideVisuelle)
			labelAideVisuelle.setVisible(false);

		paneSouth.add("East", paneInfo);

		// paneRight.setBackground(Color.WHITE);
		paneRight.setLayout(new BorderLayout());

		stylePointActif = StylePoint.petitRond;

		styleFlecheActif = StyleFleche.FlecheLonguePleine;

		styleMarqueSegmentActif = StyleMarqueSegment.marqueSegment1;
		styleRemplissageActif = StyleRemplissage.QuadrillagePleinTransp;

		JScrollPane scrollPaneStyleTrait = new JScrollPane(listeStyleTrait,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneStyleTrait.setBorder(BorderFactory.createEtchedBorder());

		JScrollPane scrollPaneStyleMarqueAngle = new JScrollPane(
				paneStyleMarqueAngle, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneStyleMarqueAngle.setBorder(BorderFactory.createEtchedBorder());
		scrollPaneStyleMarqueAngle.getVerticalScrollBar().setUnitIncrement(
				paneStyleMarqueAngle.verticalIncrement());
		Dimension d = scrollPaneStyleMarqueAngle.getPreferredSize();
		d.height = paneStyleMarqueAngle.taillePourScrollPane().height;
		// d.width += 3;
		scrollPaneStyleMarqueAngle.setPreferredSize(d);
		scrollPaneStyleMarqueAngle.setBorder(BorderFactory.createEtchedBorder());

		JScrollPane scrollPaneStylePoint = new JScrollPane(paneStylePoint,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneStylePoint.setBorder(BorderFactory.createEtchedBorder());
		scrollPaneStylePoint.getVerticalScrollBar().setUnitIncrement(
				paneStylePoint.verticalIncrement());
		d = scrollPaneStylePoint.getPreferredSize();
		d.height = paneStylePoint.taillePourScrollPane().height;
		scrollPaneStylePoint.setPreferredSize(d);
		scrollPaneStylePoint.setBorder(BorderFactory.createEtchedBorder());

		JScrollPane scrollPaneStyleRemplissage = new JScrollPane(
				paneStyleRemplissage, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneStyleRemplissage.setBorder(BorderFactory.createEtchedBorder());
		scrollPaneStyleRemplissage.getVerticalScrollBar().setUnitIncrement(
				paneStyleRemplissage.verticalIncrement());
		d = paneStyleRemplissage.getPreferredSize();
		d.height = paneStyleRemplissage.taillePourScrollPane().height;
		scrollPaneStyleRemplissage.setPreferredSize(d);
		scrollPaneStyleRemplissage.setBorder(BorderFactory.createEtchedBorder());

		JScrollPane scrollPaneCouleur = new JScrollPane(paneCouleurs,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneCouleur.setBorder(BorderFactory.createEtchedBorder());
		scrollPaneCouleur.getVerticalScrollBar().setUnitIncrement(
				paneCouleurs.verticalIncrement());
		d = scrollPaneCouleur.getPreferredSize();
		d.height = paneCouleurs.taillePourScrollPane().height;
		scrollPaneCouleur.setPreferredSize(d);
		scrollPaneCouleur.setBorder(BorderFactory.createEtchedBorder());

		paneRightHaut = new JPanel();
		paneRightHaut.setLayout(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 0;
		constraints.insets.left = 0;
		constraints.insets.right = 0;

		MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 100, 15,
				GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneRightHaut.add(scrollPaneStyleTrait, constraints);
		MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 0, 15,
				GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneRightHaut.add(scrollPaneStylePoint, constraints);
		MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 0, 15,
				GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneRightHaut.add(paneStyleMarqueSegment, constraints);
		MtgDlg.setConstraints(constraints, 0, 3, 1, 1, 0, 15,
				GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneRightHaut.add(paneStyleFleche, constraints);
		MtgDlg.setConstraints(constraints, 0, 4, 1, 1, 100, 50,
				GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneRightHaut.add(scrollPaneStyleMarqueAngle, constraints);
		MtgDlg.setConstraints(constraints, 0, 5, 1, 1, 100, 15,
				GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneRightHaut.add(scrollPaneStyleRemplissage, constraints);
		MtgDlg.setConstraints(constraints, 0, 6, 1, 1, 100, 15,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(scrollPaneCouleur, constraints);
		MtgDlg.setConstraints(constraints, 0, 7, 1, 1, 100, 15,
				GridBagConstraints.NONE, GridBagConstraints.CENTER);
		paneRightHaut.add(paneTransparence, constraints);

		paneRight.add("North", paneRightHaut);

		getContentPane().setLayout(new BorderLayout());
		/*
		 * Modifi : on ajoute les barres et on les cache si false if
		 * (allowTopToolbar) getContentPane().add("North", paneNorth); if
		 * (allowLeftToolbar) getContentPane().add("West", paneLeft); if
		 * (allowIndicationArea) getContentPane().add("South", paneSouth); if
		 * (allowRightToolbar) getContentPane().add("East", paneRight);
		 */
		getContentPane().add("North", paneNorth);
		paneNorth.setVisible(allowTopToolbar);
		getContentPane().add("West", paneLeft);
		paneLeft.setVisible(allowLeftToolbar);
		getContentPane().add("South", paneSouth);
		paneSouth.setVisible(allowIndicationArea);
		getContentPane().add("East", paneRight);
		paneRight.setVisible(allowRightToolbar);

		historiqueDlg = new HistoriqueDlg(this);
		CMathGrafDoc doc = new CMathGrafDoc(this);
		cadreActif = new MtgInternalFrame(this, doc);
		cadreActif.setTitle("MathGraph"
				+ Integer.toString(instanceNouvelleFenetre()));
		// cadreActif.setDocument(doc);  // Dplac version 4.7.2 : Doit etre appel aprs chargement de la figrue
		// pour que par exemple la couleur de fonc soit la bonne
		// Priorit  la descriptionde la figure passe en paramtre chaine de
		// caractres base 64
		// sinon charge ventuellement une figure depuis l'URL de la page si le
		// paramtre path n'est pas nul
		if (dataBase64 != null)
			loadFigureFromDataBase64(dataBase64);
		else {
			if (path != null) loadFigureFromURL();
			else doc.listePrincipale.ajouteConstantePi(); // Ajout version 4.7.2
		}
		cadreActif.setDocument(doc);
		doc.listePrincipale.setCopy(cadreActif.listeSauve);
		if (allowExercise && cadreActif.isConstruction) {
			cadreActif.listeAConstruire = cadreActif.macroPourCons.listeAssociee;
			chaineIndicationPourCons = cadreActif.macroPourCons.commentaireMacro;
			ajouteBoutonPropSol();
		}
		// if (cadreActif.isConstruction || cadreActif.isExercise)
		// etablitListeCalculsAutorises(); // Pour permettre d'utiliser certains
		// calculs Modifi version 4.6
		// Modifi version 4.6.1
		// Puis modifi version 4.7.2
		/*
		if (allowExercise) {
			if (cadreActif.isConstruction)
				etablitListeObjetsConstruits(); // Appelle etablitListeCalculsAutorises
			else {
				if (cadreActif.isExercise)
					etablitListeCalculsAutorises(); // Modifi version 4.7
			}
		}
		*/
		if (allowExercise) {
			etablitListeObjetsConstruits(); // Appelle etablitListeCalculsAutorises
		}

		// Modification version 4.4.6 : un paramtre permet de choisir le niveau de
		// fonctionnement au dpart.
		// Il peut ensuite tre chang via le menu Otpions - Prfrences.
		// Le niveau est 1 pour le niveau le plus faible, contrairement 
		// pref_niveau (invers)
		/* Modifi version 4.7.2
		if (allowExercise && cadreActif.isConstruction) {
			pref_niveau = 1;
		} else {
			String level = getParameter("level");
			if (level == null)
				pref_niveau = 3;
			else {
				if (level.equals("1"))
					pref_niveau = 4;
				else if (level.equals("2"))
					pref_niveau = 3;
				else if (level.equals("3"))
					pref_niveau = 2;
				else if (level.equals("4"))
					pref_niveau = 1;
				else
					pref_niveau = 3; // Niveau collge par dfaut
			}
			*/
		String level = getParameter("level");
		if (level == null)
			pref_niveau = 3;
		else {
			if (level.equals("1"))
				pref_niveau = 4;
			else if (level.equals("2"))
				pref_niveau = 3;
			else if (level.equals("3"))
				pref_niveau = 2;
			else if (level.equals("4"))
				pref_niveau = 1;
			else
				pref_niveau = 3; // Niveau collge par dfaut
		}
		creeMenu(); // Pas de menu si on a un exercice de validation interne
		// Une fois l'applet entirement charge, et si son paramtre Action
		// contenait "Restitution", on affecte le contenu des calculs res1, res2,
		// ..., res10
		// de la figure prcdemment sauvegardes via l'AppletManger
		// aux variables var1, var2, ..., var10 de faon  pouvoir "reconstituer"
		paneTransparence.donneValeurTransparence(doc.transparencePourSurfaces,
				cadreActif()); // Ajout version 4.6
		// la figure de l'applet prcdente
		if (action.equals("Restitution")) {
			appletManager.restitueValeurs(this);
		}
		// setVisible(true); Modifi 12-5-2012
		// cadreActif.initialiseListes();
		// metAJourIconesEtMenus(doc); Se fait dans paint de MtgInternalFrame
		// mtgDesktopPane.add(cadre); Supprim version Applet
		getContentPane().add("Center", cadreActif);
		premierAffichage = true;
		cadreActif.creePaneAffichageVariables();
		// Ajout version 4.6. On indique dans la ligne d'indication l'unit d'angle
		// de la figure
		labelUniteAngle
				.setText((cadreActif.listePr.uniteAngle == UniteAngle.Degre) ? getString("radioDegre")
						: getString("radioRadian"));

		setVisible(true); // Modifi 12-5-2012

		// On regarde si la page web qui contient l'applet contient une fonction
		// nomme mtgOnInit()
		// Si elle est prsente on l'excute.
		// En gnral cette fonction devra appeler setXML(chaine) o chaine est une
		// chaine de caractres contenant un document
		// Et c'est start qui demandera un repaint();
		// Nouveau version 4.6 : un paramtre est pass qui est en fait le nom de
		// l'applet choisi par le paramtre onInitParam
		try {
			JSObject jso = JSObject.getWindow(this);
			Object[] arg = new Object[1];
			arg[0] = mtgOnInitParam;
			jso.call("mtgOnInit", arg);
		} catch (Exception e) {
			// TODO: handle exception
		}
		chargementEnCours = false;
		// start(); // Supprim version 4.6.2
	}

	@Override
	public void start() {
		// TODO Auto-generated method stub
		super.start();
		MtgInternalFrame cadre = cadreActif();
		cadre.timerTask = new MathGraphTimerTask(cadre);
		if (cadre.macroEnCours != null) {
			if (cadre.macroEnCours.executionEnCours) {
				// macroEnCours.macroEnCours().initialiseInterruptions(this);
				cadre.macroEnCours.macroEnCours().termineAction(cadre);
			}
		}
		// repaint(); // Supprim version 4.6
	}

	@Override
	public void stop() {
		// TODO Auto-generated method stub
		outilActif.deselect();
		MtgInternalFrame cadre = cadreActif();
		/*
		 * if (cadre.timer != null) { cadre.timer.cancel(); cadre.timer = null; }
		 * cadre.timerTask = null;
		 */
		// Lignes suivantes ajoutes version 4.6.2
		if (cadre.macroEnCours != null) {
			if (cadre.macroEnCours.executionEnCours) {
				// macroEnCours.macroEnCours().initialiseInterruptions(this);
				cadre.macroEnCours.macroEnCours().termineAction(cadre);
			}
		}
		if (cadre.timer != null) {
			cadre.timer.cancel();
			cadre.timer.purge();
			cadre.timer = null;
		}
		if (cadre.timerTask != null) {
			cadre.timerTask.cancel();
			cadre.timerTask = null;
		}
		// Fin ajout 4.6.2
		cadre.runner = null;
		cadre.macroEnCours = null;
		super.stop();
		System.gc(); // Rajout version 4.6.2
	}
	
	@Override
	public void destroy() {
		// Inutile d'appeler super qui est vide
		cadreActif.flushImageBuffers();
		cadreActif.gestionnaire.cleanUp();
		cadreActif.cleanUp();
		cadreActif = null;
		// Une fois tout nettoy on appelle le garbage collector
		listeClignotante.retireTout();
		listeClignotante = null;
		listeExclusionDesignation.retireTout();
		listeExclusionDesignation = null;
		listeObjetsVisuels.retireTout();
		listeObjetsVisuels = null;
		// Destruction des icnes des outils
		iconCreerPointLibre = null;
		iconCreerPointLibreEnt = null;
		iconCreerPointLie = null;
		iconCreerMilieu = null;
		iconCreerProjeteOrthogonal = null;
		iconCreerPointParAbscisse = null;
		iconCreerPointParCoordonnees = null;
		iconCreerPointParAffixe = null;
		iconCreerCentreGravite = null;
		iconCreerPointParSommeVect = null;
		iconCreerPointParMultVect = null;
		iconCreerBarycentre = null;
		iconCreerPointInterieur = null;
		iconCurseur = null;
		iconCreerIntersection = null;
		iconCreerDroiteAB = null;
		iconCreerDroiteParallele = null;
		iconCreerDroitePerpendiculaire = null;
		iconCreerDroiteBissectrice = null;
		iconCreerDroiteMediatrice = null;
		iconCreerDroiteHorizontale = null;
		iconCreerDroiteVerticale = null;
		iconCreerDroiteParCoef = null;
		iconCreerDroiteParEquation = null;
		iconCreerSegment = null;
		iconCreerDemiDroite = null;
		iconCreerVecteur = null;
		iconCreerCercleOA = null;
		iconCreerCercleOR = null;
		iconCreerPolygone = null;
		iconPolygoneRegulier = null;
		iconParallelogramme = null;
		iconCarreDirect = null;
		iconTriangleEquilateralDirect = null;
		iconCreerLigneBrisee = null;
		iconCreerArcPetitParPointFinal = null;
		iconCreerArcPetitParRapporteur = null;
		iconCreerArcGrandParPointFinal = null;
		iconCreerArcGrandParRapporteur = null;
		iconCreerArcDirectParPointFinal = null;
		iconCreerArcIndirectParPointFinal = null;
		iconCreerMarqueAngleNonOriente = null;
		iconCreerMarqueAngleOriente = null;
		iconCreerMarqueSegment = null;
		iconMesureLongueur = null;
		iconMesureAbscisse = null;
		iconMesureAbscisseDansRepere = null;
		iconMesureOrdonneeDansRepere = null;
		iconMesurerCoefficientDirecteur = null;
		iconMesureLongueurLigne = null;
		iconMesureAirePolygone = null;
		iconMesureAffixe = null;
		iconMesureAngleNonOriente = null;
		iconMesureAngleOriente = null;
		iconMesureProduitScalaire = null;
		iconCreerImageParTranslation = null;
		iconCreerImageParTranslationParCoord = null;
		iconCreerImageParSymetrieAxiale = null;
		iconCreerImageParSymetrieCentrale = null;
		iconCreerImageParRotation = null;
		iconCreerImageParHomothetie = null;
		iconCreerImageParSimilitude = null;
		iconRapporteur = null;
		iconCreerAffichageValeur = null;
		iconCreerAffichageValeurLiePoint = null;
		iconCreerCommentaire = null;
		iconCreerCommentaireLiePoint = null;
		iconCreerLatex = null;
		iconCreerLatexLiePoint = null;
		iconCreerImage = null;
		iconCreerImageLie = null;
		iconCreerEquationDroite = null;
		iconCreerEquationDroiteLie = null;
		iconCreerEditeurFormule = null;
		iconCreerCalcul = null;
		iconCreerRepere = null;
		iconCreerLieuParPointLie = null;
		iconCreerFonctionReelle = null;
		iconCreerCourbeFonction = null;
		iconCreerPointSurCourbeParAbscisse = null;
		iconTangente = null;
		iconCreerSurface = null;
		iconCreerSurfaceLieuDroite = null;
		iconCreerSurfaceLieuDeuxPoints = null;
		iconCreerSurfaceDeuxLieux = null;
		iconPunaiser = null;
		iconDepunaiser = null;

		System.gc();
	}

	/*
	 * Supprim version Applet public int nombreCadresOuverts() { JInternalFrame[]
	 * tableCadres = mtgDesktopPane.getAllFrames(); return tableCadres.length; }
	 */

	/*
	 * Supprim version Applet public void windowActivated(WindowEvent e) { // On
	 * tablit la variable globale frameActif frameActif = this; }
	 */

	/*
	 * Supprim version Applet public void windowClosed(WindowEvent e) {
	 * System.exit(0); }
	 */

	/*
	 * Supprim version Applet public void windowClosing(WindowEvent e) {
	 * setDefaultCloseOperation(EXIT_ON_CLOSE); JInternalFrame[] table =
	 * mtgDesktopPane.getAllFrames(); for (int i = 0; i < table.length; i++) {
	 * MtgInternalFrame cadre = (MtgInternalFrame) table[i]; if
	 * (cadre.document.isDirty()) { int rep =
	 * dialogueConfirmation_YES_NO_CANCEL(textes.Chaines.ch119 + " " +
	 * cadre.nomDocument()); if (rep == JOptionPane.CANCEL_OPTION) {
	 * setDefaultCloseOperation(DO_NOTHING_ON_CLOSE); return; } if (rep ==
	 * JOptionPane.YES_OPTION) if (!((ActionFichierEnregistrer)
	 * cadre.frame.actionFichierEnregistrer) .enregistreFigure()) {
	 * setDefaultCloseOperation(DO_NOTHING_ON_CLOSE); return; } } }
	 * outilActif.deselect(); // On sauve les prfrences de l'application
	 * sauvePreferences(); }
	 */

	/*
	 * Supprim version Applet public void windowDeactivated(WindowEvent e) { }
	 */

	/*
	 * Supprim version Applet public void windowDeiconified(WindowEvent e) { }
	 */

	/*
	 * Supprim version Applet public void windowIconified(WindowEvent e) {
	 * activeOutilCapture(); }
	 */

	/*
	 * Supprim version Applet public void windowOpened(WindowEvent e) { }
	 */

	/*
	 * Supprim version Applet public void exit() { JInternalFrame[] table =
	 * mtgDesktopPane.getAllFrames(); for (int i = 0; i < table.length; i++) {
	 * MtgInternalFrame cadre = (MtgInternalFrame) table[i]; if
	 * (cadre.document.isDirty()) { int rep =
	 * dialogueConfirmation_YES_NO_CANCEL(textes.Chaines.ch119 + " " +
	 * cadre.nomDocument()); if (rep == JOptionPane.CANCEL_OPTION) { return; } if
	 * (rep == JOptionPane.YES_OPTION) if (!((ActionFichierEnregistrer)
	 * cadre.frame.actionFichierEnregistrer).enregistreFigure()) return; } } // On
	 * sauve les prfrences de l'application sauvePreferences(); dispose();
	 * System.exit(0); }
	 */

	/**
	 * Pointeur sur le cadre en cours d'activit
	 * 
	 * @return
	 */
	public final MtgInternalFrame cadreActif() {
		// return (MtgInternalFrame) mtgDesktopPane.getSelectedFrame(); Modifi
		// version Applet
		return cadreActif;
	}

	/**
	 * Activation de la fentre dont le titre est titre
	 * 
	 * @param titre
	 */
	/*
	 * Supprim version Applet public void activeCadre(MtgInternalFrame cadre) {
	 * mtgDesktopPane.getDesktopManager().activateFrame(cadre); try {
	 * cadre.setSelected(true); } catch (PropertyVetoException e) { }
	 * metAJourIconesEtMenus(cadre.document); //
	 * cadre.montreDialogueAffichageVariables(); }
	 */

	/**
	 * Renvoie la couleur active qui est donne par le bouton colorButton
	 * 
	 * @return
	 */
	public final Color couleurActive() {
		return paneTransparence.couleurActive();
	}

	/**
	 * Redessine la figure sans la recalculer via paint de MtgFigurePanel
	 */
	public final void repaintFigure() {
		cadreActif().repaintFigureSansCalcul();
	}

	/**
	 * Spcial version applet Fonction rajoutant un objet  la liste de la fentre
	 * active Il faut pour cela mettre  zro la liste clone et la cloner 
	 * nouveau
	 */
	public final void ajouteElement(CElementBase pt) {
		MtgInternalFrame cadre = cadreActif();
		// boolean b = allowExercise && cadreActif.isConstruction); // Rectifi version 4.6.1
		// Modifi version 4.7.2
		/*
		boolean b = allowExercise && (cadreActif.isConstruction || cadreActif.isExercise); // Rectifi version 4.6.1
		*/
		boolean b = allowExercise;
		if (pt.getNatureCalcul() == NatureCalcul.NVariable) {
			if (b) {
				cadre.listeConstruction.add(pt);
				cadre.listePr.add(pt);
			} else
				cadre.listePr.add(1, pt);
		} else {
			cadre.listePr.add(pt);
			if (b)
				cadre.listeConstruction.add(pt);
		}
	}

	/**
	 * Fonction ajoutant l'lment pt avant l'lment d'indice indice et dcale pt
	 * et les suivants
	 * 
	 * @param pt
	 * @param indice
	 */
	public final void insereElement(CElementBase pt, int indice) {
		MtgInternalFrame cadre = cadreActif();
		CListeObjets listePr = cadre.listePr;
		cadre.listePr.add(indice, pt);
		// if (allowExercise && cadreActif.isConstruction) { // Modifi version 4.6.1
		/* Modifi version 4.7.2
		if (allowExercise && (cadreActif.isConstruction || cadreActif.isExercise)) {
		*/
		if (allowExercise) { 
			int indice1 = listePr.indexOf(listePr.get(indice));
			cadre.listeConstruction.add(indice1, pt);
		}
	}

	public final void remplaceElement(CElementBase ancien, CElementBase nouveau) {
		MtgInternalFrame cadre = cadreActif();
		CListeObjets liste = cadre.listePr;
		int ind = liste.indexOf(ancien);
		liste.set(ind, nouveau);
		liste.positionne(cadre.infRand, cadre.dimf);
		// Ajout version 4.6
		// if (allowExercise && cadreActif.isConstruction) // Modifi version 4.6.1
		/* Modifi version 4.7.2
		if (allowExercise && (cadreActif.isConstruction || cadreActif.isExercise))
		*/
		if (allowExercise) etablitListeObjetsConstruits();
	}

	/**
	 * Fonction rajoutant un objet  la liste principale de la fentre active Il
	 * faut pour cela mettre  zro la liste clone et la cloner  nouveau Renvoie
	 * true si l'lment n'a pas dj t cr N'est pas utilis dans le cas
	 * d'image par une transformation car dans ce cas il y a des lments qui ont
	 * t ventuellement ajouts et  dtruire Modifi version 3.9 : Vrifie que
	 * l'lment existe. Sinon demande confirmation de la cration Spcial version
	 * applet
	 */
	public final boolean ajouteElementAvecVerification(CElementBase pt) {
		// if (!cadreActif().listePr.existeDeja(pt)) {
		if (!listePourConstruction().existeDeja(pt)) {
			if (pt.estDeNature(NatureObjet.NToutObjet)) {
				if (pt.antecedentDirect() != pt)
					pt.antecedentDirect().positionne(false, cadreActif().dimf);
				pt.positionne(false, cadreActif().dimf);
				if (pt.existe) {
					if (!((CElementGraphique) pt).dansFenetre(cadreActif().dimf)) {
						if (!dialogueConfirmation_YES_NO(getString("ch94")))
							return false;
					}
				} else {
					if (!dialogueConfirmation_YES_NO(getString("ch85"))) {
						return false;
					}
				}
			}
			ajouteElement(pt);
			positionne();
			return true;
		} else {
			dialogueAvertissement(getString("ch7"));
			return false;
		}
	}

	/**
	 * Fonction rajoutant un objet image par transformation  la liste principale
	 * de la fentre active Renvoie true si l'lment n'a pas dj t cr
	 * 
	 * @param nbEltsAjoutes
	 *          reprsente le nombre d'lments qui ont t ventuellement *
	 *          ajouts
	 * @param trans
	 *          est la transformation lors de la cration de l'objet (par exemple
	 *          pour un segment image les points images des extrmits) Spcial
	 *          version applet
	 */
	public final boolean ajouteElementsImageAvecVerification(CElementBase pt,
			int nbEltsAjoutes, CTransformation trans) {
		MtgInternalFrame cadre = cadreActif();
		// CListeObjets liste = cadre.listePr;
		CListeObjets liste = listePourConstruction();
		DimensionFenetre dimfen = cadre.dimf;
		if (!liste.existeDeja(pt)) {
			// On commence par positionner la tansformation qui est le premier des
			// objets ajouts  la liste
			// avant les objets ajouts ventueks
			trans.positionne(false, dimfen);
			pt.antecedentDirect().positionne(false, dimfen);
			// Il faut positionner les ventuels lments ajouts
			for (int i = nbEltsAjoutes; i >= 1; i--) {
				CElementBase el = liste.get(liste.size() - i);
				el.positionne(false, dimfen);
			}
			// On positionne aussi l'image  rajouter
			pt.positionne(false, dimfen);
			if (!((CElementGraphique) pt).dansFenetre(dimfen)) {
				if (!dialogueConfirmation_YES_NO(getString("ch94")))
					return false;
			}
			ajouteElement(pt);
			positionne();
			return true;
		} else {
			if (nbEltsAjoutes != 0)
				detruitNDerniersElements(nbEltsAjoutes);
			dialogueAvertissement(getString("ch7"));
			return false;
		}
	}

	/**
	 * Fonction dtruisant le dernier objet de la liste du cadre actif et de sa
	 * liste clone listeClone
	 */
	public final void detruitDernierElement() {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.detruitDernierObjet();
		// if (allowExercise && cadreActif.isConstruction) // Modifi version 4.6.1
		/* Modifi version 4.7.1
		if (allowExercise && (cadreActif.isConstruction || cadre.isExercise))
		*/
		if (allowExercise)
			cadre.listeConstruction.detruitDernierObjet();
	}

	/**
	 * Fonction dtruisant les n derniers objets de la liste du cadre actif et de
	 * sa liste clone listeClone
	 */
	public final void detruitNDerniersElements(int n) {
		for (int i = 0; i < n; i++) {
			detruitDernierElement();
		}
	}

	/**
	 * Retrait des listes des objets dpendant de pt Ces lments pourraont
	 * ensuite tre dtruits par le garbage collector
	 * 
	 * @param pt
	 */
	public final void retireDependants(CElementBase pt) {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.retireDependants(pt);
		etablitListeObjetsConstruits();
		cadre.videListesPourConstructions();
	}

	public int numberOfInitialObjects() {
		return cadreActif().listeSauve.size();
	}
	
	/**
	 * Ajout version 4.7.2.6 pour LaboMep
	 * Fonction renvoyant le nombre total d'objets crs
	 * @return
	 */
	public int numberOfCreatedObjects() {
		return cadreActif().listePr.size();
	}

	public void etablitListeObjetsConstruits() {
		// int nbo = numberOfInitialObjects(); // Modifi version 4.7.2
		int nbo;
		if (numberOfInitialObjects != -1) nbo = numberOfInitialObjects;
		else nbo = numberOfInitialObjects();
		CListeObjets listePr = cadreActif.listePr;
		CListeObjets listeCalculsAutorises = cadreActif.listeCalculsAutorises;
		CListeObjets listeConstruction = cadreActif.listeConstruction;
		cadreActif.listeConstruction.retireTout();
		// On ajoute  la liste des objets construits le calcul constant pi de la
		// liste principale
		// de faon  pouvoir l'utiliser dans les calculs
		listeConstruction.add(listePr.get(0));
		// Nouveau version 4.6 : On ajoute les lments de type calcul qui sont
		// autoriss via le commentaire de la macro exercice
		if (cadreActif.isExercise) {
			etablitListeCalculsAutorises();
			for (int i = 0; i < listeCalculsAutorises.size(); i++) {
				listeConstruction.add(listeCalculsAutorises.get(i));
			}
		}
		for (int i = nbo; i < listePr.size(); i++) {
			listeConstruction.add(listePr.get(i));
		}
	}

	/**
	 * Fonction retirant de la liste les N derniers lments
	 * 
	 * @param N
	 */
	public final void retireNDerniersElements(int N) {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.retireNDerniersElements(N);
		cadre.videListesPourConstructions();
	}

	/** Fonction rajoutant un objet  la liste des objets visuels */
	public final void ajouteObjetVisuel(CElementBase pt) {
		listeObjetsVisuels.add(pt);
	}

	/**
	 * Positionnement de la liste du cadre actif
	 */
	public final void positionne() {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.positionne(cadre.infRand, cadre.dimf);
	}

	/** Fonction positionnat la liste pour la cration visuelle d'objets */
	public final void positionneListeVisuelle() {
		listeObjetsVisuels.positionne(false, cadreActif().dimf);
	}

	public final void indication(String ch) {
		indicationArea.setText(ch);
		indicationArea.repaint();
	}

	/**
	 * Activation de l'outil de capture
	 */
	public final void activeOutilCapture() {
		if (outilActif != null)
			outilActif.deselect();
			outilActif = outilCapture;
			buttonCapture.setSelected(true);
			MtgInternalFrame cadre = cadreActif();
			if (cadre != null) {
				outilActif.select();
				cadre.requestFocus();
			}
	}

	/**
	 * Activation de l'outil de capture
	 */
	public final void activeOutilExecutionMacro() { // Spcial version Applet
		if (outilActif != null)
			outilActif.deselect();
		outilActif = outilExecutionMacro;
		buttonExecuterMacro.setSelected(true);
		outilActif.select();
		MtgInternalFrame cadre = cadreActif();
		if (cadre != null) {
			outilActif.select();
			cadre.requestFocus();
		}
	}

	// Pour les quatre fonctions suivantes, il faut dans toutes la mesure du
	// possible les appeler
	// plutt que d'appeler les mthodes de mme nom dans MTgDlg car alors les
	// messages sont
	// centrs par rapport  la fentre de l'application et non par rapport 
	// l'cran
	/** Fonction affichant un dialogue d'avertissement */
	public final void dialogueAvertissement(String texte) {
		JOptionPane.showMessageDialog(this, texte, getString("avertissement"),
				JOptionPane.WARNING_MESSAGE);
	}

	public final void dialogueAvertissement(Component comp, String texte) {
		JOptionPane.showMessageDialog(comp, texte, getString("avertissement"),
				JOptionPane.WARNING_MESSAGE);
	}

	/** Fonction affichant un dialogue d'information */
	public final void dialogueInformation(String texte) {
		// Modifi version 4.7.2 pour tre affich au centre de l'cran
		/*
		JOptionPane.showMessageDialog(getParentFrame(), texte,
				getString("information"), JOptionPane.WARNING_MESSAGE);
		*/
		JOptionPane.showMessageDialog(getParentFrame(), texte,
				getString("information"), JOptionPane.WARNING_MESSAGE);
	}

	/**
	 * Fonction affichant un dialogue demandant une rponse oui-non et renvoyant
	 * true si la rponse est oui
	 */
	public final boolean dialogueConfirmation_YES_NO(String texte) {
		// Modifi version 4.7.2 pour tre affich au centre de l'cran
		/*
		return (JOptionPane.showConfirmDialog(getParentFrame(), texte,
				getString("confirmation"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
		 */
		return (JOptionPane.showConfirmDialog(null, texte,
				getString("confirmation"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
	}

	public final int dialogueConfirmation_YES_NO_CANCEL(String texte) {
		// Modifi version 4.7.2 pour tre affich au centre de l'cran
		/*
		return JOptionPane.showConfirmDialog(getParentFrame(), texte,
				getString("confirmation"), JOptionPane.YES_NO_CANCEL_OPTION);
		*/
		return JOptionPane.showConfirmDialog(null, texte,
				getString("confirmation"), JOptionPane.YES_NO_CANCEL_OPTION);

	}

	public final void creeBoutonsEtActions() {

		actionAnnuler = new ActionAnnuler(this);
		buttonAnnuler = new MtgButton(actionAnnuler);

		actionRefaire = new ActionRefaire(this);
		buttonRefaire = new MtgButton(actionRefaire);

		actionFichierOuvrir = new ActionFichierOuvrir(this);
		buttonFichierOuvrir = new MtgButton(actionFichierOuvrir);

		actionTranslaterFigure = new ActionTranslaterFigure(this);
		buttonTranslationFigure = new MtgToggleButton(actionTranslaterFigure);

		actionZoomPlus = new ActionZoomPlus(this);
		buttonZoomPlus = new MtgButton(actionZoomPlus);

		actionZoomMoins = new ActionZoomMoins(this);
		buttonZoomMoins = new MtgButton(actionZoomMoins);

		actionCapture = new ActionCapture(this);
		buttonCapture = new MtgToggleButton(actionCapture);

		actionCreerPointLibre = new ActionCreerPointLibre(this);
		buttonCreerPointLibre = new MtgToggleButton(actionCreerPointLibre, couleurOutilsPoints);

		actionCreerMilieu = new ActionCreerMilieu(this);
		buttonCreerMilieu = new MtgToggleButton(actionCreerMilieu, couleurOutilsPoints);

		actionCreerPointLie = new ActionCreerPointLie(this);
		buttonCreerPointLie = new MtgToggleButton(actionCreerPointLie, couleurOutilsPoints);

		actionCreerDroiteAB = new ActionCreerDroiteAB(this);
		buttonCreerDroiteAB = new MtgToggleButton(actionCreerDroiteAB, couleurOutilsLigne);

		actionCreerDroiteParallele = new ActionCreerDroiteParallele(this);
		buttonCreerDroiteParallele = new MtgToggleButton(actionCreerDroiteParallele, couleurOutilsLigne);

		actionCreerDroitePerpendiculaire = new ActionCreerDroitePerpendiculaire(this);
		buttonCreerDroitePerpendiculaire = new MtgToggleButton(actionCreerDroitePerpendiculaire, couleurOutilsLigne);

		actionCreerDroiteBissectrice = new ActionCreerDroiteBissectrice(this);
		buttonCreerDroiteBissectrice = new MtgToggleButton(actionCreerDroiteBissectrice, couleurOutilsLigne);

		actionCreerDroiteMediatrice = new ActionCreerDroiteMediatrice(this);
		buttonCreerDroiteMediatrice = new MtgToggleButton(actionCreerDroiteMediatrice, couleurOutilsLigne);

		actionCreerSegment = new ActionCreerSegment(this);
		buttonCreerSegment = new MtgToggleButton(actionCreerSegment, couleurOutilsLigne);

		actionCreerDemiDroite = new ActionCreerDemiDroite(this);
		buttonCreerDemiDroite = new MtgToggleButton(actionCreerDemiDroite, couleurOutilsLigne);

		actionCreerIntersection = new ActionCreerIntersection(this);
		buttonCreerIntersection = new MtgToggleButton(actionCreerIntersection, couleurOutilsPoints);

		actionCreerCercleOA = new ActionCreerCercleOA(this);
		buttonCreerCercleOA = new MtgToggleButton(actionCreerCercleOA, couleurOutilsLigne);

		actionCreerCercleOR = new ActionCreerCercleOR(this);
		buttonCreerCercleOR = new MtgToggleButton(actionCreerCercleOR, couleurOutilsLigne);

		actionCreerCalcul = new ActionCreerCalculReel(this);
		buttonCreerCalcul = new MtgToggleButton(actionCreerCalcul, couleurOutilsDivers);

		actionModifierNom = new ActionModifierNom(this);
		buttonModifierNom = new MtgToggleButton(actionModifierNom);

		actionHistorique = new ActionHistorique(this);
		buttonHistorique = new MtgToggleButton(actionHistorique);

		actionAfficherObjetsNumeriques = new ActionAfficherObjetsNumeriques(this);
		buttonAfficherObjetsNumeriques = new MtgToggleButton(actionAfficherObjetsNumeriques);

		actionHelpTexteFigure = new ActionHelpTexteFigure(this);
		buttonHelpTexteFigure = new MtgToggleButton(actionHelpTexteFigure);

		actionMesureLongueur = new ActionMesureLongueur(this);
		buttonMesureLongueur = new MtgToggleButton(actionMesureLongueur, couleurOutilsMesure);

		actionMesureAbscisse = new ActionMesureAbscisse(this);
		buttonMesureAbscisse = new MtgToggleButton(actionMesureAbscisse, couleurOutilsMesure);

		actionMesureAngleNonOriente = new ActionMesureAngleNonOriente(this);
		buttonMesureAngleNonOriente = new MtgToggleButton(actionMesureAngleNonOriente, couleurOutilsMesure);

		actionMesureAngleOriente = new ActionMesureAngleOriente(this);
		buttonMesureAngleOriente = new MtgToggleButton(actionMesureAngleOriente, couleurOutilsMesure);

		actionRapporteur = new ActionRapporteur(this);
		buttonRapporteur = new MtgToggleButton(actionRapporteur, couleurOutilsTransformation);

		actionSuppressionObjetGraphique = new ActionSuppressionObjetGraphique(this);
		buttonSuppressionObjetGraphique = new MtgToggleButton(actionSuppressionObjetGraphique);

		actionSuppressionValeur = new ActionSuppressionValeur(this);
		buttonSuppressionValeur = new MtgToggleButton(actionSuppressionValeur);

		actionCreerMarqueAngleNonOriente = new ActionCreerMarqueAngleNonOriente(this);
		buttonCreerMarqueAngleNonOriente = new MtgToggleButton(actionCreerMarqueAngleNonOriente, couleurOutilsPolygones);

		actionCreerMarqueAngleOriente = new ActionCreerMarqueAngleOriente(this);
		buttonCreerMarqueAngleOriente = new MtgToggleButton(actionCreerMarqueAngleOriente, couleurOutilsPolygones);

		actionCreerImageParTranslation = new ActionCreerImageParTranslation(this);
		buttonCreerImageParTranslation = new MtgToggleButton(actionCreerImageParTranslation, couleurOutilsTransformation);

		actionCreerImageParTranslationParCoord = new ActionCreerImageParTranslationParCoord(this);
		buttonCreerImageParTranslationParCoord = new MtgToggleButton(actionCreerImageParTranslationParCoord, couleurOutilsTransformation);

		actionCreerImageParSymetrieAxiale = new ActionCreerImageParSymetrieAxiale(this);
		buttonCreerImageParSymetrieAxiale = new MtgToggleButton(actionCreerImageParSymetrieAxiale, couleurOutilsTransformation);

		actionCreerImageParSymetrieCentrale = new ActionCreerImageParSymetrieCentrale(this);
		buttonCreerImageParSymetrieCentrale = new MtgToggleButton(actionCreerImageParSymetrieCentrale, couleurOutilsTransformation);

		actionCreerImageParHomothetie = new ActionCreerImageParHomothetie(this);
		buttonCreerImageParHomothetie = new MtgToggleButton(actionCreerImageParHomothetie, couleurOutilsTransformation);

		actionCreerImageParRotation = new ActionCreerImageParRotation(this);
		buttonCreerImageParRotation = new MtgToggleButton(actionCreerImageParRotation, couleurOutilsTransformation);

		actionCreerArcPetitParPointFinal = new ActionCreerArcPetitParPointFinal(this);
		buttonCreerArcPetitParPointFinal = new MtgToggleButton(actionCreerArcPetitParPointFinal, couleurOutilsPolygones);

		actionCreerArcGrandParPointFinal = new ActionCreerArcGrandParPointFinal(this);
		buttonCreerArcGrandParPointFinal = new MtgToggleButton(actionCreerArcGrandParPointFinal, couleurOutilsPolygones);

		actionCreerArcDirectParPointFinal = new ActionCreerArcDirectParPointFinal(this);
		buttonCreerArcDirectParPointFinal = new MtgToggleButton(actionCreerArcDirectParPointFinal, couleurOutilsPolygones);

		actionCreerArcIndirectParPointFinal = new ActionCreerArcIndirectParPointFinal(this);
		buttonCreerArcIndirectParPointFinal = new MtgToggleButton(actionCreerArcIndirectParPointFinal, couleurOutilsPolygones);

		actionCreerArcPetitParRapporteur = new ActionCreerArcPetitParRapporteur(this);
		buttonCreerArcPetitParRapporteur = new MtgToggleButton(actionCreerArcPetitParRapporteur, couleurOutilsPolygones);

		actionCreerArcGrandParRapporteur = new ActionCreerArcGrandParRapporteur(this);
		buttonCreerArcGrandParRapporteur = new MtgToggleButton(actionCreerArcGrandParRapporteur, couleurOutilsPolygones);

		actionCreerDroiteHorizontale = new ActionCreerDroiteHorizontale(this);
		buttonCreerDroiteHorizontale = new MtgToggleButton(actionCreerDroiteHorizontale, couleurOutilsLigne);

		actionCreerDroiteVerticale = new ActionCreerDroiteVerticale(this);
		buttonCreerDroiteVerticale = new MtgToggleButton(actionCreerDroiteVerticale, couleurOutilsLigne);

		actionCreerMarqueSegment = new ActionCreerMarqueSegment(this);
		buttonCreerMarqueSegment = new MtgToggleButton(actionCreerMarqueSegment, couleurOutilsPolygones);

		actionPalette = new ActionPalette(this);
		buttonPalette = new MtgToggleButton(actionPalette);

		actionCreerPolygone = new ActionCreerPolygone(this);
		buttonCreerPolygone = new MtgToggleButton(actionCreerPolygone, couleurOutilsPolygones);

		actionCreerLigneBrisee = new ActionCreerLigneBrisee(this);
		buttonCreerLigneBrisee = new MtgToggleButton(actionCreerLigneBrisee, couleurOutilsPolygones);

		actionCreerSurface = new ActionCreerSurface(this);
		buttonCreerSurface = new MtgToggleButton(actionCreerSurface, couleurOutilsDivers);

		actionCreerSurfaceLieuDroite = new ActionCreerSurfaceLieuDroite(this);
		buttonCreerSurfaceLieuDroite = new MtgToggleButton(actionCreerSurfaceLieuDroite, couleurOutilsDivers);

		actionCreerSurfaceLieuDeuxPoints = new ActionCreerSurfaceLieuDeuxPoints(this);
		buttonCreerSurfaceLieuDeuxPoints = new MtgToggleButton(actionCreerSurfaceLieuDeuxPoints, couleurOutilsDivers);

		actionCreerSurfaceDeuxLieux = new ActionCreerSurfaceDeuxLieux(this);
		buttonCreerSurfaceDeuxLieux = new MtgToggleButton(actionCreerSurfaceDeuxLieux, couleurOutilsDivers);

		actionCreerProjeteOrthogonal = new ActionCreerProjeteOrthogonal(this);
		buttonCreerProjeteOrthogonal = new MtgToggleButton(actionCreerProjeteOrthogonal, couleurOutilsPoints);

		actionCreerRepere = new ActionCreerRepere(this);
		buttonCreerRepere = new MtgToggleButton(actionCreerRepere, couleurOutilsDivers);

		actionCreerPointParCoordonnees = new ActionCreerPointParCoordonnees(this);
		buttonCreerPointParCoordonnees = new MtgToggleButton(actionCreerPointParCoordonnees, couleurOutilsPoints);

		actionModifierObjetGraphique = new ActionModifierObjetGraphique(this);
		buttonModifierObjetGraphique = new MtgToggleButton(actionModifierObjetGraphique);

		actionModifierFormule = new ActionModifierFormule(this);
		buttonModifierFormule = new MtgToggleButton(actionModifierFormule);

		actionCreerDroiteParCoef = new ActionCreerDroiteParCoef(this);
		buttonCreerDroiteParCoef = new MtgToggleButton(actionCreerDroiteParCoef, couleurOutilsLigne);

		actionCreerDroiteParCoef = new ActionCreerDroiteParCoef(this);
		buttonCreerDroiteParCoef = new MtgToggleButton(actionCreerDroiteParCoef, couleurOutilsLigne);

		actionCreerAffichageValeur = new ActionCreerAffichageValeur(this);
		buttonCreerAffichageValeur = new MtgToggleButton(actionCreerAffichageValeur, couleurOutilsAffichages);

		actionCreerAffichageValeurLiePoint = new ActionCreerAffichageValeurLiePoint(this);
		buttonCreerAffichageValeurLiePoint = new MtgToggleButton(actionCreerAffichageValeurLiePoint, couleurOutilsAffichages);

		actionCreerCentreGravite = new ActionCreerCentreGravite(this);
		buttonCreerCentreGravite = new MtgToggleButton(actionCreerCentreGravite, couleurOutilsPoints);

		actionCreerPointParAffixe = new ActionCreerPointParAffixe(this);
		buttonCreerPointParAffixe = new MtgToggleButton(actionCreerPointParAffixe, couleurOutilsPoints);

		actionCreerPointParAbscisse = new ActionCreerPointParAbscisse(this);
		buttonCreerPointParAbscisse = new MtgToggleButton(actionCreerPointParAbscisse, couleurOutilsPoints);

		actionCreerImageLibre = new ActionCreerImageLibre(this);
		buttonCreerImage = new MtgToggleButton(actionCreerImageLibre, couleurOutilsAffichages);

		actionCreerImageLieePoint = new ActionCreerImageLieePoint(this);
		buttonCreerImageLie = new MtgToggleButton(actionCreerImageLieePoint, couleurOutilsAffichages);

		actionCreerCommentaire = new ActionCreerCommentaire(this);
		buttonCreerCommentaire = new MtgToggleButton(actionCreerCommentaire, couleurOutilsAffichages);

		actionCreerCommentaireLiePoint = new ActionCreerCommentaireLiePoint(this);
		buttonCreerCommentaireLiePoint = new MtgToggleButton(actionCreerCommentaireLiePoint, couleurOutilsAffichages);

		actionCreerLatex = new ActionCreerLatex(this);
		buttonCreerLatex = new MtgToggleButton(actionCreerLatex, couleurOutilsAffichages);

		actionCreerLatexLiePoint = new ActionCreerLatexLiePoint(this);
		buttonCreerLatexLiePoint = new MtgToggleButton(actionCreerLatexLiePoint, couleurOutilsAffichages);

		actionCreerEquationDroite = new ActionCreerEquationDroite(this);
		buttonCreerEquationDroite = new MtgToggleButton(actionCreerEquationDroite, couleurOutilsAffichages);

		actionCreerEquationDroiteLiePoint = new ActionCreerEquationDroiteLiePoint(this);
		buttonCreerEquationDroiteLie = new MtgToggleButton(actionCreerEquationDroiteLiePoint, couleurOutilsAffichages);

		actionCreerDroiteParEquation = new ActionCreerDroiteParEquation(this);
		buttonCreerDroiteParEquation = new MtgToggleButton(actionCreerDroiteParEquation, couleurOutilsLigne);

		actionCreerLieuParPointLie = new ActionCreerLieuParPointLie(this);
		buttonCreerLieuParPointLie = new MtgToggleButton(actionCreerLieuParPointLie, couleurOutilsDivers);

		actionCreerImageParSimilitude = new ActionCreerImageParSimilitude(this);
		buttonCreerImageParSimilitude = new MtgToggleButton(actionCreerImageParSimilitude, couleurOutilsTransformation);

		actionCreerBarycentre = new ActionCreerBarycentre(this);
		buttonCreerBarycentre = new MtgToggleButton(actionCreerBarycentre, couleurOutilsPoints);

		actionCreerPointParSommeVect = new ActionCreerPointParSommeVect(this);
		buttonCreerPointParSommeVect = new MtgToggleButton(actionCreerPointParSommeVect, couleurOutilsPoints);

		actionCreerPointParMultVect = new ActionCreerPointParMultVect(this);
		buttonCreerPointParMultVect = new MtgToggleButton(actionCreerPointParMultVect, couleurOutilsPoints);

		actionCarreDirect = new ActionCarreDirect(this);
		buttonCarreDirect = new MtgToggleButton(actionCarreDirect, couleurOutilsPolygones);

		actionTriangleEquilateralDirect = new ActionTriangleEquilateralDirect(this);
		buttonTriangleEquilateralDirect = new MtgToggleButton(actionTriangleEquilateralDirect, couleurOutilsPolygones);

		actionParallelogramme = new ActionParallelogramme(this);
		buttonParallelogramme = new MtgToggleButton(actionParallelogramme, couleurOutilsPolygones);

		actionPolygoneRegulier = new ActionPolygoneRegulier(this);
		buttonPolygoneRegulier = new MtgToggleButton(actionPolygoneRegulier, couleurOutilsPolygones);

		actionExecuterMacro = new ActionExecuterMacro(this);
		buttonExecuterMacro = new MtgToggleButton(actionExecuterMacro);

		actionGomme = new ActionGomme(this);
		buttonGomme = new MtgToggleButton(actionGomme);

		actionCreerVecteur = new ActionCreerVecteur(this);
		buttonCreerVecteur = new MtgToggleButton(actionCreerVecteur, couleurOutilsLigne);

		actionMesureAbscisseDansRepere = new ActionMesureAbscisseDansRepere(this);
		buttonMesureAbscisseDansRepere = new MtgToggleButton(actionMesureAbscisseDansRepere, couleurOutilsMesure);

		actionMesureOrdonneeDansRepere = new ActionMesureOrdonneeDansRepere(this);
		buttonMesureOrdonneeDansRepere = new MtgToggleButton(actionMesureOrdonneeDansRepere, couleurOutilsMesure);

		actionMesureAffixe = new ActionMesureAffixe(this);
		buttonMesureAffixe = new MtgToggleButton(actionMesureAffixe, couleurOutilsMesure);

		actionMesureLongueurLigne = new ActionMesureLongueurLigne(this);
		buttonMesureLongueurLigne = new MtgToggleButton(actionMesureLongueurLigne, couleurOutilsMesure);

		actionMesureAirePolygone = new ActionMesureAirePolygone(this);
		buttonMesureAirePolygone = new MtgToggleButton(actionMesureAirePolygone, couleurOutilsMesure);

		actionMesureProduitScalaire = new ActionMesureProduitScalaire(this);
		buttonMesureProduitScalaire = new MtgToggleButton(actionMesureProduitScalaire, couleurOutilsMesure);

		actionFichierNouvelleFigure = new ActionFichierNouvelleFigure(this);
		buttonFichierNouvelleFigure = new MtgButton(actionFichierNouvelleFigure);

		actionFichierEnregistrer = new ActionFichierEnregistrer(this);
		// buttonFichierEnregistrer = new MtgButton(actionFichierEnregistrer);
		// Supprim version Applet
		// buttonFichierEnregistrer.setText(""); Supprim version Applet

		actionFichierImprimer = new ActionFichierImprimer(this);
		// buttonFichierImprimer = new MtgButton(actionFichierImprimer); Supprim
		// version Applet
		// buttonFichierImprimer.setText(""); Supprim version Applet

		actionModeTrace = new ActionModeTraceActivation(this);
		buttonModeTrace = new MtgToggleButton(actionModeTrace);

		actionEditionCopier = new ActionEditionCopier(this);
		// buttonEditionCopier = new MtgButton(actionEditionCopier); Supprim
		// version Applet
		// buttonEditionCopier.setText(""); Supprim version Applet

		actionRideau = new ActionRideau(this);
		buttonRideau = new MtgToggleButton(actionRideau);

		actionPunaiser = new ActionPunaiser(this);
		buttonPunaiser = new MtgToggleButton(actionPunaiser, couleurOutilsDivers);

		actionDepunaiser = new ActionDepunaiser(this);
		buttonDepunaiser = new MtgToggleButton(actionDepunaiser, couleurOutilsDivers);

		actionCreerPointLibreEnt = new ActionCreerPointLibreEnt(this);
		buttonCreerPointLibreEnt = new MtgToggleButton(actionCreerPointLibreEnt, couleurOutilsPoints);

		actionReafficherTout = new ActionReafficherTout(this);
		buttonReafficherTout = new MtgButton(actionReafficherTout);

		actionCreerFonctionReelle = new ActionCreerFonctionReelle(this);
		buttonCreerFonctionReelle = new MtgToggleButton(actionCreerFonctionReelle, couleurOutilsDivers);

		actionCreerCourbeFonction = new ActionCreerCourbeFonction(this);
		buttonCreerCourbeFonction = new MtgToggleButton(actionCreerCourbeFonction, couleurOutilsDivers);

		actionPointSurCourbeParAbscisse = new ActionPointSurCourbeParAbscisse(this);
		buttonCreerPointSurCourbeParAbscisse = new MtgToggleButton(actionPointSurCourbeParAbscisse, couleurOutilsDivers);

		actionCurseur = new ActionCurseur(this);
		buttonCurseur = new MtgToggleButton(actionCurseur, couleurOutilsPoints);

		actionTangente = new ActionTangente(this);
		buttonTangente = new MtgToggleButton(actionTangente, couleurOutilsDivers);

		actionMesureCoefficientDirecteur = new ActionMesureCoefficientDirecteur(this);
		buttonMesurerCoefficientDirecteur = new MtgToggleButton(actionMesureCoefficientDirecteur, couleurOutilsMesure);

		actionCreerPointInterieur = new ActionCreerPointInterieurPolygone(this);
		buttonCreerPointInterieur = new MtgToggleButton(actionCreerPointInterieur, couleurOutilsPoints);

		actionRelationEntreObjets = new ActionRelationEntreObjets(this);
		buttonRelationEntreObjets = new MtgToggleButton(actionRelationEntreObjets);

		actioncreerEditeurFormule = new ActionCreerEditeurFormule(this);
		buttonCreerEditeurFormule = new MtgToggleButton(actioncreerEditeurFormule, couleurOutilsAffichages);

		actionCreerSegmentParLong = new ActionSegmentParLong(this);
		buttonSegmentParLong = new MtgToggleButton(actionCreerSegmentParLong, couleurOutilsLigne);

		// Modification version 4.6.3 : Ajout d'un bouton dans la barre horizontale pour exporter dans un fichier png avec unit
		actionFichierExporterPNGAvecUnite = new ActionFichierExporterPNGAvecUnite(this);
		buttonExporterUnite = new MtgButton(actionFichierExporterPNGAvecUnite);
		actionFichierNouvelleFigureAvecRepere = new ActionFichierNouvelleFigureAvecRepere(this);
		buttonFichierNouvelleFigureAvecRepereSansVecteurs = new MtgButton(actionFichierNouvelleFigureAvecRepere);

		// Outils sans icne
		actionCreerMarqueAngleOrienteSensDirect = new ActionCreerMarqueAngleOrienteSensDirect(this);
		actionCreerMarqueAngleOrienteSensIndirect = new ActionCreerMarqueAngleOrienteSensIndirect(this);
		actionCreerCalculComplexe = new ActionCreerCalculComplexe(this);
		actionCreerFonctionComplexe = new ActionCreerFonctionComplexe(this);
		actionCreerVariable = new ActionCreerVariable(this);
		actionCreerLieuParVariable = new ActionCreerLieuParVariable(this);
		actionCreerLieuDiscretParPointLie = new ActionCreerLieuDiscretParPointLie(this);
		actionCreerLieuDiscretParVariable = new ActionCreerLieuDiscretParVariable(this);
		actionCreerLieuObjetParPointLie = new ActionCreerLieuObjetParPointLie(this);
		actionCreerLieuObjetParVariable = new ActionCreerLieuObjetParVariable(this);
		actionCreerImageParInversion = new ActionCreerImageParInversion(this);
		actionCreerDemiPlan = new ActionCreerDemiPlan(this);
		actionCreerDerivee = new ActionCreerDerivee(this);
		actionCreerSuiteRecurrenteReelle = new ActionCreerSuiteRecurrenteReelle(this);
		actionCreerSuiteRecurrenteComplexe = new ActionCreerSuiteRecurrenteComplexe(this);
		actionCreerGrapheSuiteRecurrenteReelle = new ActionCreerGrapheSuiteRecurrenteReelle(this);
		actionCreerGrapheSuiteRecurrenteComplexe = new ActionCreerGrapheSuiteRecurrenteComplexe(this);
		actionFichierEnregistrerSous = new ActionFichierEnregistrerSous(this);
		// actionFichierQuitter = new ActionFichierQuitter(this); Supprim version
		// Applet
		actionFichierNouvelleFigureSansLongueurUnite = new ActionFichierNouvelleFigureSansLongueurUnite(this);
		actionFichierExporterPNG = new ActionFichierExporterPNG(this);
		actionFichierExporterJPEG = new ActionFichierExporterJPEG(this);
		actionModifierRepere = new ActionModifierRepere(this);
		actionPointSurCourbeParPointLie = new ActionPointSurCourbeParPointLie(this);
		actionCreerSolutionEquation = new ActionCreerSolutionEquation(this);
		actionCreerMaximum = new ActionCreerMaximum(this);
		actionCreerMinimum = new ActionCreerMinimum(this);
		actionCreerIntegrale = new ActionCreerIntegrale(this);
		actionCreerTestExistence = new ActionCreerTestExistence(this);
		actionCreerPartieReelle = new ActionCreerPartieReelle(this);
		actionCreerPartieImaginaire = new ActionCreerPartieImaginaire(this);
		actionCreerModule = new ActionCreerModule(this);
		actionCreerArgument = new ActionCreerArgument(this);
		actionCreerSommeIndicee = new ActionCreerSommeIndicee(this);
		actionCreerProduitIndice = new ActionCreerProduitIndice(this);
		actionCreerMacroApparition = new ActionCreerMacroApparition(this);
		actionCreerMacroDisparition = new ActionCreerMacroDisparition(this);
		actionCreerMacroAnimation = new ActionCreerMacroAnimation(this);
		actionCreerMacroAnimationAvecTrace = new ActionCreerMacroAnimationAvecTrace(this);
		actionCreerMacroAffectationValeurVariable = new ActionCreerMacroAffectationValeurVariable(this);
		actionCreerMacroModificationVariable = new ActionCreerMacroModificationVariable(this);
		// actionCreerMacroJouantSon = new ActionCreerMacroJouantSon(this); Supprim
		// pour version Applet
		actionCreerMacroIncrementationVariable = new ActionCreerMacroIncrementationVariable(this);
		actionCreerMacroDecrementationVariable = new ActionCreerMacroDecrementationVariable(this);
		actionCreerMacroTraceAutoParPointLie = new ActionCreerMacroTraceAutoParPointLie(this);
		actionCreerMacroTraceAutoParVariable = new ActionCreerMacroTraceAutoParVariable(this);
		actionCreerMacroAffectationPointLie = new ActionCreerMacroAffectationPointLie(this);
		actionCreerMacroClignotement = new ActionCreerMacroClignotement(this);
		actionCreerMacroAffichantTout = new ActionCreerMacroAffichantTout(this);
		actionCreerMacroPause = new ActionCreerMacroPause(this);
		actionCreerMacroBoucleAvecAnimation = new ActionCreerMacroBoucleAvecAnimation(this);
		actionCreerMacroBoucleAvecTrace = new ActionCreerMacroBoucleAvecTrace(this);
		actionCreerMacroSuiteMacros = new ActionCreerMacroSuiteMacros(this);
		actionCreerObjetDuplique = new actionCreerObjetDuplique(this);
		actionOptionsFigureEnCours = new ActionOptionsFigureEnCours(this);
		actionOptionsPreferences = new ActionOptionsPreferences(this);
		actionSuppressionImageFond = new ActionSuppressionImageFond(this);
		actionChoixImageFond = new ActionChoixImageFond(this);
		actionImageFondVisible = new ActionImageFondVisible(this);
		actionMesureLongueurLigne = new ActionMesureLongueurLigne(this);
		actionCreerCentreCercle = new ActionCreerCentreCercle(this);
		actionConstructionImplementerDepuisFigure = new ActionConstructionImplementerDepuisFigure(this);
		actionAjoutObjetsMacro = new ActionAjoutObjetsMacro(this);
		actionRetraitObjetsMacro = new ActionRetraitObjetsMacro(this);
		actionModificationMacro = new ActionModifierMacro(this);
		actionMouvoirAnimationPointLie = new ActionMouvoirAnimationPointLie(this);
		actionRenommerCalcul = new ActionRenommerCalcul(this);
		actionConstructionEnregistrer = new ActionConstructionEnregistrer(this);
		actionIncorporerConstruction = new ActionConstructionIncorporer(this);
		actionModifierConstruction = new ActionConstructionModifier(this);
		actionCreerLongueurUnite = new ActionCreerLongueurUnite(this);
		// actionChoixSourcesGraphiques = new
		// ActionConstructionChoixSourcesGraphiques(this); Supprim version Applet
		// actionChoixFinauxGraphiques = new
		// ActionConstructionChoixFinauxGraphiques(this); Supprim version Applet
		// actionChoixFinauxNonGraphiques = new
		// ActionConstructionChoixFinauxNonGraphiques(this); supprim version Applet
		// actionConstructionCreer = new ActionConstructionCreer(this); supprim
		// version Applet
		// actionConstructionReinitialiser = new
		// ActionConstructionReinitialiser(this); supprim version Applet
		actionConstructionImplementerDepuisFichier = new ActionConstructionImplementerDepuisFichier(this);
		actionConstructionSupprimer = new ActionConstructionSupprimer(this);
		actionConstructionFusionner = new ActionConstructionFusionner(this);
		actionCreationVisuelle = new ActionCreationVisuelle(this);
		actionAffichageAutoMesures = new ActionAffichageAutoMesures(this);
		actionNomsAuto = new ActionNomsAuto(this);
		actionEditerTexteFigure = new ActionEditerTexteFigure(this);
		actionCreerArcPetitParAngle = new ActionCreerArcPetitParAngle(this);
		actionCreerArcGrandParAngle = new ActionCreerArcGrandParAngle(this);
		actionCreerArcDirectParAngle = new ActionCreerArcDirectParAngle(this);
		actionCreerArcIndirectParAngle = new ActionCreerArcIndirectParAngle(this);
		actionHelpAbout = new ActionHelpAbout(this);
		// actionHelpNet = new ActionHelpNet(this); Supprim version Applet
		// actionHelpCodeHTML = new ActionHelpCodeHTML(this); Supprim version
		// Applet
		actionHelpHelp = new ActionHelpHelp(this);
		actionSupprimerLiaison = new ActionSuppressionLiaison(this);
		actionCreerLiaison = new ActionCreerLiaison(this);
		actionSupprimerLiaisonAffichage = new ActionSuppressionLiaisonAffichage(this);
		actionCreerLiaisonAffichage = new ActionCreerLiaisonAffichage(this);
		actionReclassementFinObjetGraphique = new ActionReclassementFinObjetGraphique(this);
		actionReclassementFinValeur = new ActionReclassementFinValeur(this);
		actionReclassementDebutObjetGraphique = new ActionReclassementDebutObjetGraphique(this);
		actionReclassementDebutValeur = new ActionReclassementDebutValeur(this);
		actionActiverOutilPrecedent = new ActionActiverOutilPrecedent(this);
		actionCreerMacroApparitionAPartirAutreMacro = new ActionCreerMacroApparitionAPartirAutreMacro(this);
		actionCreerMacroDisparitionAPartirAutreMacro = new ActionCreerMacroDisparitionAPartirAutreMacro(this);
		actionTraceObjets = new ActionTraceObjets(this);
		actionFichierNouvelleFigurePredefinie = new ActionFichierNouvelleFigurePredefinie(this);
		actionCreationLiaisonPointInterieurPolygone = new ActionCreationLiaisonPointInterieurPolygone(this);
		actionCreationLiaisonPointInterieurCercle = new ActionCreationLiaisonPointInterieurCercle(this);
		actionCreerFonctionReelle2Var = new ActionCreerFonctionReelle2Var(this);
		actionCreerFonctionReelle3Var = new ActionCreerFonctionReelle3Var(this);
		actionCreerFonctionComplexe2Var = new ActionCreerFonctionComplexe2Var(this);
		actionCreerFonctionComplexe3Var = new ActionCreerFonctionComplexe3Var(this);
		actionCreerMacroModeTraceActivation = new ActionCreerMacroModeTraceActivation(this);
		actionCreerMacroModeTraceDesactivation = new ActionCreerMacroModeTraceDesactivation(this);
		actionChoixOutils = new ActionChoixOutils(this);
		actionMarquerPointPourTrace = new ActionMarquerPointPourTrace(this);
		actionDemarquerPointPourTrace = new ActionDemarquerPointPourTrace(this);
		actionCreerAbscisseOrigineRep = new ActionCreerAbscisseOrigineRep(this);
		actionCreerOrdonneeOrigineRep = new ActionCreerOrdonneeOrigineRep(this);
		actionCreerUnitexRep = new ActionCreerUnitexRep(this);
		actionCreerUniteyRep = new ActionCreerUniteyRep(this);
		actionAjoutGraduationSimple = new ActionAjoutGraduationSimple(this);
		actionAjoutGraduationTrigo = new ActionAjoutGraduationTrigo(this);
		actionCreerTestEquivalence = new ActionCreerTestEquivalence(this);
		actionCreerTestEqNatOp = new ActionCreerTestEqNatOp(this);
		actionCreerDeriveePartielle = new ActionCreerDeriveePartielle(this);
		actionCreerTestDependanceVariable = new ActionCreerTestDependanceVariable(this);
		actionCreerSuiteRecurrenteComplexe2 = new ActionCreerSuiteRecurrenteComplexe2(this); // Ajout version 4.7.5
		actionCreerSuiteRecurrenteReelle2 = new ActionCreerSuiteRecurrenteReelle2(this); // Ajout version 4.7.5
		actionCreerSuiteRecurrenteComplexe3 = new ActionCreerSuiteRecurrenteComplexe3(this); // Ajout version 4.7.5
		actionCreerSuiteRecurrenteReelle3 = new ActionCreerSuiteRecurrenteReelle3(this); // Ajout version 4.7.5
		actionPropSol = new ActionPropSol(this);

		cbAffichageAutoMesures = new MtgCheckBoxMenuItem(actionAffichageAutoMesures, true);
		cbAideVisuelle = new MtgCheckBoxMenuItem(actionCreationVisuelle, true);
		cbNomsAuto = new MtgCheckBoxMenuItem(actionNomsAuto, false);
		cbModeTrace = new MtgCheckBoxMenuItem(actionModeTrace, false); // Mode trace
																																		// dsactiv
																																		// au
																																		// dmarrage
		cbImageFondVisible = new MtgCheckBoxMenuItem(actionImageFondVisible, true);

	}

	/**
	 * Mise  jour des boutons de la barre d'outils suivant le niveau
	 * 
	 * @param doc
	 *          : le document auquel la barre doit s'r
	 */
	public final void metAJourBoutonsBarreOutils(CMathGrafDoc doc) {
		int nbIcones = nombreIcones(doc);
		int hautic = btnsDispo[0].getMaximumSize().height;
		int hautdisp = cadreActif().getHeight();
		int nbIconesParCol = hautdisp / hautic;
		int oldNbCol = pref_NbCol;
		pref_NbCol = (nbIcones - 1) / nbIconesParCol + 1;
		prepareBarreOutilsGauche(oldNbCol);
		Int ind = new Int(0);
		for (int j = 0; j < btnsDispo.length; j++)
			ajouteIconeBarreVert(ind, btnsDispo[j], doc);
		System.gc();
	}

	public final Frame getParentFrame() {
		Container c = getParent();
		while (c != null) {
			if (c instanceof Frame)
				return (Frame) c;
			c = c.getParent();
		}
		return null;
	}

	private final boolean iconePermise(MtgToggleButton btn, CMathGrafDoc doc) {
		
		if (doc.itemPermis(btn.action.idMenu())) {
			// Si le dossier systme de MathGraph32 contient une figure nomm
			// default.mgj on les outils disponibles
			//  cette figure
			return docPreferencesNiveauPredefini[pref_niveau].itemPermis(btn.action
					.idMenu());
		} else return false;
	}

	/**
	 * Fonction renvoyant le nombre d'icnes de la barre d'outils de gauche
	 * 
	 * @param doc
	 * @return
	 */
	private final int nombreIcones(CMathGrafDoc doc) {
		int res = 0;
		for (int i = 0; i < btnsDispo.length; i++) {
			if (iconePermise(btnsDispo[i], doc))
				res++;
		}
		return res;
	}

	/**
	 * Ajout de l'icne n indice  la barres d'icnes verticale de gauche
	 * 
	 * @param indice
	 */
	private final void ajouteIconeBarreVert(Int indice, MtgToggleButton button,
			CMathGrafDoc doc) {
		if (doc.itemPermis(button.action.idMenu())) {
			// Si le dossier systme de MathGraph32 contient une figure nomm
			// default.mgj on les outils disponibles
			//  cette figure
			// Bug corrig version 4.6.4
			/*
			boolean b = (docPreferencesNiveauPredefini[pref_niveau] == null) ? true
					: docPreferencesNiveauPredefini[pref_niveau].itemPermis(button.action.idMenu());
					*/
			
			if (docPreferencesNiveauPredefini[pref_niveau].itemPermis(button.action.idMenu())) {
				btngroup1.add(button);
				int ind = indice.getValue();
				for (int j = 0; j < pref_NbCol; j++) {
					if ((ind % pref_NbCol) == j) leftbar[j].add(button);
				}
				indice.setValue(ind + 1);
			}
			// else btngroup1.remove(button); Inutile
		}
	}

	/**
	 * Fonction ajoutant le MtgButton button  la barre d'outils horizontale 
	 * condition qu'il soit autoris par le document doc. Ajout version 4.3
	 * 
	 * @param button
	 * @param doc
	 * @return true si l'item a t rajout
	 */
	private final boolean ajouteMtgButtonBarreHor(MtgButton button,
			CMathGrafDoc doc, Int indiceLigne) {
		int widthic = buttonFichierNouvelleFigure.getMaximumSize().width;
		if (indiceLigne.getValue() == 0) { // On calcule la largeur relle de la premire ligne de la barre
			int w = 0;
			for (int i = 0; i < topbar[0].getComponentCount(); i++) {
				w += topbar[0].getComponent(i).getMaximumSize().width;
			}
			if (w + widthic > getWidth()) indiceLigne.setValue(1);
		}
		if ((doc == null) ? true : doc.itemPermis(button.action.idMenu())) {
			// Si le dossier systme de MathGraph32 contient une figure nomm
			// default.mgj on adapte les outils disponibles
			//  cette figure
			boolean b = (docPreferencesNiveauPredefini[pref_niveau] == null) ? true
					: docPreferencesNiveauPredefini[pref_niveau].itemPermis(button.action
							.idMenu());
			if (b) {
				btngroup2.add(button);
				topbar[indiceLigne.getValue()].add(button);
				return true;
			} else
				btngroup2.remove(button);
		}
		return false;
	}

	/**
	 * Fonction ajoutant le MtgButton button  la barre d'outils horizontale 
	 * condition qu'il soit autoris par le document doc. Ajout version 4.3.
	 * 
	 * @param button
	 * @param doc
	 * @return true si l'item a t rajout
	 */
	private final boolean ajouteMtgToggleButtonBarreHor(MtgToggleButton button,
			CMathGrafDoc doc, Int indiceLigne) {
		int widthic = buttonFichierNouvelleFigure.getMaximumSize().width;
		if (indiceLigne.getValue() == 0) { // On calcule la largeur relle de la premire ligne de la barre
			int w = 0;
			for (int i =0; i < topbar[0].getComponentCount(); i++) {
				w += topbar[0].getComponent(i).getMaximumSize().width;
			}
			if (w + widthic > getWidth()) indiceLigne.setValue(1);
		}
		if ((doc == null) ? true : doc.itemPermis(button.action.idMenu())) {
			// Si le dossier systme de MathGraph32 contient une figure nomm
			// default.mgj on les outils disponibles
			//  cette figure
			boolean b = (docPreferencesNiveauPredefini[pref_niveau] == null) ? true
					: docPreferencesNiveauPredefini[pref_niveau].itemPermis(button.action
							.idMenu());
			if (b) {
				// Version 4.4 : Correction d'un bug version 4.3 o le bouton de mode
				// trace ne fonctionnait plus
				if (button != buttonModeTrace)
					btngroup1.add(button);
				topbar[indiceLigne.getValue()].add(button);
				return true;
			} else {
				btngroup1.remove(button);
			}
		}
		return false;
	}

	/**
	 * Ajout des boutons de la barre horizontale du haut
	 */
	public final void metAJourBoutonsBarreHorizontale(CMathGrafDoc doc) {

		boolean bseparator = false; // Sera true si on peut rajouter un separator
		paneNorth.removeAll();
		topbar[0].removeAll();
		topbar[1].removeAll();
		Int indligne = new Int(0);
		// if (!(allowExercise && cadreActif.isConstruction)) { // Modifi version 4.6.1
		/* Moifi version 4.7.2
		if (!(allowExercise && (cadreActif.isConstruction || cadreActif.isExercise))) {
		*/
		if (!allowExercise) {
			bseparator |= ajouteMtgButtonBarreHor(buttonFichierNouvelleFigure, doc, indligne);
			if (allowFileOpen) bseparator |= ajouteMtgButtonBarreHor(buttonFichierOuvrir, doc, indligne);
		}
		// barHor.add(buttonFichierOuvrir); Supprim version Applet
		// btngroup2.add(buttonFichierOuvrir); Supprim version Applet
		// barHor.add(buttonFichierEnregistrer); Supprim version Applet
		// btngroup2.add(buttonFichierEnregistrer); Supprim version Applet
		// barHor.add(buttonFichierImprimer); Supprim version Applet
		// btngroup2.add(buttonFichierImprimer); Supprim version Applet
		// barHor.add(buttonEditionCopier); Supprim version Applet
		// btngroup2.add(buttonEditionCopier); Supprim version Applet
		bseparator |= ajouteMtgButtonBarreHor(buttonFichierNouvelleFigureAvecRepereSansVecteurs,doc, indligne);
		bseparator |= ajouteMtgButtonBarreHor(buttonExporterUnite, doc, indligne);
		if (bseparator) topbar[indligne.getValue()].addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonCapture, doc, indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierNom, doc,
				indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonPalette, doc, indligne);
		if (bseparator)
			topbar[indligne.getValue()].addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierObjetGraphique,
				doc, indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierFormule, doc,
				indligne);
		if (bseparator)
			topbar[indligne.getValue()].addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(
				buttonSuppressionObjetGraphique, doc, indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonSuppressionValeur, doc,
				indligne);
		if (bseparator)
			topbar[indligne.getValue()].addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonGomme, doc, indligne); // null
																																							// parce
		// que
		// toujours
		// prsent
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonRideau, doc, indligne); // null
		// parce que
		// toujours
		// prsent
		if (bseparator)
			topbar[indligne.getValue()].addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgButtonBarreHor(buttonAnnuler, doc, indligne);
		bseparator |= ajouteMtgButtonBarreHor(buttonRefaire, doc, indligne);
		if (bseparator)
			topbar[indligne.getValue()].addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgButtonBarreHor(buttonZoomPlus, doc, indligne);
		bseparator |= ajouteMtgButtonBarreHor(buttonZoomMoins, doc, indligne);
		// barHor.addSeparator();
		bseparator |= ajouteMtgButtonBarreHor(buttonReafficherTout, doc, indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonTranslationFigure, doc,
				indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModeTrace, doc, indligne); // Le
		// bouton
		// fonctionne
		// en
		// bascule
		if (bseparator)
			topbar[indligne.getValue()].addSeparator();
		validate(); // Pour que les barres d'outils soient mises  jour
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonExecuterMacro, doc,
				indligne);
		// Modification versio 4.7.2 Si le paramtre allowExercise est true et si le paramtre isConstruction est true
		// on ajoute le bouton d'historique dans tous les cas
		// bseparator |= ajouteMtgToggleButtonBarreHor(buttonHistorique, doc, indligne);
		if (allowExercise && isConstructionExercise()) bseparator |= ajouteMtgToggleButtonBarreHor(buttonHistorique, null, indligne);
		else bseparator |= ajouteMtgToggleButtonBarreHor(buttonHistorique, doc, indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonAfficherObjetsNumeriques,
				doc, indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonHelpTexteFigure, doc,
				indligne);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonRelationEntreObjets, doc,
				indligne);
		// if (bseparator) barHor.addSeparator(); Modifi version 4.5.1
		boolean b = topbar[1].getComponentCount() != 0;
		paneNorth.setLayout(new GridLayout(b ? 2 : 1,1));
		paneNorth.add(topbar[0]);
		if (b) paneNorth.add(topbar[1]);
	}

	/**
	 * Renvoie un item de menu associ  l'action action Lors du suvol de l'item
	 * la barre d'information affichera l'information relative  l'outil s'il en
	 * existe un associ Utilise un inset pour que les items ne soient pas trop
	 * colls les uns aux autres
	 * 
	 * @param action
	 * @return
	 */
	private final MtgMenuItem MtgItemMenu(MtgAction action) {
		MtgMenuItem item = new MtgMenuItem(action);
		return item;
	}

	private final MtgMenu MtgMenu(String ch) {
		MtgMenu menu = new MtgMenu(ch);
		return menu;
	}

	/**
	 * Ajout du menu  la menuBar en fonction des items permis ou interdits par le
	 * document actif
	 */
	public final void creeMenu() {
		// if (cadreActif.isExercise) return;
		// A revoir pour optimisation
		if (menuFichier != null)
			menuBar.remove(menuFichier);
		if (menuEdition != null)
			menuBar.remove(menuEdition);
		if (menuCreer != null)
			menuBar.remove(menuCreer);
		if (menuCalculs != null)
			menuBar.remove(menuCalculs);
		if (menuModifier != null)
			menuBar.remove(menuModifier);
		if (menuMouvoir != null)
			menuBar.remove(menuMouvoir);
		if (menuMacros != null)
			menuBar.remove(menuMacros);
		/*
		 * Supprim version Applet if (menuFenetre != null)
		 * menuBar.remove(menuFenetre);
		 */
		if (menuHelp != null)
			menuBar.remove(menuHelp);
		if (menuOptions != null)
			menuBar.remove(menuOptions);
		if (menuConstructions != null)
			menuBar.remove(menuConstructions);
		menuFichier = MtgMenu(getString("menuFichier"));
		// if (!(allowExercise && cadreActif.isConstruction) && allowFileMenu) // Modifi version 4.6.1
		/* Modifi version 4.7.2
		if (!(allowExercise && (cadreActif.isConstruction || cadreActif.isExercise)) && allowFileMenu) menuBar.add(menuFichier);
		*/
		if (allowFileMenu) menuBar.add(menuFichier);
		menuEdition = MtgMenu(getString("menuEdition"));
		menuBar.add(menuEdition);
		menuCreer = MtgMenu(getString("menuCreer"));
		menuBar.add(menuCreer);
		menuModifier = MtgMenu(getString("menuModifier"));
		menuBar.add(menuModifier);
		menuMouvoir = MtgMenu(getString("menuMouvoir"));
		menuBar.add(menuMouvoir);
		menuCalculs = MtgMenu(getString("menuCalculs"));
		menuBar.add(menuCalculs);
		menuMacros = MtgMenu(getString("menuMacros"));
		menuBar.add(menuMacros);
		// menuBar.add(menuFenetre); Supprim version Applet
		menuOptions = MtgMenu(getString("menuOptions"));
		// if (!(allowExercise && cadreActif.isConstruction) && allowOptionsMenu) // Modifi version 4.6.1
		/* Modifi version 4.7.2
		if (!(allowExercise && (cadreActif.isConstruction || cadreActif.isExercise)) && allowOptionsMenu)
		*/
		if (allowOptionsMenu) menuBar.add(menuOptions);
		menuConstructions = MtgMenu(getString("menuConstructions"));
		menuBar.add(menuConstructions);
		menuHelp = MtgMenu(getString("menuHelp"));
		menuBar.add(menuHelp);

		JMenuItem menuItem = MtgItemMenu(actionFichierNouvelleFigure);
		menuFichier.add(menuItem);
		menuItem = MtgItemMenu(actionFichierNouvelleFigureSansLongueurUnite);
		menuFichier.add(menuItem);
		JMenu menuNouvelleFigureAvec = MtgMenu(getString("menuNouvelleFigureAvec"));
		menuFichier.add(menuNouvelleFigureAvec);
		menuItem = MtgItemMenu(actionFichierNouvelleFigureAvecRepere);
		menuNouvelleFigureAvec.add(menuItem);
		menuItem = MtgItemMenu(actionFichierNouvelleFigurePredefinie);
		menuNouvelleFigureAvec.add(menuItem);
		menuFichier.add(new JSeparator());
		if (allowFileOpen) {
			menuItem = MtgItemMenu(actionFichierOuvrir);
			menuFichier.add(menuItem);
		}
		menuItem = MtgItemMenu(actionFichierEnregistrer);
		menuFichier.add(menuItem);
		menuItem = MtgItemMenu(actionFichierEnregistrerSous);
		menuFichier.add(menuItem);
		menuFichier.add(new JSeparator());
		menuItem = MtgItemMenu(actionFichierImprimer);
		menuFichier.add(menuItem);
		menuFichier.add(new JSeparator());
		JMenu menuFichierExporter = MtgMenu(getString("menuFichierExporter"));
		menuFichier.add(menuFichierExporter);
		menuItem = MtgItemMenu(actionFichierExporterPNG);
		menuFichierExporter.add(menuItem);
		menuItem = MtgItemMenu(actionFichierExporterJPEG);
		menuFichierExporter.add(menuItem);
		menuItem = MtgItemMenu(actionFichierExporterPNGAvecUnite);
		menuFichier.add(menuItem);
		menuFichier.add(new JSeparator());
		/*
		 * Supprim version Applet menuItem = MtgItemMenu(actionFichierQuitter);
		 * menuFichier.add(menuItem);
		 */

		menuItemEditionAnnuler = MtgItemMenu(actionAnnuler);
		menuEdition.add(menuItemEditionAnnuler);
		menuItemEditionRefaire = MtgItemMenu(actionRefaire);
		menuEdition.add(menuItemEditionRefaire);
		menuEdition.add(new JSeparator());
		menuItem = MtgItemMenu(actionActiverOutilPrecedent);
		menuEdition.add(menuItem);
		menuItem = MtgItemMenu(actionEditionCopier);
		menuEdition.add(menuItem);
		menuEdition.add(new JSeparator());
		JMenu menuReclassementDebut = MtgMenu(getString("menuReclassementDebut"));
		menuEdition.add(menuReclassementDebut);
		menuItem = MtgItemMenu(actionReclassementDebutObjetGraphique);
		menuReclassementDebut.add(menuItem);
		menuItem = MtgItemMenu(actionReclassementDebutValeur);
		menuReclassementDebut.add(menuItem);
		JMenu menuReclassementFin = MtgMenu(getString("menuReclassementFin"));
		menuEdition.add(menuReclassementFin);
		menuItem = MtgItemMenu(actionReclassementFinObjetGraphique);
		menuReclassementFin.add(menuItem);
		menuItem = MtgItemMenu(actionReclassementFinValeur);
		menuReclassementFin.add(menuItem);
		menuEdition.add(new JSeparator());
		menuItem = MtgItemMenu(actionHistorique);
		menuEdition.add(menuItem);
		menuItem = MtgItemMenu(actionRelationEntreObjets);
		menuEdition.add(menuItem);
		menuItem = MtgItemMenu(actionEditerTexteFigure);
		menuEdition.add(menuItem);

		JMenu menuCreerPoint = MtgMenu(getString("menuCreerPoint"));
		menuCreer.add(menuCreerPoint);
		menuItem = MtgItemMenu(actionCreerPointLibre);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointLibreEnt);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointLie);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMilieu);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCentreGravite);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerProjeteOrthogonal);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParCoordonnees);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParAbscisse);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParSommeVect);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParMultVect);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerBarycentre);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointInterieur);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCentreCercle);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParAffixe);
		menuCreerPoint.add(menuItem);
		JMenu menuCreerPointSurCourbe = MtgMenu(getString("menuCreerPointSurCourbe"));
		menuCreerPoint.add(menuCreerPointSurCourbe);
		menuItem = MtgItemMenu(actionPointSurCourbeParAbscisse);
		menuCreerPointSurCourbe.add(menuItem);
		menuItem = MtgItemMenu(actionPointSurCourbeParPointLie);
		menuCreerPointSurCourbe.add(menuItem);
		JMenu menuCreerDroite = MtgMenu(getString("menuCreerDroite"));
		menuCreer.add(menuCreerDroite);
		menuItem = MtgItemMenu(actionCreerDroiteAB);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteParallele);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroitePerpendiculaire);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteMediatrice);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteBissectrice);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteHorizontale);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteVerticale);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteParCoef);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteParEquation);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionTangente);
		menuCreerDroite.add(menuItem);
		/*
		menuItem = MtgItemMenu(actionCreerSegment);
		menuCreer.add(menuItem);
		*/
		JMenu menuCreerSegment = MtgMenu(getString("chaineMenuCreerSegment"));
		menuCreer.add(menuCreerSegment);
		menuItem = MtgItemMenu(actionCreerSegment);
		menuCreerSegment.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSegmentParLong);
		menuCreerSegment.add(menuItem);		
		menuItem = MtgItemMenu(actionCreerDemiDroite);
		menuCreer.add(menuItem);
		menuItem = MtgItemMenu(actionCreerVecteur);
		menuCreer.add(menuItem);

		JMenu menuCreerCercle = MtgMenu(getString("menuCreerCercle"));
		menuCreer.add(menuCreerCercle);
		menuItem = MtgItemMenu(actionCreerCercleOA);
		menuCreerCercle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCercleOR);
		menuCreerCercle.add(menuItem);

		JMenu menuCreerArcDeCercle = MtgMenu(getString("menuCreerArcDeCercle"));
		menuCreer.add(menuCreerArcDeCercle);
		JMenu menuCreerArcDeCercleParParPointFinal = MtgMenu(getString("menuCreerArcDeCercleParPointFinal"));
		menuCreerArcDeCercle.add(menuCreerArcDeCercleParParPointFinal);
		menuItem = MtgItemMenu(actionCreerArcPetitParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcGrandParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcDirectParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcIndirectParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		JMenu menuCreerArcDeCercleParParAngle = MtgMenu(getString("menuCreerArcDeCercleParAngle"));
		menuCreerArcDeCercle.add(menuCreerArcDeCercleParParAngle);
		menuItem = MtgItemMenu(actionCreerArcPetitParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcGrandParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcDirectParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcIndirectParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		JMenu menuCreerArcDeCercleParParRapporteur = MtgMenu(getString("menuCreerArcDeCercleParRapporteur"));
		menuCreerArcDeCercle.add(menuCreerArcDeCercleParParRapporteur);
		menuItem = MtgItemMenu(actionCreerArcPetitParRapporteur);
		menuCreerArcDeCercleParParRapporteur.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcGrandParRapporteur);
		menuCreerArcDeCercleParParRapporteur.add(menuItem);

		JMenu menuCreerImageParTransformation = MtgMenu(getString("menuCreerImageParTransformation"));
		menuCreer.add(menuCreerImageParTransformation);
		menuItem = MtgItemMenu(actionCreerImageParTranslation);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParTranslationParCoord);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParSymetrieAxiale);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParSymetrieCentrale);
		menuCreerImageParTransformation.add(menuItem);
		JMenu menuCreerLigneOuPolygone = MtgMenu(getString("menuLigneOuPolygone"));
		menuCreer.add(menuCreerLigneOuPolygone);
		menuItem = MtgItemMenu(actionCreerPolygone);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionPolygoneRegulier);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLigneBrisee);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionParallelogramme);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionCarreDirect);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionTriangleEquilateralDirect);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParRotation);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParHomothetie);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParSimilitude);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParInversion);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionRapporteur);
		menuCreerImageParTransformation.add(menuItem);

		menuItem = MtgItemMenu(actionCurseur);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerCourbeFonction);
		menuCreer.add(menuItem);
		JMenu menuCreerLieuDePointsRelies = MtgMenu(getString("menuCreerLieuDePointsRelies"));
		menuCreer.add(menuCreerLieuDePointsRelies);
		menuItem = MtgItemMenu(actionCreerLieuParPointLie);
		menuCreerLieuDePointsRelies.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLieuParVariable);
		menuCreerLieuDePointsRelies.add(menuItem);
		JMenu menuCreerLieuDePointsNonRelies = MtgMenu(getString("menuCreerLieuDePointsNonRelies"));
		menuCreer.add(menuCreerLieuDePointsNonRelies);
		menuItem = MtgItemMenu(actionCreerLieuDiscretParPointLie);
		menuCreerLieuDePointsNonRelies.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLieuDiscretParVariable);
		menuCreerLieuDePointsNonRelies.add(menuItem);
		JMenu menuCreerLieuObjet = MtgMenu(getString("menuCreerLieuObjet"));
		menuCreer.add(menuCreerLieuObjet);
		menuItem = MtgItemMenu(actionCreerLieuObjetParPointLie);
		menuCreerLieuObjet.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLieuObjetParVariable);
		menuCreerLieuObjet.add(menuItem);

		JMenu menuCreerGrapheSuiteRecurrente = MtgMenu(getString("menuCreerGrapheSuiteRecurrente"));
		menuCreer.add(menuCreerGrapheSuiteRecurrente);
		menuItem = MtgItemMenu(actionCreerGrapheSuiteRecurrenteReelle);
		menuCreerGrapheSuiteRecurrente.add(menuItem);
		menuItem = MtgItemMenu(actionCreerGrapheSuiteRecurrenteComplexe);
		menuCreerGrapheSuiteRecurrente.add(menuItem);

		JMenu menuCreerSurface = MtgMenu(getString("menuCreerSurface"));
		menuCreer.add(menuCreerSurface);
		menuItem = MtgItemMenu(actionCreerSurface);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSurfaceDeuxLieux);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSurfaceLieuDroite);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSurfaceLieuDeuxPoints);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDemiPlan);
		menuCreerSurface.add(menuItem);
		menuCreer.add(new JSeparator());

		JMenu menuCreerMarqueAngle = MtgMenu(getString("menuCreerMarqueAngle"));
		menuCreer.add(menuCreerMarqueAngle);
		menuItem = MtgItemMenu(actionCreerMarqueAngleNonOriente);
		menuCreerMarqueAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMarqueAngleOriente);
		menuCreerMarqueAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMarqueAngleOrienteSensDirect);
		menuCreerMarqueAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMarqueAngleOrienteSensIndirect);
		menuCreerMarqueAngle.add(menuItem);

		menuItem = MtgItemMenu(actionCreerMarqueSegment);
		menuCreer.add(menuItem);

		menuCreer.add(new JSeparator());
		JMenu menuAffichage = MtgMenu(getString("menuAffichage"));
		menuCreer.add(menuAffichage);
		JMenu menuCreerAffichageValeur = MtgMenu(getString("menuCreerAffichageValeur"));
		menuAffichage.add(menuCreerAffichageValeur);
		menuItem = MtgItemMenu(actionCreerAffichageValeur);
		menuCreerAffichageValeur.add(menuItem);
		menuItem = MtgItemMenu(actionCreerAffichageValeurLiePoint);
		menuCreerAffichageValeur.add(menuItem);
		JMenu menuCreerCommentaire = MtgMenu(getString("menuCreerCommentaire"));
		menuAffichage.add(menuCreerCommentaire);
		menuItem = MtgItemMenu(actionCreerCommentaire);
		menuCreerCommentaire.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCommentaireLiePoint);
		menuCreerCommentaire.add(menuItem);
		JMenu menuCreerLatex = MtgMenu(getString("menuCreerLatex"));
		menuAffichage.add(menuCreerLatex);
		menuItem = MtgItemMenu(actionCreerLatex);
		menuCreerLatex.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLatexLiePoint);
		menuCreerLatex.add(menuItem);
		JMenu menuCreerEquationDroite = MtgMenu(getString("equationDroiteDlg1"));
		menuAffichage.add(menuCreerEquationDroite);
		menuItem = MtgItemMenu(actionCreerEquationDroite);
		menuCreerEquationDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerEquationDroiteLiePoint);
		menuCreerEquationDroite.add(menuItem);

		JMenu menuCreerAffichageImage = MtgMenu(getString("menuCreerAffichageImage"));
		menuAffichage.add(menuCreerAffichageImage);
		menuItem = MtgItemMenu(actionCreerImageLibre);
		menuCreerAffichageImage.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageLieePoint);
		menuCreerAffichageImage.add(menuItem);
		menuItem = MtgItemMenu(actioncreerEditeurFormule);
		menuAffichage.add(menuItem);

		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerIntersection);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerRepere);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerObjetDuplique);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerLongueurUnite);
		menuCreer.add(menuItem);

		menuItem = MtgItemMenu(actionCreerVariable);
		menuCalculs.add(menuItem);
		JMenu menuCalculDansR = MtgMenu(getString("menuCalculDansR"));
		menuCalculs.add(menuCalculDansR);
		menuItem = MtgItemMenu(actionCreerCalcul);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionReelle);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionReelle2Var);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionReelle3Var);
		menuCalculDansR.add(menuItem);

		menuItem = MtgItemMenu(actionCreerDerivee);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSolutionEquation);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMaximum);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMinimum);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerIntegrale);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestExistence);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteReelle);
		menuCalculDansR.add(menuItem);
		// Ajout version 4.7.5
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteReelle2);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteReelle3);
		menuCalculDansR.add(menuItem);
		//
		
		JMenu menuCalculAvance = MtgMenu(getString("avance"));
		menuCalculDansR.add(menuCalculAvance);
		menuItem = MtgItemMenu(actionCreerDeriveePartielle);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestEquivalence);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestDependanceVariable);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestEqNatOp);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSommeIndicee);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerProduitIndice);
		menuCalculAvance.add(menuItem);
		menuCalculDansR.add(new JSeparator());

		menuCalculDansR.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerPartieReelle);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPartieImaginaire);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerModule);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArgument);
		menuCalculDansR.add(menuItem);
		JMenu menuCalculDansC = MtgMenu(getString("menuCalculDansC"));
		menuCalculs.add(menuCalculDansC);
		menuItem = MtgItemMenu(actionCreerCalculComplexe);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionComplexe);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionComplexe2Var);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionComplexe3Var);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteComplexe);
		menuCalculDansC.add(menuItem);
		// Ajout version 4.7.5
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteComplexe2);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteComplexe3);
		menuCalculDansC.add(menuItem);
		//
		menuCalculs.add(new JSeparator());

		JMenu menuCalculMesurer = MtgMenu(getString("menuCalculMesurer"));
		menuCalculs.add(menuCalculMesurer);
		menuItem = MtgItemMenu(actionMesureLongueur);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAbscisse);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAngleNonOriente);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAngleOriente);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAbscisseDansRepere);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureOrdonneeDansRepere);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureCoefficientDirecteur);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAffixe);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureProduitScalaire);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureLongueurLigne);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAirePolygone);
		menuCalculMesurer.add(menuItem);
		menuCalculs.add(new JSeparator());
		menuItem = MtgItemMenu(actionRenommerCalcul);
		menuCalculs.add(menuItem);
		menuItem = MtgItemMenu(actionAfficherObjetsNumeriques);
		menuCalculs.add(menuItem);
		// Ajouts version 4.5
		menuCalculs.add(new JSeparator());
		JMenu menuInfosRepere = MtgMenu(getString("chaineMenuInfoRepere"));
		menuCalculs.add(menuInfosRepere);
		menuItem = MtgItemMenu(actionCreerAbscisseOrigineRep);
		menuInfosRepere.add(menuItem);
		menuItem = MtgItemMenu(actionCreerOrdonneeOrigineRep);
		menuInfosRepere.add(menuItem);
		menuItem = MtgItemMenu(actionCreerUnitexRep);
		menuInfosRepere.add(menuItem);
		menuItem = MtgItemMenu(actionCreerUniteyRep);
		menuInfosRepere.add(menuItem);
		//

		menuItem = MtgItemMenu(actionSuppressionObjetGraphique);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionSuppressionValeur);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionModifierObjetGraphique);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionModifierFormule);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionModifierRepere);
		menuModifier.add(menuItem);
		// Ajout version 4.5
		JMenu menuGraduation = MtgMenu(getString("chaineMenuAjouterGraduation"));
		menuModifier.add(menuGraduation);
		menuItem = MtgItemMenu(actionAjoutGraduationSimple);
		menuGraduation.add(menuItem);
		menuItem = MtgItemMenu(actionAjoutGraduationTrigo);
		menuGraduation.add(menuItem);
		//
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionCapture); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionModifierNom); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionPalette); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionGomme);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionRideau);
		menuModifier.add(menuItem);
		// Ajout version 4.4 : deux outils pour marquer et dmarquer un point pour
		// la trace
		menuItem = MtgItemMenu(actionMarquerPointPourTrace);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionDemarquerPointPourTrace);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionReafficherTout);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionTranslaterFigure);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		JMenu menuModifierCreationLiaison = MtgMenu(getString("menuModifierCreationLiaison"));
		menuModifier.add(menuModifierCreationLiaison);
		menuItem = MtgItemMenu(actionCreerLiaison);
		menuModifierCreationLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLiaisonAffichage);
		menuModifierCreationLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionCreationLiaisonPointInterieurPolygone);
		menuModifierCreationLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionCreationLiaisonPointInterieurCercle);
		menuModifierCreationLiaison.add(menuItem);

		JMenu menuModifierSuppressionLiaison = MtgMenu(getString("menuModifierSuppressionLiaison"));
		menuModifier.add(menuModifierSuppressionLiaison);
		menuItem = MtgItemMenu(actionSupprimerLiaison);
		menuModifierSuppressionLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionSupprimerLiaisonAffichage);
		menuModifierSuppressionLiaison.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionPunaiser);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionDepunaiser);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionZoomPlus); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionZoomMoins);
		menuModifier.add(menuItem);

		menuItem = MtgItemMenu(actionMouvoirAnimationPointLie);
		menuMouvoir.add(menuItem);
		menuItem = MtgItemMenu(actionCapture);
		menuMouvoir.add(menuItem);
		menuItem = MtgItemMenu(actionTraceObjets);
		menuMouvoir.add(menuItem);

		JMenu menuMacrosNouvelleMacro = MtgMenu(getString("menuMacrosNouvelleMacro"));
		menuMacros.add(menuMacrosNouvelleMacro);
		menuItem = MtgItemMenu(actionCreerMacroApparition);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroApparitionAPartirAutreMacro);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroDisparition);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroDisparitionAPartirAutreMacro);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAnimation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAnimationAvecTrace);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAffectationValeurVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroModificationVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		/*
		 * Supprim version Applet menuItem =
		 * MtgItemMenu(actionCreerMacroJouantSon);
		 * menuMacrosNouvelleMacro.add(menuItem);
		 */
		menuItem = MtgItemMenu(actionCreerMacroIncrementationVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroDecrementationVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroTraceAutoParPointLie);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroTraceAutoParVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAffectationPointLie);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroClignotement);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAffichantTout);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroPause);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroBoucleAvecAnimation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroBoucleAvecTrace);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroSuiteMacros);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroModeTraceActivation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroModeTraceDesactivation);
		menuMacrosNouvelleMacro.add(menuItem);

		menuItem = MtgItemMenu(actionExecuterMacro);
		menuMacros.add(menuItem);
		menuMacros.add(new JSeparator());
		menuItem = MtgItemMenu(actionModificationMacro);
		menuMacros.add(menuItem);
		menuMacros.add(new JSeparator());
		menuItem = MtgItemMenu(actionAjoutObjetsMacro);
		menuMacros.add(menuItem);
		menuItem = MtgItemMenu(actionRetraitObjetsMacro);
		menuMacros.add(menuItem);

		menuItem = MtgItemMenu(actionHelpAbout);
		menuHelp.add(menuItem);
		/*
		 * Supprim version Applet menuItem = MtgItemMenu(actionHelpNet);
		 * menuHelp.add(menuItem);
		 */
		menuItem = MtgItemMenu(actionHelpHelp);
		menuHelp.add(menuItem);
		/*
		 * Supprim version Applet menuItem = MtgItemMenu(actionHelpCodeHTML);
		 * menuHelp.add(menuItem);
		 */
		menuItem = MtgItemMenu(actionHelpTexteFigure);
		menuHelp.add(menuItem);

		// Menu Options
		if (allowToolsChoice) {
			menuItem = MtgItemMenu(actionChoixOutils);
			menuOptions.add(menuItem);
		}
		menuItem = MtgItemMenu(actionOptionsFigureEnCours);
		menuOptions.add(menuItem);
		menuItem = MtgItemMenu(actionOptionsPreferences);
		menuOptions.add(menuItem);
		menuOptions.add(new JSeparator());
		menuItem = MtgItemMenu(actionChoixImageFond);
		menuOptions.add(menuItem);
		menuItem = MtgItemMenu(actionSuppressionImageFond);
		menuOptions.add(menuItem);
		menuOptions.add(cbImageFondVisible);
		menuOptions.add(new JSeparator());
		/*
		 * Supprim version Applet JMenu menuPersonnaliserNiveaux =
		 * MtgMenu(getString("personnalisationNiveau"));
		 * menuPersonnaliserNiveaux.add(actionOptionPersonnaliserNiveau1);
		 * menuPersonnaliserNiveaux.add(actionOptionPersonnaliserNiveau2);
		 * menuPersonnaliserNiveaux.add(actionOptionPersonnaliserNiveau3);
		 * menuPersonnaliserNiveaux.add(actionOptionPersonnaliserNiveau4);
		 * menuOptions.add(menuPersonnaliserNiveaux);
		 */
		cbAffichageAutoMesures.setSelected(pref_affichageMesuresNiveau[1]);
		cbAideVisuelle.setSelected(pref_AideVisuelle);
		cbNomsAuto.setSelected(pref_NomsAuto);
		menuOptions.add(cbAideVisuelle);
		menuOptions.add(cbNomsAuto);
		menuOptions.add(cbAffichageAutoMesures);
		menuOptions.add(cbModeTrace);
		// Menu Constructions
		menuItem = MtgItemMenu(actionConstructionImplementerDepuisFigure);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionImplementerDepuisFichier);
		menuConstructions.add(menuItem);
		menuConstructions.add(new JSeparator());
		menuItem = MtgItemMenu(actionConstructionEnregistrer);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionIncorporerConstruction);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionSupprimer);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionModifierConstruction);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionFusionner);
		menuConstructions.add(menuItem);
		/*
		 * Supprim version Applet menuConstructions.add(new JSeparator()); MtgMenu
		 * menuConstructionSources = new MtgMenu(
		 * getString("MenuConstructionsChoixSources"));
		 * menuConstructions.add(menuConstructionSources); menuItem =
		 * MtgItemMenu(actionChoixSourcesNonGraphiques);
		 * menuConstructionSources.add(menuItem); menuItem =
		 * MtgItemMenu(actionChoixSourcesGraphiques);
		 * menuConstructionSources.add(menuItem); MtgMenu menuConstructionFinaux =
		 * new MtgMenu( getString("MenuConstructionsChoixFinaux"));
		 * menuConstructions.add(menuConstructionFinaux); menuItem =
		 * MtgItemMenu(actionChoixFinauxNonGraphiques);
		 * menuConstructionFinaux.add(menuItem); menuItem =
		 * MtgItemMenu(actionChoixFinauxGraphiques);
		 * menuConstructionFinaux.add(menuItem); menuItem =
		 * MtgItemMenu(actionConstructionCreer); menuConstructions.add(menuItem);
		 * menuItem = MtgItemMenu(actionConstructionReinitialiser);
		 * menuConstructions.add(menuItem);
		 */

		menuFichierInitial = new MtgMenu(menuFichier);
		menuEditionInitial = new MtgMenu(menuEdition);
		menuCreerInitial = new MtgMenu(menuCreer);
		menuCalculsInitial = new MtgMenu(menuCalculs);
		menuModifierInitial = new MtgMenu(menuModifier);
		menuMouvoirInitial = new MtgMenu(menuMouvoir);
		menuMacrosInitial = new MtgMenu(menuMacros);
		// menuFenetreInitial = new MtgMenu(menuFenetre); Supprim version Applet
		menuHelpInitial = new MtgMenu(menuHelp);
		menuOptionsInitial = new MtgMenu(menuOptions);
		menuConstructionsInitial = new MtgMenu(menuConstructions);

		// Modifi pour qu'on puisse cacher-montrer le menu
		// if (allowMenuBar) setJMenuBar(menuBar);
		setJMenuBar(menuBar);
		menuBar.setVisible(allowMenuBar);

	}

	/**
	 * Adaptation du menu au document en cours et en me temps, s'il est prsent
	 * au document spcifiant les outils disponibles pour le niveau de
	 * fonctionnement choisi
	 * 
	 * @param doc
	 */
	public void adapteMenuADocument(CMathGrafDoc doc) {
		cbImageFondVisible.setSelected(doc.imageFondVisible);
		if (allowToolsChoice) {
			menuFichier.adapteADocument(doc,
					docPreferencesNiveauPredefini[pref_niveau]);
			menuEdition.adapteADocument(doc,
					docPreferencesNiveauPredefini[pref_niveau]);
			menuCreer
					.adapteADocument(doc, docPreferencesNiveauPredefini[pref_niveau]);
			menuCalculs.adapteADocument(doc,
					docPreferencesNiveauPredefini[pref_niveau]);
			menuModifier.adapteADocument(doc,
					docPreferencesNiveauPredefini[pref_niveau]);
			menuMacros.adapteADocument(doc,
					docPreferencesNiveauPredefini[pref_niveau]);
			menuConstructions.adapteADocument(doc,
					docPreferencesNiveauPredefini[pref_niveau]);
		} else {
			// Cas d'un exercice de construction. On ne propose que les outils
			// autoriss par la figure
			menuFichier.adapteADocument(doc, null);
			menuEdition.adapteADocument(doc, null);
			menuCreer.adapteADocument(doc, null);
			menuCalculs.adapteADocument(doc, null);
			menuModifier.adapteADocument(doc, null);
			menuMacros.adapteADocument(doc, null);
			menuConstructions.adapteADocument(doc, null);
		}
		// menuHelp.ADocument(doc); Non car le menu Aide doit toujours tre
		// propos
		// menuOptions.ADocument(doc); Idempour le menu Options
		// }
	}

	/**
	 * Adaptation des icnes et du menu au document doc
	 * 
	 * @param doc
	 */
	public final void metAJourIconesEtMenus(CMathGrafDoc doc) {
		// Si le dossier systme de MathGraph32 contient une figure nomm
		// default.mgj on les outils disponibles
		//  cette figure
		// if (docPreferencesNiveauPredefini[pref_niveau] != null)
		// adapteMenuADocument(docPreferencesNiveauPredefini[pref_niveau]);
		adapteMenuADocument(doc);
		metAJourBoutonsBarreHorizontale(doc);
		validate(); // Pour connatre la hauteur restante pour les icnes
		metAJourBoutonsBarreOutils(doc);
		validate();
		// Version 4.4 : il faut mettre  jour dimf
		// MtgInternalFrame cadre = cadreActif();
		// cadre.dimf.setSize(cadre.paneFigure.getWidth(),
		// cadre.paneFigure.getHeight());
		// repaint();
	}

	/**
	 * Fonction renvoyant le syle de trait actif
	 * 
	 * @return
	 */
	public final int styleTraitActif() {
		return listeStyleTrait.styleTrait();
	}

	/**
	 * Fonction renvoyant le style de point actif
	 * 
	 * @return
	 */
	public final int stylePointActif() {
		return stylePointActif;
	}

	/**
	 * Fonction renvoyant le style de remplissage actif
	 * 
	 * @return
	 */
	public final int styleRemplissageActif() {
		return styleRemplissageActif;
	}

	/**
	 * Fonction renvoyant le style de marque de segment actif
	 * 
	 * @return
	 */
	public final int styleMarqueSegmentActif() {
		return styleMarqueSegmentActif;
	}

	/**
	 * Fonction renvoyant le style de flche actif pour les vecteurs et arcs
	 * orients
	 * 
	 * @return
	 */
	public final int styleFlecheActif() {
		return styleFlecheActif;
	}

	/**
	 * Fonction renvoyant le style de marque d'angle actif
	 * 
	 * @return
	 */
	public final int styleMarqueAngleActif() {
		return styleMarqueAngleActif;
	}

	/**
	 * Fonctio prparant les variables xfenetre et yfenetre pour un affichage
	 * d'une nouvelle fenetre MtgInternalFrame
	 * 
	 * @param tailleFen
	 * @return renvoie true si le positonnement est possible dans une sous fentre
	 *         et false si la fentre a besoin d'tre maximise
	 */
	/*
	 * Supprim version Applet public boolean
	 * preparePositionPourFenenetre(Dimension tailleFen) { Dimension size =
	 * mtgDesktopPane.getSize(); if ((size.width <= tailleFen.width) ||
	 * (size.height <= tailleFen.height)) { xfenetre = 0; yfenetre = 0; return
	 * false; } else { xfenetre += 10; yfenetre += 10; int x = xfenetre +
	 * tailleFen.width; int y = yfenetre + tailleFen.height; if (x > size.width)
	 * xfenetre = 0; if (y > size.height) yfenetre = 0; return true; } }
	 */

	/**
	 * Renvoie un pinteur sur le cadre correspondant au chemin d'accs et null
	 * sinon
	 * 
	 * @param path
	 * @return
	 */
	/*
	 * supprim version Applet public MtgInternalFrame cadreParChemin(String path)
	 * { JInternalFrame[] tableauCadres = mtgDesktopPane.getAllFrames(); for (int
	 * i = 0; i < tableauCadres.length; i++) { MtgInternalFrame fr =
	 * (MtgInternalFrame) tableauCadres[i]; if (fr.path != null) { if
	 * (fr.path.equals(path)) return fr; } } return null; }
	 */

	/**
	 * Proprits par dfaut du logiciel
	 * 
	 * @return
	 */
	public final Properties defaultProperties() {
		Properties prop = new Properties();
		// Unit d'angle pour les nouvelles figures
		prop.setProperty("angleUnit", "degree");
		// Nombre de points utiliss quand on utilise le menu Mouvoir Animer un
		// point li
		prop.setProperty("moveNbPoints", "100");
		// Frquence utilise quand on utilise le menu Mouvoir Animer un point li
		prop.setProperty("moveFreq", "20");
		// / Animation cyclique ou non
		prop.setProperty("moveCyclic", "true");
		// Rpertoire par dfaut pour l'ouverture des fichiers
		prop.setProperty("fileDirectory", "");
		// Rpertoire par dfaut pour l'ouverture des constructions
		prop.setProperty("constDirectory", "");
		// Niveau defonctionnement
		// prop.setProperty("level", "3"); // Niveau collge par dfaut pour la
		// version Applet Supprim version 4.4.6 car on peut chosir le niveau via le
		// paramtre level
		// Aide  la cration visuelle
		prop.setProperty("visualHelp", "true");
		// Dcalage des affichages libres lors des translations
		prop.setProperty("FreeDisplaysTranslation", "true");
		// Noms automatiques
		prop.setProperty("NomsAuto", "false");
		// Indice de taiile de caractres pour les noms
		prop.setProperty("IndexFontSizeNames", "2");
		// Indice de taiile de caractres pour les noms
		prop.setProperty("IndexFontSizeDisplays", "2");
		// Nombre de colonnes pour la barre d'outils de gauche
		prop.setProperty("NbColToolbar", "5");
		// Petites ou grandes icones
		prop.setProperty("littleIconSize", "true");
		// Ajout pour version 2.0
		prop.setProperty("transparency", "20");
		// Ajout pour version 3.2.4
		prop.setProperty("personalizedLevel", "false");
		// Version 3.2.4 : quatre boolens servant  savoir si on doit afficher les
		// mesures de longuers et angles sur la figure
		// Par dfaut c'est le cas seulement pour les deux premiers niveaux
		// (lmentaire et collge)
		prop.setProperty("measuresDisplay1", "true");
		prop.setProperty("measuresDisplay2", "true");
		prop.setProperty("measuresDisplay3", "true");
		prop.setProperty("measuresDisplay4", "true");
		prop.setProperty("coloredIcons", "true");
		prop.setProperty("antialiasing", "true");
		prop.setProperty("clipboardImagesRatio", "2"); // Ajout version 4.6
		// Version 4.6.4
		prop.setProperty("thicknessDecrease", "false");
		return prop;
	}

	private final void set_pref_language_Systeme() {
		// On regarde quelle est la langue du systme
		locale = Locale.getDefault();
		String language = getParameter("langage");
		if (language == null) {
			language = locale.getDisplayLanguage(new Locale("fr"));
			// Si la langue n'est pas une des langues supportes, on utilise l'anglais
			if (!language.startsWith("fr") && !language.startsWith("es")
					&& !language.startsWith("an"))
				language = "ang";
		} else if (language.startsWith("en"))
			language = "ang";
		if (language.startsWith("fr")) {
			pref_language = "fra";
			locale = new Locale("fr");
		} else if (language.startsWith("es")) {
			pref_language = "esp";
			locale = new Locale("es");
		} else if (language.startsWith("an")) {
			pref_language = "eng";
			locale = new Locale("en");
		}
		messages = ResourceBundle.getBundle("textes.messages", locale);
	}

	/**
	 * Chargement des prfrences de fonctionnement  partir du fichier de
	 * prfrences
	 */
	public final void chargePreferences() {
		// On charge les prfrences de l'application
		properties = defaultProperties();
		// Version Applet : on ne charge pas les prfrences locales de
		// fonctionnement du logiciel
		/*
		 * String fs = System.getProperty("file.separator"); String path; //
		 * Obligation d'intercepter exceptions du type SecuriteException sur
		 * certains systme try { String se = System.getProperty("os.name"); if
		 * (se.startsWith("Linux")) path = System.getProperty("user.home") + fs
		 * +".config" + fs + "/MathGraph32"; else if (se.startsWith("Mac")) path =
		 * System.getProperty("user.home") + fs + ".MathGraph32"; else path =
		 * System.getProperty("user.home") + fs + "MathGraph32"; } catch (Exception
		 * e) { path = System.getProperty("user.home") + fs + "MathGraph32"; }
		 * String ch = path + fs + "MathGraph32Properties"; File file = new
		 * File(ch); if (file.exists()) { FileInputStream in; try { in = new
		 * FileInputStream(file); try { properties.load(in); in.close(); } catch
		 * (IOException e) { properties = new Properties(defaultProperties());
		 * set_pref_langage_Systeme(); } } catch (FileNotFoundException e) {
		 * properties = new Properties(defaultProperties());
		 * set_pref_langage_Systeme(); } pref_langage =
		 * properties.getProperty("langage"); if (pref_langage == null)
		 * set_pref_langage_Systeme(); } else set_pref_langage_Systeme(); // Version
		 * 3.3 : Si le programme tourne pour la premire fois c'est ici qu'il faut
		 * mettre cette ligne // et non plus dans defaultProperties
		 * properties.setProperty("langage", pref_langage);
		 */
		String ch1 = properties.getProperty("angleUnit");
		if (ch1.equals("degree"))
			pref_angleUnit = UniteAngle.Degre;
		else
			pref_angleUnit = UniteAngle.Radian;
		pref_moveNbPoints = Integer
				.parseInt(properties.getProperty("moveNbPoints"));
		pref_moveFreq = Integer.parseInt(properties.getProperty("moveFreq"));
		pref_moveCyclic = Boolean
				.parseBoolean(properties.getProperty("moveCyclic"));
		// Pas utilis version Applet
		/*
		 * pref_fileDirectory = properties.getProperty("fileDirectory");
		 * pref_constDirectory = properties.getProperty("constDirectory");
		 */
		// pref_niveau = Integer.parseInt(properties.getProperty("level"));
		pref_AideVisuelle = Boolean.parseBoolean(properties
				.getProperty("visualHelp"));
		pref_decalageAffichagesLibres = Boolean.parseBoolean(properties
				.getProperty("FreeDisplaysTranslation"));
		pref_NomsAuto = Boolean.parseBoolean(properties.getProperty("NomsAuto"));
		pref_indiceTaillePoliceNom = Integer.parseInt(properties
				.getProperty("IndexFontSizeNames"));
		pref_IndiceTaillePoliceAffichages = Integer.parseInt(properties
				.getProperty("IndexFontSizeDisplays"));
		pref_NbCol = Integer.parseInt(properties.getProperty("NbColToolbar"));
		pref_LittleIconSize = Boolean.parseBoolean(properties
				.getProperty("littleIconSize"));
		// Nouveau version 3.2.4 : personnalisation possible des niveaux
		pref_NiveauPersonnalise = Boolean.parseBoolean(properties
				.getProperty("personalizedLevel"));
		pref_affichageMesuresNiveau[1] = Boolean.parseBoolean(properties
				.getProperty("measuresDisplay4"));
		pref_affichageMesuresNiveau[2] = Boolean.parseBoolean(properties
				.getProperty("measuresDisplay3"));
		pref_affichageMesuresNiveau[3] = Boolean.parseBoolean(properties
				.getProperty("measuresDisplay2"));
		pref_affichageMesuresNiveau[4] = Boolean.parseBoolean(properties
				.getProperty("measuresDisplay1"));

		// On charge aussi le document qui spcifie les outils permis ou interdits
		// nomm
		// default1.mgj pour le niveau lmentaire
		// default2.mgj pour le niveau collge
		// default3.mgj pour le niveau avanc sans complexes
		// default4.mgj pour le niveau avanc avec complexesation
		// On le cherche dans le dossier contenant le jar excutable.
		// Version 3.2.4 : on initialise les chanes de descripteur de niveau qui
		// seront ventuellement
		// modifies dans chargePreferences
		descripteurNiveau[1] = getString("niveauAvanceAvecComplexes");
		descripteurNiveau[2] = getString("niveauAvanceSansComplexes");
		descripteurNiveau[3] = getString("niveauMoyen");
		descripteurNiveau[4] = getString("niveauElementaire");
		// Version 3.7 : Possibilit de regrouper les icnes de mme type par
		// couleur d'encadrement
		pref_ColoredIcons = Boolean.parseBoolean(properties
				.getProperty("coloredIcons"));
		// Version 4.5.1 : Possibilit de choisir d'utiliser le mode antialiasint
		// activ ou non
		pref_antialiasing = Boolean.parseBoolean(properties
				.getProperty("antialiasing"));

		// Modification version 3.2.4 : on charge ventuellement dans ces quatre
		// documents
		// le texte de la figure. S'il n'est pas vide les 80 premiers caractres
		// deviennent le descripteur du niveau
		// Si pref_NiveauPersonnalise est false, les documents chargs sont default1
		//  default4
		// sinon ce sont les documents lelve1  level4 s'ils existent

		for (int i = 1; i <= 4; i++) {
			docPreferencesNiveauPredefini[i] = chargeDocumentDansDossierMtgFrame("files/default"
					+ Integer.toString(5 - i) + ".mgj");
		}
		// Ajout version 4.6
		pref_ClipboardImagesRatio = Double.parseDouble(properties
				.getProperty("clipboardImagesRatio"));

	}

	/**
	 * Sauvegarde des prfrences de fonctionnement dans le fichier de prfrences
	 */
	/*
	 * Supprim version Applet private void sauvePreferences() {
	 * 
	 * // Les quatre prfrences qui ne se font pas via le menu Preferences
	 * properties.setProperty("NomsAuto", Boolean.toString(pref_NomsAuto));
	 * properties.setProperty("visualHelp", Boolean.toString(pref_AideVisuelle));
	 * properties.setProperty("IndexFontSizeNames", Integer
	 * .toString(pref_indiceTaillePoliceNom));
	 * properties.setProperty("IndexFontSizeDisplays", Integer
	 * .toString(pref_IndiceTaillePoliceAffichages));
	 * 
	 * String fs = System.getProperty("file.separator"); String path; try { String
	 * se = System.getProperty("os.name"); if (se.startsWith("Linux")) path =
	 * System.getProperty("user.home") + fs +".config" + fs + "/MathGraph32"; else
	 * if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs +
	 * ".MathGraph32"; else path = System.getProperty("user.home") + fs +
	 * "MathGraph32"; } catch (Exception e) { path =
	 * System.getProperty("user.home") + fs + "MathGraph32"; } String ch = path +
	 * fs + "MathGraph32Properties"; File filePath = new File(path); File file =
	 * new File(ch); FileOutputStream out; try { if (!filePath.exists())
	 * filePath.mkdir(); out = new FileOutputStream(file); try {
	 * properties.store(out, "Mathgraph32 Properties"); out.close(); } catch
	 * (IOException e1) { } } catch (FileNotFoundException e2) { } }
	 */
	/**
	 * Activation de l'icne associe  l'outil choisi
	 * 
	 * @param outil
	 */
	public final void activeIconeAssocieeAction(MtgAction action) {
		int i;
		for (int j = 0; j < pref_NbCol; j++) {
			for (i = 0; i < leftbar[j].getComponentCount(); i++) {
				MtgToggleButton but = (MtgToggleButton) leftbar[j].getComponent(i);
				if (but.action == action) {
					but.setSelected(true);
					return;
				}
			}
		}
		for (int j = 0; j <= 1; j++) {
			for (i = 0; i < topbar[j].getComponentCount(); i++) {
				Component c = topbar[j].getComponent(i);
				if (c.getClass() == MtgToggleButton.class) {
					MtgToggleButton but = (MtgToggleButton) topbar[j].getComponent(i);
					if (but.action == action) {
						but.setSelected(true);
						return;
					}
				} else {
					if (c.getClass() == MtgButton.class) {
						MtgButton but1 = (MtgButton) topbar[j].getComponent(i);
						if (but1.action == action) {
							but1.setSelected(true);
							return;
						}
					}
				}
			}
		}
	}

	/**
	 * Prparation de la barre d'outils de gauche suivant le nombre de colonnes
	 * 
	 * @param ancienNbCol
	 */
	public final void prepareBarreOutilsGauche(int ancienNbCol) {
		for (int i = 0; i < ancienNbCol; i++)
			leftbar[i].removeAll();
		paneLeft.removeAll();
		paneLeft.setLayout(new GridLayout(1, pref_NbCol));
		leftbar = new JToolBar[pref_NbCol];
		for (int i = 0; i < pref_NbCol; i++) {
			leftbar[i] = new JToolBar(JToolBar.VERTICAL);
			leftbar[i].setFloatable(false);
			leftbar[i].setBorder(BorderFactory.createEmptyBorder()); // Ajout version
																																// 3.3 pour que
																																// les icnes
																																// soient moins
																																// espaces
																																// horizontalement
			paneLeft.add(leftbar[i]);
		}
	}

	/**
	 * Gnration automatique d'un nom quand la gnration automatique de noms de
	 * points est active
	 * 
	 * @return
	 */
	public final String genereNomPourPoint() {
		if (pref_NomsAuto)
			return cadreActif().listePr.genereNomPourPoint();
		else
			return "";
	}

	// Ajout version 3.2.4
	public final CMathGrafDoc chargeDocumentDansDossierMtgFrame(String nomFichier) {
		BufferedInputStream buff = null;
		CMathGrafDoc doc = new CMathGrafDoc(this);
		try {
			URL adresse = MtgFrame.class.getResource(nomFichier);
			try {
				buff = new BufferedInputStream(adresse.openStream());
				MathGraphDataInputStream dataStream = new MathGraphDataInputStream(buff);
				doc.chargeDepuisFlux(dataStream, false); // Modifi version 4.4.1
			}
			finally {
				buff.close();			
			}
		} catch (Exception e) {
		}
		return doc;
	}

	// Ajout version 4.3
	public final CPrototype chargeConstructionDansDossierMtgFrame(
			String nomFichier) {
		BufferedInputStream buff = null;
		CPrototype prot = new CPrototype();
		try {
			try {
				URL adresse = MtgFrame.class.getResource("constructions/" + nomFichier);
				buff = new BufferedInputStream(adresse.openStream());
				MathGraphDataInputStream dataStream = new MathGraphDataInputStream(buff);
				prot.read(dataStream);
			}
			finally {
				buff.close();
			}
		} catch (Exception e) {
		}
		;
		return prot;
	}

	/** Mthode chargeant l'applet et appele depuis un thread */
	private final void loadFigureFromURL() {
		BufferedInputStream buff = null;
		MathGraphDataInputStream dataStream = null;
		try {
			// chargementEnCours = true; // Modifi version 4.6. Mis au dbut de
			// init()
			// On regarde si un chargement d'une nouvelle figure a t demand via
			// chargeFigure
			String chAcces = cheminAcces();
			URL adresse = new URL(getDocumentBase(), path);
			MtgInternalFrame cadre = cadreActif();
			CMathGrafDoc doc = cadre.document;
			try {
				buff = new BufferedInputStream(adresse.openStream());
				dataStream = new MathGraphDataInputStream(buff);
				doc.chargeDepuisFluxFromURL(dataStream);
			}
			finally {
				dataStream.close(); // Ajout version 4.6.2 pour librer des ressources
				buff.close(); // Ajout version 4.6.2 pour librer des ressources
			}
			CListeObjets listePr = doc.listePrincipale;
			// On regarde si une couleur de fond est spcifie dans la figure.
			// listePr.setCopy(listeClone);
			// listePr.setCopy(listeSauve);
			// listeDependantsDansPrincipale = new CListeObjets(listePr);
			CListeObjets listeMacrosJouantSon = new CListeObjets(listePr);
			listeMacrosJouantSon.ajouteMacrosJouantSon(listePr);
			// Si ncessaire on charge les fichiers sonores des macros jouant des sons
			if (listeMacrosJouantSon.size() != 0) {
				CListeObjets list = listeMacrosJouantSon;
				ListIterator<CElementBase> li = list.listIterator();
				while (li.hasNext()) {
					CMacroJouantSon mac = (CMacroJouantSon) (li.next());
					mac.audioClip = getAudioClip(getDocumentBase(), chAcces
							+ mac.wavePath);
				}
			}
			// Dplac version 4.7.2
			// listePr.setCopy(cadre.listeSauve); // Spcial version applet : On
																					// sauvegarde la figure initiale pour
																					// pouvoir y revenir
			cadre.listeConstruction.add(listePr.get(0)); // Spcial version applet :
																										// Constante pi de la liste
																										// principale
			cadre.macroPourCons = macroPourConstruction();
			cadre.isConstruction = (cadre.macroPourCons != null);
			cadre.macroPourCalculs = macroCalculsAutorises();
			cadre.isExercise = (cadre.macroPourCalculs != null);
			// chargementEnCours = false; // Modifi version 4.6. Mis  la fin de
			// Init()
		} catch (Exception e) {
			chargementEnCours = false;
			e.printStackTrace();
			// dialogueAvertissement(getString("ch35")); // Modifi version 4.6.2
		}
	}

	private void loadFigureFromDataBase64(String data) {
		CMathGrafDoc doc;
		MathGraphDataInputStream inps = null;
		MtgInternalFrame cadre = cadreActif();
		ByteArrayInputStream ba = new ByteArrayInputStream(
				Kernel.base64Decode(data));
		try {
			// chargementEnCours = true; // Modifi version 4.6. Mis au dbut de
			// init()
			try { // Ajout version 4.6.2
				inps = new MathGraphDataInputStream(ba);
				doc = cadre.document;
				doc.chargeDepuisFlux(inps, true); // true car chargement en ligne des
																					// anciennes images ou macro jouant son
			}
			finally {
				inps.close();
			}
			String chAcces = cheminAcces();
			CListeObjets listePr = doc.listePrincipale;
			CListeObjets listeMacrosJouantSon = new CListeObjets(listePr);
			listeMacrosJouantSon.ajouteMacrosJouantSon(listePr);
			// Si ncessaire on charge les fichiers sonores des macros jouant des sons
			if (listeMacrosJouantSon.size() != 0) {
				CListeObjets list = listeMacrosJouantSon;
				ListIterator<CElementBase> li = list.listIterator();
				while (li.hasNext()) {
					CMacroJouantSon mac = (CMacroJouantSon) (li.next());
					mac.audioClip = getAudioClip(getDocumentBase(), chAcces
							+ mac.wavePath);
				}
			}
			// Dplac version 4.7.2
			// listePr.setCopy(cadre.listeSauve); // Spcial version applet : On
																					// sauvegarde la figure initiale pour
																					// pouvoir y revenir
			cadre.listeConstruction.add(listePr.get(0)); // Spcial version applet :
																										// Constante pi de la liste
																										// principale
			cadre.macroPourCons = macroPourConstruction();
			cadre.isConstruction = (cadre.macroPourCons != null);
			cadre.macroPourCalculs = macroCalculsAutorises();
			cadre.isExercise = (cadre.macroPourCalculs != null);
			// chargementEnCours = false; // Modifi version 4.6. Mis  la finde
			// init()
		} catch (IOException e) {
			// TODO Auto-generated catch block
			chargementEnCours = false;
			e.printStackTrace();
			// dialogueAvertissement(getString("ch35")); // Modifi version 4.6.2
		}
	}

	/** Fonction renvoyant le chemin d'accs  la figure sans le nom */
	String cheminAcces() {
		String ch = getParameter("Path");
		if (ch == null)
			return "";
		int i = ch.lastIndexOf('/');
		if (i == -1)
			return "";
		else
			return ch.substring(0, i + 1);
	}

	// Fonctions spcifiques version Applet
	public final String getScript() {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(2000);
		MathGraphDataOutputStream oups = new MathGraphDataOutputStream(baos);
		// On enregistre la liste d'objets principale dans l'archive
		try {
			cadreActif().document.write(oups);
			oups.flush();
			oups.close();
			// ch = baos.toString();
			baos.flush();
			baos.close();
			byte[] ba = baos.toByteArray();
			// return Kernel.base64Encode(ba);
			return Kernel.base64Encode(ba);
		} catch (IOException e) {
			return "";
		}
	}

	public final void setScript(String xmlString, String paramCallBack) {
		CMathGrafDoc doc;
		// stop(); // Supprim version 4.6
		isReady = false; // Ajout version 4.6.1 : Sera mis  true dans paint de MtgFigurePanel lors du premier affichage
		chargementEnCours = true;
		MtgInternalFrame cadre = cadreActif();
		// Lignes suivantes ajoutes version 4.6.2
		if (cadre.timer != null) {
			cadre.timer.cancel();
			cadre.timer.purge();
			cadre.timer = null;
		}
		// cadre.timerTask = null; // Supprim version 4.6.2
		cadre.macroEnCours = null;
		if( cadre.timerTask != null) {
			cadre.timerTask.cancel();
			cadre.timerTask = null;
		}
		// cadre.macroAnimation = null;
		// cadre.timer = null; // Supprim version 4.6.2
		// On dtruit le document actuel
		cadre.retireComposants(); // Ncessaire pour que si on charge une autre
															// figure via setXML les anciens
															// champs d'dition et dialogues associs 
															// variables soient dtruits.
		cadre.listePr.retireTout();
		// On charge le document  partir de la chane de caractres hexadecimal
		// passe en argument
		// On traduit la chane de caractres codant la figure en base 64 en un
		// tabelau de bytes

		ByteArrayInputStream ba = new ByteArrayInputStream(
				Kernel.base64Decode(xmlString));
		MathGraphDataInputStream inps = null;
		try {
			try {
				inps = new MathGraphDataInputStream(ba);
				doc = new CMathGrafDoc(this);
				doc.chargeDepuisFlux(inps, true); // Modif version 4.4.1 true car on ne
																					// charge pas les anciennes images et
																					// CMacroJouantSon
			}
			finally {
				inps.close();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			doc = cadre.document;
			// dialogueAvertissement("Error while loading document"); // Modifi version 4.6.2
			e.printStackTrace();
		}
		cadre.setDocument(doc);
		CListeObjets listePr = doc.listePrincipale;
		labelUniteAngle.setText((listePr.uniteAngle == UniteAngle.Degre) ? getString("radioDegre")
						: getString("radioRadian"));
		listePr.setCopy(cadre.listeSauve); // Spcial version applet : On sauvegarde
																				// la figure initiale pour pouvoir y
																				// revenir
		// Pour setXML qui peut tre appel aprs qu'une autre figure ait t charge il faut rinitialiser
		// la liste des objets construits. Version 4.6.1
		cadre.listeConstruction.retireTout();
		// Modification version 4.7.2.6
		if (allowExercise) {
			etablitListeObjetsConstruits(); // Appelle etablitListeCalculsAutorises
		}
		else cadre.listeConstruction.add(listePr.get(0)); // Spcial version applet :
																									// Constante pi de la liste
																									// principale
		cadre.macroPourCons = macroPourConstruction();
		cadre.isConstruction = (cadre.macroPourCons != null);
		if (allowExercise && cadreActif.isConstruction) {
			cadre.listeAConstruire = cadre.macroPourCons.listeAssociee;
			chaineIndicationPourCons = cadre.macroPourCons.commentaireMacro;
			ajouteBoutonPropSol();
		}
		cadre.macroPourCalculs = macroCalculsAutorises();
		cadre.isExercise = (cadre.macroPourCalculs != null);
		try {
			cadre.gestionnaire.initialise(); // Rajout version 4.6. Important !
		} catch (IOException e) {
			// dialogueAvertissement("Error initialising setScript"); // Modifi version 4.6.2
			e.printStackTrace();
		}
		// creeMenu(); Inutile se fait dans paint() de MtgFigurePanel
		// metAJourIconesEtMenus(doc); Se fait dans paint de MtgInternalFrame
		premierAffichage = true; // C'est dans paint() de MtgFigurePanel que
															// s'initialisent les barres d'outil
		// C'est aussi dans paint de MtgFigurePanel que isReady sera mis  true lors du premier affichage
		chargementEnCours = false;
		cadreActif().creePaneAffichageVariables(); // Car la figure peut contenir
																								// des variables avec un panneau
																								// associ
		// On adapte le bouton de choix de transparence
		paneTransparence.donneValeurTransparence(doc.transparencePourSurfaces,
				cadreActif()); // Ajout version 4.6

		// associes  des dialogues
		// Modifi pour la version applet : on initialise par l'outil de capture
		// dans tous les cas
		// if (!cadre.isConstruction && !cadre.isExercise &&
		// (cadre.listePr.nombreObjetsParNature(NatureObjet.NMacro) != 0))
		// activeOutilExecutionMacro();
		// else activeOutilCapture();
		// activeOutilCapture();
		// start(); Supprim version 4.6
		// cadre.redessineAvecCalcul(); // Supprim 30:4/2012
		// repaint();
		validate();
		repaint(); // Rajout 30/4/2012 pour essayer d'viter les problmes avec
								// FireFox
		// Pour LaboMep : si tout s'est bien pass applet de mtgCallback() du JavaScript de la page possdant l'applet
		try {
			JSObject jso = JSObject.getWindow(this);
			Object[] arg = new Object[1];
			arg[0] = paramCallBack;
			jso.call("mtgCallback", arg);
		} catch (Exception e) {
			// Pas de message si la fonction Javascript mtgCallcBack est absente.
			}
	}

	public static String getString(String key) {
		try {
			return messages.getString(key);
		} catch (Exception e) {
			return "!" + key + "!";
		}
	}

	/**
	 * Spcial Applet
	 * 
	 * @return Fonction renvoyant null si la figure ne contient pas un exercice de
	 *         construction. Dans le cas contraire, la figure doit contenir une
	 *         macro d'apparition d'objet nomme SolutionConstruction dont la
	 *         liste d'objets  faire apparatre sera en fait la liste des objets
	 *         que l'utilisateur de l'applet doit construire. le commentaire de
	 *         cette macro contiendra des informations que l'on affichera dans la
	 *         ligne d'indication pour dire quel est le type d'objet que
	 *         l'utilisateur doit cliquer. Chacun commencera par un caractre du
	 *         type #1: suivi de l'indication  afficher
	 */
	private CMacroApparition macroPourConstruction() {
		ListIterator<CElementBase> li = cadreActif().listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) li.next();
			if (elb.getClass() == CMacroApparition.class) {
				CMacroApparition mac = (CMacroApparition) elb;
				if ((mac.intitule
						.compareToIgnoreCase(getString("solutionConstruction")) == 0)
						|| (mac.intitule.compareToIgnoreCase("SolutionConstruction") == 0))
					return mac;
			}
		}
		return null;
	}

	/*
	 * Spcial Applet version 4.6 : Renvoie si elle existe un pointeur sur une
	 * macro de pause dont le commentaire contiendra les noms des calculs
	 * autoriss  l'utilisateur
	 */
	private CMacroPause macroCalculsAutorises() {
		ListIterator<CElementBase> li = cadreActif().listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) li.next();
			if (elb.getClass() == CMacroPause.class) {
				CMacroPause mac = (CMacroPause) elb;
				// Modifi version 4.7.2 . Si une macro de pause nomme calculs est prsente, elle fournit dans son commentaire
				// la liste des calculs que l'lve peut utiliser contenus dans des accolades
				/*
				if ((mac.intitule.compareToIgnoreCase("exercice") == 0)
						|| (mac.intitule.compareToIgnoreCase(getString("exercice")) == 0)) return mac;
				*/
				if ((mac.intitule.compareToIgnoreCase("calcul") == 0)
						|| (mac.intitule.compareToIgnoreCase(getString("calcul")) == 0)) return mac;
			}
		}
		return null;
	}

	public void etablitListeCalculsAutorises() {
		if (cadreActif.isExercise) {
			cadreActif.listeCalculsAutorises.retireTout();
			// Modifi version 4.7
			// CMacro mac;
			// if (cadreActif.isConstruction) mac = cadreActif.macroPourCons;
			// else mac = cadreActif.macroPomacurCalculs;
			CMacro mac = cadreActif.macroPourCalculs;
			ListIterator<CElementBase> li = cadreActif.listePr.listIterator();
			while (li.hasNext()) {
				CElementBase elb = li.next();
				if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculNommeSaufConst)) {
					CCalculAncetre calc = (CCalculAncetre) elb;
					if (mac.commentaireMacro.indexOf("{" + calc.nomCalcul + "}") != -1) {
						cadreActif.listeCalculsAutorises.add(calc);
						// cadreActif.listeConstruction.add(calc); Modifi version 4.6
					}
				}
			}
		}
	}

	/**
	 * Spcial version applet version 4.6: Renvoie un pointeur sur la liste
	 * utilise pour les constructions suivant que l'applet sert ou non  faire un
	 * exercice de construction
	 * 
	 * @return
	 */
	public CListeObjets listePourConstruction() {
		MtgInternalFrame cadre = cadreActif();
		/* Modifi version 4.7.2
		if ((cadre.isConstruction || cadre.isExercise) && allowExercise)
		*/
		if (allowExercise)
			return cadre.listeConstruction;
		else
			return cadre.listePr;
	}

	/**
	 * Fonction externe spcial applet  appeler pour recharger la figure  son
	 * tat initial.
	 */
	public final void reset() {
		MtgInternalFrame cadre = cadreActif();
		if (cadre.macroEnCours != null) {
			cadre.macroEnCours.termineAction(cadre);
			// cadre.timerTask = null; // Supprim version 4.6.2
			if (cadre.timer != null) {
				cadre.timer.cancel();
				cadre.timer.purge();
				cadre.timer = null;
			}
			// Lignes suivantes ajoutes version 4.6.2
			if (cadre.timerTask != null) {
				cadre.timerTask.cancel();
				cadre.timerTask = null;
			}
			// Fin ajout 4.6.2
			if (cadre.runner != null)
				cadre.runner = null;
			cadre.macroEnCours.termineAction(cadre);
			cadre.macroEnCours = null;
		}
		cadre.listeConstruction.retireTout();
		cadre.listePr.retireTout();
		cadre.listeSauve.setCopy(cadre.listePr);
		cadre.listePr.setClone(cadre.listeSauve);
		etablitListeObjetsConstruits();
		// premierAffichage = true;
		activeOutilCapture();
		cadre.repaintFigureAvecCalcul();
	}

	/**
	 * Fonction renvoyant une chane permettant  l'utilisateur de savoir quel est
	 * l'objet ni qu'il doit dsigner quand il dsigne les objets qu'on lui a
	 * demand de construire
	 * 
	 * @param i
	 * @return
	 */
	public String indicationSol(int i) {
		String ret = "";
		String ch = "#" + glob.Kernel.chaineNombre(i, 0) + ":";
		int j = chaineIndicationPourCons.indexOf(ch);
		if (j != -1) {
			int k = chaineIndicationPourCons.indexOf("#", j + ch.length());
			if (k != -1) {
				ret = chaineIndicationPourCons.substring(j + ch.length(), k);
			} else
				ret = chaineIndicationPourCons.substring(j + ch.length());
			// Recherche de retour chariot et saut
			int m = ret.indexOf("\r\n");
			if (m != -1)
				ret = ret.substring(0, m);
			// On ne prend que les 80 premiers caractres
			if (ret.length() > 80)
				ret = ret.substring(0, 81);
		}
		return ret;
	}

	/**
	 * 
	 * @return true si les objets qui ont t cliqus et sont contenus dans
	 *         listeProposee sont confondus avec ceux qu'ils faut construire
	 *         contenus dans listeAConstruire
	 */
	private boolean reponseValideUnePasse() {
		boolean res = true;
		MtgInternalFrame cadre = cadreActif();
		CListeObjets listeAConstruire = cadre.listeAConstruire;
		int nbo = listeAConstruire.size();
		for (int i = 0; (i < nbo) && res; i++) {
			CElementBase elb1 = (CElementBase) listeAConstruire.get(i);
			boolean res2 = false;
			for (int j = 0; (j < nbo) && !res2; j++) {
				CElementBase elb2 = (CElementBase) cadre.listeProposee.get(j);
				res2 = res2 || elb1.coincideAvec(elb2);
			}
			res = res && res2;
		}
		return res;
	}

	private void epsilon(CPointLie ptl) {
		double absmin = ptl.abscisseMinimale();
		double absmax = ptl.abscisseMaximale();
		// on dcale l'abscisse au maximum de 5% vers la haut ou vers la bas au
		// hasard
		double a = Math.floor(Math.random() * 5) + 1;
		double s1 = Math.floor(Math.random() * 2);
		if (s1 == 0)
			a = -a;
		double nabs = ptl.abscisse + a * (absmax - absmin) / 10000;
		// if (nabs > absmax) nabs = absmax;
		// else if (nabs < absmin) nabs = absmin;
		if ((nabs > absmin) && (nabs < absmax))
			ptl.donneAbscisse(nabs);
		ptl.donneAbscisse(nabs);
	}

	private void epsilon(CPointBase ptb) {
		double x1 = 0;
		double y1 = 0;
		double a = (Math.floor(Math.random() * 5) + 1) * 0.001;
		double b = (Math.floor(Math.random() * 5) + 1) * 0.001;
		double s1 = Math.floor(Math.random() * 2);
		double s2 = Math.floor(Math.random() * 2);
		if (s1 == 0)
			x1 = a;
		else
			x1 = -a;
		if (s2 == 0)
			y1 = b;
		else
			y1 = -b;
		ptb.placeEn(ptb.x + x1, ptb.y + y1);
	}

	public boolean reponseValide() {
		// On regarde d'abord si les objets proposs coincident bien avec ceux 
		// construire
		CListeObjets listePr = cadreActif.listePr;
		CListeObjets listeSauve = cadreActif.listeSauve;
		CListeObjets listeAConstruire = cadreActif.listeAConstruire;
		listeSauve.positionne(cadreActif.infRand, cadreActif.dimf);
		// On clone l'tat actuel de la liste
		CListeObjets listeClone = new CListeObjets();
		listePr.setCopy(listeClone);
		listeClone.positionne(cadreActif.infRand, cadreActif.dimf); // Employer
																																// pasoitionne
																																// et pas
																																// setClone 
																																// cause des
																																// leiux
																																// d'objets
		boolean res = reponseValideUnePasse();
		if (!res)
			return false;
		// La liste listeClone contient des clones des objets dans l'tat o ils
		// sont actuellement
		// On remet les lments du texte de la figure dans l'tat o ils taient au
		// chargement
		// de la figure.
		// C'est ncessaire car la mthode de vrification va dplacer lgrement
		// les points mobiles pour tester la validit de la rponse et ils faut que
		// la
		// construction reste possible lors de ces dplacements
		listePr.setClone(listeSauve); // Le clonage n'est fait que pour les lments
																	// initiaux de la figure
		listePr.positionne(cadreActif.infRand, cadreActif.dimf);
		CListeObjets listePtsLibres = new CListeObjets();
		CListeObjets listePtsLies = new CListeObjets();
		int nbo = listeSauve.size();
		for (int j = 0; j < nbo; j++) {
			CElementBase elb = (CElementBase) listePr.get(j);
			if ((elb.getNature() == NatureObjet.NPointBase)
					&& (listeAConstruire.dependDe(elb)))
				listePtsLibres.add(elb);
			if ((elb.getNature() == NatureObjet.NPointLie)
					&& (listeAConstruire.dependDe(elb)))
				listePtsLies.add(elb);
		}
		// On va 3 fois de suite dplacer un peu les points libres et lis dont
		// dpendent
		// les objets  construire et qui font partie de la figure initiale et
		// regarder
		// si la solution reste valide. Si oui elle sera accepte sinon refuse
		// listeClone.setClone(listePr);
		for (int i = 0; (i < 3) && res; i++) {
			// On dplace un peu les points libres et lis
			ListIterator<CElementBase> li1 = listePtsLibres.listIterator();
			while (li1.hasNext()) {
				CPointBase ptb = (CPointBase) li1.next();
				epsilon(ptb);
			}
			ListIterator<CElementBase> li2 = listePtsLies.listIterator();
			while (li2.hasNext()) {
				CPointLie ptl = (CPointLie) li2.next();
				epsilon(ptl);
			}
			listePr.positionne(cadreActif.infRand, cadreActif.dimf);
			res = res && reponseValideUnePasse();
			listePr.setClone(listeSauve);
			listePr.positionne(cadreActif.infRand, cadreActif.dimf);
		}
		// On remet les objets dans la position o les a mis l'utilisateur
		listePr.setClone(listeClone);
		return res;
	}

	public boolean isConstructionExercise() {
		return cadreActif.isConstruction;
	}

	/**
	 * Fonction retirant tous les objets de la liste utilise pour proposer une
	 * solution
	 * 
	 */
	public void discardSolutionSubmitted() {
		cadreActif.listeProposee.retireTout();
	}

	/**
	 * Fonction renvoyant true lorsque l'exercice n'est pas un exercice de
	 * construction ou lorsque l'exercice est unn exercice de construction et que
	 * l'utilisateur a bien dsign tous les objets qu'il pense tre ceux qu'on
	 * lui demandait de construire
	 * 
	 * @return
	 */
	public boolean solutionSubmitted() {
		if (!cadreActif.isConstruction)
			return true;
		return (cadreActif.listeAConstruire.size() == cadreActif.listeProposee
				.size());
	}

	// Pour compatibilit avec l'ancienne version
	public boolean solutionProposee() {
		return solutionSubmitted();
	}

	/**
	 * Version 1.8 : Mthode publique destine  connaitre la valeur d'un calcul
	 * Renvoie -1 si le calcul n'est pas dfini ou n'existe pas. Sinon renvoie sa
	 * valeur
	 * 
	 * @param nomCalcul
	 * @return
	 */
	public final double valeurDe(String nomCalcul) {
		// Si l'exercice est une exercice de construction et si on demande
		// la valeur d'un calcul nomm reponse qui en fait n'existe pas on revoie
		// true
		// si les objets dsigns avec l'outil outilPropSol sont les bons
		if (allowExercise && cadreActif.isConstruction
				&& nomCalcul.equalsIgnoreCase("reponse")) {
			if (constructionExacte)
				return 1;
			else
				return 0;
		}
		CCalculAncetre calcul = cadreActif.listePr.pointeurCalculReelNomme(
				nomCalcul, -1);
		if (calcul == null)
			return -1;
		// Pas de retirage alatoire
		else {
			if (!calcul.existe)
				return -1;
			else
				return calcul.rendValeur(false);
		}
	}

	public final double valueOf(String nameCalculus) {
		return valeurDe(nameCalculus);
	}

	/**
	 * Version rectifie le 18/3/09 Le troisime paramtre n'est plus utilis et
	 * n'est gard que pour compatibilit avec les versions antrieures Version
	 * 1.8 : Mthode publique donnant la valeur correspond au contenu de la chane
	 * chCalcul au calcul ou  la fonction de nom nomCalcul. Renvoie 1 si tout
	 * s'est bien pass. Renvoie 0 si mne  un calcul n'existant pas Renvoie -1
	 * s'il y a une faute de syntaxe Renvoie -2 si le calcul ou la fonction
	 * n'existe pas. La figure n'est pas repositionnes et raffiche aprs
	 * l'affectation de la valeur  la variable. Il faut pour cela appeler
	 * redessine()
	 * 
	 * @param nomCalcul
	 * @param chCalcul
	 * @param variableFormelle
	 * @return
	 */

	public final int donneValeurA(String nomCalcul, String chCalcul) {
		@SuppressWarnings("unused")
		double v;
		CCalculBase calc;
		CListeObjets listePr = cadreActif.listePr;
		// On remplace les virgules par des points dcimaux
		chCalcul = chCalcul.replace(',', '.');
		CCalcul valeur = listePr.pointeurCalculOuFonction(nomCalcul, -1);
		if (valeur == null)
			return -2;
		else {
			String[] tabvarf = null;
			boolean bFonction = false;
			if (valeur.getNatureCalcul() == NatureCalcul.NFonction) {
				bFonction = true;
				tabvarf = new String[1];
				tabvarf[0] = ((CFonc) valeur).chaineVariable;
			} else {
				if (valeur.estDeNatureCalcul(NatureCalcul.NFonction2Var
						| NatureCalcul.NFonction3Var)) {
					bFonction = true;
					CFoncNVar foncnvar = (CFoncNVar) valeur;
					int nbvar = foncnvar.nombreVariables();
					tabvarf = new String[nbvar];
					for (int k = 0; k < nbvar; k++)
						tabvarf[k] = foncnvar.chaineVariable[k];
				}
			}
			Int indErr = new Int();
			if (!CCalculBase.verifieSyntaxe(listePr, chCalcul, indErr,
					listePr.indexOf(valeur) - 1, tabvarf))
				return -1;
			if (bFonction) {
				calc = CCalculBase.creeCalculBase(chCalcul, listePr, 0,
						chCalcul.length() - 1, tabvarf);
				valeur.chaineCalcul = chCalcul;
				valeur.calcul = calc;
				listePr.recalculeDeriveesDependantDe(valeur);
				return 1;
			} else {
				calc = CCalculBase.creeCalculBase(chCalcul, listePr, 0,
						chCalcul.length() - 1, tabvarf);
				try {
					v = calc.resultat(false);
				} catch (ErreurCalculException e) {
					return 0;
				}
				valeur.chaineCalcul = chCalcul;
				valeur.calcul = calc;
				return 1;
			}
		}
	}

	/**
	 * public method giving formula in chCalcul to object named chCalcul This
	 * object may be a real calculus or a real function of one too three variables
	 * 
	 * @param nomCalcul
	 * @param chCalcul
	 * @return 1 if no error Renvoie 0 if leads to a calculus non existing Renvoie
	 *         -1 if there is a syntax error Renvoie -2 if there is no object
	 *         named nomCalcul
	 */
	public final int giveFormulaTo(String nomCalcul, String chCalcul) {
		return donneValeurA(nomCalcul, chCalcul);
	}

	/**
	 * public method giving formula in chCalcul to object named chCalcul This
	 * object may be a complex calculus or a complex function of one too three
	 * variables
	 * 
	 * @param nomCalcul
	 * @param chCalcul
	 * @return 1 if no error Renvoie 0 if leads to a calculus non existing Renvoie
	 *         -1 if there is a syntax error Renvoie -2 if there is non object
	 *         named nomCalcul
	 */
	public final int giveComplexFormulaTo(String nomCalcul, String chCalcul) {
		@SuppressWarnings("unused")
		double v;
		CCalculBase calc;
		CListeObjets listePr = cadreActif.listePr;
		// On remplace les virgules par des points dcimaux
		chCalcul = chCalcul.replace(',', '.');
		CCalcul valeur = listePr.pointeurCalculOuFonctionComplexe(nomCalcul, -1);
		if (valeur == null)
			return -2;
		else {
			String[] tabvarf = null;
			boolean bFonction = false;
			if (valeur.getNatureCalcul() == NatureCalcul.NFonctionComplexe) {
				bFonction = true;
				tabvarf = new String[1];
				tabvarf[0] = ((CFoncComplexe) valeur).chaineVariable;
			} else {
				if (valeur.estDeNatureCalcul(NatureCalcul.NFonctionComplexe2Var
						| NatureCalcul.NFonctionComplexe3Var)) {
					bFonction = true;
					CFoncComplexeNVar foncnvar = (CFoncComplexeNVar) valeur;
					int nbvar = foncnvar.nombreVariables();
					tabvarf = new String[nbvar];
					for (int k = 0; k < nbvar; k++)
						tabvarf[k] = foncnvar.chaineVariable[k];
				}
			}
			Int indErr = new Int();
			if (!CCalculBase.verifieSyntaxeComplexe(listePr, chCalcul, indErr,
					listePr.indexOf(valeur) - 1, tabvarf))
				return -1;
			if (bFonction) {
				calc = CCalculBase.creeCalculBaseComplexe(chCalcul, listePr, 0,
						chCalcul.length() - 1, tabvarf);
				valeur.chaineCalcul = chCalcul;
				valeur.calcul = calc;
				listePr.recalculeDeriveesDependantDe(valeur);
				return 1;
			} else {
				calc = CCalculBase.creeCalculBaseComplexe(chCalcul, listePr, 0,
						chCalcul.length() - 1, tabvarf);
				try {
					v = calc.resultat(false);
				} catch (ErreurCalculException e) {
					return 0;
				}
				valeur.chaineCalcul = chCalcul;
				valeur.calcul = calc;
				return 1;
			}
		}
	}

	/**
	 * public method redrawing the figure without recalculating it
	 */
	public final void redraw() {
		// cadreActif.redessineSansCalcul(); Modifi version 4.6
		aRedessiner = true;
		cadreActif.paneFigure.repaint();
	}

	/**
	 * public method recalculating the figure.
	 * 
	 * @param updateRandom
	 *          : if true, calls to rand function will be updated
	 */
	public final void recalculateAndRedraw(boolean updateRandom) {
		// cadreActif.infRand = true; // Dfait corrig version 4.6
		cadreActif.infRand = updateRandom;
		aRecalculerEtRedessiner = true;
		cadreActif.paneFigure.repaint();
	}

	// Ajout version 4.6
	/**
	 * Fonction publique renvoyant 0 si un des champs d'dition de la figure a une
	 * faute soit de syntaxe soit donnant un calcul non existant Si tous les
	 * champs sont valides, recalcule la figure et renvoie 1. Un champ vide est
	 * considr comme valide. Si la figure ne content pas de champ d'dition,
	 * renvoie -1.
	 * 
	 * @return
	 */
	public final int fieldsValidation() {
		ListIterator<CElementBase> li = cadreActif.listePr.listIterator();
		boolean trouve = false;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				if (!ed.mtgFormulaEditor.validation())
					return 0;
				else
					trouve = true;
			}
		}
		// Si tous les champs sont valides on recalcule et redessine la figure
		if (trouve) {
			recalculateAndRedraw(false);
			return 1;
		} else
			return -1;
	}

	// Ajout version 4.6
	/**
	 * Fonction regardant si un ou plusieurs champs d'dition sont associs  un
	 * diteur de formule associ  un calcul ou une fonction nomm nomCalcul Si
	 * oui, renvoie 1 si la rponse est valide et non vide et 0 sinon. Renvoie - 1
	 * s'il n'y a pas de champ associ  ce calcul ou  cette fonction.
	 */
	public final int fieldValidation(String nomCalcul) {
		ListIterator<CElementBase> li = cadreActif.listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				if (ed.calculAssocie.nomCalcul.equals(nomCalcul)) {
					MtgFormulaEditor mtgfe = ed.mtgFormulaEditor;
					// A noter :validation recalcule la figure mais ne la raffiche pas
					if ((mtgfe.getText().isEmpty()) || !mtgfe.validation())
						return 0;
					else {
						recalculateAndRedraw(false);
						return 1;
					}
				}
			}
		}
		// Si pas de champ associ on renvoie -1
		return -1;
	}

	// Ajout version 4.6
	/**
	 * Fonction publique donnant le focus au premier champ d'dition invalide s'il
	 * en existe.
	 */
	public final void activateFirstInvalidField() {
		ListIterator<CElementBase> li = cadreActif.listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				if (!ed.mtgFormulaEditor.validation()) {
					ed.mtgFormulaEditor.requestFocus();
					break;
				}
			}
		}
	}

	// Ajout version 4.6
	/**
	 * Fonction publique donnant le focus au premier champ d'dition invalide
	 * associ au calcul nomm nomCalcul s'il en existe.
	 */
	public final void activateField(String nomCalcul) {
		ListIterator<CElementBase> li = cadreActif.listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				if (ed.calculAssocie.nomCalcul.equals(nomCalcul)) {
					ed.mtgFormulaEditor.requestFocus();
					break;
				}
			}
		}
	}

	/**
	 * Marque le champ d'dition associ au calcul nomm nomCalcul comme invalide
	 * en l'entourat de rouge.
	 * 
	 * @param nomCalcul
	 */
	public void setInvalidField(String nomCalcul) {
		ListIterator<CElementBase> li = cadreActif.listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				if (ed.calculAssocie.nomCalcul.equals(nomCalcul)) {
					ed.mtgFormulaEditor.marquePourErreur();
					ed.mtgFormulaEditor.validate();
					break;
				}
			}
		}
	}

	public final void setLevel(int level) {
		if ((level >= 1) && (level <= 4)) {
			pref_niveau = 5 - level;
			// creeMenu();
			setMenuVisible(allowMenuBar);
			metAJourIconesEtMenus(cadreActif().document);
			recalculateAndRedraw(false); // Modifi version 4.7.2.1 pour envoyer un oordre repaint
		}
	}

	public final void setLeftToolbarVisible(boolean visible) {
		allowLeftToolbar = visible;
		paneLeft.setVisible(visible);
		validate();
	}

	public final void setMenuVisible(boolean visible) {
		allowMenuBar = visible;
		menuBar.setVisible(visible);
		validate();
	}

	public final void setTopToolbarVisible(boolean visible) {
		allowTopToolbar = visible;
		paneNorth.setVisible(visible);
		validate();
	}

	public final void setRightToolbarVisible(boolean visible) {
		allowRightToolbar = visible;
		paneRight.setVisible(visible);
		validate();
	}

	public final void setIndicationAreaVisible(boolean visible) {
		allowIndicationArea = visible;
		paneSouth.setVisible(visible);
		validate();
	}

	/**
	 * Mthode publique destine  excuter une macro via JavaScript. Prend comme
	 * paramtre l'initul de la mthode (doit respecter la casse et les espaces)
	 * Renvoie true si l'excution a public avoir lieu et false sinon
	 * 
	 * @param nameMacro
	 * @return
	 */
	public final boolean executeMacro(String nameMacro) {
		// Refus si la macro possde une macro par dfaut qui est une macro
		// d'animation
		// if (macroAnimation != null) return false; // Supprim version logiciel
		// dans Applet
		// Modifi version 4.6 : Refus aussi si une autre macro est en cours
		// d'excution.
		// if (macroAnimation != null) return false;
		if (cadreActif.macroEnCours != null)
			return false;
		// On cherche si NomMacro contient bien un intitul de macro valide
		CMacro macroAExecuter = cadreActif.listePr.pointeurMacro(nameMacro);
		if (macroAExecuter == null)
			return false;
		/*
		 * Supprim version 4.6 if (macroEnCours != null) { // Si aucune macro n'est
		 * d?j? en cours d'ex?cution // on lance l'ex?cution de la macro clique si
		 * possible CMacro mac = macroEnCours.macroEnCours();
		 * mac.termineAction(cadreActif); macroEnCours.termineAction(cadreActif); }
		 */
		if (macroAExecuter.executionPossible()) {
			cadreActif.macroEnCours = macroAExecuter;
			macroAExecuter.initialise(cadreActif);
			CMacro mac = macroAExecuter.macroEnCours();
			mac.actionAvantExecution(cadreActif);
			// Si la macro utilise un thread, il faut le crer.
			// ? revoir
			mac.initialiseInterruptions(cadreActif);

			// if (macroEnCours.macroEnCours().typeAnimation() ==
			// CMacro.sansAnimation) repaint(); Modifi version 4.6
			if (mac.typeAnimation() == CMacro.sansAnimation)
				cadreActif.repaintFigureImmediatly();
			return true;
		} else
			return false;
	}

	public final boolean macroEnCours() {
		return executingMacro();
	}

	public final boolean executingMacro() {
		return (cadreActif.macroEnCours != null);
	}

	// Version 1.8 : Mthode publique destine  savoir si la figure est bien
	// charg?e
	public final boolean figureChargee() {
		return !chargementEnCours;
	}

	public final boolean figureLoaded() {
		return figureChargee();
	}

	/**
	 * Ajout pour la version 2.5.4 pour sauvegarder les valeurs dans une applet de
	 * validation interne.
	 */
	public final void sauvegarde() {
		if (action.equals("Save")) {
			appletManager.sauveValeurs(this);
		}
	}

	/**
	 * public method saving values or formulas of calculuses or functions named z1
	 * to z20 and calculuses named res1 to res2
	 */
	public final void saveData() {
		appletManager.sauveValeurs(this);
	}

	/**
	 * public method retrieving values or formulas of calculuses or functions
	 * named z1 to z20 and calculuses named res1 to res2 The values or formula
	 * retrieved must have been saved before with saveData() method
	 */
	public final void retrieveData() {
		appletManager.restitueValeurs(this);
	}

	/**
	 * Fonction ajoutant le bouton de proposition de solution au mileiu et en bas
	 * dans le cas d'un exercice de construction
	 */
	private void ajouteBoutonPropSol() {
		// Modifi version 4.7.2 : Activ seulement si allowModification est true
		if (allowSolutionButton) {
			buttonPropSol = outilPropSol.boutonAssocie();
			cadreActif.add(buttonPropSol, JLayeredPane.DRAG_LAYER);
			cadreActif.springl.putConstraint(SpringLayout.SOUTH, buttonPropSol, 0,
					SpringLayout.SOUTH, cadreActif);
			cadreActif.springl.putConstraint(SpringLayout.HORIZONTAL_CENTER,
					buttonPropSol, 0, SpringLayout.HORIZONTAL_CENTER, cadreActif);
			validate(); // Ajoutt version 4.7.2
		}
	}

	public void retireBoutonPropSol() {
		if (buttonPropSol != null) {
			cadreActif.remove(buttonPropSol);
			buttonPropSol = null;
		}
	}

	/**
	 * Renvoie la nature des objets dont on peut crer l'image par une
	 * transformation Les points sont autoriss. Ne sert que pour un exercice de
	 * construction Les droites sont autorises si la macro macroPourCons un
	 * commentaire qui contient la squence de caractres ImageDroite. Idem pour
	 * les demi-droites cercles et arcs de cercles
	 * 
	 * @return
	 */
	public long natureObjetsPourImages() {
		if (!(cadreActif.isConstruction && allowExercise))
			return NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur
					| NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NPolygone
					| NatureObjet.NLigneBrisee;
		long res = NatureObjet.NToutPoint;
		if ((chaineIndicationPourCons.indexOf(getString("imageDroite")) != -1)
				|| (chaineIndicationPourCons.indexOf("imageDroite") != -1))
			res |= NatureObjet.NDroite;
		if ((chaineIndicationPourCons.indexOf(getString("imageDemiDroite")) != -1)
				|| (chaineIndicationPourCons.indexOf("imageDemiDroite") != -1))
			res |= NatureObjet.NDemiDroite;
		if ((chaineIndicationPourCons.indexOf(getString("imageCercle")) != -1)
				|| (chaineIndicationPourCons.indexOf("imageCercle") != -1))
			res |= NatureObjet.NCercle;
		if ((chaineIndicationPourCons.indexOf(getString("imageArc")) != -1)
				|| (chaineIndicationPourCons.indexOf("imageArc") != -1))
			res |= NatureObjet.NArc;
		return res;
	}

	/**
	 * Ajout version 4.6. Met  vide touys les champs d'dition
	 */
	public void setEditorsEmpty() {
		CListeObjets listePr = cadreActif.listePr;
		ListIterator<CElementBase> li = listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) li.next();
			if (elb.getNature() == NatureObjet.NEditeurFormule) {
				CEditeurFormule ed = (CEditeurFormule) elb;
				ed.mtgFormulaEditor.setText("");
				ed.formula = null; // Ajout version 4.7.2
			}
		}
	}

	public final boolean isReady() {
		return isReady;
	}
	
	/**
	 * Fonction chargeant une icne dans le dossier images de mathgraph32
	 * @param iconName
	 * @return
	 */
	private ImageIcon loadIcon(String iconName) {
    URL adr = MtgFrame.class.getResource("images/" + iconName + ".png");
    return new ImageIcon(adr);
	}
	
	private void loadToolsIcons() {
		if(iconCreerPointLibre == null) iconCreerPointLibre = loadIcon("PointLibre");
		if(iconCreerPointLibreEnt == null) iconCreerPointLibreEnt = loadIcon("PointLibreEnt");
		if(iconCreerPointLie == null) iconCreerPointLie = loadIcon("PointLie");
		if(iconCreerMilieu == null) iconCreerMilieu = loadIcon("Milieu");
		if(iconCreerProjeteOrthogonal == null) iconCreerProjeteOrthogonal = loadIcon("Projete");
		if(iconCreerPointParAbscisse == null) iconCreerPointParAbscisse = loadIcon("PointParAbs");
		if(iconCreerPointParCoordonnees == null) iconCreerPointParCoordonnees = loadIcon("PointParCoordonnees");
		if(iconCreerPointParAffixe == null) iconCreerPointParAffixe = loadIcon("PointParAff");
		if(iconCreerCentreGravite == null) iconCreerCentreGravite = loadIcon("CentreGravite");
		if(iconCreerPointParSommeVect == null) iconCreerPointParSommeVect = loadIcon("PointParSommeVect");
		if(iconCreerPointParMultVect == null) iconCreerPointParMultVect = loadIcon("PointParMultVect");
		if(iconCreerBarycentre == null) iconCreerBarycentre = loadIcon("Barycentre");
		if(iconCreerPointInterieur == null) iconCreerPointInterieur = loadIcon("PointInterieurPolygone");
		if(iconCurseur == null) iconCurseur = loadIcon("Curseur");
		if(iconCreerIntersection == null) iconCreerIntersection = loadIcon("Int");
		if(iconCreerDroiteAB == null) iconCreerDroiteAB = loadIcon("DteAB");
		if(iconCreerDroiteParallele == null) iconCreerDroiteParallele = loadIcon("DtePar");
		if(iconCreerDroitePerpendiculaire == null) iconCreerDroitePerpendiculaire = loadIcon("DtePer");
		if(iconCreerDroiteBissectrice == null) iconCreerDroiteBissectrice = loadIcon("DteBis");
		if(iconCreerDroiteMediatrice == null) iconCreerDroiteMediatrice = loadIcon("DteMed");
		if(iconCreerDroiteHorizontale == null) iconCreerDroiteHorizontale = loadIcon("DteHor");
		if(iconCreerDroiteVerticale == null) iconCreerDroiteVerticale = loadIcon("DteVer");
		if(iconCreerDroiteParCoef == null) iconCreerDroiteParCoef = loadIcon("DteParCoef");
		if(iconCreerDroiteParEquation == null) iconCreerDroiteParEquation = loadIcon("DteParEq");
		if(iconCreerSegment == null) iconCreerSegment = loadIcon("Segment");
		if(iconCreerDemiDroite == null) iconCreerDemiDroite = loadIcon("DemiDt");
		if(iconCreerVecteur == null) iconCreerVecteur = loadIcon("Vecteur");
		if(iconCreerCercleOA == null) iconCreerCercleOA = loadIcon("CercOA");
		if(iconCreerCercleOR == null) iconCreerCercleOR = loadIcon("CercOR");
		if(iconCreerPolygone == null) iconCreerPolygone = loadIcon("Polygone");
		if(iconPolygoneRegulier == null) iconPolygoneRegulier = loadIcon("PolygoneRegulier");
		if(iconParallelogramme == null) iconParallelogramme = loadIcon("Parallelogramme");
		if(iconCarreDirect == null) iconCarreDirect = loadIcon("CarreDirect");
		if(iconTriangleEquilateralDirect == null) iconTriangleEquilateralDirect = loadIcon("TriangleEquilateralDirect");
		if(iconCreerLigneBrisee == null) iconCreerLigneBrisee = loadIcon("LigneBrisee");
		if(iconCreerArcPetitParPointFinal == null) iconCreerArcPetitParPointFinal = loadIcon("ArcPetitParPoint");
		if(iconCreerArcPetitParRapporteur == null) iconCreerArcPetitParRapporteur = loadIcon("ArcPetitParRapporteur");
		if(iconCreerArcGrandParPointFinal == null) iconCreerArcGrandParPointFinal = loadIcon("ArcGrandParPoint");
		if(iconCreerArcGrandParRapporteur == null) iconCreerArcGrandParRapporteur = loadIcon("ArcGrandParRapporteur");
		if(iconCreerArcDirectParPointFinal == null) iconCreerArcDirectParPointFinal = loadIcon("ArcDirectParPoint");
		if(iconCreerArcIndirectParPointFinal == null) iconCreerArcIndirectParPointFinal = loadIcon("ArcIndirectParPoint");
		if(iconCreerMarqueAngleNonOriente == null) iconCreerMarqueAngleNonOriente = loadIcon("MarqueAngNor");
		if(iconCreerMarqueAngleOriente == null) iconCreerMarqueAngleOriente = loadIcon("MarqueAngOr");
		if(iconCreerMarqueSegment == null) iconCreerMarqueSegment = loadIcon("MarqueSegment");
		if(iconMesureLongueur == null) iconMesureLongueur = loadIcon("MesLong");
		if(iconMesureAbscisse == null) iconMesureAbscisse = loadIcon("MesAbs");
		if(iconMesureAbscisseDansRepere == null) iconMesureAbscisseDansRepere = loadIcon("MesAbsRep");
		if(iconMesureOrdonneeDansRepere == null) iconMesureOrdonneeDansRepere = loadIcon("MesOrdRep");
		if(iconMesurerCoefficientDirecteur == null) iconMesurerCoefficientDirecteur = loadIcon("MesCoefDir");
		if(iconMesureLongueurLigne == null) iconMesureLongueurLigne = loadIcon("MesureLongueurLigne");
		if(iconMesureAirePolygone == null) iconMesureAirePolygone = loadIcon("MesureAire");
		if(iconMesureAffixe == null) iconMesureAffixe = loadIcon("MesAff");
		if(iconMesureAngleNonOriente == null) iconMesureAngleNonOriente = loadIcon("MesAngNor");
		if(iconMesureAngleOriente == null) iconMesureAngleOriente = loadIcon("MesAngOr");
		if(iconMesureProduitScalaire == null) iconMesureProduitScalaire = loadIcon("ProduitScalaire");
		if(iconCreerImageParTranslation == null) iconCreerImageParTranslation = loadIcon("Translation");
		if(iconCreerImageParTranslationParCoord == null) iconCreerImageParTranslationParCoord = loadIcon("TranslationParCoord");
		if(iconCreerImageParSymetrieAxiale == null) iconCreerImageParSymetrieAxiale = loadIcon("SymAxiale");
		if(iconCreerImageParSymetrieCentrale == null) iconCreerImageParSymetrieCentrale = loadIcon("SymCentrale");
		if(iconCreerImageParRotation == null) iconCreerImageParRotation = loadIcon("Rotation");
		if(iconCreerImageParHomothetie == null) iconCreerImageParHomothetie = loadIcon("Homothetie");
		if(iconCreerImageParSimilitude == null) iconCreerImageParSimilitude = loadIcon("Similitude");
		if(iconRapporteur == null) iconRapporteur = loadIcon("Rapporteur");
		if(iconCreerAffichageValeur == null) iconCreerAffichageValeur = loadIcon("AffichageValeur");
		if(iconCreerAffichageValeurLiePoint == null) iconCreerAffichageValeurLiePoint = loadIcon("AffichageValeurLie");
		if(iconCreerCommentaire == null) iconCreerCommentaire = loadIcon("Commentaire");
		if(iconCreerCommentaireLiePoint == null) iconCreerCommentaireLiePoint = loadIcon("CommentaireLie");
		if(iconCreerLatex == null) iconCreerLatex = loadIcon("Latex");
		if(iconCreerLatexLiePoint == null) iconCreerLatexLiePoint = loadIcon("LatexLie");
		if(iconCreerImage == null) iconCreerImage = loadIcon("Image");
		if(iconCreerImageLie == null) iconCreerImageLie = loadIcon("ImageLie");
		if(iconCreerEquationDroite == null) iconCreerEquationDroite = loadIcon("EquationDroite");
		if(iconCreerEquationDroiteLie == null) iconCreerEquationDroiteLie = loadIcon("EquationDroiteLie");
		if(iconCreerEditeurFormule == null) iconCreerEditeurFormule = loadIcon("EditeurFormule");
		if(iconCreerCalcul == null) iconCreerCalcul = loadIcon("Calcul");
		if(iconCreerRepere == null) iconCreerRepere = loadIcon("Repere");
		if(iconCreerLieuParPointLie == null) iconCreerLieuParPointLie = loadIcon("Lieu");
		if(iconCreerFonctionReelle == null) iconCreerFonctionReelle = loadIcon("Fonction");
		if(iconCreerCourbeFonction == null) iconCreerCourbeFonction = loadIcon("CourbeFonction");
		if(iconCreerPointSurCourbeParAbscisse == null) iconCreerPointSurCourbeParAbscisse = loadIcon("PointSurCourbeParAbscisse");
		if(iconTangente == null) iconTangente = loadIcon("Tangente");
		if(iconCreerSurface == null) iconCreerSurface = loadIcon("Surface");
		if(iconCreerSurfaceLieuDroite == null) iconCreerSurfaceLieuDroite = loadIcon("SurfaceLieuDroite");
		if(iconCreerSurfaceLieuDeuxPoints == null) iconCreerSurfaceLieuDeuxPoints = loadIcon("SurfaceLieuDeuxPoints");
		if(iconCreerSurfaceDeuxLieux == null) iconCreerSurfaceDeuxLieux = loadIcon("SurfaceDeuxLieux");
		if(iconPunaiser == null) iconPunaiser = loadIcon("Punaiser");
		if(iconDepunaiser == null) iconDepunaiser = loadIcon("Depunaiser");
		if(iconSegmentParLongueur == null) iconSegmentParLongueur = loadIcon("SegmentParLongueur");
		if(iconAnnuler == null) iconAnnuler = loadIcon("Undo24");
		if(iconRefaire == null) iconRefaire = loadIcon("Redo24");
		if(iconVoirObjetsNumeriques == null) iconVoirObjetsNumeriques = loadIcon("VoirCalculs");
		if(iconCapture == null) iconCapture = loadIcon("Main24");
		if(iconCopier == null) iconCopier = loadIcon("Copy24");
		if(iconExecuterMacro == null) iconExecuterMacro = loadIcon("Macro");
		if(iconExporterAvecUnite == null) iconExporterAvecUnite = loadIcon("ExportUnite");
		if(iconNouvelleFigure == null) iconNouvelleFigure = loadIcon("New24");
		if(iconNouvelleFigureAvecRepere == null) iconNouvelleFigureAvecRepere = loadIcon("RepQuadrille");
		if(iconOuvrir == null) iconOuvrir = loadIcon("Open24");
		if(iconGomme == null) iconGomme = loadIcon("Gomme");
		if(iconTexteFigure == null) iconTexteFigure = loadIcon("TexteFigure");
		if(iconHistorique == null) iconHistorique = loadIcon("Historique24");
		if(iconModifierFormule == null) iconModifierFormule = loadIcon("ModificationObjetNumerique");
		if(iconNommer == null) iconNommer = loadIcon("Nommer");
		if(iconModifierObjetGraphique == null) iconModifierObjetGraphique = loadIcon("ModificationObjetGraphique");
		if(iconPalette == null) iconPalette = loadIcon("Palette");
		if(iconReafficher == null) iconReafficher = loadIcon("Reafficher");
		if(iconRideau == null) iconRideau = loadIcon("Rideau");
		if(iconSupprimerObjetGraphique == null) iconSupprimerObjetGraphique = loadIcon("Delete24");
		if(iconSupprimerObjetNumerique == null) iconSupprimerObjetNumerique = loadIcon("DeleteCalcul24");
		if(iconTranslaterFigure == null) iconTranslaterFigure = loadIcon("Translation24");
		if(iconZoomPlus == null) iconZoomPlus = loadIcon("ZoomIn24");
		if(iconZoomMoins == null) iconZoomMoins = loadIcon("ZoomOut24");
		if(iconDemarquerPoint == null) iconDemarquerPoint = loadIcon("DemarquerPt");
		if(iconMarquerPoint == null) iconMarquerPoint = loadIcon("MarquerPt");
		if(iconActiverModeTrace == null) iconActiverModeTrace = loadIcon("Traces24");
		if(iconRelationEntreObjets == null) iconRelationEntreObjets = loadIcon("Relation");
	}
}